# -*- coding: utf-8 -*-

from report import report_sxw
from datetime import datetime
import os


class report_absence_control(report_sxw.rml_parse):
    _name = 'report.absence.control'

    def __init__(self, cr, uid, name, context):
        super(report_absence_control, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'get_empresa': self.get_empresa,
            'get_final_date': self.get_final_date,
            'format_sign_ins': self.format_sign_ins,
            'emp_data': self.emp_data
        })
        self.context = context

    def emp_data(self, employee_cod, day_id):
        employee_id = self.pool.get('absence.control.wizard.lines').search(self.cr, self.uid, [('employee_ids', '=', day_id), ('cod_number', '=', employee_cod)])[0]
        employee_data = self.pool.get('absence.control.wizard.lines').browse(self.cr, self.uid, employee_id)
        return employee_data

    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id = obj_addr.search(self.cr, self.uid, [('type', '=', 'invoice'), ('partner_id', '=', partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
                'street': row.street,
                'phone': row.phone,
                'fax': row.fax,
                'email': row.email,
                'city': row.city,
                'name': row.name,
                'country': row.country_id.name,
            }
        return res

    def get_final_date(self):
        return datetime.now().strftime('%d/%m/%Y %H:%M')

    def format_sign_ins(self, sign_in):
        if isinstance(sign_in, basestring):
            if sign_in != 'None':
                return datetime.strptime(sign_in, '%Y-%m-%d %H:%M:%S').strftime('%a %H:%M:%S')
        else:
            if sign_in != None:
                return sign_in.strftime('%a %H:%M:%S')


report_sxw = report_sxw.report_sxw


class TxtParser(report_sxw):
    """Custom class that use webkit to render HTML reports
       Code partially taken from report openoffice. Thanks guys :)
    """

    def __init__(self, name, table, rml=False, parser=False,
                 header=True, store=False):
        self.parser_instance = False
        self.localcontext = {}
        report_sxw.__init__(self, name, table, rml, parser,
                            header, store)

    def create_single_pdf(self, cursor, uid, ids, data, report_xml, context):
        """generate the PDF"""

        if context is None:
            context = {}

        if data['is_pdf']:
            return super(TxtParser, self).create_single_pdf(cursor, uid, ids, data, report_xml, context=context)

        self.parser_instance = self.parser(
            cursor,
            uid,
            self.name2,
            context=context
        )
        if data['is_csv']:
            ret_file_name = open(data['ret_file_name'], 'r')
            salida_csv = ret_file_name.read()
            os.remove(data['ret_file_name'])
            return (salida_csv, 'csv')


TxtParser(
    'report.absence.control',
    'absence.control.wizard',
    'diana_control_ausencias/report/report_absence_control.rml',
    parser=report_absence_control, header=False)

TxtParser(
    'report.absence.control.by.emp',
    'absence.control.wizard',
    'diana_control_ausencias/report/report_absence_control_by_emp.rml',
    parser=report_absence_control, header=False)
