# -*- coding: utf-8 -*-

from osv import fields, osv
from datetime import datetime, timedelta
import unicodedata, codecs
from tools.translate import _
import sys


class biostar_fingerprint(osv.osv):
    _inherit = 'biostar.fingerprint'
    _columns = {
        'absence_control': fields.boolean('Absence Control'),
    }
    _defaults = {
        'absence_control': lambda *a: False,
    }


biostar_fingerprint()


class absence_control_wizard(osv.osv_memory):
    def _default_employees(self, cr, uid, context=None):
        if context['active_model'] == 'hr.employee':
            return context['active_ids']
        else:
            return []

    def get_departments(self, cr, uid, context=None):
        th_group_model_data_id = self.pool.get('ir.model.data').search(cr, uid, [('name', '=', 'group_signins_interface')])
        th_group = self.pool.get('ir.model.data').browse(cr, uid, th_group_model_data_id[0]).res_id
        groups = [group.id for group in self.pool.get('res.users').browse(cr, uid, uid).groups_id]
        if th_group in groups or uid == 1:
            return str(self.pool.get('hr.department').search(cr, uid, []))
        else:
            emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)])
            if len(emp_id) < 1:
                raise osv.except_osv('ERROR', _('%s has no employee profile configured') % (self.pool.get('res.users').browse(cr, uid, uid).name))
            else:
                emp_id = emp_id[0]
            childs = [dept.id for dept in self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.child_ids]
            pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
            while len(pending_childs) > 0:
                next_round = []
                for child in pending_childs:
                    childs += [c.id for c in child.child_ids]
                    next_round += [c.id for c in child.child_ids]
                pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
            childs.append(self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.id)
            return str(childs)

    _name = 'absence.control.wizard'
    _description = 'Employee absence control.'
    _columns = {
        'start_date': fields.date('Start date', required=True),
        'end_date': fields.date('End date', required=True),
        'employees': fields.many2many('hr.employee', 'absence_control_employee_rel', 'absence_control_id', 'employee_id', 'Employees'),
        'data_per_day': fields.one2many('absence.control.wizard.day', 'data_day_id', 'Data per day'),
        'group_by_employee': fields.boolean('Group by employee', help=_('If set, the report will be grouped by employee, showing all the days after each employee, else it will be grouped by default, showing each day and the employee who came that day.')),
        'is_pdf': fields.boolean('PDF'),
        'is_csv': fields.boolean('CSV'),
        'department_id': fields.many2one('hr.department', 'Department'),
        'allowed_depts': fields.text('Allowed Departments')
    }
    _defaults = {
        'is_pdf': lambda *a: True,
        'employees': _default_employees,
        'allowed_depts': lambda self, cr, uid, context=None: self.get_departments(cr, uid, context=context),
        'group_by_employee': lambda *a: False
    }

    def add_dept(self, cr, uid, ids, context=None):
        data = self.browse(cr, uid, ids)[0]
        if not data.department_id:
            raise osv.except_osv('ERROR', _('You must select a department'))
        childs = [data.department_id.id]
        pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
        while len(pending_childs) > 0:
            next_round = []
            for child in pending_childs:
                childs += [c.id for c in child.child_ids]
                next_round += [c.id for c in child.child_ids]
            pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
        employees_ids = [emp.id for emp in data.employees]
        employees_ids += self.pool.get('hr.employee').search(cr, uid, [('state', 'not in', ['egresado', 'borrador']), ('department_id', 'in', childs), ('id', 'not in', employees_ids)], context=context)
        return self.write(cr, uid, ids, {'department_id': None, 'employees': [(6, 0, employees_ids)]})

    def clean_list(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'employees': [(6, 0, [])]})

    def print_results(self, cr, uid, ids, context=None):
        # Max delay must be set
        max_delay_in = timedelta(minutes=75)
        max_delay_out = timedelta(minutes=45)
        # Getting data from wizard
        data = self.browse(cr, uid, ids)[0]
        wiz_id = data.id
        start_date = data.start_date
        end_date = data.end_date
        idsw = [emp.id for emp in data.employees]

        # Check if there are employees selected
        if len(idsw) == 0:
            raise osv.except_osv('ERROR', _('There are no employees loaded'))

        # Getting the absence_control_wizard_lines object
        acwl_obj = self.pool.get('absence.control.wizard.lines')
        acwd_obj = self.pool.get('absence.control.wizard.day')

        # Checking csv or pdf is selected
        if not data.is_csv and not data.is_pdf:
            raise osv.except_osv('ERROR', _('You must check between PDF or CSV.'))

        # Getting conection with the biostar SQL database
        conection = self.conection(cr, uid, ids, context=None)
        self.event = self.pool.get('biostar.event.data').even_id(cr, uid, ids)
        if not self.event:
            raise osv.except_osv('ERROR', _('There are no events set on fingerprint reader settings.'))

        # Getting fingerprint reader's ids
        driver = self.reader_id(cr, uid, ids)
        if not driver:
            raise osv.except_osv('ERROR', _('There are no fingerprint readers or you have not selected anything on settings.'))

        # Getting employees codes
        employees = self.pool.get('hr.employee').browse(cr, uid, idsw)
        employees_codes = []
        employee_data = {}
        for e in employees:
            employees_codes.append(e.cod_number)
            employee_data[e.cod_number] = e  # Also saving each employee data for easy access trough its code

        # Setting the query
        search_tuple = self.list_2_tuple(employees_codes)
        sql = self.query(search_tuple, start_date, end_date, driver)
        conection.execute(sql)

        # Fetching data from the query
        rows = conection.fetchall()
        if not rows:
            raise osv.except_osv('ERROR', _('There are no results since %s to %s.') % (start_date, end_date))

        # Reading data from rows
        reader_data = {}  # Saves whom employees came which day
        employee_signs = {}  # Saves all sign ins for each employee
        employee_sign_ins = {}  # Saving the first and last sing in for each employee
        # row: (hour, date, reader.name, cod_number, employee.name)
        for row in rows:
            """ Saving data as dict, where every day is the key and its value is a
                list of the employee codes whom came to work that day
            """
            try:
                if row[3] not in reader_data[row[1]]:
                    reader_data.setdefault(row[1], []).append(row[3])
            except:
                reader_data.setdefault(row[1], []).append(row[3])

            """ Saving data as dict, where the key is a mix between the employee cod_number
                and the date, ex: '104698-05-02-2017' and its values are a list with the hours
                at which the employee signed in that day.
            """
            employee_signs.setdefault(str(row[3] + '-' + row[1]), []).append(row[1] + '-' + row[0])

        # Iterating over each day from the period
        sd = datetime.strptime(start_date, '%Y-%m-%d')
        ed = datetime.strptime(end_date, '%Y-%m-%d')
        cur_day = sd

        data_to_rml = {}
        data_to_rml['is_pdf'] = data.is_pdf
        data_to_rml['is_csv'] = data.is_csv

        everyday_data_ids = []
        while cur_day <= ed:
            """ Checkig which worker came per day, getting the difference
                between the codes of each day from reader_data and
                employees_codes
            """
            if cur_day.weekday() == 0:
                yesterday_day_name = 'sunday'
                day_name = 'monday'
                tomorrow_day_name = 'tuesday'
            elif cur_day.weekday() == 1:
                yesterday_day_name = 'monday'
                day_name = 'tuesday'
                tomorrow_day_name = 'wednesday'
            elif cur_day.weekday() == 2:
                yesterday_day_name = 'tuesday'
                day_name = 'wednesday'
                tomorrow_day_name = 'thursday'
            elif cur_day.weekday() == 3:
                yesterday_day_name = 'wednesday'
                day_name = 'thursday'
                tomorrow_day_name = 'friday'
            elif cur_day.weekday() == 4:
                yesterday_day_name = 'thursday'
                day_name = 'friday'
                tomorrow_day_name = 'saturday'
            elif cur_day.weekday() == 5:
                yesterday_day_name = 'friday'
                day_name = 'saturday'
                tomorrow_day_name = 'sunday'
            elif cur_day.weekday() == 6:
                yesterday_day_name = 'saturday'
                day_name = 'sunday'
                tomorrow_day_name = 'monday'

            today_rows_query_turn = self.query_employee_turn(cr, cur_day, day_name, idsw)
            employees_turns = {}
            # row2: (cod_number, rota, turn, ci, sede, department, desde, hasta)
            for row2 in today_rows_query_turn:
                employees_turns[row2[0]] = [row2[1], row2[2], row2[3], row2[4], row2[5], row2[6], row2[7], row2[8]]

            yesterday = cur_day - timedelta(days=1)
            yesterday_rows_query_turn = self.query_employee_turn(cr, yesterday, yesterday_day_name, idsw)
            employees_past_turns = {}
            for row2 in yesterday_rows_query_turn:
                employees_past_turns[row2[0]] = [row2[1], row2[2], row2[3], row2[4], row2[5], row2[6], row2[7], row2[8]]

            tomorrow = cur_day + timedelta(days=1)
            tomorrow_rows_query_turn = self.query_employee_turn(cr, tomorrow, tomorrow_day_name, idsw)
            employees_after_turns = {}
            for row2 in tomorrow_rows_query_turn:
                employees_after_turns[row2[0]] = [row2[1], row2[2], row2[3], row2[4], row2[5], row2[6], row2[7], row2[8]]

            absence_list_per_day = []
            assisted_list_per_day = []
            assistance_list_ids = []

            for emp in employees_codes:
                try:
                    """ In case there is no turn assigned to that employee,
                        the employee will be marked as absent.
                    """
                    flex_start_hour = datetime.strptime(cur_day.strftime('%Y-%m-%d') + '-' + employees_turns[emp][5], '%Y-%m-%d-%H:%M:%S') - timedelta(hours=1)
                    flex_end_hour = datetime.strptime(cur_day.strftime('%Y-%m-%d') + '-' + employees_turns[emp][6], '%Y-%m-%d-%H:%M:%S') + timedelta(minutes=30)

                    if employees_past_turns[emp][5] > employees_past_turns[emp][6]:
                        last_end_hour = datetime.strptime(cur_day.strftime('%Y-%m-%d') + '-' + employees_past_turns[emp][6], '%Y-%m-%d-%H:%M:%S') + timedelta(minutes=30)
                    else:
                        last_end_hour = datetime.strptime(cur_day.strftime('%Y-%m-%d') + '-' + employees_past_turns[emp][6], '%Y-%m-%d-%H:%M:%S') + timedelta(minutes=30) - timedelta(days=1)
                    if employees_after_turns[emp][5] == '00:00:00' and employees_after_turns[emp][6] == '00:00:00':
                        next_start_hour = datetime.strptime(cur_day.strftime('%Y-%m-%d') + '-23:59:59', '%Y-%m-%d-%H:%M:%S') - timedelta(hours=1) + timedelta(days=1)
                    else:
                        next_start_hour = datetime.strptime(cur_day.strftime('%Y-%m-%d') + '-' + employees_after_turns[emp][5], '%Y-%m-%d-%H:%M:%S') - timedelta(hours=1) + timedelta(days=1)

                    assisted = False
                    try:
                        # first_one, last_one: the ones on turn
                        # f_pt, l_pt: the ones past turn
                        # f_ot, l_ot: the ones over turn
                        first_one = None
                        last_one = None
                        f_pt = None
                        l_pt = None
                        f_ot = None
                        l_ot = None
                        employee_current_signs = []

                        if flex_start_hour + timedelta(hours=1) < flex_end_hour - timedelta(minutes=30):
                            # employee_current_signs = employee_signs[str(emp+'-'+cur_day.strftime('%Y-%m-%d'))]
                            for sign in employee_signs[str(emp + '-' + cur_day.strftime('%Y-%m-%d'))]:
                                sign_in_f = datetime.strptime(sign, '%Y-%m-%d-%H:%M:%S')
                                if sign_in_f > last_end_hour:
                                    employee_current_signs.append(sign)
                            # try:
                            #     for sign in employee_signs[str(emp+'-'+(cur_day+timedelta(days=1)).strftime('%Y-%m-%d'))]:
                            #         sign_in_f = datetime.strptime(sign,'%Y-%m-%d-%H:%M:%S')
                            #         if sign_in_f < next_start_hour:
                            #             employee_current_signs.append(sign)
                            # except:
                            #     pass

                        else:
                            flex_end_hour += timedelta(days=1)
                            for sign in employee_signs[str(emp + '-' + cur_day.strftime('%Y-%m-%d'))]:
                                sign_in_f = datetime.strptime(sign, '%Y-%m-%d-%H:%M:%S')
                                if sign_in_f > last_end_hour:
                                    employee_current_signs.append(sign)
                            try:
                                for sign in employee_signs[str(emp + '-' + (cur_day + timedelta(days=1)).strftime('%Y-%m-%d'))]:
                                    sign_in_f = datetime.strptime(sign, '%Y-%m-%d-%H:%M:%S')
                                    if sign_in_f < next_start_hour:
                                        employee_current_signs.append(sign)
                            except:
                                pass

                        for sign_in in employee_current_signs:
                            # sign_in_f: formated datetime sign_in
                            sign_in_f = datetime.strptime(sign_in, '%Y-%m-%d-%H:%M:%S')
                            # Checking if current sign-in is between the employee current turn
                            if sign_in_f >= flex_start_hour and sign_in_f <= flex_end_hour:
                                if first_one:
                                    if sign_in_f < first_one and not last_one:
                                        last_one = first_one
                                        first_one = sign_in_f
                                    elif sign_in_f < first_one and last_one:
                                        if first_one > last_one:
                                            last_one = first_one
                                        first_one = sign_in_f
                                    elif not last_one:
                                        last_one = sign_in_f
                                    elif sign_in_f > last_one:
                                        last_one = sign_in_f
                                else:
                                    first_one = sign_in_f
                                # Checking if current sign-in is not between the employee past turn
                            else:
                                # if current sign-in is before the current turn
                                if sign_in_f < flex_start_hour:
                                    if not f_pt:
                                        f_pt = sign_in_f
                                    elif sign_in_f < f_pt:
                                        f_pt = sign_in_f
                                    elif not l_pt:
                                        l_pt = sign_in_f
                                    elif sign_in_f > l_pt:
                                        l_pt = sign_in_f
                                else:
                                    if not f_ot:
                                        f_ot = sign_in_f
                                    elif sign_in_f < f_ot:
                                        f_ot = sign_in_f
                                    elif not l_ot:
                                        l_ot = sign_in_f
                                    elif sign_in_f > l_ot:
                                        l_ot = sign_in_f

                            # ***** assisted or not *****
                            if sign_in_f >= flex_start_hour and sign_in_f <= flex_end_hour:
                                assisted = True

                        if f_pt and l_ot:
                            emp_si, emp_so, note_si, note_so = f_pt, l_ot, _('Overtime'), _('Overtime')
                        elif f_pt and f_ot:
                            emp_si, emp_so, note_si, note_so = f_pt, f_ot, _('Overtime'), _('Overtime')
                        elif f_pt and last_one:
                            if last_one > flex_end_hour - max_delay_out:
                                emp_si, emp_so, note_si, note_so = f_pt, last_one, _('Overtime'), None
                            else:
                                emp_si, emp_so, note_si, note_so = f_pt, last_one, _('Overtime'), _('Not finished')
                        elif f_pt and first_one:
                            emp_si, emp_so, note_si, note_so = f_pt, first_one, _('Overtime'), _('Not finished')
                        elif f_pt and l_pt:
                            emp_si, emp_so, note_si, note_so = f_pt, l_pt, _('Turn changed'), _('Turn changed')
                        elif f_pt:
                            emp_si, emp_so, note_si, note_so = f_pt, None, _('Overtime'), _('No sign in')
                        elif first_one and l_ot:
                            if first_one < flex_start_hour + max_delay_in:
                                emp_si, emp_so, note_si, note_so = first_one, l_ot, None, _('Overtime')
                            else:
                                emp_si, emp_so, note_si, note_so = first_one, l_ot, _('Delayed'), _('Overtime')
                        elif first_one and f_ot:
                            if first_one < flex_start_hour + max_delay_in:
                                emp_si, emp_so, note_si, note_so = first_one, f_ot, None, _('Overtime')
                            else:
                                emp_si, emp_so, note_si, note_so = first_one, f_ot, _('Delayed'), _('Overtime')
                        elif first_one and last_one:
                            if first_one < flex_start_hour + max_delay_in and last_one > flex_end_hour - max_delay_out:
                                emp_si, emp_so, note_si, note_so = first_one, last_one, None, None
                            elif first_one < flex_start_hour + max_delay_in and last_one <= flex_end_hour - max_delay_out:
                                emp_si, emp_so, note_si, note_so = first_one, last_one, None, _('Not finished')
                            elif first_one >= flex_start_hour + max_delay_in and last_one > flex_end_hour - max_delay_out:
                                emp_si, emp_so, note_si, note_so = first_one, last_one, _('Delayed'), None
                            elif first_one >= flex_start_hour + max_delay_in and last_one <= flex_end_hour - max_delay_out:
                                emp_si, emp_so, note_si, note_so = first_one, last_one, _('Delayed'), _('Not finished')
                        elif first_one:
                            emp_si, emp_so, note_si, note_so = first_one, None, None, _('No sign in')
                        elif f_ot and l_ot:
                            emp_si, emp_so, note_si, note_so = f_ot, l_ot, _('Turn changed'), _('Turn changed')
                        elif f_ot:
                            emp_si, emp_so, note_si, note_so = f_ot, None, _('Overtime'), _('No sign in')
                        else:
                            emp_si, emp_so, note_si, note_so = None, None, _('Absent'), _('Absent')

                        # Saving employee data to a new line record
                        rota, turn, ci, site, department, start_hour, end_hour, holiday_status = employees_turns[emp]
                        emp_data = {
                            'cod_number': emp,
                            'identification': ci,
                            'name': employee_data[emp].name,
                            'site': site,
                            'department': department,
                            'turn': turn,
                            'rotation': rota,
                            'sign_in': emp_si,
                            'sign_out': emp_so,
                            'si_note': note_si if not employee_data[emp].no_sujeto_marcaje else None,
                            'so_note': note_so if not employee_data[emp].no_sujeto_marcaje else None,
                            'holiday': holiday_status
                        }
                        crl = acwl_obj.create(cr, uid, emp_data)
                        assistance_list_ids.append(crl)
                    except Exception, e:
                        # If there is an error, it's mostly because the employee didn't have any sign-in
                        rota, turn, ci, site, department, start_hour, end_hour, holiday_status = employees_turns[emp]
                        emp_data = {
                            'cod_number': emp,
                            'identification': ci,
                            'name': employee_data[emp].name,
                            'site': site,
                            'department': department,
                            'turn': turn,
                            'rotation': rota,
                            'sign_in': None,
                            'sign_out': None,
                            'si_note': _('Absent'),
                            'so_note': _('Absent'),
                            'holiday': holiday_status
                        }
                        crl = acwl_obj.create(cr, uid, emp_data)
                        assistance_list_ids.append(crl)
                except:
                    # If there is an error, it's because the employee had no turn for that day
                    emp_data = {
                        'cod_number': emp,
                        'identification': employee_data[emp].identification_id,
                        'name': employee_data[emp].name,
                        'site': employee_data[emp].hr_sede_id.name,
                        'department': employee_data[emp].department_id.name,
                        'turn': _('no turn'),
                        'rotation': _('no turn'),
                        'sign_in': None,
                        'sign_out': None,
                        'si_note': _('No info') if not employee_data[emp].no_sujeto_marcaje else None,
                        'so_note': _('No info') if not employee_data[emp].no_sujeto_marcaje else None,
                        'holiday': _('No info') if not employee_data[emp].no_sujeto_marcaje else None
                    }
                    crl = acwl_obj.create(cr, uid, emp_data)
                    assistance_list_ids.append(crl)

            this_day_data_id = acwd_obj.create(cr, uid, {
                'name': cur_day.strftime('%A %d/%m/%Y'),
                'employees_list': [(6, 0, assistance_list_ids)],
            })

            everyday_data_ids.append(this_day_data_id)
            cur_day += timedelta(days=1)
        self.pool.get('absence.control.wizard').write(cr, uid, [wiz_id], {'data_per_day': [(6, 0, everyday_data_ids)]})
        # In case csv is required
        if data.is_csv and not data.is_pdf:
            ret_file_name = '/tmp/tmp_marcaje_%s.csv' % (datetime.now().strftime('%H%M%S%d%m%Y'))
            data_to_rml['ret_file_name'] = ret_file_name

            header = _("Date,Code,Identification,Full name,Site,Department,Turn,Rotation,Sign In,Sign Out,Holidays\n")
            out = ''
            out += header
            # Checking the order
            if data.group_by_employee:
                for employee in self.pool.get('absence.control.wizard').browse(cr, uid, wiz_id).employees:
                    for day_row in self.pool.get('absence.control.wizard').browse(cr, uid, wiz_id).data_per_day:
                        fdate = datetime.strptime(day_row.name, '%A %d/%m/%Y').strftime('%d/%m/%Y')
                        emp = self.emp_data_day(cr, uid, employee.cod_number, day_row.id)
                        csv_si = str(emp.sign_in.strftime('%a %H:%M:%S')).decode('utf-8') + ' ' if emp.sign_in else ''
                        csv_si += emp.si_note if emp.si_note else ''
                        csv_so = str(emp.sign_out.strftime('%a %H:%M:%S')).decode('utf-8') + ' ' if emp.sign_out else ''
                        csv_so += emp.so_note if emp.so_note else ''
                        txt = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s' % (fdate, emp.cod_number, emp.identification, emp.name, emp.site, emp.department, emp.turn, emp.rotation, csv_si, csv_so, emp.holiday)
                        out += txt + '\n'
            else:
                for day_row in self.pool.get('absence.control.wizard').browse(cr, uid, wiz_id).data_per_day:
                    fdate = datetime.strptime(day_row.name, '%A %d/%m/%Y').strftime('%d/%m/%Y')
                    for emp in day_row.employees_list:
                        csv_si = str(emp.sign_in.strftime('%a %H:%M:%S')).decode('utf-8') + ' ' if emp.sign_in else ''
                        csv_si += emp.si_note if emp.si_note else ''
                        csv_so = str(emp.sign_out.strftime('%a %H:%M:%S')).decode('utf-8') + ' ' if emp.sign_out else ''
                        csv_so += emp.so_note if emp.so_note else ''
                        txt = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s' % (fdate, emp.cod_number, emp.identification, emp.name, emp.site, emp.department, emp.turn, emp.rotation, csv_si, csv_so, emp.holiday)
                        out += txt + '\n'
            temp_file = codecs.open(ret_file_name, 'wb+', 'utf-8')
            temp_file.write(out)
            temp_file.close()
        report_name = 'absence.control.by.emp' if data.group_by_employee else 'absence.control'
        return {
            'type': 'ir.actions.report.xml',
            'report_name': report_name,
            'datas': data_to_rml,
        }

    def reader_id(self, cr, uid, ids):
        dev_obj = self.pool.get('biostar.fingerprint')
        dev_id = dev_obj.search(cr, uid, ['&', ('valido', '=', True), ('absence_control', '=', True)])
        dev_brw = dev_obj.browse(cr, uid, dev_id)
        t = [n.reader_id for n in dev_brw]
        if len(t) > 1:
            t = 'IN %s' % str(tuple(t))
        elif len(t) == 1:
            t = '= %s' % t[0]
        else:
            t = ''
        return t

    def conection(self, cr, uid, ids, context=None):
        bio_obj = self.pool.get('biostar.settings')
        try:
            bio_conect = bio_obj.conexion(cr, uid, ids, context=None)
            cursor = bio_conect.cursor()
        except:
            raise osv.except_osv('ERROR', _('Could not establish communication with the server.'))

        return cursor

    def list_2_tuple(self, list):
        list = [str(i) for i in list]
        t = ()
        if len(list) == 1:
            t = "('%s')" % "".join(list)
        else:
            t = tuple(list)
        return str(t)

    def query(self, cod, fd, fh, t):
        cod = str(cod).strip()
        t = str(t).strip()

        sql = '''
        SELECT CONVERT(VARCHAR(11), DATEADD(s, nDateTime, '1970-01-01'),108), CONVERT(nVarchar(10), DATEADD(s, EVENT.nDateTime, '1970-01-01'), 20), READER.sName,  C.sFieldValue5
        FROM TB_EVENT_LOG AS EVENT
        JOIN TB_USER AS U ON EVENT.nUserID = U.sUserId
        JOIN TB_READER AS READER ON EVENT.nReaderIdn = READER.nReaderIdn
        JOIN TB_EVENT_DATA AS E_DATA ON EVENT.nEventIdn = E_DATA.nEventIdn
        JOIN TB_USER_CUSTOMINFO As C ON C.nUserIdn = U.nUserIdn
        WHERE E_DATA.nEventIdn %s
        AND CONVERT(nVarchar(10), DATEADD(s, EVENT.nDateTime, '1970-01-01'), 20) >= '%s'
        AND CONVERT(nVarchar(10), DATEADD(s, EVENT.nDateTime, '1970-01-01'), 20) <= '%s'
        AND READER.nReaderIdn  %s 

        AND C.sFieldValue5 IN %s
        ORDER BY C.sFieldValue5, CONVERT(nVarchar(10), DATEADD(s, EVENT.nDateTime, '1970-01-01'), 20),  CONVERT(VARCHAR(11), DATEADD(s, nDateTime, '1970-01-01'),108)
        ''' % (self.event, fd, fh, t, cod)
        return sql

    def switch_pdf_csv(self, cr, uid, ids, is_pdf, context=None):
        if is_pdf:
            return {'value': {'is_csv': False}}
        else:
            return {'value': {'is_pdf': False}}

    def emp_data_day(self, cr, uid, employee_cod, day_id):
        employee_id = self.pool.get('absence.control.wizard.lines').search(cr, uid, [('employee_ids', '=', day_id), ('cod_number', '=', employee_cod)])[0]
        employee_data = self.pool.get('absence.control.wizard.lines').browse(cr, uid, employee_id)
        return employee_data

    def query_employee_turn(self, cr, day, day_name, employees):
        day = day.strftime('%Y-%m-%d')
        query = '''
            SELECT
                distinct employees.cod_number AS "cod_number",
                rota.name AS "rota", 
                turn.name AS "turn",
                employees.identification_id AS "ci",
                sede.name AS "site",
                department.name AS "department",
                hours.hour_start AS "hour_start",
                hours.hour_end AS "hour_end",
                CASE
                    WHEN
                        holidays.date_from <= date '%s' and
                        holidays.date_to >= date '%s'
                    THEN holidays.type_holidays_status
                    ELSE 'Ninguno'
                END AS "holidays"
            FROM
                hr_hours_per_day AS rota
            JOIN
                hr_shift_week AS shifts ON shifts.%s_hours_day_id = rota.id
            JOIN
                hr_shift AS turn ON turn.id = shifts.shift_id
            JOIN
                hr_employee AS employees ON employees.id = shifts.employee_id
            JOIN
                hr_period AS periods ON periods.id = shifts.period_id
            JOIN
                hr_sede AS sede ON sede.id = employees.hr_sede_id
            JOIN
                hr_department AS department ON department.id = employees.department_id
            JOIN
                hr_hours AS hours ON hours.id = rota.%s_hours_id
            FULL JOIN hr_holidays AS holidays
                    ON holidays.employee_id = employees.id
                    and holidays.date_from <= date '%s'
                    and holidays.date_to >= date '%s'
            WHERE
                employees.id IN %s AND
                periods.date_start <= date '%s' AND
                periods.date_end >= date '%s'
        ''' % (day, day, day_name, day_name, day, day, self.list_2_tuple(employees), day, day)
        cr.execute(query)
        res = cr.fetchall()
        return res


absence_control_wizard()


class absence_control_wizard_lines(osv.osv_memory):
    _name = 'absence.control.wizard.lines'
    _description = 'Employee absence control data.'
    _columns = {
        'cod_number': fields.char('Employee Code', size=10),
        'identification': fields.char('Identification', size=10),
        'name': fields.char('Name', size=256),
        'site': fields.char('Site', size=128),
        'department': fields.char('Department', size=256),
        'turn': fields.char('Turn', size=128),
        'rotation': fields.char('Rotation', size=128),
        'sign_in': fields.datetime('Sign in'),
        'sign_out': fields.datetime('Sign out'),
        'si_note': fields.char('Sign in note', size=32),
        'so_note': fields.char('Sign out note', size=32),
        'holiday': fields.char('Holiday status', size=32),
        'employee_ids': fields.many2one('absence.control.wizard.day', 'Absent Employees'),
    }


absence_control_wizard_lines()


class absence_control_wizard_day(osv.osv_memory):
    _name = 'absence.control.wizard.day'
    _columns = {
        'name': fields.char('Date', size=32),
        'employees_list': fields.one2many('absence.control.wizard.lines', 'employee_ids', 'Absent employees'),
        'data_day_id': fields.many2one('absence.control.wizard')
    }


absence_control_wizard_day()
