#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from osv import fields, osv
import netsvc
import stock
import product
from datetime import date
import time
import datetime
from tools.translate import _
import unicodedata
import decimal_precision as dp

def elimina_tildes(s):
    """
    Funcion para eliminar las tildes de algun texto utilizando el modulo unicodedata.
    """
    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn')).replace("'","")
    

class caja_reposicion(osv.osv):
    _name = 'caja.reposicion'
    _description = 'Reposicion de Caja Chica'
caja_reposicion()


class caja_chica_configuracion(osv.osv):
    _name = 'caja.chica.configuracion'
    _descripcion = 'Configuraciones de Caja Chica'
    
    
    _columns = {
        'name': fields.char('Referencia', size=128, readonly=True),
        'max_reposicion': fields.integer('U.T. por Caja Chica', required=True),
        'max_linea': fields.integer('U.T. por Linea de Reposicion', required=True),
        'product_categ': fields.many2one('product.category', 'Categoria de Servicios', required=True),
    }
    
    _defaults = {
        'name': 'Configuracion para Caja Chica',
    }
    
caja_chica_configuracion()


class caja_chica(osv.osv):
    _name = "caja.chica"
    _description = "Caja Chica"
    
    
    def _get_caja_en_uso(self, cr, uid, ids, name, args, context=None):
        """
        Metodo para campo funcional se encarga de verificar si que la caja
        chica se encuentra en uso.
        """
        if context is None:
            context = {}
        
        repo_obj = self.pool.get('caja.reposicion')
        
        res = {}
        for id in ids:
            res[id] = True
            repo_sea = repo_obj.search(cr, uid, [('caja_chica_id','=',id),('state','in',('draft','confirm','done'))], context=context)
            if repo_sea == []:
                res[id] = False

        return res
    
    
    def _get_configuracion(self, cr, uid, context=None):
        """
        Funcion para extraer la configuracion para la caja chica
        """
        if context is None:
            context = {}
        
        fecha_actual = time.strftime('%Y-%m-%d')
        
        config_obj = self.pool.get('caja.chica.configuracion')
        config_ser = config_obj.search(cr, uid,[])
        if config_ser:
            return config_ser[0]
        return False
    
    
    def _calculate_amount(self, cr, uid, ids, field_names, arg=None, context=None):
        """ compute the Petty Cash Amount, Reposition Amount, Differential Amount, Balance Total
        for the provided petty cash
        Arguments:
        `ids`: patty cash ids
        `field_names`: the fields to compute (a list of any of
                       'cch_amount', 'repo_amount' 'diff_amount' and 'balance_total')
        `arg`: unused fields.function stuff
        """
        
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        for cch_id in id_brw:
            campo1 = 0
            campo2 = 0
            campo3 = 0
            campo4 = 0
            
            campo1 = cch_id.amount
            campo2 = cch_id.repo_id and cch_id.repo_id.amount_total or 0.0
            campo3 = cch_id.differential_move_id and cch_id.differential_move_id.amount or 0.0
            campo4 = campo1 - (campo2 + campo3)
            
            res[cch_id.id] = {'cch_amount': campo1, 'repo_amount': campo2, 'diff_amount': campo3, 'balance_total': campo4}
        
        return res
    
    _columns = {
        'name': fields.char('Referencia', size=128, required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'user_ids': fields.many2many('res.users', 'caja_chica_res_users_rel',
                            'caja_chica_id', 'user_id', 'Responsables',
                            required=True),
        'amount': fields.float('Monto Total', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'account_id': fields.many2one('account.account', 'Cuenta Contable', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'company_id': fields.many2one('res.company', 'Company', readonly=True),
        'journal_id': fields.many2one('account.journal', 'Diario de Pago', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'journal_move_id': fields.many2one('account.journal', 'Diario de Gasto', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'en_uso': fields.function(_get_caja_en_uso, method=True, string='En Uso', type='boolean'),
        'active': fields.boolean('Activa', help="Si este campo esta marcado la caja chica estara activa para ser usada", readonly=False, states={'opened':[('readonly',True)]}),
        'config': fields.many2one('caja.chica.configuracion', 'Configuracion', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'analytic_id': fields.many2one('account.analytic.account', 'Cuenta Analitica', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'fiscalyear_id': fields.many2one('account.fiscalyear', 'Fiscal Year', required=True, select=True, readonly=True, states={'draft':[('readonly',False)]}),
        'date_from': fields.date('Date From', required=True, select=True, readonly=True, states={'draft':[('readonly',False)]}),
        'date_to': fields.date('Date To', required=True, select=True, readonly=True, states={'draft':[('readonly',False)]}),
        'state': fields.selection([('draft','Draft'), ('opened','Opened'),('pre-closed','Pre Closed'), ('closed','Closed')], 'State', required=True, readonly=True,
            help='When you manually create a new record will be set up in state \' Borrardor \', setting the option to open it. In that case, the opening entry will be created and remain in the state \' Opened \' until the pre-closing and closing occurs, from petty cash to the state \' Closed \'.'),
        'move_id': fields.many2one('account.move', 'Open Move', readonly=True),
        'open_voucher_id': fields.many2one('account.voucher', 'Open Voucher', readonly=True),
        'close_move_id': fields.many2one('account.move', 'Close Move', readonly=True),
        'repo_id': fields.many2one('caja.reposicion', 'Reposition', readonly=True, states={'opened':[('readonly',False)]}),
        'differential_move_id': fields.many2one('account.move', 'Differential Move', readonly=True, states={'pre-closed':[('readonly',False)]}),
        'cch_amount': fields.function(_calculate_amount, digits_compute=dp.get_precision('Account'), method=True, string='Petty Cash Amount', multi='balance'),
        'repo_amount': fields.function(_calculate_amount, digits_compute=dp.get_precision('Account'), method=True, string='Reposition Amount', multi='balance'),
        'diff_amount': fields.function(_calculate_amount, digits_compute=dp.get_precision('Account'), method=True, string='Differential Amount', multi='balance'),
        'balance_total': fields.function(_calculate_amount, digits_compute=dp.get_precision('Account'), method=True, string='Balance Total', multi='balance'),
    }
    
    _defaults = {
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'active': 1,
        'config': _get_configuracion,
        'state': 'draft',
    }
    
    
    def _check_monto_caja_chica(self, cr, uid, ids, context=None):
        """ Verificar que el monto de la caja chica, no exceda la cantidad de
        unidades permitidas configurado para caja chica.
        @return: True (bien) or False (mal)
        """
        config_obj = self.pool.get('caja.chica.configuracion')
        ut_obj = self.pool.get('l10n.ut')
        
        fecha_actual = time.strftime('%Y-%m-%d')
        ut_ser = ut_obj.search(cr, uid, [('date', '<=', fecha_actual)], order='date desc')
        
        if not ut_ser:
            raise osv.except_osv(('Error !'), ('No existe unidad tributaria creada'))
        
        for ca_ch in self.browse(cr, uid, ids):
            if not ca_ch.config:
                raise osv.except_osv(('Error !'), ('No existe configuracion para caja chica creada'))
            
            maximo_ut = float(ca_ch.config.max_reposicion)
            ut_brw = ut_obj.browse(cr, uid, [ut_ser[0]])[0]
            ut_amount = float(ut_brw.amount)
            monto_caja = float(ca_ch.amount)
            monto_permitido = maximo_ut * ut_amount
            if (monto_permitido - monto_caja) < 0:
                raise osv.except_osv(('Error !'), ('El saldo de la caja chica es superior a las %s UT permitida'%(maximo_ut)))
                return False

        return True
        
    _constraints = [(_check_monto_caja_chica, 'El saldo de la caja chica es superior al monto permitido para una caja chica', ['amount'])]
    
    
    def log_caja_chica(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        for caja_brw in self.browse(cr, uid, ids,context=context):
            name_ref = caja_brw.name 
            if context.get('estado', '') == 'opened':
                mensaje = _('Petty Cash opened ') + name_ref
            elif context.get('estado', '') == 'pre-closed':
                mensaje = _('Petty Cash pre-closed ') + name_ref
            elif context.get('estado', '') == 'closed':
                mensaje = _('Petty Cash closed ') + name_ref
            else:
                mensaje = _('Petty Cash updated ') + name_ref

            self.log(cr, uid, caja_brw.id, mensaje)

    
    def create_open_move(self, cr, uid, ids, context=None):
        
        am_obj = self.pool.get("account.move")
        aml_obj = self.pool.get("account.move.line")
        seq_obj = self.pool.get('ir.sequence')
        period_obj = self.pool.get("account.period")
        date_doc = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        
        for ca_ch in self.browse(cr, uid, ids, context=context):
            
            seq_journal_move_id = ca_ch.journal_move_id.sequence_id and ca_ch.journal_move_id.sequence_id.id or False
            journal_move_id = ca_ch.journal_move_id and ca_ch.journal_move_id.id or False
            period_id = period_obj.find(cr, uid, date_doc)[0] or False
            
            name_line_deb = ca_ch.journal_id.default_debit_account_id and ca_ch.journal_id.default_debit_account_id.name or '/'
            account_id_deb = ca_ch.journal_id.default_debit_account_id and ca_ch.journal_id.default_debit_account_id.id or False
            name_line_cred = ca_ch.account_id and ca_ch.account_id.name or '/'
            account_id_cred = ca_ch.account_id and ca_ch.account_id.id or '/'
            partner_id = ca_ch.company_id.partner_id and ca_ch.company_id.partner_id.id or False
        
            move = {
                    'name': seq_obj.get_id(cr, uid, seq_journal_move_id),
                    'journal_id': journal_move_id,
                    'date': date_doc,
                    'ref': 'Apertura caja chica %s'%ca_ch.name,
                    'period_id': period_id
                    }
            move_id = am_obj.create(cr, uid, move, context=context)
            
            #create lines
            #cuenta deudora
            debit = ca_ch.amount
            credit = 0.0
            
            move_line = {
                'name': name_line_deb,
                'debit': debit,
                'credit': credit,
                'account_id': account_id_deb,
                'move_id': move_id,
                'journal_id': journal_move_id,
                'period_id': period_id,
                'date': date_doc,
                'date_maturity': date_doc,
                'analytic_account_id': ca_ch.analytic_id and ca_ch.analytic_id.id or False,
            }
            account_credit = aml_obj.create(cr, uid, move_line, context=context)
            #cuenta acreedora
            debit = 0.0
            credit = ca_ch.amount
            move_line = {
                'name': name_line_cred,
                'debit': debit,
                'credit': credit,
                'account_id': account_id_cred,
                'move_id': move_id,
                'journal_id': journal_move_id, 
                'period_id': period_id,
                'partner_id': partner_id,
                'date': date_doc,
                'date_maturity': date_doc,
                'analytic_account_id': ca_ch.analytic_id and ca_ch.analytic_id.id or False,
            }
            account_debit = aml_obj.create(cr, uid, move_line, context=context)
            isok = am_obj.button_validate(cr, uid, [move_id], context=context)
            
            #Crear orden de pago de apertura
            voucher_id = self.voucher_open(cr, uid, [ca_ch.id], cch_aml_id=account_debit, context=context)
            
            #Actualizacion de datos creados
            self.write(cr, uid, [ca_ch.id], {'move_id': move_id, 'open_voucher_id': voucher_id}, context=context)
        return True
        
    
    def voucher_open(self, cr, uid, ids, cch_aml_id=None, context=None):
        """
        Crea un account.voucher para la apertura de la caja chica
        """
        
        if context is None:
            context = {}
        
        voucher_obj = self.pool.get('account.voucher')
        voucher_line_obj = self.pool.get('account.voucher.line')
        
        caja_chica_brw = self.browse(cr, uid, ids, context=context)
        
        for caja in caja_chica_brw:
            journal_id = caja.journal_id and caja.journal_id.id or False
            account_id = caja.journal_id.default_debit_account_id and caja.journal_id.default_debit_account_id.id or False
            partner_id = caja.company_id.partner_id and caja.company_id.partner_id.id or False
            
            voucher_dict = {
                'type': 'payment',
                'name': 'Apertura ' + caja.name,
                'journal_id': journal_id,
                'account_id': account_id,
                'amount': caja.amount,
                'partner_id': partner_id,
                'pay': 'check',
                'imputable': False,
            }
            voucher_id = voucher_obj.create(cr, uid, voucher_dict, context=context)
            vou_lin_dict = {
                'voucher_id': voucher_id,
                'partner_id': partner_id,
                'name': 'Apertura ' + caja.name,
                'account_id': caja.account_id and caja.account_id.id or False,
                'amount': caja.amount,
                'type': 'dr',
                'move_line_id': cch_aml_id,
            }
            vou_lin_id = voucher_line_obj.create(cr, uid, vou_lin_dict, context=context)
            
            # Mensaje Usuario
            message = _('Created account voucher in draft: ') + 'Apertura ' + caja.name
            voucher_obj.log(cr, uid, voucher_id, message, context=context)
        
        return voucher_id
        
    
    def action_open(self, cr, uid, ids, context=None):
        """ Abrir la caja chica para crear asiento de apertura
        @return: True
        """
        #validaciones
        for cch_value in self.browse(cr, uid, ids, context=context):
            if cch_value.fiscalyear_id:
                #validaciones de fecha
                #debe estar dentro del rango del periodo fiscal
                if (cch_value.date_from < cch_value.fiscalyear_id.date_start \
                    or cch_value.date_from > cch_value.fiscalyear_id.date_stop or \
                    cch_value.date_to < cch_value.fiscalyear_id.date_start  \
                    or cch_value.date_to > cch_value.fiscalyear_id.date_stop):
                    raise osv.except_osv(_('Message'),_('You cant open! The init date or the end date have an error.'))
                #validaciones de periodo fiscal
                #deben estar cerradas las cajas chicas del periodo fiscal anterior
                fiscalyear_id_cch = cch_value.fiscalyear_id.id
                
                cch_ids = self.search(cr, uid, [('fiscalyear_id','!=',fiscalyear_id_cch),('state','!=','closed')], context=context)

                if cch_ids:
                    raise osv.except_osv(_('Message'),_('You cant open! Petty cash of previous fiscal year is open.'))
            else:
                raise osv.except_osv(_('Message'),_('You cant open! Must be indicate fiscal year before open it.'))
                
        #crear asiento
        result = self.create_open_move(cr, uid, ids, context=context)
        
        #cambiar estado de la caja chica a opened
        if result:
            self.write(cr, uid, ids, {'state': 'opened'}, context=context)
            context.update({'estado': 'opened'})
            self.log_caja_chica(cr, uid, ids, context=context)
        else:
            raise osv.except_osv(_('Message'),_('You cant open!'))
        return True
        
    
    def create_close_move(self, cr, uid, ids, move_differential, context=None):
        
        am_obj = self.pool.get("account.move")
        aml_obj = self.pool.get("account.move.line")
        seq_obj = self.pool.get('ir.sequence')
        period_obj = self.pool.get("account.period")
        date_doc = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        
        for ca_ch in self.browse(cr, uid, ids, context=context):
            
            seq_journal_move_id = ca_ch.journal_move_id.sequence_id and ca_ch.journal_move_id.sequence_id.id or False
            journal_move_id = ca_ch.journal_move_id and ca_ch.journal_move_id.id or False
            period_id = period_obj.find(cr, uid, date_doc)[0] or False
            
            name_line_deb = ca_ch.journal_id.default_debit_account_id and ca_ch.journal_id.default_debit_account_id.name or '/'
            account_id_deb = ca_ch.journal_id.default_debit_account_id and ca_ch.journal_id.default_debit_account_id.id or False
            name_line_cred = ca_ch.account_id and ca_ch.account_id.name or '/'
            account_id_cred = ca_ch.account_id and ca_ch.account_id.id or False
            partner_id = ca_ch.company_id.partner_id and ca_ch.company_id.partner_id.id or False
            
            #Apunte del diferencial Caja Chica x Pagar
            if move_differential:
                apunte_id = aml_obj.search(cr, uid, [('move_id','=',move_differential.id),('account_id','=',account_id_cred)], context=context)
                if not apunte_id:
                    raise osv.except_osv(_('Error !'), _('Dont exist differential move'))
                    
                apunte_brw = aml_obj.browse(cr, uid, apunte_id, context=context)[0]
                
                move = {
                        'name': seq_obj.get_id(cr, uid, seq_journal_move_id),
                        'journal_id': journal_move_id,
                        'date': date_doc,
                        'ref': 'Cierre caja chica %s'%ca_ch.name,
                        'period_id': period_id
                        }
                move_id = am_obj.create(cr, uid, move, context=context)
                
                #create lines
                #cuenta deudora
                if apunte_brw.debit:
                    credit = 0.0
                    debit = ca_ch.amount
                else:
                    credit = ca_ch.amount
                    debit = 0.0
                move_line = {
                    'name': name_line_deb,
                    'debit': debit,
                    'credit': credit,
                    'account_id': account_id_deb,
                    'move_id': move_id,
                    'journal_id': journal_move_id,
                    'period_id': period_id,
                    'partner_id': partner_id,
                    'date': date_doc,
                    'date_maturity': date_doc,
                    'analytic_account_id': ca_ch.analytic_id and ca_ch.analytic_id.id or False,
                }
                account_credit = aml_obj.create(cr, uid, move_line, context=context)
                #cuenta acreedora
                if apunte_brw.debit:
                    credit = ca_ch.amount
                    debit = 0.0
                else:
                    credit = 0.0
                    debit = ca_ch.amount
                move_line = {
                    'name': name_line_cred,
                    'debit': debit,
                    'credit': credit,
                    'account_id': account_id_cred,
                    'move_id': move_id,
                    'journal_id': journal_move_id, 
                    'period_id': period_id,
                    'date': date_doc,
                    'date_maturity': date_doc,
                    'analytic_account_id': ca_ch.analytic_id and ca_ch.analytic_id.id or False,
                }
                account_debit = aml_obj.create(cr, uid, move_line, context=context)
                isok = am_obj.button_validate(cr, uid, [move_id], context=context)
                
                #asociar asiento de cierre
                self.write(cr, uid, [ca_ch.id], {'close_move_id':move_id}, context=context)

                
                #conciliar
                conciliation_ids = []
                #Caja chica por pagar del asiento del cierre
                conciliation_ids.append(account_debit)
                
                #caja chica por pagar del asiento de reposicion
                for k in ca_ch.repo_id.move_ids:
                    if k.account_id.id == account_id_cred:
                        conciliation_ids.append(k.id)
                
                #caja chica por pagar  del asiento diferencial
                conciliation_ids.append(apunte_brw.id)
                
                #conciliar todo
                aml_obj.reconcile_partial(cr, uid, conciliation_ids, 'auto', context=context)
                
        return True
    
    def action_pre_close(self, cr, uid, ids, context=None):
        """ Pre Cierre de la caja chica
        @return: True
        """
        repo_obj = self.pool.get('caja.reposicion')
        for caja in self.browse(cr, uid, ids, context=context):
            if caja.repo_id:
                #Verificar si la reposicion esta asociada en otra caja chica
                existe_repo = self.search(cr, uid, [('repo_id','=',caja.repo_id.id),('id','!=',caja.id)], context=context)
                #Verificar el estado de la reposicion
                if caja.repo_id.state not in ('done'):
                    raise osv.except_osv(_('Message'),_('You cant pre-close! Repositioning is not state done.'))
                #Verificar que las reposiciones de la caja chica esten en estado final menos la asociada al cierre
                repo_state_final = repo_obj.search(cr, uid, [('caja_chica_id','=',caja.id),('id','!=',caja.repo_id.id),('state','in',('draft','confirm','done','paid'))], context=context)
                if repo_state_final:
                    raise osv.except_osv(_('Message'),_('You cant pre-close! Exist repositioning of petty cash in state different of paid.'))
                    
                if not existe_repo:
                    self.write(cr, uid, caja.id, {'state': 'pre-closed'}, context=context)
                    context.update({'estado': 'pre-closed'})
                    self.log_caja_chica(cr, uid, [caja.id], context=context)
                else:
                    raise osv.except_osv(_('Message'),_('You cant pre-close! Repositioning associated in other document.'))
            else:
                raise osv.except_osv(_('Message'),_('You cant pre-close! You must indicate repositioning document.'))
        return True
        
        
    def action_close(self, cr, uid, ids, context=None):
        """ Cerrar la caja chica para crear asiento de cierre
        @return: True
        """
        for caja in self.browse(cr, uid, ids, context=context):
            if caja.differential_move_id:
                #Verificar que el diferencial no este asociado en otra caja chica
                existe_diferencial = self.search(cr, uid, [('differential_move_id','=',caja.differential_move_id.id),('id','!=',caja.id)], context=context)
                
                if existe_diferencial:
                    raise osv.except_osv(_('Message'),_('You cant close! Differential is associated in other document.'))
                    
                #Verificar que el asiento tenga la cuenta a conciliar
                move_line_obj = self.pool.get('account.move.line')
                existe_cta = move_line_obj.search(cr, uid, [('move_id','=',caja.differential_move_id.id),('account_id','=',caja.account_id.id),('reconcile_partial_id','=',False),('reconcile_id','=',False)], context=context)
                if not existe_cta:
                    raise osv.except_osv(_('Message'),_('You cant close! The differential move associated is not valid.'))
                    
            elif caja.balance_total != 0.00:
                raise osv.except_osv(_('Message'),_('You cant close! You must associate the differential.'))
            
            if (caja.differential_move_id and caja.balance_total == 0.00):
                #crear asiento
                result = self.create_close_move(cr, uid, [caja.id], caja.differential_move_id, context=context)
                
                #cambiar estado de la caja chica a closed
                if result: 
                    self.write(cr, uid, [caja.id], {'state': 'closed'}, context=context)
                    #CAMBIAR ESTADO DE LA REPOSICION A PAGADO
                    repo_obj = self.pool.get('caja.reposicion')
                    repo_obj.write(cr, uid, [caja.repo_id.id], {'state': 'final'}, context=context)
                    
                    context.update({'estado': 'closed'})
                    self.log_caja_chica(cr, uid, [caja.id], context=context)
            else:
                raise osv.except_osv(_('Message'),_('You cant close! You should check the differential.'))
        return True
        
    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(_('Message'),_('You cant do it!'))
        return False
        
    def onchange_fiscalyear(self, cr, uid, ids, fiscalyear):
        result = {}
        if fiscalyear:
            fy_brw = self.pool.get('account.fiscalyear').browse(cr, uid, fiscalyear).date_stop
            fecha_actual = date.today().strftime('%Y-%m-%d')
            result = {'value': {
                'date_from': fecha_actual,
                'date_to': fy_brw,
                }
            }
        else:
            result = {'value': {
                'date_from': '',
                'date_to': '',
                }
            }
        return result
caja_chica()


class caja_reposicion_linea(osv.osv):
    _name = "caja.reposicion.linea"
    _description = "Lineas la Reposicion de Caja"
    
    
    def _get_cuenta_analitica(self, cr, uid, context=None):
        if context is None:
            context = {}
            
        repo_obj = self.pool.get('caja.reposicion')
        if context.get('caja_reposicion_id', False):
             repo_brw = repo_obj.browse(cr, uid, [context['caja_reposicion_id']])[0]
             return repo_brw.caja_chica_id.analytic_id and repo_brw.caja_chica_id.analytic_id.id or False
        
        return False
    
    
    _columns = {
        'name': fields.char('Referencia', size=128, required=True),
        'concepto': fields.char('Concepto', size=128, required=True),
        'caja_reposicion_id': fields.many2one('caja.reposicion', 'Caja Reposicion'),
        'amount': fields.float('Monto', required=True),
        'account_id': fields.many2one('account.account', 'Cuenta Contable', required=True),
        'analytic_id': fields.many2one('account.analytic.account', 'Cuenta Analitica', required=True),
        'product_id': fields.many2one('product.product', 'Servicio', required=True),
        'date' : fields.date('Fecha de Emision', required=True),
        'state': fields.selection([('draft','Borrador'), ('done','Pagado'), 
                            ('cancel','Cancelado')], 'Estado', readonly=True),
        'move_line_id': fields.many2one('account.move.line', 'Move Line', help="The move line of this entry line.", select=2),
    }
    
    _defaults = {
        'state': 'draft',
        'analytic_id': _get_cuenta_analitica,
    }
    
    def _check_linea_reposicion(self, cr, uid, ids, context=None):
        """ Verificar que las lineas de la reposicion de caja chica,
        no exceda la cantidad de unidades permitidas configurado para caja chica.
        @return: True (bien) or False (mal)
        """
        if context is None:
            context = {}
        
        repo_obj = self.pool.get('caja.reposicion')
        config_obj = self.pool.get('caja.chica.configuracion')
        ut_obj = self.pool.get('l10n.ut')
        
        fecha_actual = time.strftime('%Y-%m-%d')
        ut_ser = ut_obj.search(cr, uid, [('date', '<=', fecha_actual)], order='date desc')
        config_ser = config_obj.search(cr, uid, [])
        
        if not ut_ser:
            raise osv.except_osv(('Error !'), ('No existe unidad tributaria creada'))
            
        for lin_rep in self.browse(cr, uid, ids):
            if lin_rep.amount <= 0:
                raise osv.except_osv(('Error !'), ('Debe colocar un monto mayor a 0'))
            
            ut_brw = ut_obj.browse(cr, uid, [ut_ser[0]])[0]
            max_line_total = 0
            config_brw = lin_rep.caja_reposicion_id.caja_chica_id and lin_rep.caja_reposicion_id.caja_chica_id.config or False
            if not config_brw:
                config_brw = config_obj.browse(cr, uid, [config_ser[0]])[0]
            if not config_brw:
                raise osv.except_osv(('Error !'), ('Se debe crear una configuracion para el manejo de caja chica.'))
                
            ut_amount = ut_brw.amount
            max_line_ut = config_brw.max_linea
            max_line_total = max_line_ut * ut_amount
            if lin_rep.amount - max_line_total > 0.0001:
                raise osv.except_osv(('Error !'), ('La linea "%s" excede las %s Unidades Tributarias. No puede ser aprobada la reposicion.'%(lin_rep.name,max_line_ut)))
                return False
                
        return True
        
    _constraints = [(_check_linea_reposicion, 'El monto de la linea excede las Unidades Tributarias permitidas. No puede ser aprobada la reposicion.', ['amount'])]
    
    
    def onchange_product_id(self, cr, uid, ids, product_id, context=None):
        """
        On_change para extraer la cuenta de gasto del producto.
        """
        if context is None:
            context = {}
        
        res = {}
        product_obj = self.pool.get('product.product')
        if product_id:
            product_brw = product_obj.browse(cr, uid, [product_id], context)[0]
            res.update({
                'account_id': product_brw.property_account_expense and product_brw.property_account_expense.id or False,
            })
            
        return {
            'value':res
        }
        
caja_reposicion_linea()


class caja_reposicion(osv.osv):
    _name = 'caja.reposicion'
    _description = 'Reposicion de Caja Chica'
    _order = "id desc"
    
    
    def _get_amount_total(self, cr, uid, ids, name, args, context=None):
        """
        Metodo para calcular el monto total de la reposicion
        """
        if context is None:
            context = {}
        
        repo_brw = self.browse(cr, uid, ids, context=context)
        res = {}
        for id in ids:
            res[id] = 0.00
        
        for repo in repo_brw:
            for line in repo.reposicion_linea_ids:
                res[repo.id] = res[repo.id] + line.amount
                
        return res
    
    
    _columns = {
        'name': fields.char('Secuencia de movimiento', size=128, readonly=True),
        'ref': fields.char('Referencia de movimiento', size=128, required=True,
                        readonly=True,
                        states={'draft':[('readonly',False)]}),
        'reposicion_linea_ids': fields.one2many('caja.reposicion.linea', 
                            'caja_reposicion_id', 'Lineas Reposicion', 
                            states={'draft':[('readonly',False)], 
                                    'confirm':[('readonly',True)], 
                                    'done':[('readonly',True)], 
                                    'paid':[('readonly',True)], 
                                    'final':[('readonly',True)], 
                                    'cancel':[('readonly',True)]}),
        'confirmado_por': fields.many2one('res.users', 'Confirmado por', readonly=True),
        'aprobado_por': fields.many2one('res.users', 'Aprobado por', readonly=True),
        'fecha_aprobado': fields.date('Fecha de Aprobacion', readonly=True),
        'fecha_confirmado': fields.date('Fecha Confirmado', readonly=True),
        'caja_chica_id': fields.many2one('caja.chica', 'Caja Chica', required= True,
                            states={'draft':[('readonly',False)], 
                                    'confirm':[('readonly',True)], 
                                    'done':[('readonly',True)], 
                                    'paid':[('readonly',True)], 
                                    'final':[('readonly',True)], 
                                    'cancel':[('readonly',True)]}),
        'move_ids': fields.many2many('account.move.line', 'caja_reposicion_move_line_rel',
                            'reposicion_id', 'imputacion_id', 'Asientos Contable', 
                            readonly=True),
        'amount_total': fields.function(_get_amount_total, method=True, string='Monto Total'),
        'company_id': fields.many2one('res.company', 'Company'),
        'state': fields.selection([('draft','Borrador'), ('confirm','Confirmado'), 
                            ('done','Aprobado'), ('paid', 'Orden de Pago Creada'), ('final','Pagado'),('cancel','Rechazada')],
                            'Estado', readonly=True),
        #~ 'department_id': fields.many2one('hr.department','Departamento'),
        'period_id': fields.many2one('account.period', 'Periodo', readonly=True),
        'product_categ': fields.related('caja_chica_id','config','product_categ',type='many2one',relation='product.category',string='Categoria', store=True, readonly=True),
    }
    
    _defaults = {
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'state': 'draft',
    }
    
    
    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(('Error !'), ('No se permite duplicar una reposición'))

    
    def create(self, cr, uid, values, context=None):
        if context is None:
            context = {}
        
        sequence_obj = self.pool.get('ir.sequence')
        caja_obj = self.pool.get('caja.chica')
        
        if values.get('caja_chica_id', False):
            caja_brw = caja_obj.browse(cr, uid, [values['caja_chica_id']])[0]
            if caja_brw.en_uso:
                raise osv.except_osv(('Error !'), ('No se puede crear una reposicion para esta caja chica, ya existe una reposicion para %s'%caja_brw.name))
        
        values['name'] = sequence_obj.get(cr, uid, 'caja.reposicion')
        return super(caja_reposicion, self).create(cr, uid, values, context=context)


    def log_reposicion(self, cr, uid, repo_id, context=None):
        if context is None:
            context = {}
        
        repo_brw = self.browse(cr, uid, [repo_id])[0]
        name_ref = repo_brw.name + '/' + repo_brw.ref
        if context.get('estado', '') == 'confirm':
            mensaje = 'Se ha confirmado la reposicion "' + name_ref + '" de la caja chica "' + repo_brw.caja_chica_id.name + '"'
        elif context.get('estado', '') == 'done':
            mensaje = 'Se ha aprobado la reposicion "' + name_ref + '" de la caja chica "' + repo_brw.caja_chica_id.name + '"'
        elif context.get('estado', '') == 'paid':
            mensaje = 'Se ha creado orden de pago de la reposicion "' + name_ref + '" de la caja chica "' + repo_brw.caja_chica_id.name + '"'
        elif context.get('estado', '') == 'cancel':
            mensaje = 'Se ha cancelado la reposicion "' + name_ref + '" de la caja chica "' + repo_brw.caja_chica_id.name + '"'
        else:
            mensaje = 'Se modifico reposicion ' + name_ref

        self.log(cr, uid, repo_id, mensaje)


    def cancel_draft(self, cr, uid, ids, context=None):
        """
        Funcion para pasar a cancel una reposicion, solo cuando esta en draft
        """
        if context is None:
            context = {}
            
        self.write(cr, uid, ids, {'state':'cancel','confirmado_por': None,
        'fecha_confirmado': None,}, context=context)
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        return True
        
        
    def cancel_reposicion(self, cr, uid, ids, context=None):
        """
        Funcion para pasar a cancel una reposicion, solo cuando esta confirmada
        """
        if context is None:
            context = {}
        
        self.write(cr, uid, ids, {'state':'cancel','confirmado_por': None,
        'fecha_confirmado': None,}, context=context)
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        return True

        
    def confirmar_reposicion(self, cr, uid, ids, context=None):
        """
        Funcion para confirmar la reposicion de la caja chica.
        """
        if context is None:
            context = {}
        
        for repo in self.browse(cr, uid, ids):
            """
            Inicio de validacion de monto de reposicion no exceda monto de caja chica.
            """
            if (repo.amount_total - repo.caja_chica_id.amount) > 0.0001:
                raise osv.except_osv(('Error !'), ('La reposicion %s no se puede confirmar, excede el monto total de la caja chica (%s)'%(repo.name, repo.caja_chica_id.amount)))
            if not repo.reposicion_linea_ids:
                raise osv.except_osv(('Error !'), ('Debe agregar lineas a la reposicion antes de confirmar'))
            for line in repo.reposicion_linea_ids:
                if line.amount <= 0:
                    raise osv.except_osv(('Error !'), ('La linea "%s" debe tener un monto mayor a 0'%(line.name)))
            
            #validacion en el caso de que exista la la orden de pago de apertura
            #debe estar en realizado antes de confirmar la reposicion
            if repo.caja_chica_id.open_voucher_id:
                if repo.caja_chica_id.open_voucher_id.state != 'posted':
                    raise osv.except_osv(_('Message'), _('You cant confirm the document! The opening payment order should be posted.'))
            
        self.write(cr, uid, ids, {'state': 'confirm', 'confirmado_por': uid, 'fecha_confirmado': time.strftime('%Y-%m-%d')}, context=context)
        
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        
        return True
        
        
    def validar_reposicion(self, cr, uid, ids, context=None):
        """
        Aqui se aprueba y se generan los asientos contables al momento de aprobar
        la reposicion.
        """
        if context is None:
            context = {}
        
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get('account.move.line')
        repo_line_obj = self.pool.get('caja.reposicion.linea')
        config_obj = self.pool.get('caja.chica.configuracion')
        obj_company = self.pool.get('res.company')
        ut_obj = self.pool.get('l10n.ut')
        
        fecha_actual = time.strftime('%Y-%m-%d')
        
        repo_brw = self.browse(cr, uid, ids, context=context)
        
        """
        Inicio de validacion de las lineas de la caja chica para que no excedan
        el limite definido para las cajas chicas.
        """
        ut_ser = ut_obj.search(cr, uid, [('date', '<=', fecha_actual)], order='date desc')
        config_ser = config_obj.search(cr, uid, [])
        if not ut_ser:
            raise osv.except_osv(('Error !'), ('No existe unidad tributaria creada'))
        
        ut_brw = ut_obj.browse(cr, uid, [ut_ser[0]])[0]
        
        for repo in repo_brw:
            max_line_total = 0
            config_brw = repo.caja_chica_id.config
            if not config_brw:
                config_brw = config_obj.browse(cr, uid, [config_ser[0]])[0]
            if not config_brw:
                raise osv.except_osv(('Error !'), ('Se debe crear una configuracion para el manejo de caja chica.'))
            ut_amount = ut_brw.amount
            max_line_ut = config_brw.max_linea
            max_line_total = max_line_ut * ut_amount
            for line in repo.reposicion_linea_ids:
                if line.amount - max_line_total > 0.0001:
                    raise osv.except_osv(('Error !'), ('La linea "%s" excede las %s Unidades Tributarias. No puede ser aprobada la reposicion.'%(line.name,max_line_ut)))
                if line.account_id.type == 'view':
                    raise osv.except_osv(('Error !'), ('Imposible aprobar la reposicion. En la linea %s, se eligio una cuenta de tipo vista. (%s %s)' % (line.name, line.account_id.code, line.account_id.name)))
                    
        #Fin de la validacion
        
        for repo in repo_brw:
            caja_brw = repo.caja_chica_id
            move_dict = {}
            ref = elimina_tildes(repo.name + ' / ' + repo.ref)
            move_dict = {
                'ref': ref,
                'journal_id': caja_brw.journal_move_id and caja_brw.journal_move_id.id or None,
                'to_check': True,
            }
            move_id = move_obj.create(cr, uid, move_dict)
            move_brw = move_obj.browse(cr, uid, [move_id])[0]
            periodo_id = move_brw.period_id.id
            context.update({'period_id': periodo_id})
            
            move_line_ids = []
            company_id = obj_company._company_default_get(cr, uid, 'account.move', context=context)
            for line in repo.reposicion_linea_ids:
                move_line_dict = {}
                name = elimina_tildes(line.name + ' / ' + line.concepto)
                move_line_dict = {
                    'name': name[:64],
                    'ref': ref[:64],
                    #~ 'quantity': ,
                    #~ 'product_uom_id': ,
                    'product_id': line.product_id and line.product_id.id or None,
                    'debit': line.amount,
                    'credit': 0.00,
                    'account_id': line.account_id and line.account_id.id or None,
                    'move_id': move_id,
                    'journal_id': caja_brw.journal_move_id and caja_brw.journal_move_id.id or None,
                    'analytic_account_id': line.analytic_id and line.analytic_id.id or 'NULL',
                    'state': 'valid',
                    'period_id': periodo_id,
                }
                
                #Creacion de apunte contable para debito por medio de query
                cr.execute("insert into account_move_line (account_id,name,ref,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date,product_id,analytic_account_id) values (%s,'%s','%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s',now(),%s,False,%s,now(),%s,%s)"%(move_line_dict['account_id'],move_line_dict['name'],move_line_dict['ref'],move_line_dict['journal_id'],int(company_id),move_line_dict['credit'],move_line_dict['state'],move_line_dict['period_id'],move_line_dict['debit'],str(fecha_actual),move_line_dict['move_id'],uid,move_line_dict['product_id'],move_line_dict['analytic_account_id']))
                cr.commit()

                move_line_dict.update({'debit': 0.00, 'credit': line.amount, 'account_id': caja_brw.account_id and caja_brw.account_id.id or None, 'partner_id': repo.company_id.partner_id.id})

                #Creacion de apunte contable para credito por medio de query
                #se incorpora el partner en el apunte
                cr.execute("insert into account_move_line (account_id,name,ref,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date,product_id,analytic_account_id,partner_id) values (%s,'%s','%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s',now(),%s,False,%s,now(),%s,%s,%s)"%(move_line_dict['account_id'],move_line_dict['name'],move_line_dict['ref'],move_line_dict['journal_id'],int(company_id),move_line_dict['credit'],move_line_dict['state'],move_line_dict['period_id'],move_line_dict['debit'],str(fecha_actual),move_line_dict['move_id'],uid,move_line_dict['product_id'],move_line_dict['analytic_account_id'],move_line_dict['partner_id']))
                cr.commit()
                
                #relacionar la linea de la reposicion con el apunte contable asociado a la caja chica
                apunte_id = move_line_obj.search(cr, uid, [('move_id', '=',move_id),('partner_id','!=',False)], context=context)
                if apunte_id:
                    dict_linea = {
                        'move_line_id':apunte_id[0],
                    }
                    repo_line_obj.write(cr, uid, [line.id], dict_linea, context=context)
                else:
                    raise osv.except_osv(('Error !'), ('Error in create the move.'))
                    
            #Asentar el asiento contable de la caja chica.
            move_obj.button_validate(cr, uid, [move_id], context=context)
            
            #Unir lineas de asiento a la reposicion
            move_line_ids = [mo.id for mo in move_brw.line_id]
            self.write(cr, uid, [repo.id], {'move_ids': [(6, 0, move_line_ids)]}, context=context)
            
            
        self.write(cr, uid, ids, {'state': 'done', 'aprobado_por': uid, 
            'fecha_aprobado': fecha_actual, 'period_id': periodo_id}, context=context)
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)

        return True
        

    def voucher_reposicion(self, cr, uid, ids, context=None):
        """
        Crea un account.voucher para pagar la caja chica
        """
        if context is None:
            context = {}
        
        voucher_obj = self.pool.get('account.voucher')
        voucher_line_obj = self.pool.get('account.voucher.line')
        
        repo_brw = self.browse(cr, uid, ids, context=context)
        
        #Validar que la reposicion no este asociada a un cierre de caja chica
        existe_repo = self.pool.get('caja.chica').search(cr, uid, [('repo_id','in',tuple(ids))])
        if existe_repo:
            raise osv.except_osv(_('Message'), _('You cant create the pay order! Repositioning associated in the closed petty cash.'))
        
        
        for repo in repo_brw:
            journal_id = repo.caja_chica_id.journal_id and repo.caja_chica_id.journal_id.id or False
            account_id = repo.caja_chica_id.journal_id and repo.caja_chica_id.journal_id.default_credit_account_id.id or False
            partner_id = repo.company_id.partner_id.id
            
            voucher_dict = {
                'type': 'payment',
                'name': repo.name + ' / ' + repo.ref,
                'journal_id': journal_id,
                'account_id': account_id,
                #~ 'narration': 'NARRATION SERA EL MEMORIAL?',
                #~ 'currency_id': PUEDE SER QUE LO USE,
                'amount': repo.amount_total,
                'partner_id': partner_id,
                #~ 'comment': 'ESTE SERA EL MEMORIAL',
                'pay': 'check',
                #~ 'analytic_id': PUEDE SER QUE LO USE,
            }
            voucher_id = voucher_obj.create(cr, uid, voucher_dict)
            
            for line in repo.reposicion_linea_ids:
                
                vou_lin_dict = {
                    'voucher_id': voucher_id,
                    'partner_id': partner_id,
                    'name': line.name + ' / ' + line.concepto,
                    'account_id': repo.caja_chica_id.account_id.id,
                    #~ 'untax_amount': fields.float('Untax Amount'),
                    'amount': line.amount,
                    'type': 'dr',#fields.selection([('dr','Debit'),('cr','Credit')], 'Cr/Dr'),
                    #~ 'account_analytic_id': line.analytic_id.id,
                    'move_line_id':line.move_line_id and line.move_line_id.id or False,
                }
                vou_lin_id = voucher_line_obj.create(cr, uid, vou_lin_dict, context=context)
            # Mensaje Usuario
            message = 'Se genero orden de pago ' + repo.name + '/' + repo.ref
            voucher_obj.log(cr, uid, voucher_id, message, context=context)
        
        self.write(cr, uid, ids, {'state': 'paid'})
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        
        return True
        
        
    def action_cancel_draft(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        self.write(cr, uid, ids, {'state': 'draft', 'confirmado_por': None,
            'fecha_confirmado': None,}, context=context)
        
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
            
        return True
        
    
caja_reposicion()
