#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################
from osv import osv
from osv import fields
from tools.translate import _
from tools import config
import calendar
import time
import datetime
import decimal_precision as dp
import string

class presupuesto_modificacion(osv.osv):
    _name="presupuesto.modificacion"
    _description="Presupuesto Modificacion"
    
    
    def _get_amount_total_colum(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        for id in id_brw:
            acum1 = 0
            acum2 = 0
            for line in id.line_up_ids:
                acum1 = acum1 + line.amount
                
            for line in id.line_down_ids:
                acum2 = acum2 + line.amount
            
            res[id.id] = {'amount_total_up': acum1, 'amount_total_down': acum2}
        return res
    
    
    def _get_accion_especifica_transferencia(self, cr, uid, ids, name, args, context=None):
        res = {}
        modif_brw = self.browse(cr, uid, ids, context=context)
        for line in modif_brw:
            if line.type_document == 'transferencia' and line.aumento:
                query = """
                SELECT pml.accion_especifica_id
                FROM presupuesto_modificacion_linea pml
                WHERE pml.modificacion_id = %s
                    AND pml.type = 'up'
                GROUP BY pml.accion_especifica_id
                """%(line.id)
                cr.execute(query)
                accion = cr.dictfetchall()
                
                if accion:
                    res[line.id] = accion[0]['accion_especifica_id']
                else:
                    res[line.id] = None
            else:
                res[line.id] = None
                
        return res
    
    
    _columns = {
        'name': fields.char('Description', size=128, required=True, select=True),
        'sequence_id': fields.char('Sequence', size=64, required=True, select=True),
        'motive': fields.char('Motive', size=64),
        'period_id': fields.many2one('periodo.presupuestario', 'Periodo Presupuestario', required=True),
        'ejercicio_id': fields.many2one('ejercicio.presupuestario','Ejercicio Presupuestario', required=True),
        'amount_total_up': fields.function(_get_amount_total_colum, digits_compute=dp.get_precision('Account'), method=True, string='Total Columna 1', multi='total_column'),
        'amount_total_down': fields.function(_get_amount_total_colum, digits_compute=dp.get_precision('Account'), method=True, string='Total Columna 2', multi='total_column'),
        'type_document': fields.selection([('transferencia','Transferencia'),('incremento_disminucion','Incremento y Disminucion')],'Tipo de Documento', required=True),
        'line_up_ids': fields.one2many('presupuesto.modificacion.linea','modificacion_id','Lineas Arriba',
            domain=[('type','=','up')], context={'default_type':'up'}, readonly=True, states={'draft':[('readonly',False)]}),
        'line_down_ids': fields.one2many('presupuesto.modificacion.linea','modificacion_id','Lineas Abajo',
            domain=[('type','=','down')], context={'default_type':'down'}, readonly=True, states={'draft':[('readonly',False)]}),
        'aumento': fields.boolean('Aumento / Trasferencia Unica'),
        'company_id': fields.many2one('res.company', 'Company', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'accion_especifica_id': fields.function(_get_accion_especifica_transferencia, method=True, relation='accion.especifica', type='many2one', string='Accion Especifica'),
        'state':fields.selection([
            ('draft','Draft'),
            ('cancel','Cancel'),
            ('done','Done'),
        ],'State', select=True, readonly=True),
    }
    
    
    _defaults = {
        'company_id': lambda self,cr,uid,c: self.pool.get('res.company')._company_default_get(cr, uid, 'presupuesto.modificacion',context=c),
        'state': 'draft',
        'sequence_id': '/',
    }
    
    
    def check_lines_transf(self, cr, uid, modif_brw, context=None):
        if modif_brw.aumento:
            query = """
                SELECT pml.accion_especifica_id
                FROM presupuesto_modificacion_linea pml
                WHERE pml.modificacion_id = %s
                GROUP BY pml.accion_especifica_id
            """%(modif_brw.id)
            cr.execute(query)
            count = cr.dictfetchall()
            if len(count) > 1:
                return False
                
        return True
    
    
    def process_modification(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        line_modif = self.pool.get('presupuesto.modificacion.linea')
        modif_brw = self.browse(cr, uid, ids, context=context)
        
        for modif in modif_brw:
            if (modif.amount_total_up - modif.amount_total_down) > 0.00:
                raise osv.except_osv(_('Amount Error!'), ("Los montos totales deben ser iguales, modificacion: %s " % modif.name))
                
            operador_up = 'positive'
            operador_down = 'positive'
            if modif.type_document == 'transferencia':
                status = self.check_lines_transf(cr, uid, modif, context=context)
                if not status:
                    raise osv.except_osv(_('Amount Error!'), ("Existen lineas que no corresponden con el tipo de modificacion presupuestaria a realizar. Modificacion: %s " % modif.name))
                operador_up = 'negative'
            else:
                if not modif.aumento:
                    operador_up = 'negative'
                    operador_down = 'negative'
            
            line_srch_up = line_modif.search(cr, uid, [('modificacion_id', '=', modif.id), ('type', '=', 'up')])
            line_srch_down = line_modif.search(cr, uid, [('modificacion_id', '=', modif.id), ('type', '=', 'down')])
            line_modif.write(cr, uid, line_srch_up, {'state': 'valid', 'operation': operador_up}, context=context)
            line_modif.write(cr, uid, line_srch_down, {'state': 'valid', 'operation': operador_down}, context=context)
                
            self.write(cr, uid, modif.id, {'state': 'done'}, context=context)
            
        return True
    
    
    def to_draft_modification(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        line_modif = self.pool.get('presupuesto.modificacion.linea')
        
        modif_brw = self.browse(cr, uid, ids, context=context)
        for modif in modif_brw:
            line_srch = line_modif.search(cr, uid, [('modificacion_id', '=', modif.id)])
            line_modif.write(cr, uid, line_srch, {'state': 'draft'}, context=context)
                
            self.write(cr, uid, modif.id, {'state': 'draft'}, context=context)
        
        return True
        
        
    def calc_dummy(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
            
        return True
    
    
    def create(self, cr, uid, values, context=None):
        if context is None:
            context = {}
        
        sequence_obj = self.pool.get('ir.sequence')
        values['sequence_id'] = sequence_obj.get(cr, uid, 'presupuesto.modificacion')
        
        return super(presupuesto_modificacion, self).create(cr, uid, values, context=context)
    
    
presupuesto_modificacion()


class presupuesto_modificacion_linea(osv.osv):
    _name = "presupuesto.modificacion.linea"
    _description = "Presupuesto Modificacion Linea"
    _rec_name = "modificacion_id"
    
    
    def default_get(self, cr, uid, fields, context=None):
        """
        Get default values
        @param fields: List of fields for default value
        """
        if context is None:
            context = {}
        #~ print "default_get"
        #~ print "fields",fields
        #~ print "context",context
        
        res = super(presupuesto_modificacion_linea, self).default_get(cr, uid, fields, context=context)
        
        if context.has_key('accion_especifica_id'):
            if context.get('accion_especifica_id', False):
                res.update({'accion_especifica_id': context.get('accion_especifica_id', False)})
            #~ else:
                #~ modif_brw = self.pool.get('presupuesto.modificacion').browse(cr, uid, context.get('active_id'), context=context)
                #~ print modif_brw
                #~ print modif_brw.type_document
                #~ print modif_brw.aumento
                #~ if modif_brw.type_document == 'transferencia' and modif_brw.aumento:
                    #~ query = """
                    #~ SELECT pml.accion_especifica_id
                    #~ FROM presupuesto_modificacion_linea pml
                    #~ WHERE pml.modificacion_id = %s
                    #~ AND pml.type = 'up'
                    #~ GROUP BY pml.accion_especifica_id
                    #~ """%(modif_brw.id)
                    #~ cr.execute(query)
                    #~ accion = cr.dictfetchall()
                    #~ if accion:
                        #~ res.update({'accion_especifica_id': accion[0]['accion_especifica_id']})
        #~ print "res",res
        return res
    
    
    def _get_amount_total_amount(self, cr, uid, ids, name, args, context=None):
        if context is None:
            context = {}
            
        clasi_presu = self.pool.get('clasificador.presupuestario')
        period_presu = self.pool.get('periodo.presupuestario')
        
        
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        for line_id in id_brw:
            #~ period_ids = period_presu.search(cr, uid, [('date_start','>=',line_id.modificacion_id.ejercicio_id.date_start),('date_stop','<=',line_id.modificacion_id.period_id.date_stop)])
            period_ids = period_presu.search(cr, uid, [('date_start','>=',line_id.modificacion_id.ejercicio_id.date_start),('date_stop','<=',line_id.modificacion_id.ejercicio_id.date_stop)])
            context.update({'periods': period_ids, 'accion_especifica_id': [line_id.accion_especifica_id.id]})
            dict_parcial = {'amount_availability': 0, 'amount_modified': 0}
            parti = clasi_presu.browse(cr, uid, line_id.partida_id.id, context=context)
            signo = 1
            if (line_id.modificacion_id.type_document == 'transferencia' and line_id.type == 'up') \
                or (line_id.modificacion_id.type_document == 'incremento_disminucion' and not line_id.modificacion_id.aumento):
                signo = -1
            amount = line_id.amount * signo
            if line_id.modificacion_id.state == 'done':
                dict_parcial['amount_availability'] = (parti.presupuesto - parti.modificado + amount) - parti.comprometido
            else:
                dict_parcial['amount_availability'] = (parti.presupuesto - parti.modificado) - parti.comprometido
            dict_parcial['amount_modified'] = dict_parcial['amount_availability'] + amount
            res[line_id.id] = dict_parcial
            
        return res
    
    
    _columns = {
        #~ 'name': fields.char('Descripcion', size=128),
        'partida_id': fields.many2one('clasificador.presupuestario', 'Partida Presupuestaria', required=True),
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica', required=True),
        'ejercicio_id': fields.related('modificacion_id', 'ejercicio_id', relation='ejercicio.presupuestario', type='many2one', string='Ejercicio Presupuestario', store=True, readonly=True),
        'period_id': fields.related('modificacion_id', 'period_id', relation='periodo.presupuestario', type='many2one', string='Periodo Presupuestario', store=True, readonly=True),
        'type_document': fields.related('modificacion_id', 'type_document', type='selection', string='Tipo de Documento', store=True),
        'amount': fields.float('Amount', digits_compute=dp.get_precision('Account'), required=True),
        'amount_availability': fields.function(_get_amount_total_amount, digits_compute=dp.get_precision('Account'), method=True, string='Amount Availability', multi='total_amount'),
        #~ 'amount_availability': fields.related('partida_id', 'disponibilidad', string='Availability', readonly=True),
        'amount_modified': fields.function(_get_amount_total_amount, digits_compute=dp.get_precision('Account'), method=True, string='Amount Modified', multi='total_amount'),
        #~ 'amount_modified': fields.function(_get_amount_total_amount, digits_compute=dp.get_precision('Account'), method=True, string='Amount Modified'),
        'type': fields.selection([('up','Up'),('down','Down')], 'Type', required=True),
        'operation': fields.selection([('positive','Positive'),('negative','Negative')], 'Operation'),
        'modificacion_id': fields.many2one('presupuesto.modificacion', 'Presupuesto Modificacion', ondelete='cascade'),
        'state': fields.selection([('draft','Draft'),('valid','Valid')],'State'),
        'company_id': fields.related('modificacion_id','company_id', relation='res.company', type='many2one', string='Company', store=True, readonly=True),
    }
    
    
    _defaults = {
        'state': 'draft',
    }
    
    _sql_constraints = [
        ('amount_not_negative', 'CHECK (amount > 0)', 'No se permiten montos negativos.'),
    ]
    
    
    def check_lines(self, cr, uid, ids, context=None):
        """Verificar que las lineas de la modificacion se ajusten al tipo de modificacion presupuestaria.
        @return: True (bien) or False (mal)
        """
        line_brw = self.browse(cr, uid, ids, context=context)

        for line in line_brw:
            #~ print "line.type",line.type
            #~ print "line.amount_availability",line.amount_availability
            #~ print "line.amount_modified",line.amount_modified
            
            #Se verifica que el monto modificado no deje la partida en negativo o que la disponibilidad sea negativo
            #~ if line.modificacion_id.type_document == 'transferencia' and line.type == 'up' and (line.amount_availability < 0 or line.amount_modified < 0):
                #~ raise osv.except_osv(_('Amount Error!'),_("No se puede crear esta modificacion porque el monto sera negativo. Monto Disponible: %s, Monto Modificado: %s"%(line.amount_availability,line.amount_modified)))
                #~ return False
            #~ elif line.amount_modified < 0:
                #~ raise osv.except_osv(_('Amount Error!'),_("No se puede crear esta modificacion porque el monto modificado quedara negativo. Monto Modificado: %s"%(line.amount_modified)))
                #~ return False
                
            if line.modificacion_id.type_document == 'transferencia':
                query = """
                    SELECT pml.accion_especifica_id
                    FROM presupuesto_modificacion_linea pml
                    WHERE pml.modificacion_id = %s
                """%(line.modificacion_id.id)
                
                if not line.modificacion_id.aumento:
                    query = query + "AND pml.type = 'up'"
                
                query = query + " GROUP BY pml.accion_especifica_id"
                #~ print query
                cr.execute(query)
                count = cr.dictfetchall()
                if len(count) > 1:
                    return False
        
        return True


    _constraints = [(check_lines, 'Solo se permite una accion especifica por transferencia', ['accion_especifica_id'])]
    
    
    def onchange_accion_especifica(self, cr, uid, ids, accion_especifica_line, accion_especifica_parent, type_document, aumento, context=None):
        if context is None:
            context={}
        #~ print "onchange_accion_especifica"
        #~ print "accion_especifica_line",accion_especifica_line
        #~ print "accion_especifica_parent",accion_especifica_parent
        #~ print "type_document",type_document
        #~ print "aumento",aumento
        #~ print "context",context
        result = {'value':{}}
        if type_document == 'transferencia' and aumento == True and accion_especifica_line != False:
            #~ if accion_especifica_parent is False:
                #~ print "ids",ids
                #~ line_brw = self.browse(cr, uid, context.get('active_id', False), context=context)[0]
                #~ accion_especifica_parent = line_brw.modificacion_id.accion_especifica_id and line_brw.modificacion_id.accion_especifica_id.id
            if accion_especifica_parent and accion_especifica_line != accion_especifica_parent:
                result['value']['accion_especifica_id'] = accion_especifica_parent
                warning = {
                'title': 'Accion Especifica Diferente',
                'message':
                    'Esta realizando una transferencia presupuestaria entre una misma accion especifica no puede cambiar dicha accion.'
                }
                result['warning'] = warning
        #~ print "result",result
        return result
    
    #~ def create(self, cr, uid, vals, context=None, check=True):
        #~ if not context:
            #~ context={}
            #~ 
        #~ code = self.pool.get('ir.sequence').get(cr, uid, 'islr.wh.doc')
        #~ vals['code'] = code
        #~ return super(islr_wh_doc, self).create(cr, uid, vals, context)
        
presupuesto_modificacion_linea()
