# -*- coding: utf-8 -*-
"""
Módulo utilizado para desarrollar el modulo de comisiones
"""
import time

import decimal_precision as dp

import netsvc
import pooler
from osv import fields, osv, orm
from tools.translate import _

import unicodedata

def elimina_tildes(s):
    """
    Funcion para eliminar las tildes de algun texto utilizando el modulo unicodedata.
    """
    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn')).replace("'","")

class sisb_commission_to_user(osv.osv):
    _name = "sisb.commission.to.user"
    _description = "Commissions to user"
    _order = "id desc"
    
    
    _columns = {
        'name': fields.char('Description', size=128, select=True, required=True),
        'salesmen_id': fields.many2one('res.users', 'Salesmen', required=True),
        'percentage_commission':fields.float('Commission', digits_compute=dp.get_precision('Account'), help="Percentage commission for salesman"),
        'salesteam_id': fields.many2one('crm.case.section', 'Sales Team', required=True),
    }
    
        
    _defaults = {
        'percentage_commission': lambda *a: 0.00,
    }
    
    def _check_percentage(self, cr, uid, ids, context=None):
        #Garantizar que el porcentaje en la excepcion no sea mayor al porcentaje del equipo de venta
        for obj in self.browse(cr, uid, ids, context=context):
            if obj.percentage_commission > obj.salesteam_id.percentage_commission:
                return False
        return True
        
    _sql_constraints = [
        ('sisb_commission_to_user_salesmen_id', 'unique(salesmen_id)', 'The line of the exeptions must be attached only once to salesmen!'),
    ]
    
    _constraints = [
        (_check_percentage, 'The percentage to user must be minor that percentage of sales team. If you upgraded sales team, please write changes before record commissions to user', ['percentage_commission']),
    ]
    
sisb_commission_to_user()


class sisb_commission(osv.osv):
    _name = "sisb.commission"
    _description = "Commissions"
    _order = "id desc"
    
    _columns = {
        'name': fields.char('Description', size=256, select=True, readonly=True, required=True),
        'state': fields.selection([
            ('draft', 'Draft'),
            ('confirmed', 'Confirmed'),
            ('done', 'Done'),
            ], 'State', select=True, readonly=True, required=True, 
            help=' * The \'Draft\' state is used when a invoice is open. \
            \n* The \'Confirmed\'  state is used when a invoice is paid. \
            \n* The \'Done\' state is used when the commission is paid to the salesman.'),        
        'invoice_id': fields.many2one('account.invoice', 'Invoice Reference', ondelete='restrict', select=True, readonly=True, required=True),
        'company_id': fields.related('invoice_id', 'company_id', type='many2one',relation='res.company', string='Company', store=True, readonly=True),
        'user_id': fields.many2one('res.users', 'Salesman', select=True, readonly=True, required=True),
        'context_section_id': fields.many2one('crm.case.section', 'Sales Team', select=True, readonly=True, required=True),
        'condition': fields.selection([
            ('salesman', 'Salesman'),
            ('responsible', 'Responsible User'),
            ], 'Condition',  select=True, readonly=True, required=True,
            help=' * The \'salesman\' condition is used when a comission is paid  to the salesman. \
            \n* The \'responsible\' condition is used when the commission is paid to the responsible user.'),
        'date_invoice': fields.date('Invoice Date', readonly=True, select=True, required=True),
        'paid_date_invoice': fields.date('Date Invoice Paid', readonly=True, select=True),
        'base': fields.float('Base', digits_compute=dp.get_precision('Account'), readonly=True, required=True, help="Invoice Base"),
        'percentage': fields.char('Percentage (%)', size=16, readonly=True, required=True, help="Commission Percentage"), 
        'commission': fields.float('Commission', digits_compute=dp.get_precision('Account'), readonly=True, required=True, help="Invoice commission for salesmen"),
        'partner_id': fields.related('invoice_id', 'partner_id', type='many2one',relation='res.partner', string='Customer', store=True, select=True, readonly=True),
        'shop_id': fields.related('invoice_id', 'journal_id','sale_shop_id', type='many2one',relation='sale.shop', string='Sale Shop', store=True, select=True, readonly=True),
    }
    
    _defaults = {
        'name': lambda self, cr, uid, context: '/',
        'state': 'draft',
    }
    
    _sql_constraints = [
        ('sisb_commission_invoice_user_id', 'unique(invoice_id,user_id)', 'The line of the comission must be attached only once to salesmen and invoice!'),
    ]
    
    def date_reconcile(self, cr, uid, move_id, context=None):
        if context is None:
            context = {}
        
        aml_obj = self.pool.get('account.move.line')
        aml_id = aml_obj.search(cr, uid, [('move_id','=',move_id), ('reconcile_id','!=','')])
        aml_brw = aml_obj.browse(cr, uid, aml_id, context=context)
        
        return aml_brw[0].reconcile_id and aml_brw[0].reconcile_id.create_date or False
    
    def sales_team_invoice(self, cr, uid, invoice_id, salesmen_id, context=None):
        
        ai_obj = self.pool.get('account.invoice')
        ai_brw = ai_obj.browse(cr, uid, invoice_id, context=context)
        user_obj = self.pool.get('res.users')
        
        if ai_brw.type in ('out_invoice') and not ai_brw.parent_id:
            sale_order_brw = ai_brw.sale_ids
        else:
            sale_order_brw = ai_brw.parent_id.sale_ids
        
        if sale_order_brw:
            #Determinar el equipo de venta del pedido
            salesteam_brw = sale_order_brw[0].section_id
        else:
            #Determinar el equipo de venta del usario
            salesteam_brw= user_obj.browse(cr, uid, salesmen_id, context=context).context_section_id
        
        return salesteam_brw
    
    def salesman_percentage(self, cr, uid, invoice_id, salesmen_id, context=None):
        if context is None:
            context = {}
        
        salesteam_brw = self.sales_team_invoice(cr, uid, invoice_id, salesmen_id, context)
        
        #Calculo del porcentaje del vendedor del equipo de venta
        if salesteam_brw:
            #verificar que el usuario se encuentra dentro de las excepciones            
            for x in salesteam_brw.exceptions_commission_id:
                if x.salesmen_id.id == salesmen_id:
                    percent_commission = x.percentage_commission

                    return percent_commission

            return salesteam_brw.percentage_commission
            
        else:
            return 0.0
        
    
    def invoice_commision_exist(self, cr, uid, invoice_id, context=None):
        if context is None:
            context = {}
        
        result = self.search(cr, uid, [('invoice_id','=',invoice_id)])
        
        return result
        
    def responsible_exist(self, cr, uid, salesman_id, context=None):
        if context is None:
            context = {}
        
        team_obj = self.pool.get('crm.case.section')
        team_responsible = team_obj.search(cr, uid, [('user_id','=',salesman_id)])
        
        return team_responsible
        
    
    def commission_data(self, cr, uid, invoice_id, context=None):
        if context is None:
            context = {}
        
        for invoice in self.pool.get('account.invoice').browse(cr, uid, invoice_id, context=context):
            #facturas con vendedor asociado
            #facturas que no se ha registrado comisiones
            #facturas que no le han aplicado comisiones
            #facturas a clientes y rectificativas de clientes
            #facturas registradas a partir de la fecha indicada en la configuracion de la compañia
            if invoice.user_id \
                and not self.invoice_commision_exist(cr, uid, invoice.id, context) \
                and not invoice.commission_paid and invoice.type in ('out_invoice','out_refund')  \
                and  invoice.date_invoice >= invoice.company_id.date_init_commission:
                #Armar los datos en funcion a la factura y la comision del vendedor
                
                if invoice.type in ('out_refund'):
                    if (invoice.parent_id.date_invoice >= invoice.parent_id.company_id.date_init_commission):
                        #Es rectificativa de una factura de cliente aprobada despues de la fecha inicial
                        
                        #Para el vendedor
                        commission_ai_id = self.search(cr, uid, [('invoice_id','=',invoice.parent_id.id),('condition','=','salesman')])
                        commission_ai_brw = self.browse(cr, uid, commission_ai_id)
                        for comm_ai in commission_ai_brw:
                            commission_vals = {}
                            descripcion = comm_ai.name
                            commission_vals = {
                                'name': descripcion,
                                'invoice_id': invoice.id,
                                'user_id': comm_ai.user_id.id,
                                'context_section_id': comm_ai.context_section_id.id,
                                'condition': 'salesman',
                                'date_invoice': invoice.date_invoice,
                                'base': invoice.amount_untaxed * -1,
                                'percentage':comm_ai.percentage,
                                'commission': (invoice.amount_untaxed * float(comm_ai.percentage))/100 * -1,
                            }
                            self.create(cr, uid, commission_vals, context=context)
                            
                        #Para los responsables
                        commission_ai_id = self.search(cr, uid, [('invoice_id','=',invoice.parent_id.id),('condition','=','responsible')])
                        commission_ai_brw = self.browse(cr, uid, commission_ai_id)
                        for comm_ai in commission_ai_brw:
                            commission_vals = {}
                            descripcion = comm_ai.name
                            commission_vals = {
                                'name': descripcion,
                                'invoice_id': invoice.id,
                                'user_id': comm_ai.user_id.id,
                                'context_section_id': comm_ai.context_section_id.id,
                                'condition': 'responsible',
                                'date_invoice': invoice.date_invoice,
                                'base': invoice.amount_untaxed * -1,
                                'percentage': comm_ai.percentage,
                                'commission': (invoice.amount_untaxed * float(comm_ai.percentage))/100 * -1,
                            }
                            self.create(cr, uid, commission_vals, context=context)
                else:
                    
                    #Facturas de cliente
                    sales_team_id = self.sales_team_invoice(cr, uid, invoice.id, invoice.user_id.id, context)
                    
                    #Si aplica comision y el vendedor es diferente al responsable
                    if sales_team_id.allow_commission and not (self.responsible_exist(cr, uid, invoice.user_id.id, context)):
                        
                        percent =  self.salesman_percentage(cr, uid, invoice.id, invoice.user_id.id, context=context)
                        
                        #Si aplica comision se genera el registro
                        if percent:
                            
                            commission_vals = {}
                            descripcion = (invoice.date_invoice or '*') + ' [V] ' + ((invoice.user_id and invoice.user_id.context_section_id and elimina_tildes(invoice.user_id.context_section_id.name)) or '--') + ' ' + (elimina_tildes(invoice.user_id.name) or '--') + ' ' + (elimina_tildes(invoice.partner_id.name) or '--')
                            commission_vals = {
                                'name': descripcion,
                                'invoice_id': invoice.id,
                                'user_id': invoice.user_id.id,
                                'context_section_id': sales_team_id.id,
                                'condition': 'salesman',
                                'date_invoice': invoice.date_invoice,
                                'base': invoice.type == 'out_invoice' and invoice.amount_untaxed or invoice.amount_untaxed * -1,
                                'percentage':percent,
                                'commission': invoice.type == 'out_invoice' and (invoice.amount_untaxed * percent)/100 or (invoice.amount_untaxed * percent)/100 * -1,
                            }
                            self.create(cr, uid, commission_vals, context=context)
                            
                            #Recorrer el arbol del equipo de ventas para establecer comision de los responsables
                            parent = sales_team_id
                            while parent:
                                user_responsible = parent.user_id and parent.user_id.id or False
                                if user_responsible:
                                    percent =  parent.commission_to_responsible
                                    #Si aplica comision se genera el registro
                                    if percent:
                                        commission_vals = {}
                                        descripcion = (invoice.date_invoice or '*')  + ' [R] ' + ((invoice.user_id and invoice.user_id.context_section_id and elimina_tildes(invoice.user_id.context_section_id.name))  or '--') + ' ' + (elimina_tildes(invoice.user_id.name) or '--') + ' ' + (elimina_tildes(invoice.partner_id.name) or '--')
                                        commission_vals = {
                                            'name': descripcion,
                                            'invoice_id': invoice.id,
                                            'user_id': parent.user_id.id,
                                            'context_section_id': parent.id,
                                            'condition': 'responsible',
                                            'date_invoice': invoice.date_invoice,
                                            'base': invoice.type == 'out_invoice' and invoice.amount_untaxed or invoice.amount_untaxed * -1,
                                            'percentage':percent,
                                            'commission': invoice.type == 'out_invoice' and (invoice.amount_untaxed * percent)/100 or (invoice.amount_untaxed * percent)/100 * -1,
                                        }
                                        self.create(cr, uid, commission_vals, context=context)
                                
                                parent = parent.parent_id
                           
                    
        return True
    
    
sisb_commission()


class account_invoice(osv.osv):
    _inherit = "account.invoice"
    
    _columns = {
        'commission_paid':fields.boolean('Commission paid', help="This selection indicates whether generate the commission.")
    }
    
    def validar_cambio_vendedor(self, cr, uid, ids):
        ai_brw = self.browse(cr, uid, ids)
        if ai_brw[0].type in ('out_invoice','out_refund'):
            for so in ai_brw[0].sale_ids:
                if so.user_id.id <> ai_brw[0].user_id.id:
                    raise osv.except_osv(_('Error'),_("You can not change the salesman."))
                    
        return True
    
    #Validar que no exista cambios en el vendedor
    def action_date_assign(self, cr, uid, ids, *args):
        #Se hereda la función action_date_assign para llamar funcion de validacion
        self.validar_cambio_vendedor(cr, uid, ids)
        return super(account_invoice, self).action_date_assign(cr, uid, ids, args)
    
    def action_move_create(self, cr, uid, ids, *args):
        #Se hereda la función action_move_create para registrar comisiones en estado borrador
        #Se hereda este metodo para obtener la fecha de la factura registrada (date_invoice)
        valor_retorno = super(account_invoice, self).action_move_create(cr, uid, ids, args)
        
        if valor_retorno:
            #LLAMADO DEL METODO PARA REGISTRAR COMISIONES
            commission_obj = self.pool.get('sisb.commission')
            
            resultado = commission_obj.commission_data(cr, uid, ids)
            
            if not resultado:
                return False
                
        return valor_retorno
        
        
    def confirm_paid(self, cr, uid, ids, context=None):
        #Se hereda la función confirm_paid para cambiar de estado el registro de comisiones a confirmado
        if context is None:
            context = {}
            
        valor_retorno = super(account_invoice, self).confirm_paid(cr, uid, ids, context=context)
        
        if valor_retorno:
            ai_brw = self.browse(cr, uid, ids)
            if ai_brw[0].type in ('out_invoice','out_refund') and not ai_brw[0].commission_paid:
                #LLAMADO DEL METODO PARA CONFIRMAR COMISIONES
                commission_obj = self.pool.get('sisb.commission')
                commision_ids = commission_obj.search(cr, uid, [('invoice_id','=',ids)])
                
                valores = {
                    'state':'confirmed',
                    'paid_date_invoice': commission_obj.date_reconcile(cr, uid, ai_brw[0].move_id.id, context=context),
                }
                
                commission_obj.write(cr, uid, commision_ids, valores, context=context)
                self.write(cr, uid, ids, {'commission_paid':True}, context=context)
        
        
        return valor_retorno
        
account_invoice()

class crm_case_section(osv.osv):
    _inherit = "crm.case.section"
    
    _columns = {
        'commission_to_responsible':fields.float('Commission to responsible (%)', digits_compute=dp.get_precision('Account'), help="Percentage commission to responsible"),
        'allow_commission': fields.boolean('Allow commission', help="Allows to apply commission to members sales team"),
        'percentage_commission':fields.float('Commission', digits_compute=dp.get_precision('Account'), help="Percentage commission for members of sales team"),
        'exceptions_commission_id': fields.one2many('sisb.commission.to.user', 'salesteam_id', 'Exceptions commission'),
        'salesmen_member_id': fields.one2many('res.users', 'context_section_id', 'Salesman Member'),
    }
    
    def _check_percentage(self, cr, uid, ids, context=None):
        #Garantizar que el porcentaje en la excepcion no sea mayor al porcentaje del equipo de venta
        for obj in self.browse(cr, uid, ids, context=context):
            for com_u in obj.exceptions_commission_id:
                if com_u.percentage_commission > obj.percentage_commission:
                    return False
        return True
    
    _constraints = [
        (_check_percentage, 'The percentage to user must be minor that percentage of sales team.', ['percentage_commission']),
    ]
    
    _defaults = {
        'allow_commission': lambda *a: 0,
        'percentage_commission': lambda *a: 0.00,
    }
    
    def if_allow_commission(self, cr, uid, ids, valor, context=None):
        if context is None:
            context = {}
        
        result = {}
        
        if not valor:
            result = {'value': {
                'commission_to_responsible': 0.00,
                'percentage_commission': 0.00,
                }
            }
        
        return result
    

crm_case_section()

class sale_order(osv.osv):
    _inherit = 'sale.order'
    
    def check_limit(self, cr, uid, ids, context={}):
        if context is None:
            context = {}
        
        #Verificar que el vendedor pertenece la equipo de venta seleccionado en el pedido
        so = self.browse(cr, uid, ids[0], context)
        if so.user_id:
            sales_team_salesman = so.user_id.context_section_id and so.user_id.context_section_id.id or False
            sales_team_so = so.section_id and so.section_id.id or False
            if sales_team_salesman:
                if sales_team_salesman != sales_team_so:
                    raise osv.except_osv(_('Error !'), _('The salesman %s does not belong to the sales team!')%so.user_id.name)
                    return False
            else:
                raise osv.except_osv(_('Error !'), _('the user %s is not salesman!')%so.user_id.name)
                return False

        return super(sale_order, self).check_limit(cr, uid, ids, context=context)
        
        
sale_order()




