#!/usr/bin/env python
# -*- coding: utf-8 -*-

from osv import fields, osv
from tools.translate import _
import datetime
from datetime import timedelta
from functools import wraps


class horas_extras_planificacion_tipos(osv.osv):

    _name = 'horas.extras.planificacion.tipos'
    _description = 'tipos de planificacion de sobretiempo'
    _rec_name = 'descripcion'
    _columns = {
        'name': fields.char('Motivo', size=60),
        'descripcion': fields.char('Descripcion', size=60),
        'parent_id': fields.many2one('horas.extras.planificacion.tipos', 'Padre Tipos', select=True),
        'child_ids': fields.one2many('horas.extras.planificacion.tipos', 'parent_id', 'tipos hijos'),
    }

    _sql_constraints = [('unique_code', 'unique(name)', _('El parametro motivo debe ser unico'))]


horas_extras_planificacion_tipos()


class horas_extras_planificacion(osv.osv):

    _name = 'horas.extras.planificacion'
    _description = 'horas extras'
    _order = 'fecha desc'
    _rec_name = 'referencia'

    def search_manager_department(self, cr, uid, emp_id):
        obj_employee = self.pool.get('hr.employee')
        obj_department = self.pool.get('hr.department')
        department_id = obj_department.search(cr, uid, [('manager_id', 'in', emp_id)])
        department_brw = obj_department.browse(cr, uid, department_id)
        subordinado = []
        for emp in department_brw:
            subordinado = [i.id for i in emp.member_ids]
        return subordinado

    def search_childs_department(self, cr, uid, emp_id):
        emp_ids = []
        sql = """
        select id from hr_employee
            where department_id in (select id from (select distinct slh.id as child_id from hr_department slp
            left outer join hr_department slh on slh.parent_id between slp.id and slh.parent_id
            where slh.parent_id =%s) as hijos join hr_department as hrd ON hrd.id = hijos.child_id or hrd.parent_id = hijos.child_id
            or id = %s order by id asc) or (department_id = %s);
        """

        obj_employee = self.pool.get('hr.employee')
        employee_brw = obj_employee.browse(cr, uid, emp_id)
        department_id = [i.department_id.id for i in employee_brw if i.job_id.type_job_id.autoriza_sobretiempo]
        if department_id:
            sql = sql % (department_id[0], department_id[0], department_id[0])
            cr.execute(sql)
            emp_ids = cr.fetchall()
            emp_ids = list(set([i for t in emp_ids for i in t]))
        return emp_ids

    def validar_departamentos(self, cr, uid):
        departamento_id = self._get_department(cr, uid)
        ids = []
        if departamento_id:
            sql = """
            select id from (select distinct slh.id as child_id from hr_department slp
            left outer join hr_department slh on slh.parent_id between slp.id and slh.parent_id
            where slh.parent_id =%s) as hijos join hr_department as hrd ON hrd.id = hijos.child_id
            or hrd.parent_id = hijos.child_id
            """ % departamento_id
            ids.append(departamento_id)
            cr.execute(sql)
            ids_fech = cr.fetchall()
            if ids:
                ids = ids + [id[0] for id in ids_fech]

        return ids

    def _get_trabajadores_nomina(self, cr, uid, group_name, param_nomina=None, context=None):
        if not param_nomina:
            parameter_obj = self.pool.get('hr.parameters')
            parameters_id = parameter_obj.search(cr, uid, [('type', '=', 'cargar_sobretiempos_nomina')])
            if not parameters_id:
                return False
            parameters_brw = parameter_obj.browse(cr, uid, parameters_id)[0]
            param_nomina = parameters_brw.check_cargar_sobretiempos_nomina
        if param_nomina:
            grupo = self._get_grupo_id(cr, uid, group_name)
            if grupo:
                grupos_usuario = self._get_grupos_usuario(cr, uid, context)
                if grupo in grupos_usuario:
                    return True
        return False

    def _get_subordinados_ids(self, cr, uid, ids, field_name, arg, context):
        res = {}
        obj_employee = self.pool.get('hr.employee')
        obj_department = self.pool.get('hr.department')
        load_nom = self._get_trabajadores_nomina(cr, uid, 'group_sisb_horas_extras_th', param_nomina=None, context=None)
        params = [] if uid == 1 or load_nom else [('user_id', '=', uid)]
        emp_id = obj_employee.search(cr, uid, params)
        subordinado = emp_id
        if uid != 1 and not load_nom:
            busqueda_recursiva_emp = self.search_childs_department
            subordinado = subordinado + busqueda_recursiva_emp(cr, uid, emp_id)
        subordinado = list(set(subordinado))
        if emp_id:
            if emp_id[0] in subordinado:
                subordinado.remove(emp_id[0])
        subordinado = str(subordinado)
        for i in ids:
            res[i] = subordinado
        return res

    def _get_back_draft(self, cr, uid, ids, field_name, arg, context):
        res = {}
        brw = self.browse(cr, uid, ids)
        grupo = self._get_grupo_id(cr, uid, 'group_sisb_horas_extras_volver_borrador')
        grupos_usuario = self._get_grupos_usuario(cr, uid, context)

        for i in brw:
            if (uid == 1 and i.state in ['confirmed', 'processing']) or (grupo in grupos_usuario and i.state in ['confirmed', 'processing']) or i.state == 'confirmed':
                res[i.id] = True
            else:
                res[i.id] = False
        return res

    def _get_department(self, cr, uid):
        res = False
        if uid == 1:
            return res
        obj_employee = self.pool.get('hr.employee')
        params = [('user_id', '=', uid)]
        emp_id = obj_employee.search(cr, uid, params)
        emp_brw = obj_employee.browse(cr, uid, emp_id)
        if emp_brw:
            res = emp_brw[0].department_id.id
        else:
            raise osv.except_osv(_('Error'), _('El usuario conectado debe poseer un perfil asociado en Talento Humano'))
        return res

    def _get_grupos_usuario(self, cr, uid, context=None):
        return [group.id for group in self.pool.get('res.users').browse(cr, uid, uid, context=None).groups_id]

    def _get_grupo_id(self, cr, uid, group_name, context=None):
        """Devuelve el id del grupo 'group_name'. Devuelve False en caso de no existir """
        model_data_obj = self.pool.get('ir.model.data')
        grupo_search = model_data_obj.search(cr, 1, [('name', '=', group_name), ('model', '=', 'res.groups')], context=None)
        grupo = model_data_obj.read(cr, 1, grupo_search, ['res_id'], context)[0]['res_id']
        if grupo:
            return grupo
        return False

    _columns = {
        'name': fields.char('name', size=64),
        'user_id': fields.many2one('res.users', 'Usuario', readonly=True),
        'department_id': fields.many2one('hr.department', 'Departamento', readonly=True),
        'observacion': fields.text('Observacion'),
        'fecha': fields.date('Fecha', required=True, readonly=True),
        'referencia': fields.char('Referencia', size=64, required=True, readonly=True),
        'permitidos': fields.function(_get_subordinados_ids, string='Permitidos', method=True, type='text', store=False, ),
        'permitir_back_draf': fields.function(_get_back_draft, string='Volver a borrador', method=True, type='boolean', store=False),
        'state': fields.selection([('draft', 'Borrador'), ('confirmed', 'Confirmado'), ('approved', 'Aprobado'),
                                   ('processing', 'Procesado'), ('exception', 'Excepcion'), ('load', 'Cargado'),
                                   ('done', 'Listo'), ('cancel', 'Cancelado')], 'Estado', readonly=True),
    }

    _defaults = {
        'fecha': lambda *a: datetime.datetime.now().strftime("%Y-%m-%d"),
        'state': lambda *a: 'draft',
        'referencia': lambda *a: 'ST-#',
        'user_id': lambda self, cr, uid, context=None: uid,
        'department_id': lambda self, cr, uid, context=None: self._get_department(cr, uid)
    }


horas_extras_planificacion()


class horas_extras_planificacion_lineas(osv.osv):

    _name = 'horas.extras.planificacion.lineas'
    _description = 'Horas extras lineas'
    _order = 'id desc'

    def _calc_turno(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        brw = self.browse(cr, uid, ids, context=context)
        days = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday']
        hsw_obj = self.pool.get('hr.shift.week')
        for i in brw:
            hsw_id = hsw_obj.search(cr, uid, [('employee_id', '=', i.empleado_id.id), ('period_id.date_start', '<=', i.fecha_desde), ('period_id.date_end', '>=', i.fecha_desde)], context=context)
            res[i.id] = hsw_obj.read(cr, uid, hsw_id, context=context)[0][days[datetime.datetime.strptime(i.fecha_desde, '%Y-%m-%d').weekday()] + '_hours_day_id'][1] if hsw_id else None
        return res

    _columns = {
        'name': fields.char('Lineas de horas extras', size=64),
        'empleado_id': fields.many2one('hr.employee', 'Trabajador', required=True),
        'department_id': fields.related('empleado_id', 'department_id', relation='hr.department', string="Departamento",
                                        type='many2one', store=True, readonly=True, select=False),
        'fecha_desde': fields.date('Fecha desde', required=True),
        'fecha_hasta': fields.date('Fecha hasta', required=True),
        'hora_desde': fields.time('Hora desde', required=True),
        'hora_hasta': fields.time('Hora hasta', required=True),
        'horas_extras_planificacion_id': fields.many2one('horas.extras.planificacion', 'Referencia', readonly=True),
        'motivo_id': fields.many2one('horas.extras.planificacion.tipos', 'Motivo', required=True),
        'excepciones': fields.boolean('excepciones'),
        'turno_id': fields.function(_calc_turno, method=True, string='Turno calculado', type='meny2one', relation='hr.hour.per.day'),
        'state': fields.selection([('draft', 'Borrador'), ('confirmed', 'Confirmado'), ('approved', 'Aprobado'),
                                   ('processing', 'Procesado'), ('exception', 'Excepcion'), ('exceed', 'Excedido'), ('load', 'Cargado'), ('done', 'Listo'),
                                   ('cancel', 'Cancelado')], 'Estado', readonly=True),
        'exceed': fields.boolean("Excedido")
    }
    _defaults = {
        'state': lambda *a: 'draft',
        'excepciones': lambda *a: False,
        'exceed': False
    }

    def _check_hour(self, cr, uid, ids, conetxt=None):
        convert_datetime = self.convert_datetime
        hel_brw = self.browse(cr, uid, ids)
        for i in hel_brw:
            fecha_desde = "%s %s" % (i.fecha_desde, i.hora_desde)
            fecha_hasta = "%s %s" % (i.fecha_hasta, i.hora_hasta)
            if fecha_desde and fecha_hasta:
                hora_inicio = convert_datetime(fecha_desde, tipo='object')
                hora_fin = convert_datetime(fecha_hasta, tipo='object')
                if hora_fin <= hora_inicio:
                    return False
        return True

    def _check_24(self, cr, uid, ids, conetxt=None):
        hel_brw = self.browse(cr, uid, ids)
        convert_datetime = self.convert_datetime
        for i in hel_brw:
            fecha_desde = "%s %s" % (i.fecha_desde, i.hora_desde)
            fecha_hasta = "%s %s" % (i.fecha_hasta, i.hora_hasta)
            if fecha_desde and fecha_hasta:
                hora_inicio = convert_datetime(fecha_desde, tipo='object')
                hora_fin = convert_datetime(fecha_hasta, tipo='object')
                total_horas = abs(hora_fin - hora_inicio)
                days = total_horas.days * 24
                hours = total_horas.seconds / 3600.0
                total = days + hours
                if total > 24.0:
                    return False
        return True

    def _check_last_hour(self, cr, uid, ids, context=None):
        hel_brw = self.browse(cr, uid, ids)
        convert_datetime = self.convert_datetime
        now = datetime.datetime.now()
        last = now - timedelta(days=1)
        for i in hel_brw:
            fecha_desde = "%s %s" % (i.fecha_desde, i.hora_desde)
            fecha_hasta = "%s %s" % (i.fecha_hasta, i.hora_hasta)
            if fecha_desde and fecha_hasta:
                hora_inicio = convert_datetime(fecha_desde, tipo='object')
                hora_fin = convert_datetime(fecha_hasta, tipo='object')
                if hora_inicio > last and hora_fin > now:
                    return False
        return True

    _constraints = [(_check_hour, 'La Hora final del sobretiempo no puede ser menor o igual a la hora de inicio', ['hora_hasta']),
                    (_check_24, 'Las lineas de sobretiempos no puede ser mayor de 24 horas', ['hora_hasta']),
                    (_check_last_hour, 'Las lineas de sobretiempos no puede ser mayor a la fecha actual', ['hora_hasta']), ]

    def onchange_fecha(self, cr, uid, ids, fecha_desde):
        result = {}
        if not ids:
            result = {'value': {'fecha_hasta': fecha_desde}}
        return result

    def onchange_hora(self, cr, uid, ids, hora_desde):
        result = {}
        if not ids:
            result = {'value': {'hora_hasta': hora_desde}}
        return result

    def convert_datetime(self, value_date, tipo='object', format_inp=None):
        if not format_inp:
            format_inp = "%Y-%m-%d %H:%M:%S"
        value_date = datetime.datetime.strptime(value_date, format_inp)
        d = {'date': value_date.strftime("%Y-%m-%d"),
             'time': value_date.strftime("%H:%M:%S"),
             'object': value_date}
        return d[tipo]

    def _validate_hour_lines(self, cr, uid, ids, emp_id, fecha_desde, fecha_hasta, hora_desde, hora_hasta):
        date_desde = "%s %s" % (fecha_desde, hora_desde)
        date_hasta = "%s %s" % (fecha_hasta, hora_hasta)
        sql = """
             select id from
                (select id, fecha_desde + hora_desde as date_desde, fecha_hasta + hora_hasta as date_hasta
                from horas_extras_planificacion_lineas
                where empleado_id = %s) as hxl
            where (('%s' BETWEEN  hxl.date_desde and hxl.date_hasta)
            or ('%s' BETWEEN  hxl.date_desde and hxl.date_hasta))
            and hxl.id != %s
            """ % (emp_id, date_desde, date_hasta, ids)
        cr.execute(sql)
        validar = cr.fetchall()
        validar = [t for i in validar for t in i]
        if validar:
            brw = self.browse(cr, uid, validar)[0]
            msg = """
                 Las horas del trabajador %s coinciden con un registro anterior
                 """ % brw.empleado_id.name
            raise osv.except_osv(_('Error'), _(msg))
        return True

    def _check_nomina(self, cr, uid, ids, emp_id, conetxt=None):
        hel_obj = self.pool.get('horas.extras.planificacion.lineas')
        emp_obj = self.pool.get('hr.employee')
        emp_brw = emp_obj.browse(cr, uid, emp_id)
        hel_brw = hel_obj.browse(cr, uid, ids)
        nomina_emp = emp_brw.hr_payroll_id.id
        for i in hel_brw:
            if nomina_emp != i.empleado_id.hr_payroll_id.id:
                return False
        return True

    def diff_hours(self, hour_start, hour_end):
        hd = datetime.datetime.strptime(hour_start, '%H:%M:%S')
        hh = datetime.datetime.strptime(hour_end, '%H:%M:%S')
        return (hh - hd).seconds / float(3600)

    def validate_lines(self, cr, uid, vals, cid=0, context=None):
        emp_id = vals['empleado_id'] if type(vals['empleado_id']) == int else vals.empleado_id.id
        horas_extras_planificacion_id = vals['horas_extras_planificacion_id'] if type(vals['horas_extras_planificacion_id']) == int else vals.horas_extras_planificacion_id.id
        fecha_desde = vals['fecha_desde']
        fecha_hasta = vals['fecha_hasta']
        hora_desde = vals['hora_desde']
        hora_hasta = vals['hora_hasta']
        dept_obj = self.pool.get('hr.employee').browse(cr, uid, emp_id, context=context).department_id
        # Validando que no exceda de las horas diarias permitidas por el departamento
        week_lines = self.search(cr, uid, ['&', '&', '&', '|', ('state', 'not in', ['draft', 'cancel']), ('horas_extras_planificacion_id', '=', horas_extras_planificacion_id), ('fecha_desde', '=', fecha_desde), ('empleado_id', '=', emp_id), ('id', '!=', cid)], context=context)
        sum_hours = sum(self.diff_hours(l.hora_desde, l.hora_hasta) for l in self.browse(cr, uid, week_lines, context=context))
        if sum_hours + self.diff_hours(hora_desde, hora_hasta) > dept_obj.hours_per_day:
            return False
        # Validando que no pase de las horas semanales permitidas por el departamento
        day = datetime.datetime.strptime(fecha_desde, '%Y-%m-%d')
        weekstart = day - timedelta(days=day.weekday())
        weekend = day + timedelta(days=6 - day.weekday())
        week_lines = self.search(cr, uid, ['&', '&', '&', '&', '|', ('state', 'not in', ['draft', 'cancel']), ('horas_extras_planificacion_id', '=', horas_extras_planificacion_id), ('fecha_desde', '>=', weekstart.strftime('%Y-%m-%d')), ('fecha_desde', '<=', weekend.strftime('%Y-%m-%d')), ('empleado_id', '=', emp_id), ('id', '!=', cid)], context=context)
        sum_hours = sum(self.diff_hours(l.hora_desde, l.hora_hasta) for l in self.browse(cr, uid, week_lines, context=context))
        if sum_hours + self.diff_hours(hora_desde, hora_hasta) > dept_obj.hours_per_week:
            return False
        # Validando que no exceda de las horas mensuales para todos los trabajadores del departamento
        monthstart = day - timedelta(days=day.day - 1)
        next_month = day.replace(day=28) + timedelta(days=4)
        monthend = next_month - timedelta(days=next_month.day)
        while dept_obj:  # Iterando por cada departamento, desde el actual hasta sus padres
            depts_ids = pending_ids = dept_obj.child_ids
            while pending_ids:  # Haciendo la búsqueda hacia sus hijos
                pending_ids = reduce(lambda x, y: x + y, [c.child_ids for c in pending_ids], [])
                depts_ids += pending_ids
            depts_ids = [i.id for i in depts_ids] + [dept_obj.id]
            week_lines = self.search(cr, uid, ['&', '&', '&', '&', '|', ('state', 'not in', ['draft', 'cancel']), ('horas_extras_planificacion_id', '=', horas_extras_planificacion_id), ('fecha_desde', '>=', monthstart.strftime('%Y-%m-%d')), ('fecha_desde', '<=', monthend.strftime('%Y-%m-%d')), ('empleado_id.department_id', 'in', depts_ids), ('id', '!=', cid)], context=context)
            sum_hours = sum(self.diff_hours(l.hora_desde, l.hora_hasta) for l in self.browse(cr, uid, week_lines, context=context))
            if sum_hours + self.diff_hours(hora_desde, hora_hasta) > dept_obj.hours_per_dept:
                return False
            else:
                dept_obj = dept_obj.parent_id
        return True

    def create(self, cr, uid, vals, context=None):
        lines_ids = self.search(cr, uid, [('horas_extras_planificacion_id', '=', vals['horas_extras_planificacion_id'])])
        if lines_ids:
            check_nomina = self._check_nomina(cr, uid, lines_ids, vals['empleado_id'])
            if not check_nomina:
                raise osv.except_osv(_('Error'), _('Los trabajadores deben pertenecer a una misma nomina por hoja'))
        return super(horas_extras_planificacion_lineas, self).create(cr, uid, vals, context=context)

    def write(self, cr, uid, ids, vals, context=None):
        ids = [ids] if type(ids) != list else ids
        for record in self.browse(cr, uid, ids, context=context):
            lines_ids = self.search(cr, uid, [('horas_extras_planificacion_id', '=', record.horas_extras_planificacion_id.id)])
            if lines_ids:
                check_nomina = self._check_nomina(cr, uid, lines_ids, record.empleado_id.id)
                if not check_nomina:
                    raise osv.except_osv(_('Error'), _('Los trabajadores deben pertenecer a una misma nomina por hoja'))
            super(horas_extras_planificacion_lineas, self).write(cr, uid, record.id, vals, context=context)
        return True

    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(_('Accion no permitida!'), _('No se pueden duplicar las lineas de sobretiempos'))

    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if not isinstance(ids, list):
            ids = [ids]
        brw = self.browse(cr, uid, ids)
        load_nom = self.pool.get('horas.extras.planificacion')._get_trabajadores_nomina(cr, uid, 'group_sisb_horas_extras_th', param_nomina=False)
        if uid != 1 and not load_nom:
            if not reduce(lambda x, y: x and y, map(lambda x: x.horas_extras_planificacion_id.user_id.id == uid, brw)):
                raise osv.except_osv(_('Error'), _('No cuenta con los permisos para borrar lineas a este documento'))
        return super(horas_extras_planificacion_lineas, self).unlink(cr, uid, ids, context=context)


horas_extras_planificacion_lineas()


class horas_extras_planificacion_calculos(osv.osv):

    _name = 'horas.extras.planificacion.calculos'
    _description = 'Historico de horas extras'
    _columns = {
        'trabajador_id': fields.many2one('hr.employee', 'Trabajador', readonly=True),
        'horas_extras_planificacion_id': fields.many2one('horas.extras.planificacion', 'Planificacion de horas extras lineas', readonly=True),
        'value': fields.float('Cantidad', readonly=True),
        'concept_id': fields.many2one('hr.concept', 'Concepto', readonly=True),
    }


horas_extras_planificacion_calculos()


class horas_extras_planificacion(osv.osv):
    _inherit = 'horas.extras.planificacion'

    def _nota_nomina_editable(self, cr, uid, ids, name, args, context=None):
        res = {}
        grupo = self._get_grupo_id(cr, uid, 'group_hr_user')
        if grupo:
            grupos_usuario = self._get_grupos_usuario(cr, uid, context)
            horas_extras_brw = self.browse(cr, uid, ids[0])
            for line in horas_extras_brw.planificacion_lineas_ids:
                if grupo in grupos_usuario and line.state in ['approved', 'processing']:
                    res[line.horas_extras_planificacion_id.id] = True
                else:
                    res[line.horas_extras_planificacion_id.id] = False

        return res

    _columns = {
        'planificacion_lineas_ids': fields.one2many('horas.extras.planificacion.lineas', 'horas_extras_planificacion_id', 'Lineas de sobretiempos'),
        'calculos_ids': fields.one2many('horas.extras.planificacion.calculos', 'horas_extras_planificacion_id', 'Calculos de sobretiempos'),
        'variable_concept_id': fields.many2one('hr.variable.concepts', 'Concepto variable', readonly=True),
        'nota_nomina': fields.text('Nota de nomina'),
        'nota_nomina_editable': fields.function(_nota_nomina_editable, method=True, string='¿Es editable?', type='boolean'),
    }

    def check_lines(self, cr, uid, ids, context=None):
        validar = self.pool.get('horas.extras.planificacion.lineas').validate_lines
        for record in self.browse(cr, uid, ids, context=context):
            for line in record.planificacion_lineas_ids:
                self.write(cr, uid, record.id, {'planificacion_lineas_ids': [(1, line.id, {'exceed': not validar(cr, uid, line, cid=line.id, context=context)})]}, context=context)
        return True

    def validate_doc(func):
        @wraps(func)
        def wraper(*args, **kargs):
            proccess_name = 'confirmar' if func.__name__ == 'confirmed' else 'aprobar' if func.__name__ == 'approved' else 'cargar'
            self, cr, uid, ids, context = args
            today = datetime.datetime.now().strftime('%Y-%m-%d')
            for record in self.browse(cr, uid, ids, context=context):
                for line in record.planificacion_lineas_ids:
                    allowed_days = line.empleado_id.hr_payroll_id.period_days * line.empleado_id.hr_payroll_id.periodos_sobretiempo
                    period_id = self.pool.get('hr.period').search(cr, 1, [('date_start', '<=', today), ('date_end', '>=', today), ('type', '=', line.empleado_id.hr_payroll_id.hr_payment_calendar_id.type)], context=context)
                    period = self.pool.get('hr.period').browse(cr, 1, period_id, context=context)[0] if len(period_id) > 0 else False
                    if not period:
                        raise osv.except_osv(_('Error'), _('No existe periodo creado en Talento Humano para el dia actual'))
                    if line.fecha_hasta >= period.date_start:
                        raise osv.except_osv(_('Error'), _('No se puede %s sobretiempo al empleado %s en el periodo actual de la nomina') % (proccess_name, line.empleado_id.name))
                    if datetime.datetime.strptime(period.date_start, '%Y-%m-%d') - timedelta(days=allowed_days) > datetime.datetime.strptime(line.fecha_desde, '%Y-%m-%d'):
                        raise osv.except_osv(_('Error'), _('No se puede %s sobretiempo al empleado %s fuera de los periodos configurados por nomina') % (proccess_name, line.empleado_id.name))
            return func(*args, **kargs)
        return wraper

    @validate_doc
    def confirmed(self, cr, uid, ids, context=None):
        validar = self.pool.get('horas.extras.planificacion.lineas').validate_lines
        #  Validando que no vaya a saltar el flujo
        if not isinstance(ids, list):
            ids = [ids]
        for record in self.browse(cr, uid, ids):
            if record.state != 'draft':
                return False
        # Fin de la validacion
        group_id = self.pool.get('ir.model.data').search(cr, uid, [('name', '=', 'group_sisb_horas_extras_autoriza_exceso')])
        group_id = self.pool.get('ir.model.data').browse(cr, uid, group_id[0]).res_id
        autoriza = True if group_id in map(int, self.pool.get('res.users').browse(cr, uid, uid).groups_id) else False

        lines_obj = self.pool.get('horas.extras.planificacion.lineas')
        lines_id = lines_obj.search(cr, uid, [('horas_extras_planificacion_id', '=', ids)])
        line_brw = lines_obj.browse(cr, uid, lines_id)
        if not line_brw:
            raise osv.except_osv(_('Error'), _('No hay lineas cargadas'))
        for i in line_brw:
            emp_id = i.empleado_id.id
            fecha_desde = i.fecha_desde
            fecha_hasta = i.fecha_hasta
            hora_desde = i.hora_desde
            hora_hasta = i.hora_hasta
            lines_obj._validate_hour_lines(cr, uid, i.id, emp_id, fecha_desde, fecha_hasta, hora_desde, hora_hasta)
            if not validar(cr, uid, i, cid=i.id, context=context):
                if autoriza:
                    lines_obj.log(cr, uid, i.id, 'Se autorizo sobretiempo con exceso a %s (%s) desde %s %s hasta %s %s' % (i.empleado_id.name, i.empleado_id.cod_number, i.fecha_desde, i.hora_desde, i.fecha_hasta, i.hora_hasta))
                else:
                    raise osv.except_osv(_('Error'), _('No se puede confirmar sobretiempo al empleado %s porque ha excedido las horas permitidas') % i.empleado_id.name)
        vals = {'state': 'confirmed'}
        if self.browse(cr, uid, ids[0]).referencia == 'ST-#':
            vals.update({'referencia': self.pool.get('ir.sequence').get(cr, uid, 'sobretiempos_tseq')})
        self.write(cr, uid, ids, vals)
        self.write_state_lines(cr, uid, ids, 'confirmed')
        lines_obj.write(cr, uid, lines_id, {'exceed': False}, context=context)
        return True

    def write_state_lines(self, cr, uid, ids, state, lines=None, field=None):
        horas_extras_lines_obj = self.pool.get('horas.extras.planificacion.lineas')
        brw = self.browse(cr, uid, ids)[0]
        if not lines:
            lines = [obj.id for obj in brw.planificacion_lineas_ids]
        if not field:
            field = 'state'
        write_id = horas_extras_lines_obj.write(cr, uid, lines, {field: state})
        return write_id

    def back_draft(self, cr, uid, ids, context=None):
        #  Validando que no vaya a saltar el flujo
        if not isinstance(ids, list):
            ids = [ids]
        for record in self.browse(cr, uid, ids):
            if record.state not in ['confirmed', 'processing']:
                return False
        # Fin de la validacion
        self.write(cr, uid, ids, {'state': 'draft'})
        self.write_state_lines(cr, uid, ids, 'draft')
        horas_extras_calc_obj = self.pool.get('horas.extras.planificacion.calculos')
        calc_ids = horas_extras_calc_obj.search(cr, uid, [('horas_extras_planificacion_id', 'in', ids)])
        if calc_ids:
            horas_extras_calc_obj.unlink(cr, uid, calc_ids)
        return True

    def cancel(self, cr, uid, ids, context=None):
        for i in self.browse(cr, uid, ids):
            if not i.nota_nomina:
                raise osv.except_osv(_('Error'), _('Debe colocar una observacion para poder cancelar la hoja de sobretiempo'))
        self.write(cr, uid, ids, {'state': 'cancel'})
        self.write_state_lines(cr, uid, ids, 'cancel')
        return True

    @validate_doc
    def approved(self, cr, uid, ids, context=None):
        #  Validando que no vaya a saltar el flujo
        if not isinstance(ids, list):
            ids = [ids]
        for record in self.browse(cr, uid, ids):
            if record.state != 'confirmed':
                return False
        # Fin de la validacion
        self.write(cr, uid, ids, {'state': 'approved'})
        self.write_state_lines(cr, uid, ids, 'approved')
        return True

    def chequear_turno(self, cr, uid, ids, date_start, date_end, emp_id):
        shitf_week_obj = self.pool.get('horas.extras.wizard.concept.variable')
        shitf_week = shitf_week_obj.get_shitf_week(cr, uid, ids, date_start, date_end, emp_id)
        return shitf_week

    def calcular_cantidad_conceptos(self, cr, uid, ids, date_start, date_end, emp_id, line_id, shift_id=None):
        horas_extras_obj = self.pool.get('horas.extras.wizard.concept.variable')
        value = horas_extras_obj.calcular_sobretiempo(cr, uid, ids, date_start, date_end, emp_id, ids, shift_id=shift_id)
        return value

    def processing(self, cr, uid, ids, context=None):
        #  Validando que no vaya a saltar el flujo
        if not isinstance(ids, list):
            ids = [ids]
        for record in self.browse(cr, uid, ids):
            if record.state != 'approved':
                return False
        # Fin de la validacion
        self.write(cr, uid, ids, {'state': 'processing'})
        dict_emp_values = {}
        for i in self.browse(cr, uid, ids, context):
            for line in i.planificacion_lineas_ids:
                dict_emp_values[line.empleado_id.id] = {}

        for i in self.browse(cr, uid, ids, context):
            for line in i.planificacion_lineas_ids:
                fecha_desde = "%s %s" % (line.fecha_desde, line.hora_desde)
                fecha_hasta = "%s %s" % (line.fecha_hasta, line.hora_hasta)
                if not self.chequear_turno(cr, uid, ids, fecha_desde, fecha_hasta, line.empleado_id.id):
                    self.write_state_lines(cr, uid, ids, 'exception', lines=line.id)
                    self.write_state_lines(cr, uid, ids, True, lines=line.id, field='excepciones')
                else:
                    self.write_state_lines(cr, uid, ids, 'processing', lines=line.id)
                    valor = self.calcular_cantidad_conceptos(cr, uid, ids, fecha_desde, fecha_hasta, line.empleado_id.id, line.id, None)
                    for concepto in valor:
                        if concepto not in dict_emp_values[line.empleado_id.id]:
                            dict_emp_values[line.empleado_id.id][concepto] = valor[concepto]
                        else:
                            dict_emp_values[line.empleado_id.id][concepto] += valor[concepto]

        horas_extras_calc_obj = self.pool.get('horas.extras.planificacion.calculos')

        for emp in dict_emp_values:
            vals = {'trabajador_id': emp, 'horas_extras_planificacion_id': ids[0]}
            for c in dict_emp_values[emp]:
                vals['concept_id'] = c
                vals['value'] = dict_emp_values[emp][c]
                horas_extras_calc_obj.create(cr, uid, vals)
        return True

    def unlink(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        brw = self.browse(cr, uid, ids)
        if brw[0].planificacion_lineas_ids:
            raise osv.except_osv(_('Accion no permitida!'), _('Solo se pueden eliminar hojas vacias'))
        if brw[0].referencia != 'ST-#':
            raise osv.except_osv(_('Accion no permitida!'), _('No se pueden borrar hojas con numero de referencia asignado'))
        return super(horas_extras_planificacion, self).unlink(cr, uid, ids, context)

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context = {}
        load_nom = self._get_trabajadores_nomina(cr, uid, 'group_sisb_horas_extras_th', param_nomina=False)
        hr_user = self._get_trabajadores_nomina(cr, uid, 'group_hr_user', param_nomina=True)
        if (uid != 1 and not load_nom and not hr_user):
            lista = self.validar_departamentos(cr, uid)
            dic = 'department_id', 'in', lista
            args.append(dic)
        return super(horas_extras_planificacion, self).search(cr, uid, args, offset, limit, order, context, count)

    def write(self, cr, uid, ids, values, context=None):
        if context is None:
            context = {}
        if not isinstance(ids, list):
            ids = [ids]
        brw = self.browse(cr, uid, ids)
        user_id = brw[0].user_id
        load_nom = self._get_trabajadores_nomina(cr, uid, 'group_sisb_horas_extras_th', param_nomina=False)
        if uid != 1 and not load_nom and brw[0].state == 'draft' and 'state' not in values:
            if uid != user_id.id:
                raise osv.except_osv(_('Error'), _('Solo el usuario %s puede editar esta hoja' % user_id.name))
        return super(horas_extras_planificacion, self).write(cr, uid, ids, values, context=context)

    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(_('Accion no permitida!'), _('No se pueden duplicar las hojas de sobretiempos'))
        return False


horas_extras_planificacion()


class horas_extras_planificacion_lineas_excepciones(osv.osv):

    _name = 'horas.extras.planificacion.lineas.excepciones'
    _description = 'Lineas con excepcioneses'
    _columns = {
        'horas_extras_planificacion_lineas_id': fields.many2one('horas.extras.planificacion.lineas', 'Planificacion de horas extras lineas', readonly=True),
        'value': fields.float("Calculo de hora extras", required=True),
        'concept_id': fields.many2one('hr.concept', 'Concepto', required=True),
    }


horas_extras_planificacion_lineas_excepciones()

# Herencia sobre hr_payroll (nominas) para agregar campos de dias permitidos para sobretiempo


class hr_payroll(osv.osv):
    _inherit = 'hr.payroll'
    _columns = {
        'periodos_sobretiempo': fields.integer('Periodos para sobretiempo', help='Periodos permitidos para cargar sobretiempos, segun el tipo de nomina')
    }
    _defaults = {
        'periodos_sobretiempo': lambda *a: 1
    }


hr_payroll()


class horas_extras_historial_parametros(osv.osv):
    _name = 'horas.extras.historial.parametros'
    _description = 'Historial de cambios en sobretiempos por departamento'
    _columns = {
        'user_id': fields.many2one('res.users', 'Usuario'),
        'date': fields.datetime('Fecha'),
        'department_id': fields.many2one('hr.department', 'Departamento'),
        'prev_hours_per_dept': fields.integer('Departamento - previo'),
        'prev_hours_per_week': fields.integer('Semanal - previo'),
        'prev_hours_per_day': fields.integer('Diario - previo'),
        'hours_per_dept': fields.integer('Departamento - nuevo'),
        'hours_per_week': fields.integer('Semanal - nuevo'),
        'hours_per_day': fields.integer('Diario - nuevo')
    }
    _order = 'date desc'


horas_extras_historial_parametros()
