#!/usr/bin/env python
# -*- coding: utf-8 -*--

from osv import fields, osv
from datetime import datetime


class hr_department(osv.osv):
    _inherit = 'hr.department'
    _columns = {
        'hours_per_dept': fields.integer('Horas por departamento', help="Sumatoria de las horas permitidas al personal del departamento, mensualmente"),
        'hours_per_week': fields.integer('Horas por semana', help="Maximo de horas que se le permiten a un trabajador en una semana"),
        'hours_per_day': fields.integer('Horas por dia', help="Maximo de horas que se le permiten a un trabajador en una dia"),
        'historial_ids': fields.one2many('horas.extras.historial.parametros', 'department_id', 'Historial de cambios de sobretiempos', readonly=True)
    }
    _defaults = {
        'hours_per_dept': 0,
        'hours_per_week': 0,
        'hours_per_day': 0
    }
    _constraints = [
        (lambda self, cr, uid, ids, context=None: all(x.hours_per_dept >= 0 and x.hours_per_week >= 0 and x.hours_per_day >= 0 for x in self.browse(cr, uid, ids, context=context)), 'Valores deben ser positivos', ['hours_per_dept', 'hours_per_week', 'hours_per_day']),
        (lambda self, cr, uid, ids, context=None: all(x.hours_per_week <= 168 and x.hours_per_day <= 24 for x in self.browse(cr, uid, ids, context=context)), 'No pueden haber mas de 24 horas en un dia ni 168 horas en una semana', ['hours_per_week', 'hours_per_day']),
    ]

    def write(self, cr, uid, ids, vals, context=None):
        ids = ids if type(ids) == list else [ids]
        if set(vals) - {'hours_per_dept', 'hours_per_week', 'hours_per_day'} == set(vals):
            return super(hr_department, self).write(cr, uid, ids, vals, context=context)
        for record in self.browse(cr, uid, ids, context=context):
            vals.update({'historial_ids': [(0, 0, {
                'user_id': uid,
                'date': datetime.now(),
                'prev_hours_per_dept': record.hours_per_dept,
                'prev_hours_per_week': record.hours_per_week,
                'prev_hours_per_day': record.hours_per_day,
                'hours_per_dept': vals['hours_per_dept'] if 'hours_per_dept' in vals else record.hours_per_dept,
                'hours_per_week': vals['hours_per_week'] if 'hours_per_week' in vals else record.hours_per_week,
                'hours_per_day': vals['hours_per_day'] if 'hours_per_day' in vals else record.hours_per_day
            })]})
            super(hr_department, self).write(cr, uid, [record.id], vals, context=context)
        return True


hr_department()
