#-*- coding:utf-8 -*-
##############################################################################
#
#    SISB, Sistema Integral Simón Bolivar
#    Copyright (C) 2012-2013 Industrias Diana, C.A. All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
from datetime import datetime
import logging
from lxml import etree


class hr_variable_concepts(osv.osv):
    """
    Herencia para:
    1) Incluir funcion para obtener el nombre de los conceptos
    variables
    """
    _inherit = 'hr.variable.concepts'

    def __init__(self, *args, **kargs):
        """ Corregida la manera en que se agregaba el campo 'tipo_de_proceso', para que
            se tome en cuenta si ya existe y se deba agregar el tipo al selection, o si
            se debe crear por primera vez
        """
        super(hr_variable_concepts, self).__init__(*args, **kargs)
        if 'tipo_de_proceso' in self._columns:
            if ('sobretiempo', 'Sobretiempo') not in self._columns['tipo_de_proceso'].selection:
                self._columns['tipo_de_proceso'].selection.append(('sobretiempo', 'Sobretiempo'))
        else:
            self._columns['tipo_de_proceso'] = fields.selection([('sobretiempo', 'Sobretiempo')], "Tipo de proceso")

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        """ Se sobrecarga el metodo fields_view_get para hacer solo-lectura todos los campos en la vista de
            formulario de los conceptos variables cuando el tipo de proceso sea de sobretiempo. Se hace de
            esta forma para tomar en cuenta cuando otros modulos alteran los atributos 'readonly' de esta vista
        """
        view = super(hr_variable_concepts, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar, submenu=submenu)
        if view_type == 'form':
            doc = etree.XML(view['arch'])
            # In case 'tipo_de_proceso' field is not in the view, add it to compare it
            node = doc.xpath("//field[@name='tipo_de_proceso']")
            if not node:
                tdp = etree.Element('field', name='tipo_de_proceso', invisible="1")
                if 'tipo_de_proceso' not in view['fields']:
                    view['fields'].update(self.pool.get('hr.variable.concepts').fields_get(cr, uid, fields=['tipo_de_proceso'], context=context))
                doc.append(tdp)
            # Now, we must iterate all fields to set them all readonly
            for n in doc.xpath("//field"):
                attrs = eval(n.get('attrs', '{}'))
                if attrs:
                    if 'readonly' in attrs:
                        attrs['readonly'].insert(0, '|')
                        for i, d in enumerate(attrs['readonly']):
                            f, o, v = d if type(d) == tuple else False, False, False
                            if f == 'tipo_de_proceso':
                                if o == 'in':
                                    v = v + ['sobretiempo'] if type(v) == list else v + ('sobretiempo',)
                                else:  # assuming '=' is the operator, 'not in' or another operator should not be used
                                    o, v = 'in', ['sobretiempo', v]
                                attrs['readonly'][i] = (f, v, o)
                                break
                        else:
                            attrs['readonly'].append(('tipo_de_proceso', 'in', ('sobretiempo',)))
                    else:
                        attrs.update({'readonly': [('tipo_de_proceso', 'in', ('sobretiempo',))]})
                    n.set('attrs', str(attrs))
                else:
                    n.set('attrs', "{'readonly': [('tipo_de_proceso', 'in', ('sobretiempo',))]}")
            view['arch'] = etree.tostring(doc)
        return view

    def unlink(self, cr, uid, ids, context=None):
        brw = self.browse(cr, uid, ids[0])
        horas_extras_obj = self.pool.get('horas.extras.planificacion')
        variables_lines_obj = self.pool.get('hr.variable.concept.line')
        context = {} if context is None else context
        if brw.tipo_de_proceso == 'sobretiempo':
            if brw.state not in ['done', 'execute']:
                context.update({"eliminar_sobretiempo": True})
                horas_extras_id = horas_extras_obj.search(cr, uid, [('variable_concept_id', 'in', ids)])
                horas_extras_obj.write(cr, uid, horas_extras_id, {'state': 'processing', 'variable_concept_id': False})
                horas_extras_obj.write_state_lines(cr, uid, horas_extras_id, 'processing')
                lines = [i.id for i in brw.variable_concept_line_ids]
                variables_lines_obj.unlink(cr, uid, lines, context=context)
        return super(hr_variable_concepts, self).unlink(cr, uid, ids)


hr_variable_concepts()


class hr_variable_concept_line(osv.osv):
    _inherit = 'hr.variable.concept.line'
    _description = 'Variable Concept Lines'

    def unlink(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        if ids:
            brw = self.browse(cr, uid, ids[0])
            if brw.variable_concept_id.tipo_de_proceso == 'sobretiempo' and not context.get('eliminar_sobretiempo', False):
                raise osv.except_osv(_('ERROR!'), _("No se pueden borrar las lineas de conceptos variables tipo %s " % (brw.variable_concept_id.tipo_de_proceso)))
        return super(hr_variable_concept_line, self).unlink(cr, uid, ids, context=context)


hr_variable_concept_line()
