#!/usr/bin/env python
# -*- coding: utf-8 -*-

from osv import fields, osv
import datetime
from datetime import datetime
from datetime import timedelta
import time 
from time import time
import base64
from tools.translate import _

class horas_retardo_descuentos_lineas(osv.osv_memory):
	_name = 'horas.retardo.descuentos.lineas'
	_description = 'lineas de calculo' 
	_columns = {
		'employee_id': fields.many2one('hr.employee', 'Trabajador', readonly=True),
		'concept_id': fields.many2one('hr.concept', 'Concepto', readonly=True),
		'qty':fields.float("cantidad", readonly=True),
        'code_employee': fields.related('employee_id', 'cod_number', type='char', size=64, string='Código Trabajdor', readonly=True),
        'color':fields.integer('color'),
	}
	_defaults={
	'color':1,
	}

horas_retardo_descuentos_lineas()

class horas_retardo_descuentos_detalles(osv.osv_memory):
	_name = 'horas.retardo.descuentos.detalles'
	_description = 'lineas de calculo' 
	_columns = {
		'employee_id': fields.many2one('hr.employee', 'Trabajador', readonly=True),
        'code_employee': fields.related('employee_id', 'cod_number', type='char', size=64, string='Código Trabajdor', readonly=True),
		'turno_id': fields.many2one('hr.shift', 'Turno', readonly=True),
		'concept_id': fields.many2one('hr.concept', 'Concepto', readonly=True),
		'fecha':fields.date("Fecha", readonly=True),
		'entrada':fields.time("Entrada", readonly=True),
		'salida':fields.time("Salida", readonly=True),
		'color':fields.integer('color'),
	}
	_defaults={
		'color':1,
	}


horas_retardo_descuentos_detalles()

class horas_retardo_descuentos(osv.osv_memory):
    _name = 'horas.retardo.descuentos'
    _description = 'calculo de retardo automatico contra biometrico' 
    _columns = {
        'calculado':fields.boolean("calculado"),
        'process_id': fields.many2one('hr.process', 'Proceso', required=True),
        'descuentos_lines': fields.one2many('horas.retardo.descuentos.lineas', 'descuentos_id', "Lineas de descuentos", required=True),
        'descuentos_detalles': fields.one2many('horas.retardo.descuentos.detalles', 'descuentos_id', "Detalles de descuentos", required=True),
    }

    def elimina_tildes(self, s):
        import unicodedata
        return ''.join((c for c in unicodedata.normalize('NFD', unicode(s)) if unicodedata.category(c) != 'Mn'))  

    def normalizar_string(self, unicode_string):
        """
        Funcion para normalizar los caracteres ASCII no aceptados en python 
        
        retorna en string normalizado en mayuscula 
        
        parametros:
        unicode_string -- String en unicode
        """
        from unicodedata import normalize
        #~ unicode_string=unicode(unicode_string, 'utf-8')
        unicode_string=unicode(unicode_string)
        return normalize('NFKD',unicode_string).encode('ASCII', 'ignore').upper()
        
    def create_attachment(self, cr, uid, ids, line, res_id, process,context=None):
        """
        Funcion para crear archivo csv adjunto  
        
        retorna un string con la informacion del csv
        
        parametros:
        line -- objeto (horas_retardo_descuentos_detalles) iterable 
        res_id -- id del modelo 
        process -- objeto (hr_process) iterable del proceso 
        """
        elimtil = self.elimina_tildes
        nstring = self.normalizar_string
        name = "Calculo de deducciones "+process.name+'.csv' 
        name = name.upper()
        txt_string=""
        titulo= "DETALLES DE DEDUCCIONES\n"
        titulo= titulo+'|fecha|nombre|codigo|turno|concepto|entrada|salida|\n'
        txt_string = txt_string + titulo
        for l in line:
            fecha = l.fecha
            nombre = nstring(elimtil(l.employee_id.name))
            codigo = nstring(elimtil(l.employee_id.cod_number))
            turno = nstring(elimtil(l.turno_id.name))
            concepto =  nstring(elimtil(l.concept_id.name))
            entrada = l.entrada
            salida = l.salida
            txt_string= txt_string + "|%s|%s|%s|%s|%s|%s|%s|\n"%(fecha,nombre,codigo,turno,concepto,entrada,salida)
        id_attachment = self._write_attachment(cr, uid, res_id, txt_string, name, 'hr.variable.concepts', context)
        return txt_string
         
    
    def _write_attachment(self, cr, uid, ids, root, name, res_model, context):
        id_att = self.pool.get('ir.attachment').create(cr, uid, {
            'name': name,
            'voucher_support': name,
            'res_name': name,
            'datas': base64.encodestring(root),
            'datas_fname': name,
            'res_model': res_model,
            'res_id': str(ids),
        }, context=context)
        return id_att
    
    def cargar_concepto_variable(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        conceptos_variables_obj =  self.pool.get('hr.variable.concepts')
        hr_variable_concept_line_obj = self.pool.get('hr.variable.concept.line')
        hr_process_obj = self.pool.get('hr.process')
        descuentos_brw = self.browse(cr, uid, ids[0], context=None)
        process_id = descuentos_brw.process_id.id
        if descuentos_brw.process_id.calculado_descuento:
            raise osv.except_osv(_('Error!'),_("""El proceso %s ya pertenece a un concepto variable del tipo deducciones 
                                                  si desea volver a calcular debe eliminar el concepto variable """%descuentos_brw.process_id.name))
            
        tipo_de_proceso = 'deducciones'
        values = {'tipo_de_proceso': tipo_de_proceso,
                  'hr_process_id': process_id}
        conceptos_variables_id = conceptos_variables_obj.create(cr, uid, values, context)
        for line in descuentos_brw.descuentos_lines:
            values_lines = {'variable_concept_id':conceptos_variables_id,
                 'employee_id': line.employee_id.id,
                 'concept_id': line.concept_id.id,
                 'type': 'qty',
                  'value': line.qty}
            hr_variable_concept_line_obj.create(cr, uid, values_lines)
        hr_process_obj.write(cr, uid, process_id, {'calculado_descuento':True})
        self.create_attachment(cr, uid, ids, descuentos_brw.descuentos_detalles, conceptos_variables_id, descuentos_brw.process_id)
        return {
            'type': 'ir.actions.act_window_close',
            'active_id': ids
        } 

    def list_2_typle(self, list):
        """
        Convierte listas en tuplas  
        
        retorna una tupla
        
        parametros:
        list -- una lista
        """
        list = [str(i) for i in list]    
        t = ()
        if len(list) == 1:
           t = "('%s')"%"".join(list)
        else:
            t = tuple(list)
        return str(t)
        
    def query(self, codigo, fecha_inicio, fecha_fin, captahuellas, eventos):
        """
        Funcion para generar el string sql para la consulta en el biostar 
        
        retorna el string sql
        
        parametros:
        codigo -- lista de codigos con los trabajadores
        fecha_inicio -- fecha inicio en formato Anno-mes-dia
        fecha_fin --    fecha fin en formato Anno-mes-dia
        captahuellas -- lista con los ids de los captahuellas registrados 
						en  biostar.fingerprint usar (get_captahuellas) 
	    eventos -- lista de eventos registrados en biostar.event.data 
				   usar (get_eventos)
        """
        sql = '''
            SELECT  C.sFieldValue5 , COUNT(CONVERT(VARCHAR(32), DATEADD(s, EVENT.nDateTime, '1970-01-01'),108)),MIN(CONVERT(VARCHAR(11), DATEADD(s, nDateTime, '1970-01-01'),108)) As hour, MAX(CONVERT(VARCHAR(11), DATEADD(s, nDateTime, '1970-01-01'),108)) As hour,CONVERT(nVarchar(10), DATEADD(s, EVENT.nDateTime, '1970-01-01'), 20)
            FROM TB_EVENT_LOG AS EVENT 
            JOIN TB_USER AS U ON EVENT.nUserID = U.sUserId 
            JOIN TB_READER AS READER ON EVENT.nReaderIdn = READER.nReaderIdn JOIN TB_EVENT_DATA AS E_DATA ON EVENT.nEventIdn = E_DATA.nEventIdn 
            JOIN TB_USER_CUSTOMINFO As C ON C.nUserIdn = U.nUserIdn
            WHERE E_DATA.nEventIdn %s
            AND CONVERT(nVarchar(10), DATEADD(s, EVENT.nDateTime, '1970-01-01'), 20) BETWEEN '%s' and '%s'
            AND READER.nReaderIdn in %s 
            AND C.sFieldValue5 in  %s
            GROUP BY CONVERT(nVarchar(10), DATEADD(s, EVENT.nDateTime, '1970-01-01'), 20),  C.sFieldValue5 
            '''%(eventos, fecha_inicio, fecha_fin, captahuellas, codigo)    
        return sql   
        
    def get_eventos(self, cr, uid, ids):
        eventos = self.pool.get('biostar.event.data').even_id(cr, uid, ids)
        return eventos
        
    def get_captahuellas(self, cr, uid):
        """
        Funcion que busca la lista de IDS que identifican cada uno de los 
        capta huellas conectadas al servidor SQL SERVER que contiene el 
        Biometrico, esta funcion es usada para armar la consulta en SQL para 
        el Biometrico retorna
        @return t: un estring t
        
        """
        dev_obj = self.pool.get('biostar.fingerprint')
        dev_id = dev_obj.search(cr, uid, [('valido', '=', True)])
        dev_brw = dev_obj.browse(cr, uid, dev_id)
        t = [n.reader_id for n in dev_brw]
        t = self.list_2_typle(t)
        return t  
    
    def crear_sql(self, cr, uid, codigo, fecha_inicio, fecha_fin):
        eventos = self.get_eventos(cr, uid, 1)
        codigo = self.list_2_typle(codigo)
        captahuellas = self.get_captahuellas(cr, uid)
        sql = self.query(codigo, fecha_inicio, fecha_fin, captahuellas,eventos)
        return sql  

    def get_cursor(self, cr, uid, ids, context=None):
        biostar_obj = self.pool.get("biostar.settings")
        biostar_cxn = biostar_obj.conexion(cr, uid, ids, context=None)
        cursor = biostar_cxn.cursor()
        return cursor
    
    def generate_period(self, date_start, date_end):
        """
        Funcion encargada de generar un periodo a paratir de dos fechas
        @param: fh: es una lista que contiene la fecha de inicio y fin del periodo
        @return: l: una lista con todos las fechas intermedias  a la fecha fin 
        e inicio
        """
        l = []
        dia1 = self.conver_date(date_start)        
        dia2 = self.conver_date(date_end)
        dias = abs(dia2 - dia1)
        l.append(date_start)
        for i in range(1, dias.days):
            dia_periodo =  dia1 + timedelta(i)
            l.append(dia_periodo.strftime("%Y-%m-%d"))
        l.append(date_end)    
        return l
    
    def get_feriados(self, cr, uid):
        """
        Obtiene todas las fechas feriadas en el calendario
        retorna una lista con las fechas feriadas
        parametros:
        cr -- cursos de la BD
        uid -- Id del usuario
        """
        hr_holidays_obj = self.pool.get('hr.holidays.calendar')
        hr_holidays_id = hr_holidays_obj.search(cr, uid, [])
        hr_holidays = hr_holidays_obj.browse(cr, uid, hr_holidays_id)
        hr_holidays_calendar = []
        if hr_holidays:
           hr_holidays_calendar = [i.date for i in hr_holidays]
        return hr_holidays_calendar
    
    def filtrar_trabajadores(self, trabajadores, date):
        """
        Filtra los trabajadores y deja solo los activos y que la
        fecha de inicio del periodo sea mayor a la de la fecha de 
        ingreso laboral y este seleccionado la opccion no sujeto
        a marcajes
        retorna una lista con el objeto de hr_employee
        parametros:
        trabajadores -- objecto de hr.employee
        date -- String con la fecha formato Anno-mes-dia
        """
        l = []
        for i in trabajadores:
            if i.start_rel_lab < date and i.state == 'activo' and not i.no_sujeto_marcaje:
               l.append(i)
        return l

    
    def day_week(self, fecha):
        """
		Devuelve el dia de la semana segun la fecha
        retorna un numero entero que indica el dia de la semana
        siendo 1 lunes y 7 domingo
        parametros:
        fecha -- String con la fecha formato Anno-mes-dia
		"""
        dia = datetime.strptime(fecha, '%Y-%m-%d').isoweekday()
        return dia    
    
    def subtract_hours(self,  hr, ha, criterio=3600):
        """
		Resta dos horas
        retorna el valor absoluto de la resta de dos horas
        parametros:
        hr -- Hora desde
        ha -- Hora hasta
		"""
        if not isinstance(hr, datetime):
            hr =  self.cvr_hr(hr)
        if not isinstance(ha, datetime):
            ha =  self.cvr_hr(ha)
        dif = ha - hr 
        ab = abs(dif)
        time =  float(ab.seconds / float(criterio))
        time = round(time, 2)
        return time 
        
    def cvr_hr(self, hours):
        """
        Convierte una cadena en formato datetime
        """
        return datetime.strptime(hours, '%H:%M:%S')   
    
    def conver_date(self, fecha):
        """
		Convierte una cadena de fecha en formato datetime
        retorna una fecha en formato datetime
        parametros:
        fecha -- fecha
		"""
        d = datetime.strptime(fecha,"%Y-%m-%d")
        return d 

    def get_periodo_anterior(self, date_start, date_end):
        """
		Obtiene el periodo anterior apartir de dos fechas
		"""
        dc_start = self.conver_date(date_start)
        dc_end = self.conver_date(date_end)
        dt = dc_end-dc_start
        #~ delta=dt.days+1
        delta=dt.days
        date_start = dc_start - timedelta(days=delta+1)
        date_end = date_start + timedelta(days=delta)
        return [date_start.strftime("%Y-%m-%d"), date_end.strftime("%Y-%m-%d")]           

    def get_shitf_week(self, cr, date_start, date_end, emp_id):
        """
		Obtiene las rotaciones de un trabajador apartir de una fecha
		"""
        sql = """
            select max(shit.id) from hr_shift_week as shit 
            join hr_period as period ON shit.period_id = period.id 
            join hr_employee as emp ON shit.employee_id = emp.id 
            join hr_shift as sh ON shit.shift_id = sh.id 
            where emp.id = '%s' 
            and period.date_start <= '%s'
            """%(emp_id, date_end)
        cr.execute(sql)
        shitf_week_obj = self.pool.get('hr.shift.week')
        shitf_week_id = cr.fetchall()[0][0]
        shitf_week_brw = None
        if shitf_week_id:
            shitf_week_brw = shitf_week_obj.browse(cr, 1, shitf_week_id)
        return shitf_week_brw
        
    def ejecutar_query(self, cr, uid, ids, codigo, fecha_inicio, fecha_fin):
        """
        Ejecuta la consulta de los marcajes y retorna un diccionario
        con los datos:
        @trabajador: Codigo del trabajador en el biostar
        @cant_marcajes: cantidas de marcajes en el dia
        @entrada: Hora de entrada registrado en esa fecha
        @salida: Hora de salida registrado en esa fecha
        @fecha: fecha registrada en el biostar
        """
        sql = self.crear_sql(cr, uid, codigo, fecha_inicio, fecha_fin)
        cursor = self.get_cursor(cr, uid, ids, context=None)
        cursor.execute(sql)
        rows = cursor.fetchall()
        marcajes = [] 
        for i in rows:
            d = {}
            d['trabajador'] = i[0]
            d['cant_marcajes']=i[1]
            d['entrada']=i[2]
            d['salida']=i[3]
            d['fecha']=i[4]
            marcajes.append(d)
        return marcajes
    
    def buscar_marcaje_ant(self, cr, uid, ids, codigo, fecha):
        """
        Busca los marcajes del dia anterior 
        
        Retorna la consulta del dia anterior 
        
        Parametros:
        codigo -- Codigo del trabajador
        fecha -- Fecha para buscar el marcaje
        """
        fecha = self.dia_anterior(fecha) 
        result = self.ejecutar_query(cr, uid, ids, [codigo], fecha, fecha)
        return result
        
    def fechas_validas(self, fecha, horario_entrada, horario_salida, dias_laborales):
        """
        Verifica si los dias evaluados son laborales para el trabajador 
        
        Retorna True si es laboral y False de los contrario 
        
        Parametros:
        fecha -- Fecha trabajada
        horario_entrada -- Hora de entrada obtenida del sistema
        horario_salida --  Hora de salida obtenida del sistema
        dias_laborales --  Lista de dias validos para trabajar
        """
        if horario_entrada != '00:00:00' and horario_salida != '00:00:00' and fecha in dias_laborales:
           return True
        return False 
    
    def dia_anterior(self, fecha):
        """
        Le resta un dia a una fecha 
        
        Retorna el dia anterior a la fecha
        
        Parametros:
        fecha -- Fecha trabajada
        """
        
        fecha = self.conver_date(fecha) - timedelta(days=1)
        fecha = fecha.strftime("%Y-%m-%d")
        return fecha 
    
    def get_ausencias(self, cr, ids, trabajador_id, date_start, dias_laborales, dias_trabajados, concepto):
        """
        Calcula las ausencias del trabajador mediante la comparacion de los dias laborados
        con los dias laborales 
        
        Retorna la sumatoria de los dias no laborados por ausencias del trabajador
        Parametros:
        cr -- Cursor de la BD
        trabajador_id -- id del trabajador evaluado
        date_start --  Fecha del inicio de la relacion laboral
        dias_laborales --  Lista de dias validos para trabajar
        dias_trabajados --  Lista de dias rejistrados por el biostar
        concepto --  Concepto cargado en los parametros para mostralo en
                     la linea de los detalles
        """
        ausencia = 0
        gethorario = self.get_horario
        for dl in dias_laborales:
            if dl > date_start: 
                turno = self.get_shitf_week(cr, dl, dl, trabajador_id)
                hr = gethorario(cr, dl, trabajador_id, turno=turno)
                if dl not in dias_trabajados and hr["horario_entrada"] != '00:00:00' and hr["horario_salida"] != '00:00:00':
                    falta = self.get_value_falta(cr,  trabajador_id, dl, turno=turno)
                    ausencia +=falta
                    self.crear_lineas_detalles(cr, 1, ids[0], turno.shift_id.id, trabajador_id, dl, '00:00:00', '00:00:00', {concepto:falta})
        return ausencia
        
    def crear_lineas_detalles(self, cr, uid, ids, turno, employee, fecha, entrada, salida, descuentos):
        """
        Crea lineas en la tabla en memoria  horas.retardo.descuentos.detalles
        Retorna True
        Parametros:
        cr -- Cursor de la BD
        uid -- id des usuario (res_users)
        ids -- id de descuentos_id
        employee -- id del trabajador evaluado
        fecha --  Fecha 
        entrada --  Hora de entrada registrada por el biostar
        salida --  Hora de salida registrada por el biostar
        descuentos --  Diccionario con los conceptos y sumatoria de las
                       deducciones
        """
        
        detalles = {'employee_id':employee, 'turno_id':turno, 
        'fecha':fecha, 'descuentos_id':ids,  'concept_id':None,
        'entrada':entrada, 'salida':salida,}
        detalles_obj = self.pool.get('horas.retardo.descuentos.detalles')
        for i in descuentos:
            if descuentos[i] > 0.00:
                detalles['concept_id'] = i
                detalles_obj.create(cr, uid, detalles)
        return True 
            
    def get_hours_per_day_obj(self, fecha, turno, dia=None):
        dia = self.day_week(fecha)
        dia_id = {1:'monday', 2:'tuesday', 3:'wednesday', 4:'thursday', 5:'friday', 6:'saturday', 7:'sunday'}
        func = getattr(turno, dia_id[dia]+'_hours_day_id')
        return func
    
    def get_hours_obj(self, fecha, obj, dia=None):
        dia = self.day_week(fecha)
        dia_id = {1:'monday', 2:'tuesday', 3:'wednesday', 4:'thursday', 5:'friday', 6:'saturday', 7:'sunday'}
        func = getattr(obj, dia_id[dia]+'_hours_id')
        return func
    
    def get_hours_per_day(self, cr, fecha, trabajador_id, turno=None):
        if not turno:
           turno = self.get_shitf_week(cr, fecha, fecha, trabajador_id).shift_id
        hours_per_day = self.get_hours_per_day_obj(fecha, turno)
        return hours_per_day
    
    def get_horario(self, cr, fecha, trabajador_id, turno=None):
        hours_per_day = self.get_hours_per_day(cr, fecha, trabajador_id, turno=turno)
        horario= self.get_hours_obj(fecha, hours_per_day)
        return {'horario_entrada':horario.hour_start, 'horario_salida':horario.hour_end}
     
    def get_value_falta(self, cr, trabajador, fecha, turno=None):
		valor = 0.00
		if turno:
			valor = turno.shift_id.jor_day 
		else:
			turno = self.get_shitf_week(cr, fecha, fecha, trabajador).shift_id
			if turno:
				valor = turno.jor_day 
		return valor
    
    def calcular(self, cr, ids, trabajador, cant_marcajes, entrada, salida, 
                fecha, horario_entrada, horario_salida, dd_retardo, dd_tiem_no_lab, 
                falta_injust, min_marcajes=None, tolerancia=0, dias_laborales=[]):
       
        """
        Funcion para realizar los calculos de las ausencias y descuentos por retardos
        esta funcion retorna un diccionario con el id de los conceptos variables
        configurados en los parametros
        """
        falta = self.get_value_falta(cr, trabajador.id, fecha)
        diff = self.subtract_hours
        cvr_hr = self.cvr_hr
        descuentos = {dd_retardo:0.00, dd_tiem_no_lab:0.00, falta_injust:0.00}
        hora_entrada = cvr_hr(horario_entrada)
        hora_salida = cvr_hr(horario_salida)
        entrada = cvr_hr(entrada)
        salida = cvr_hr(salida)
        laborable = self.fechas_validas(fecha, horario_entrada, horario_salida, dias_laborales) 
        if laborable:
            if hora_salida > hora_entrada:
               if cant_marcajes >= min_marcajes:
                    if entrada > hora_entrada + timedelta(hours=tolerancia):
                        descuentos[dd_retardo] += diff(hora_entrada, entrada)
                    if salida < hora_salida:
                       descuentos[dd_tiem_no_lab] += diff(hora_salida, salida)
               else:
                   descuentos[falta_injust] += falta
            else:
                    # calculos para [grupo 3]
                marcaje_ant = self.buscar_marcaje_ant(cr, 1, 1, trabajador.cod_number, fecha)
                if marcaje_ant:
                   marcaje_ant = marcaje_ant[0]
                   entrada_dia_anterior = cvr_hr(marcaje_ant['salida'])
                   salida = entrada
                   entrada = entrada_dia_anterior
                   if entrada > hora_entrada:
					  descuentos[dd_retardo] += diff(entrada, hora_entrada) 
                   if salida < hora_salida:
                      descuentos[dd_tiem_no_lab] += diff(salida, hora_salida) 
                else:
                    descuentos[falta_injust] += falta
        return descuentos
        
    
    def calcular_descuentos(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        object_brw = self.browse(cr, uid, ids[0], context=None)
        proceso = object_brw.process_id
        trabajadores = [i.employee_id for i in proceso.hr_payslip_ids] 
        periodos= [(i.date_start, i.date_end) for i in proceso.hr_period_ids ]
        periodo_anterior = self.get_periodo_anterior(periodos[0][0], periodos[0][1])
        fecha_inicio = periodo_anterior[0]
        fecha_fin = periodo_anterior[1]
        trabajadores = self.filtrar_trabajadores(trabajadores, fecha_fin)
        #~ trabajadores = self.pool.get('hr.employee').browse(cr, uid, [666, 684, 268, 176, 30])
        #~ trabajadores = self.pool.get('hr.employee').browse(cr, uid, [505])
        feriados = self.get_feriados(cr, uid)
        dias_peridos = self.generate_period(fecha_inicio,fecha_fin)
        dias_laborales = [i for i in dias_peridos if i not in feriados]
        
        deducciones = {}
        codigos = {}
        calculos = {}
        dias_trabajados  = {}
        conceptos_deducciones = {}
        turno_trabajador={}
        employee_id = {}
        for i in trabajadores:
            deducciones[i.id] = {}
            codigos[i.cod_number] = i
            dias_trabajados[i.id] = []
            conceptos_deducciones[i.id] = {}
            turno_trabajador[i.id] = {}
            employee_id[i.id] = i
            
        marcajes = self.ejecutar_query(cr, uid, ids, codigos.keys(), fecha_inicio, fecha_fin)
        for m in marcajes:
            trabajador_id = codigos[m['trabajador']].id
            turno_trabajador[trabajador_id][m['fecha']] = {}
        
        for m in marcajes:
            trabajador_id = codigos[m['trabajador']].id
            m['trabajador'] = codigos[m['trabajador']]
            turno = self.get_shitf_week(cr, m['fecha'], m['fecha'], trabajador_id)
            horario_por_dia = self.get_hours_per_day(cr, m['fecha'], trabajador_id, turno=turno)
            dd_retardo = horario_por_dia.deduccion_retardo.id
            dd_tiem_no_lab = horario_por_dia.deduccion_tiempo_no_laborado.id 
            falta_injust = horario_por_dia.deduccion_falta_injustificada.id 
            horario = self.get_horario(cr, m['fecha'], trabajador_id, turno=turno)
            struct_fecha = {'turno':turno, 'horario':horario}
            turno_trabajador[trabajador_id][m['fecha']].update(struct_fecha)
            dias_trabajados[trabajador_id].append(m['fecha'])
            deducciones[trabajador_id].update({dd_retardo:0.00, dd_tiem_no_lab:0.00, falta_injust:0.00})
            conceptos_deducciones[trabajador_id].update({'dd_retardo':dd_retardo, 'dd_tiem_no_lab':dd_tiem_no_lab, 'falta_injust':falta_injust})
            m.update(horario)
            m.update(conceptos_deducciones[trabajador_id]) 
        
       
        for m in marcajes:
            trabajador_id =m['trabajador'].id
            calculos = self.calcular(cr, ids[0], min_marcajes=2, dias_laborales=dias_laborales, **m)
            self.crear_lineas_detalles(cr, 1, ids[0], turno_trabajador[trabajador_id][m['fecha']]['turno'].shift_id.id, trabajador_id, m['fecha'], m['entrada'], m['salida'], calculos)
            for c in calculos:
                if c in deducciones[trabajador_id]:
                   deducciones[trabajador_id][c]+= calculos[c]

        for i in deducciones:
            if 'falta_injust' in conceptos_deducciones[i]:
               deducciones[i][conceptos_deducciones[i]['falta_injust']] +=self.get_ausencias(cr, ids, i, employee_id[i].start_rel_lab, dias_laborales, turno_trabajador[i].keys(), conceptos_deducciones[i]['falta_injust'] )
        self.crear_lineas(cr, uid, ids[0], deducciones)
        self.write(cr, uid, ids, {'calculado':True})
        return True 
    
    def crear_lineas(self, cr, uid, ids, dicclineas):
        lineas = []
        for e in dicclineas:
            for c in dicclineas[e]:
                if dicclineas[e][c] > 0.00:
                    dlineas = {'employee_id':e, 'concept_id':c, 'qty':dicclineas[e][c], 'descuentos_id':ids}
                    lineas.append(dlineas)
        lineas_obj = self.pool.get('horas.retardo.descuentos.lineas') 
        
        for c in lineas:
            id = lineas_obj.create(cr, uid,  c)
        return lineas
 
    
    
horas_retardo_descuentos()

class horas_retardo_descuentos_lineas(osv.osv_memory):
    _inherit = 'horas.retardo.descuentos.lineas'
    _columns = {
        'descuentos_id': fields.many2one('horas.retardo.descuentos', 'Descuentos', required=True),
    }

horas_retardo_descuentos_lineas()

class horas_retardo_descuentos_detalles(osv.osv_memory):
    _inherit = 'horas.retardo.descuentos.detalles'
    _columns = {
        'descuentos_id': fields.many2one('horas.retardo.descuentos', 'Descuentos', required=True),
    }

horas_retardo_descuentos_detalles()

