#!/usr/bin/python
# -*- coding: utf-8 -*-
   
###############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

from datetime import date
import decimal_precision as dp
import netsvc
import pooler
from osv import fields, osv, orm
from tools.translate import _
import base64


class ml_subir_csv(osv.osv):
    _name = 'ml.subir.csv'
    
    _rec_name = "fecha"
    
    
    _columns = {
        'fecha': fields.date('Date of purchase', required=True),
        'adjunto': fields.binary('Archivo CSV', required=True),
        'state':fields.selection([('draft', 'Draft'),('done','Done'),('inprocess','in process')], 'State'),
        'lineas_ids': fields.one2many('ml.csv.lineas', 'csv_id', 'Lines'),
        'key': fields.char('key', size=16),
        'valid_item':fields.integer('Valid Item'),
        'amount':fields.float('Amount'),
    }
    
    _order = 'id desc'
    
    _sql_constraints = [
        ('uniq_validation', 'unique (valid_item)', 'Fecha no disponible / Ya existe un archivo cargado con la fecha que esta seleccionando!'),('uniq_validation_cabecera', 'unique (fecha)', 'fecha!')
        ]
    
    def create(self, cr, uid, vals, context=None):   
        if 'key' in vals.keys():
            if vals['key'] == 'wizard':
                return  super(ml_subir_csv, self).create(cr, uid, vals, context)
            else:
                raise osv.except_osv(_('Error'), _('To load a new document you must do it from online market, CSV file upload.'))
        raise osv.except_osv(_('Error'), _('To load a new document you must do it from online market, CSV file upload'))
    
    
    def copy(self, cr, uid, id, defaults, context=None):
        raise osv.except_osv(_('Action Denied!'),_('Are not permitted duplicate!'))
        return False
    
    
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        if not isinstance(ids, list):
            ids = [ids]
            
        ml_brw = self.browse(cr, uid, ids, context=context)
        for item_ml in ml_brw:            
            if item_ml.state != 'draft':
                raise osv.except_osv(_('Action Denied!'), _('You can not be deleted if its status is %s' %item_ml.state))
                                                
        return super(ml_subir_csv, self).unlink(cr, uid, ids, context)
    
ml_subir_csv()


class ml_csv_lineas(osv.osv):
    _name = 'ml.csv.lineas'
    _columns = {
    
        'employee_id':fields.many2one('hr.employee', "Empleado", required=True, select=True),
        'cedula': fields.char('Cedula', size=256),
        'fecha':fields.char('Fecha', size=256),
        'name':fields.char('Orden', size=256),
        'cantidad':fields.float('Cantidad'),
        'monto': fields.float('Monto'),
        'deferred': fields.float('Por deducir'),
        'state':fields.selection([('draft', 'Draft'), ('cancel', 'Cancel'), ('done','Done')], "State"),
        'csv_id': fields.many2one('ml.subir.csv', 'Archivo csv', required=True, ondelete='cascade', select=True, readonly=True, states={'draft':[('readonly',False)]}),
    }
    
    _sql_constraints = [
        ('uniq_validation_cedula', 'unique (csv_id, cedula)', 'Cedula duplicada en el documento!')
        ]
    
    def copy(self, cr, uid, id, defaults, context=None):
        raise osv.except_osv(_('Action Denied!'),_('Are not permitted duplicate!'))
        return False
        
    def unlink(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Action Denied!'),_('Are not permitted unlink!'))
        return False
    
ml_csv_lineas()


class ml_montos_lineas(osv.osv):
    _name='ml.montos.lineas'
    _columns = {
    
        'csv_linea':fields.many2one('ml.csv.lineas', 'linea', select=True),
        'monto':fields.float('Amount'),
    
    }
    
ml_montos_lineas()



