#~ # -*- coding: utf-8 -*-
from report import report_sxw
import time
from datetime import datetime, timedelta
from lxml import etree
import base64
import unicodedata
from osv import fields, osv
from tools.translate import _
import math

class report_pago_guarderia(report_sxw.rml_parse):

    _name= 'report.pago.guarderia'
    def __init__(self, cr, uid, name, context):
        super(report_pago_guarderia, self).__init__(cr, uid, name, context)
        self.localcontext.update(
        {	
            'get_empresa': self._get_empresa,
            'get_final_date': self._get_final_date,
            'get_empleado':self._get_empleado,
            'get_number_invoice': self._get_number_invoice,
            'get_date_invoice':self._get_date_invoice,
            'get_guarderia':self._get_guarderia,
            'get_data': self._get_data,
            'get_total': self._get_total,
        })
        self.context= context
        self.ids= context.get('active_ids')

    def _get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),
          ('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res

    def _get_final_date(self):
        return datetime.now().strftime('%d-%m-%Y %H:%M')
        
    def _get_empleado(self, form):
        empleado_id = form['employee_id']
        if not isinstance(empleado_id, list):
            empleado_id = [empleado_id]
        empleado_brw = self.pool.get('hr.employee').browse(self.cr, self.uid, empleado_id)
        return empleado_brw
        
    def _get_number_invoice(self, form):
        return form['number_invoice']
        
    def _get_date_invoice(self, form):
        return form['date_invoice']
        
    def _get_guarderia(self, form):
        return form['guarderia']
    
    def _get_total(self, form):
        pago = self.pool.get('hr.pagos.guarderias').browse(self.cr, self.uid, form['id'])
        return pago.amount
        
    def _get_data(self, form):
        data = []
        self.cr.execute("""
            select distinct he.name as trabajador, hef.name as hijo, hm.name as mes, 
                lpg.type as tipo,
                --ai.reference as ref_factura, ai.date_document as fecha_factura, 
                case when av.state = 'posted' then 'pagado' else 'no pagado' end as estado_op,
                av.date as fecha_op, lpg.amount as total
                from hr_linea_pago_guarderias lpg 
                join hr_pagos_guarderias pg on pg.id = lpg.pago_guarderia_id
                join hr_employee he on he.id = lpg.employee_id
                join hr_employee_family hef on hef.id = lpg.employee_family_id
                join hr_month hm on hm.id = lpg.month_id
                join hr_guarderias hg on hg.id = pg.guarderia_id
                join res_partner rp on rp.id = hg.partner_id
                left join account_invoice ai on ai.pago_guarderia_id = pg.id --and ai.state in ('open','paid')
                left join account_voucher_line avl on avl.invoice_id = ai.id
                left join account_voucher av on av.id = avl.voucher_id --and av.state = 'posted'
                where pg.id = %s
        """%(form['id']))
        
        data = self.cr.dictfetchall()
        
        if not data:
            data = [{'no_data': False}]
        else:
            data[0].update({'no_data': True})
        return data
        
report_sxw.report_sxw('report.report.pago.guarderia',
        'hr.pagos.guarderias',
        parser= report_pago_guarderia,
        rml= 'sisb_nomina/report/report_pago_guarderia.rml'
    )
