#-*- coding:utf-8 -*-

##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    d$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import datetime
from report import report_sxw
import time
import pooler

class year_salary_report(report_sxw.rml_parse):

    def __init__(self, cr, uid, name, context):
        super(year_salary_report, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'time': time,
            'get_employee': self.get_employee,
            'get_periods': self.get_periods,
            'get_months_tol': self.get_months_tol,
            'get_total': self.get_total,
            'rif':self.rif,
        })

        self.mnths =[]
        self.mnths_tol = []
        self.total=0.0
        self.param_concept = []

    def get_periods(self,form):
#       Get start year-month-date and end year-month-date
        fy = int(form['date_from'][0:4])
        ly = int(form['date_to'][0:4])

        fm = int(form['date_from'][5:7])
        lm = int(form['date_to'][5:7])
        no_months = (ly-fy)*12+lm-fm + 1
        cm = fm
        cy = fy

#       Get name of the months from integer
        mnth_name = []
        for count in range(0,no_months):
            m = datetime.date(cy, cm, 1).strftime('%b')
            mnth_name.append(m)
            self.mnths.append(str(cm)+'-'+str(cy))
            if cm == 12:
                cm = 0
                cy = ly
            cm = cm +1
        for c in range(0,(12-no_months)):
            mnth_name.append('None')
            self.mnths.append('None')
        return [mnth_name]

    def get_employee(self,form):
        ls1=[]
        ls = []
        tol_mnths=['Total',0,0,0,0,0,0,0,0,0,0,0,0]
        emp = pooler.get_pool(self.cr.dbname).get('hr.employee')
        
        if form['report']=='resumen_utilidades':
            emp_ids = emp.search(self.cr, self.uid, [('hr_sede_id','=', form['sede_id'])] )
        else:
            emp_ids = form['employee_ids']
        
        self.param_concept = eval(form['param'])
        empll  = emp.browse(self.cr,self.uid, emp_ids)
         
        cnt = 1
        for emp_id in empll:
            payroll = emp_id.hr_payroll_id.id
            ls1.append(emp_id.name)
            tol = 0.0
            for mnth in self.mnths:
                if mnth <> 'None':
                    if len(mnth) != 7:
                        mnth = '0' + str(mnth)
                    sal = self.get_salary_year(emp_id, mnth, form['report'], payroll)
                    if sal:
                        ls1.append(sal[0][0])
                        tol += sal[0][0]
                        tol_mnths[cnt] = tol_mnths[cnt] + sal[0][0]
                    else:
                        ls1.append(0.00)
                        tol_mnths[cnt] = 0.0
                else:
                    ls1.append('')
                    tol_mnths[cnt] = ''
                cnt = cnt + 1
            cnt = 1
            if tol != 0.0:
                ls1.append(tol)
                ls.append(ls1)
            ls1 = []
        self.mnths_tol.append(tol_mnths)

        return ls

    def get_months_tol(self):
        return self.mnths_tol

    def get_total(self):
        for item in self.mnths_tol:
            for count in range(1,len(item)):
              if item[count] == '':
                  continue
              self.total += item[count]
        return self.total
    
    def get_salary_year(self, emp, mes, report, payroll):
        
        process_id = self.get_calc_period(mes)
        procesos = process_id and ' and  hp.hr_process_id in ' + str(tuple(process_id)) or ''
        conceptos = self.param_concept and ' and hpl.concept_id in ' + str(tuple(self.param_concept)) or ''
        if procesos and conceptos:
            query = """
            select employee_id, sum(total)  as total from (
                select hpl.employee_id, case when hc.type_calc = 'allocation' then hpl.total else hpl.total * -1 end as total from hr_payslip hp
                inner join hr_payslip_line hpl on hpl.slip_id = hp.id and hpl.employee_id = hp.employee_id
                inner join hr_concept hc on hpl.concept_id = hc.id
                where hc.type_calc in('allocation','deduction') and hpl.employee_id = %s and
                hp.state = 'done'  
                %s
                 
                %s
                ) as tbl
            group by employee_id
            
            """%(emp.id,procesos,conceptos)
            
            self.cr.execute(query)
            res = self.cr.fetchall()
            total = res and res[0][-1] or 0.00
        else:
            total = 0.00
            
        if report:
            obj_hr_parameters = self.pool.get('hr.parameters')
            parameters_id = obj_hr_parameters.search(self.cr, self.uid, [('type','=','utilidades'),('payroll_id','=',payroll)])
            parameters_browse = obj_hr_parameters.browse(self.cr, self.uid, parameters_id)
            valor = parameters_browse[0].value
            total = total * (valor / 360)
        return [[total]]
    
    def get_calc_period(self, mnth):
        dates = datetime.datetime.strptime(mnth, '%m-%Y')
        year_id =  self.pool.get('hr.year').search(self.cr, self.uid, [('year','=',dates.year)])
        month_id =  self.pool.get('hr.month').search(self.cr, self.uid, [('month_year','=',dates.month), ('hr_year_id', '=', year_id)])
        period_id =  self.pool.get('hr.period').search(self.cr, self.uid, [('hr_month_id','in',month_id)])
        process_id =  self.pool.get('hr.process').search(self.cr, self.uid, [('hr_period_ids','in',period_id)])
        return process_id
    
    def company(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, self.uid).company_id
    
    def rif(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, self.uid).company_id.partner_id.vat[2:]
        

#~ report_sxw.report_sxw('report.year.salary', 'hr.payslip', 'hr_payroll/report/report_year_report.rml', parser=year_salary_report,header='internal landscape')
report_sxw.report_sxw('report.year.salary', 'hr.payslip', 'hr_payroll/report/report_year_report.rml', parser=year_salary_report,header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
