#!/usr/bin/env python
# -*- coding: utf-8 -*--

from report import report_sxw
from datetime import datetime
import pooler
from osv import fields, osv
import calendar



class csv_reporte_general(report_sxw.rml_parse):
    _name =  'csv.reporte.general'
    
    def __init__(self, cr, uid, name, context):
        super(csv_reporte_general, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'get_empresa':self.get_empresa,
            'get_final_date':self.get_final_date,
            'cvr_date':self.cvr_date,
        })
        
        self.context = context

    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res
    
    def get_final_date(self):
        return datetime.now().strftime('%d/%m/%Y %H:%M')    
    
    def cvr_date(self, fecha):
        return  datetime.datetime.strptime(fecha, '%Y-%m-%d').strftime('%Y%m%d')
        
report_sxw = report_sxw.report_sxw     
   
   
class TxtParser(report_sxw):
    """Custom class that use webkit to render HTML reports
       Code partially taken from report openoffice. Thanks guys :)
    """
    def __init__(self, name, table, rml=False, parser=False,
        header=True, store=False):
        self.parser_instance = False
        self.localcontext={}
        report_sxw.__init__(self, name, table, rml, parser,
            header, store)
    
    
    def all_employee(self, cr, uid, estados, actual, fecha_consulta):
        
        pool= pooler.get_pool(cr.dbname)    
        obj_employee = pool.get('hr.employee')
        
        if actual:
            if estados:
                employee_ids = obj_employee.search(cr, uid, [('state', 'in', estados)])
            else:
                employee_ids = obj_employee.search(cr, uid, [])
        else:
            employee_ids = obj_employee.search(cr, uid, [])
        
        if employee_ids == []:
            raise osv.except_osv(('ERROR!'), ('La combinacion de datos para dicha consulta no arrojo ningun resultado!!!'))
        
        browse_id = obj_employee.browse(cr, uid, employee_ids)        
        
        lista_empleados = []
        for data_employee in browse_id:
            fin_contrato = self.fin_relacion_laboral(data_employee.id, cr, uid)
            if data_employee.start_rel_lab <= fecha_consulta:
                if data_employee.state == 'egresado':
                    if fin_contrato >= fecha_consulta: 
                        lista_empleados.append(data_employee)
                else:
                    lista_empleados.append(data_employee)
        return lista_empleados
    

    def fin_relacion_laboral(self, employee_id, cr, uid):
        if employee_id:
            query = """select date_end FROM hr_contract WHERE employee_id = %s ORDER BY id"""
            cr.execute(query %employee_id)
            fin_rel_lab = cr.fetchall()
            fin_rel_lab = fin_rel_lab[-1][-1]
            if fin_rel_lab or fin_rel_lab != None:
                return str(fin_rel_lab)
        return ''
    
    def _sueldo(self, employee_id, date_start, cr, uid):
            if employee_id:
                query = """SELECT amount FROM hr_attribute WHERE employee_id = %s and date_start <= %s and concept_id = 59 
                        ORDER BY id"""
                cr.execute(query, (employee_id, date_start))
                salario = cr.fetchall()
                if salario:
                    return str(salario[-1][-1])
            return ''
                
    def _sede(self, employee_id, date_start, cr, uid):
            if employee_id:
                query = """SELECT sede FROM hr_department_employee WHERE employee_id = %s and date_start <= %s
                        ORDER BY id """
                cr.execute(query, (employee_id, date_start))
                sede = cr.fetchall()
                if sede:
                    return sede[-1][-1]
            return ''
    
    def _nomina(self, employee_id, date_stard, cr, uid):
            if employee_id:
                query = """SELECT payroll FROM hr_department_employee WHERE employee_id = %s and date_start <= %s 
                        ORDER BY id"""
                cr.execute(query, (employee_id, date_stard))
                nomina = cr.fetchall()
                if nomina:
                    return nomina[-1][-1]
            return ''
    
    def _cargo(self, employee_id, date_stard, cr, uid):
            if employee_id:
                query = """SELECT job FROM hr_department_employee WHERE employee_id = %s and date_start <= %s
                        ORDER BY id"""
                cr.execute(query, (employee_id, date_stard))
                cargo = cr.fetchall()
                if cargo:
                    return cargo[-1][-1]
            return ''
        
    def _departamento(self, actual, departamento_id, employee_id, date_start, cr, uid):
        
        if actual:
            SQL = """SELECT nombre_departamento(id) as departamento from hr_department where id = """+str(departamento_id)+""" """
            cr.execute(SQL)
            full_departamento = cr.fetchall()
            if full_departamento:
                full_departamento = full_departamento[0][0].replace(',','')
                return full_departamento
        else:
            
            query = """select nombre_departamento(department_id) 
            from hr_payslip WHERE employee_id = %s and date_from <= %s 
            and type_process = 'nomina'
            order by date_from desc limit 1 """
            cr.execute(query, (employee_id, date_start))
            resultado = cr.fetchall()
            if resultado:
                departamento = resultado[0][0].replace(',','')
                return departamento
            return ''
            
    def get_final_date(self):
        return datetime.now().strftime('%Y-%m-%d')
    
    def elimina_tildes(self, s):
        return ''.join((c for c in unicodedata.normalize('NFD', unicode(s)) if unicodedata.category(c) != 'Mn'))
    
    
    def date_end_rel(self, fecha, date_start, date_end):
        date_start = datetime.strptime(date_start, '%Y-%m-%d')
        fin = datetime.strptime(fecha, '%Y-%m-%d') 
        if date_end:
            date_end = datetime.strptime(date_end, '%Y-%m-%d')
            res = self._tiempo_servicio(date_start, date_end)
        else:
            res = self._tiempo_servicio(date_start, fin)
        return str(res)
        
    def date_end_rel_egresados(self, date_start, date_end):
        if date_end:
            date_start = datetime.strptime(date_start, '%Y-%m-%d')
            date_end = datetime.strptime(date_end, '%Y-%m-%d') 
            return self._tiempo_servicio(date_start, date_end)
        return ''
        
    def cvr_date(self, fecha):
        if fecha:
            return datetime.datetime.strptime(fecha, '%Y-%m-%d').strftime('%d%m%Y')                
        return False    
    
    def _tiempo_servicio(self, desde, hasta):
            res = ""
            if hasta.month == desde.month:
                if hasta.day < desde.day:
                    anos = hasta.year - desde.year - 1
                    meses = 12 - desde.month + hasta.month - 1
                    if hasta.month == 1:
                        dias = calendar.monthrange(hasta.year-1,12)[1] - desde.day + hasta.day
                    else:
                        dias = calendar.monthrange(hasta.year,hasta.month-1)[1] - desde.day + hasta.day 
                else: 
                    anos = hasta.year - desde.year
                    meses = desde.month - hasta.month
                    dias =  hasta.day - desde.day
                    
            elif hasta.month < desde.month:
                anos = hasta.year - desde.year - 1
                if hasta.day < desde.day:
                    meses = 12 - desde.month + hasta.month - 1
                    if hasta.month == 1:
                        dias = calendar.monthrange(hasta.year-1,12)[1] - desde.day + hasta.day
                    else:
                        dias = calendar.monthrange(hasta.year,hasta.month-1)[1] - desde.day + hasta.day 
                else:
                    meses = 12 - desde.month + hasta.month
                    dias =  hasta.day - desde.day  
            else:
                anos = hasta.year - desde.year
                meses = hasta.month - desde.month
                if hasta.day < desde.day:
                    meses = hasta.month - desde.month - 1
                    if hasta.month == 1:
                        dias = calendar.monthrange(hasta.year-1,12)[1] - desde.day + hasta.day
                    else:
                        dias = calendar.monthrange(hasta.year,hasta.month-1)[1] - desde.day + hasta.day
                else:
                    meses = hasta.month - desde.month
                    dias =  hasta.day - desde.day
            res = str(anos) + '/' + str(meses) + '/' + str(dias) 
            return res
    
    def create_single_pdf(self, cr, uid, ids, data, report_xml, context=None):
        """generate the csv"""
        
        if context is None:context={}

        self.parser_instance = self.parser(cr,uid,self.name2,context=context)
         
        csv = '|'.join(data['form']) + '\n'
         
        lista = []
         
        browse_employee = self.all_employee(cr, uid, data['estados'], data['tipo'], data['fecha'])
        
        for empleado in browse_employee:
            
            carga = []
            
            for cab in data['form']:
                
                if cab == 'codigo':
                    carga.append(empleado.cod_number)
                if cab == 'cedula':
                    carga.append(empleado.identification_id)
                if cab == 'nombre':
                    carga.append(empleado.name)
                if cab == 'sexo':
                    carga.append("Masculino" if empleado.gender == 'male' else "Femenino")
                if cab == 'telefono':
                    carga.append(empleado.mobile_phone or '')
                if cab == 'correo':
                    carga.append(empleado.work_email or '')
                if cab == 'fecha_nacimiento':
                    carga.append(empleado.birthday)
                if cab == 'edad':
                    carga.append(empleado.edad)
                if cab == 'inicio_relacion_laboral':
                    carga.append(empleado.start_rel_lab)
                if cab == 'fin_relacion_laboral':
                    carga.append(self.fin_relacion_laboral(empleado.id, cr, uid))
                     
                #~ """datos que varian segun el mes"""
                
                if cab == 'sueldo':
                    carga.append(self._sueldo(empleado.id, data['fecha'], cr, uid))
                if cab == 'sede':
                    carga.append(self._sede(empleado.id, data['fecha'], cr, uid))
                if cab == 'nomina':
                    carga.append(self._nomina(empleado.id, data['fecha'], cr, uid))
                if cab == 'departamento':
                    carga.append(self._departamento(data['tipo'], empleado.department_id.id, empleado.id, data['fecha'], cr, uid))
                if cab == 'cargo':
                    carga.append(self._cargo(empleado.id, data['fecha'], cr, uid))
                if cab == 'anos_antiguedad':
                    if empleado.state != "egresado":
                        servicio = self.date_end_rel(data['fecha'], empleado.start_rel_lab, False)
                        servicio.split('/')
                        timer = servicio.split('/')
                        carga.append(timer[0])
                        carga.append(timer[1])
                        carga.append(timer[2])
                    else:
                        query = """SELECT date_end FROM hr_contract WHERE employee_id = """+str(empleado.id)+""" ORDER BY id """
                        cr.execute(query)
                        _contrato_ = cr.fetchall()
                        try:
                            servicio = self.date_end_rel_egresados(empleado.start_rel_lab, _contrato_[-1][0])
                            servicio.split('/')
                            timer = servicio.split('/')
                            if len(timer) < 3:
                                carga.append(' ')
                                carga.append(' ')
                                carga.append(' ')
                            else:
                                carga.append(timer[0])
                                carga.append(timer[1])
                                carga.append(timer[2])
                        except:
                            """si llega ocurrir un error en el tiempo"""
                            carga.append(' ')
                            carga.append(' ')
                            carga.append(' ')
                        
                if cab == 'fecha':
                    carga.append(data['fecha'])
                if cab == 'estado':
                    carga.append(empleado.state)
        
            lista.append(carga)
        for x in sorted(lista):
            csv = csv + '|'.join(x) + '\n'

        return (csv,'csv')
        
TxtParser(
    'report.general.trabajadores',
    'hr.employee',
    'sisb_nomina/report/reporte_general.rml',
    parser=csv_reporte_general, header=False)
