#-*- coding:utf-8 -*-
#!/usr/bin/env python

from time import strftime
from datetime import date
from datetime import datetime
from datetime import timedelta
import re
from dateutil import relativedelta
import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import decimal_precision as dp
import logging
from tools.safe_eval import safe_eval as eval
import calendar
from itertools import groupby
from operator import itemgetter
import logging

class hr_guarderias(osv.osv):
    '''
        Clase para definir los partner que son guarderías
    '''
    _description = "Guarderias"
    _name = 'hr.guarderias'

    _columns = {
        'partner_id': fields.many2one('res.partner', 'Partner'),
        'name': fields.related('partner_id', 'name', type='string', 
          relation='res.partner', string="Nombre"),
        'employee_family_ids': fields.one2many('hr.employee.family', 
          'guarderia_id', 'Familiares'),
    }

    _sql_constraints = [('unique_partner','unique(partner_id)', 
      _('El partner debe ser unico'))]

hr_guarderias()

class hr_pagos_guarderias(osv.osv):
    ''''
        Clase para definir los pagos que se haran por concepto de guarderias
    '''
    _description = "Pagos de guarderias"
    _name = 'hr.pagos.guarderias'
    
    _columns = {
        'employee_id': fields.many2one('hr.employee', 'Trabajador', select=True,
          required=True),
        'name': fields.related('employee_id', 'name', type='string', 
          relation='hr.employee', string="Nombre"),
        'amount': fields.float('Monto factura', digits_compute=dp.get_precision('Payroll'),
          required=True),
        'number_invoice': fields.char('Número de factura', required=True, size=256),
        'date_invoice': fields.date('Fecha de la factura', select=True, required=True),
        'guarderia_id': fields.many2one('hr.guarderias', 'Guardería', required=True),
        'state': fields.selection([
              ('draft','Borrador'), 
              ('checked','Validado'),
              ('review','Procesado'), 
              ('done','Aprobado'),
              ('invoiced','Facturado'),
              ('paid', 'Pagado')],
          'Estado', readonly=True),
        'date_approved': fields.date('Fecha de aprobado', select=True, readonly=True),
        'date_paid': fields.date('Fecha de pago', select=True, readonly=True),
        'payslip_id': fields.many2one('hr.payslip', 'Hoja de pago'),
    }
    
    _defaults = {
        'state': 'draft',
    }
    
    _sql_constraints = [
      ('check_amount',"CHECK(amount > 0.0)",'El monto debe ser mayor que cero (0)'),
      ('check_amount',"UNIQUE(guarderia_id,number_invoice)",'Ya se encuentra registrado \
        un pago para la misma guarderia con el mismo numero de factura.'),
    ]

    def _check_unique_fact_number_guar(self, cr, uid, ids, context=None):
        for pago in self.browse(cr, uid, ids, context=context):
            #Se debe extraer la parte entera del numero de factura para luego 
            #buscar facturas ya registradas para el mismo partner con ese numero.
            num = pago.number_invoice
            exp = re.sub(r'\D',"",num)
            exp = int(exp)
            
            if not pago.account_invoice_ids:
                fact_ids = self.pool.get('account.invoice').search(cr, uid, [
                  ('reference', 'ilike', str(exp)),
                  ('partner_id', '=', pago.guarderia_id.partner_id.id)])
                fact_brw = self.pool.get('account.invoice').browse(cr, uid, fact_ids)
                for fact in fact_brw:
                    num_fact = fact.reference
                    exp_fact = re.sub(r'\D',"",num_fact)
                    exp_fact = int(exp_fact)
                    if exp_fact == exp:
                        return False
        return True

    _constraints = [
      (_check_unique_fact_number_guar, 
        "Ya se encuentra registrada una factura con el mismo numero. \
        El numero de factura debe ser unico para cada guarderia", 
        ['number_invoice,guarderia_id']),
    ]
        
    def set_to_draft(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for id in ids:
            pago = self.browse(cr, uid, id)
            if pago.state not in ('checked'):
                raise osv.except_osv(_('ERROR!'),_('No se puede pasar a Borrador \
                  un pago de guarderia en estado distinto a Validado.'))
            result = self.write(cr, uid, id, {'state' : 'draft'})
    
    def set_to_checked(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for id in ids:
            pago = self.browse(cr, uid, id)
            if not pago.linea_pago_guarderias_ids:
                raise osv.except_osv(_('ERROR!'),_('No se puede Validar \
                  un pago sin Datos de pagos.'))
            total = sum([x.amount for x in pago.linea_pago_guarderias_ids])
            if total <> pago.amount:
                raise osv.except_osv(_('ERROR!'),_('El monto total de la factura \
                    debe coincidir con la suma de los montos de las líneas.'))
            result = self.write(cr, uid, id, {'state' : 'checked'})
        return result
        
    def set_to_review(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for id in ids:
            pago = self.browse(cr, uid, id)
            if pago.state == 'invoiced':
                raise osv.except_osv(_('ERROR!'),_('El pago de %s \
                  ya ha sido Facturado.'%(pago.employee_id.name)))
            if pago.state not in ('checked', 'done'):
                raise osv.except_osv(_('ERROR!'),_('No se puede pasar a Revisado \
                  un pago de guarderia en estado distinto a Validado.'))
            result = self.write(cr, uid, id, 
              {'state' : 'review',
              'date_approved': False,
              })
        return result
        
    def set_to_done(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for id in ids:
            pago = self.browse(cr, uid, id)
            if pago.state not in ('review','invoiced','done'):
                raise osv.except_osv(_('ERROR!'),_('No se puede pasar a Aprobado \
                  un pago de guarderia en estado distinto a Procesado.'))
            result = self.write(cr, uid, id, 
              {'state' : 'done',
              'date_approved' : datetime.now().strftime('%Y-%m-%d'),
              'date_paid' : False,
              })
        return result
        
    def set_to_invoiced(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        result = True
        for id in ids:
            if id:
                pago = self.browse(cr, uid, id)
                if pago.state not in ('done','invoiced'):
                    raise osv.except_osv(_('ERROR!'),_('No se puede pasar a Facturado \
                      un pago de guarderia en estado distinto a Aprobado.'))
                result = self.write(cr, uid, id, 
                  {'state' : 'invoiced',
                  })
        return result
        
    def set_to_paid(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for id in ids:
            pago = self.browse(cr, uid, id)
            if pago.state not in ('invoiced','done','paid'):
                raise osv.except_osv(_('ERROR!'),_('No se puede pasar a Pagado \
                  un pago de guarderia en estado distinto a Aprobado.'))
            result = self.write(cr, uid, id, 
              {'state' : 'paid', 
               'date_paid' : datetime.now().strftime('%Y-%m-%d'),
              })
        return result
        
hr_pagos_guarderias()

class hr_linea_pago_guarderias(osv.osv):
    ''''
        Clase para definir las lineas de pagos de guarderias
    '''
    _description = "Línea de Pago de guarderias"
    _name = 'hr.linea.pago.guarderias'

    _columns = {
        'pago_guarderia_id': fields.many2one('hr.pagos.guarderias', 'Pago de guarderia', ondelete='cascade'),
        'employee_id': fields.related('pago_guarderia_id', 'employee_id', 
          type='many2one', relation='hr.employee', string="Trabajador", store=True),
        'name': fields.related('employee_id', 'name', type='string', 
          relation='hr.employee', string="Nombre"),
        'employee_family_id': fields.many2one('hr.employee.family', 'Hijo', required=True),
        'amount': fields.float('Monto', digits_compute=dp.get_precision('Payroll'),
          required=True),
        'month_id': fields.many2one('hr.month', 'Mes', required=True),
        'type': fields.selection([('mensualidad','Mensualidad'),('inscripcion','Inscripcion')], 
          'Tipo', required=True),
    }
    
    _sql_constraints = [
        ('check_amount',"CHECK(amount > 0.0)", 
          'El monto debe ser mayor que cero (0)')
    ]
    
    def _check_unique_relation(self, cr, uid, ids, context=None):
        for line in self.browse(cr, uid, ids, context=context):
            if self.search(cr, uid, [('id', '<>', line.id),
              ('employee_family_id', '=', line.employee_family_id.id), 
              ('month_id', '=', line.month_id.id),
              ('type', '=', line.type)]):
                return False
        return True
        
    def _check_unique_guar(self, cr, uid, ids, context=None):
        for line in self.browse(cr, uid, ids, context=context):
            if self.search(cr, uid, [('id', '<>', line.id),
              ('employee_family_id', '=', line.employee_family_id.id), 
              ('month_id', '=', line.month_id.id),
              ('pago_guarderia_id.guarderia_id', '<>', line.pago_guarderia_id.guarderia_id.id)]):
                return False
        return True
        
    def _check_inscription_in_range(self, cr, uid, ids, context=None):
        for line in self.browse(cr, uid, ids, context=context):
            if line.type == 'inscripcion':
                last_inscription_brw = self.search(cr, uid, [('id', '<>', line.id),
                  ('employee_family_id', '=', line.employee_family_id.id), 
                  ('type', '=', 'inscripcion')], order='month_id desc')
                if last_inscription_brw:
                    last_month = self.browse(cr, uid, last_inscription_brw[0], context=context).month_id
                    actual_month = line.month_id
                    actual_date = datetime(actual_month.hr_year_id.year, actual_month.month_year, 01)
                    last_date = datetime(last_month.hr_year_id.year, last_month.month_year, 01)
                    if actual_month.month_year >= 8 and actual_month.month_year <= 12: #entre agosto y diciembre
                        periodo_inicio = datetime(actual_month.hr_year_id.year, 8, 01)
                        periodo_fin = datetime(actual_month.hr_year_id.year + 1, 7, 31)
                    elif actual_month.month_year >= 01 and actual_month.month_year <= 07: #entre enero y julio
                        periodo_inicio = datetime(actual_month.hr_year_id.year -1, 8, 01)
                        periodo_fin = datetime(actual_month.hr_year_id.year, 7, 31)
                    #~ ####################### se cambio la validacion del periodo de 10 meses por el periodo escolar
                    #~ period = abs((actual_date.month - last_date.month) + 12 * (actual_date.year - last_date.year))
                    #~ if period < 10:
                        #~ return False
                    #~ #######################
                    if last_date >= periodo_inicio and last_date <= periodo_fin:
                        return False

                    
        return True
    
    def _get_age(self, cr, uid, employee_family_id, month_id, context=None):
        birth = datetime.strptime(employee_family_id.birthday,"%Y-%m-%d")
        years = ((month_id.hr_year_id.year - birth.year - 1) + 
            (1 if (month_id.month_year, 1) >= \
            (birth.month, birth.day) else 0))
        return years
        
    def _check_edad_maxima(self, cr, uid, ids, context=None):
        param_edad_max_id = self.pool.get('hr.parameters').search(cr, uid, 
          [('type', '=', 'edad_max_pago_guarderia')])
        if not param_edad_max_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro \
              definido para Máxima edad de hijos para pago de guarderías. \
              Ir a la pestaña Configuracion -> Configuraciones\
              de nomina -> Parametros de nomina y registrar un \
              parametro de tipo Máxima edad de hijos para pago de guarderías'))
        
        edad_max = self.pool.get('hr.parameters').browse(cr, uid, 
          param_edad_max_id)[0].value
        edad_max = int(edad_max)
        
        for line in self.browse(cr, uid, ids, context=context):
            edad = self._get_age(cr, uid, line.employee_family_id, line.month_id, context)
            if edad > edad_max:
                return False
        return True
        
    def _check_monto_maximo(self, cr, uid, ids, context=None):
        param_monto_max_id = self.pool.get('hr.parameters').search(cr, uid, 
          [('type', '=', 'guarderia')])
        if not param_monto_max_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro \
              definido para Monto máximo a pagar por Guardería. \
              Ir a la pestaña Configuracion -> Configuraciones\
              de nomina -> Parametros de nomina y registrar un \
              parametro de tipo Monto máximo a pagar por Guardería'))
        
        monto_max = self.pool.get('hr.parameters').browse(cr, uid, 
          param_monto_max_id)[0].value
                  
        for line in self.browse(cr, uid, ids, context=context):
            if line.amount > monto_max:
                return False
        return True

    _constraints = [
      (_check_unique_relation, 
        "Ya existe un pago para el mismo hijo, mes y tipo. El registro debe ser único.", 
        ['employee_family_id,month_id,type']),
      (_check_unique_guar, 
        "Ya existe un pago para el mismo hijo y mes para otra guarderia. En el mismo mes\
        no deben haber pagos para distintas guarderias.", 
        ['employee_family_id,month_id,pago_guarderia_id.guarderia_id']),
      (_check_inscription_in_range, 
        "Ya existe un pago de inscripcion para el mismo hijo en un rango menor de 10 meses.", 
        ['employee_family_id,month_id']),
      (_check_edad_maxima, 
        "La edad del niño para el mes en el que se esta cargando el pago supera\
        la edad máxima permitida para pago de guarderia.", 
        ['employee_family_id']),
      (_check_monto_maximo, 
        "El monto supera el monto máximo para pago de guardería. Por favor, verificar.", 
        ['employee_family_id']),
        
    ]
        
    def on_change_employee_family_id(self, cr, uid, ids, type, employee_family_id, context):
        """ Funcion usada para setear el valor del monto si el tipo es mensualidad.
        En ese caso el monto debe ser la mensualidad que se coloco en la maestra de
        familiares para el pago de guarderia del correspondiente hijo.
        """
        if context is None:
            context = {}
        if type == 'inscripcion' or not employee_family_id:
            result = {
              'amount': 0
            }
        else:
            family_brw = self.pool.get('hr.employee.family').browse(cr, uid, employee_family_id)
            result = {
              'amount': family_brw.guarderia_amount \
                if (family_brw and family_brw.guarderia_amount) else 0.0,
            }
        
        return {'value': result}
        
    def on_change_type(self, cr, uid, ids, type, employee_family_id, context):
        """ Funcion usada para setear el valor del monto si el tipo es mensualidad.
        En ese caso el monto debe ser la mensualidad que se coloco en la maestra de
        familiares para el pago de guarderia del correspondiente hijo.
        """
        if context is None:
            context = {}
        if type == 'inscripcion' or not employee_family_id:
            result = {
              'amount': 0
            }
        else:
            family_brw = self.pool.get('hr.employee.family').browse(cr, uid, employee_family_id)
            result = {
              'amount': family_brw.guarderia_amount \
                if (family_brw and family_brw.guarderia_amount) else 0.0,
            }
        
        return {'value': result}
        
    def on_change_amount(self, cr, uid, ids, amount, type, employee_family_id, context):
        """ Funcion usada para setear el valor del monto si el tipo es mensualidad.
        En ese caso el monto debe ser la mensualidad que se coloco en la maestra de
        familiares para el pago de guarderia del correspondiente hijo.
        """
        if context is None:
            context = {}
        if type == 'inscripcion' or not employee_family_id:
            result = {
              'amount': amount,
            }
        else:
            family_brw = self.pool.get('hr.employee.family').browse(cr, uid, employee_family_id)
            result = {
              'amount': family_brw.guarderia_amount \
                if (family_brw and family_brw.guarderia_amount) else 0.0,
            }
        
        return {'value': result}
        
hr_linea_pago_guarderias()

class hr_pagos_guarderias(osv.osv):
    
    _inherit = 'hr.pagos.guarderias'
    
    _columns = {
        'linea_pago_guarderias_ids': fields.one2many('hr.linea.pago.guarderias', 
          'pago_guarderia_id', 'Lineas de pago de guarderia'),
    }
    
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for id in ids:
            record = self.browse(cr, uid, id)
            if record.state not in ('draft'):
                raise osv.except_osv(_('ERROR!'),_('No se permite eliminar pagos \
                  en estado distinto a Borrador.'))
        return super(hr_pagos_guarderias, self).unlink(cr, uid, ids, context)

hr_pagos_guarderias()

class hr_employee_family(osv.osv):
    
    _inherit = 'hr.employee.family'
    
    _columns = {
        'linea_pago_guarderias_ids': fields.one2many('hr.linea.pago.guarderias', 
          'employee_family_id', 'Linea de pago de guarderia'),
    }

hr_employee_family()

class hr_payslip_line(osv.osv):

    _inherit = 'hr.payslip.line'
    _description = 'Pay Slip Line'
    
    _columns = {
        'linea_pago_guarderias_id': fields.many2one('hr.linea.pago.guarderias', 
          'Linea de pago de guarderia'),
    }

hr_payslip_line()

class hr_linea_pago_guarderias(osv.osv):

    _description = "Línea de Pago de guarderias"
    _inherit = 'hr.linea.pago.guarderias'
    
    _columns = {
        'payslip_line_ids': fields.one2many('hr.payslip.line', 
          'linea_pago_guarderias_id', 'Lineas de hojas de pago'),
    }
    
    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        
        if '_terp_view_name' in context and \
          context['_terp_view_name'] == 'Lineas pago de guarderias':
            raise osv.except_osv(_('ERROR!'),_('No se permite crear lineas \
              de pago de guarderias desde esta opcion.'))
        
        return super(hr_linea_pago_guarderias, self).create(cr, uid, vals, context)
    
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if '_terp_view_name' in context and \
          context['_terp_view_name'] == 'Lineas pago de guarderias':
            raise osv.except_osv(_('ERROR!'),_('No se permite eliminar lineas \
              de pago de guarderias desde esta opcion.'))
              
        return super(hr_linea_pago_guarderias, self).unlink(cr, uid, ids, context)
        
hr_linea_pago_guarderias()

class hr_payslip(osv.osv):

    _inherit = 'hr.payslip'
    _description = 'Pay Slip'
    
    _columns = {
        'pago_guarderia_ids': fields.one2many('hr.pagos.guarderias', 
          'payslip_id', 'Pago de guarderia'),
    }
    
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if not isinstance(ids, list):
            ids = []
        for id in ids:
            payslip = self.browse(cr, uid, id)
            if not payslip.hr_process_id.type == 'guarderias':
                break
            self.pool.get('hr.pagos.guarderias').set_to_checked(cr, uid, \
              [x.id for x in payslip.pago_guarderia_ids])
            self.pool.get('hr.pagos.guarderias').write(cr, uid, 
              [x.id for x in payslip.pago_guarderia_ids], {'payslip_id':False}, context=context)
            
        return super(hr_payslip, self).unlink(cr, uid, ids, context)
        
    def get_payslip_lines_guarderias(self, cr, uid, contract_ids, payslip_id, context):
        """
        Método para el cálculo de pago de guarderias
        """

        parameter_obj = self.pool.get('hr.parameters')
        payslip_obj = self.pool.get('hr.payslip')
        concept_obj = self.pool.get('hr.concept')
        obj_rule = self.pool.get('hr.salary.rule')
        blacklist = []
        result_dict = {}
        rules = {}
        rules_dict = {}

        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)
        
        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'), _('La nomina %s no tiene \
              calendario de pago. Debe asignarle un calendario de pago \
              antes de ejecutar cualquier proceso')
              %(payslip.employee_id.hr_payroll_id.name))
              
        param_guarderias_id = parameter_obj.search(cr, uid, [('type', '=', 'guarderia')])
        if not param_guarderias_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido \
              para el monto a pagar por guardería. Ir a la pestaña Configuracion \
              -> Configuraciones de nomina -> Parametros de nomina y \
              registrar un parametro de tipo Monto máximo a pagar por Guardería'))

        param_guarderias_brw = parameter_obj.browse(cr, uid, param_guarderias_id)[0]

        localdict = {
            'monto_guarderia': 0.0,
        }
        
        monto_guarderia = {}
                    
        for pago_guarderia in payslip.pago_guarderia_ids:
            for line in pago_guarderia.linea_pago_guarderias_ids:
                #Si el monto de la guarderia <= que lo que se paga por guarderia
                if line.amount <= param_guarderias_brw.value: 
                    monto_guarderia[line.employee_family_id, line.month_id, line.type, line] = line.amount
                else:
                    monto_guarderia[line.employee_family_id, line.month_id, line.type, line] = param_guarderias_brw.value
        
        period = list(set(payslip.hr_process_id.hr_period_ids) & 
          set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
        structure_ids = []
        structure_brw = [j for j in payslip.employee_id.hr_payroll_id.hr_payroll_structure_ids]
        #El tipo de la estructura debe ser del mismo tipo del proceso
        for str_brw in structure_brw: 
            if str_brw.type == payslip.hr_process_id.type:
                structure_ids.append(str_brw.id)
        if not structure_ids:
            raise osv.except_osv(_('ERROR!'), _('No hay una estructura de \
              tipo %s definida para la nomina %s') %(payslip.hr_process_id.type, 
              payslip.employee_id.hr_payroll_id.name))

        #get the rules of the structure and their children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, 
          uid, structure_ids, payslip.employee_id, period.id, context=context)
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]

        for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
            localdict['result'] = None
            localdict['result_qty'] = 1.0
            #check if the rule can be applied
            if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, 
              context=context) and rule.id not in blacklist:
                for hijo in monto_guarderia:
                    key = rule.code+'/'+hijo[0].name+'/'+hijo[1].name+'/'+hijo[2]
                    localdict['monto_guarderia'] = monto_guarderia[hijo]
                    #compute the amount of the rule
                    amount, qty, rate = obj_rule.compute_rule(cr, uid, 
                      rule.id, localdict, context=context)

                    #check if there is already a rule computed with that code
                    previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                    #set/overwrite the amount computed for this rule in the localdict
                    tot_rule = amount * qty * rate / 100.0
                    localdict[rule.code] = tot_rule
                    rules[rule.code] = rule

                    result_dict[key] = {
                        'salary_rule_id': rule.id,
                        'name': rule.name+'/'+hijo[0].name+'/'+hijo[1].name+'/'+hijo[2].capitalize(),
                        'code': rule.code,
                        'sequence': rule.sequence,
                        'appears_on_payslip': rule.appears_on_payslip,
                        'register_id': rule.register_id.id,
                        'amount': amount,
                        'employee_id': payslip.employee_id.id,
                        'quantity': qty,
                        'rate': rate,
                        'slip_id': payslip.id,
                        'state': 'draft',
                        'employee_family_id': hijo[0].id,
                        'linea_pago_guarderias_id': hijo[3].id,
                    }
            else:
                #blacklist this rule and its children
                blacklist += [id for id, seq in 
                  self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, 
                  uid, [rule], context=context)]
        
        result = [value for code, value in result_dict.items()]
        return result
        
hr_payslip()
