# -*- coding: utf-8 -*-
##############################################################################
#
# Facturar pago de guarderias
#
##############################################################################

from osv import fields, osv
from tools.translate import _


class guarderia_invoice(osv.osv_memory):
    
    _name = "guarderia.invoice"
    _description = "Facturas pago guarderias"
    
    def _get_journal_id(self, cr, uid, context=None):
        if context is None:
            context = {}
        model = context.get('active_model')
        if not model or model != 'hr.pagos.guarderias':
            return []

        model_pool = self.pool.get(model)
        journal_obj = self.pool.get('account.journal')
        res_ids = context and context.get('active_ids', [])
        vals = []
        pagos_brw = model_pool.browse(cr, uid, res_ids, context=context)
        for hc in pagos_brw:
            journal_type = ('purchase',)
            value = journal_obj.search(cr, uid, [('type','in',journal_type )])
            for jr_type in journal_obj.browse(cr, uid, value, context=context):
                t1 = jr_type.id,jr_type.name
                if t1 not in vals:
                    vals.append(t1)
        if not vals:
            raise osv.except_osv(_('Mensaje !'), _('No hay diario configurado para facturas de proveedor!'))
        
        return vals
                
    _columns = {
        'journal_id': fields.selection(_get_journal_id, 'Diario destino',required=True),
    }
    
    def open_invoice(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        invoice_ids = []
        data_pool = self.pool.get('ir.model.data')
        res = self.create_invoice(cr, uid, ids, context=context)
        invoice_ids += res.values()
        inv_type = context.get('inv_type', False)
        action_model = False
        action = {}
        if not invoice_ids:
            raise osv.except_osv(_('Error'), _('No se pudo crear factura'))
        if inv_type == "in_invoice":
            action_model,action_id = data_pool.get_object_reference(cr, uid, 'account', "action_invoice_tree2")
        else:
            raise osv.except_osv(_('Error'), _('No se puede crear facturas que no sean al proveedor'))
        return True
        
    def create_invoice(self, cr, uid, ids, context=None):
        journal_mapping_type_inv = {
            'sale': 'out_invoice',
            'purchase': 'in_invoice',
            'sale_refund': 'out_refund',
            'purchase_refund': 'in_refund',
        }
        
        if context is None:
            context = {}
        payslip_pool = self.pool.get('hr.payslip')
        invoice_data_obj = self.read(cr, uid, ids, ['journal_id'])
        journal_pool = self.pool.get('account.journal')

        active_ids = context.get('active_ids', [])
        
        model = context.get('active_model')
        model_pool = self.pool.get(model)
        pagos_brw = model_pool.browse(cr, uid, active_ids, context=context)
        
        message = 'Ya se genero factura para este pago de guarderia '
        inv_exist = False
        
        for pago in pagos_brw:
            context['date_invoice'] = pago.date_invoice
            context['number_invoice'] = pago.number_invoice
        
            if pago.account_invoice_ids:
                message = message + pago.payslip_id.employee_id.name + ', '
                inv_exist = True
            if inv_exist:
                raise osv.except_osv(_('Error'), _(message))
                
        journal_type = journal_pool.browse(cr, uid, invoice_data_obj[0]['journal_id']).type
        inv_type = journal_mapping_type_inv[journal_type]
        context['inv_type'] = inv_type
        
        res = self.action_invoice_create(cr, uid, pagos_brw,
              journal_id = invoice_data_obj[0]['journal_id'],
              type = inv_type,
              context=context)
        return res
        
    def _get_address_invoice(self, cr, uid, partner_id):
        """ Gets invoice address of a partner
        @return {'contact': address, 'invoice': address} for invoice
        """
        partner_obj = self.pool.get('res.partner')
        return partner_obj.address_get(cr, uid, [partner_id.id],
                ['contact', 'invoice'])
                
    def new_invoice(self, cr, uid, pago, partner, lines, journal_id, type='in_invoice', context=None):
        invoice_obj = self.pool.get('account.invoice')
        invoice_line_obj = self.pool.get('account.invoice.line')
        address_obj = self.pool.get('res.partner.address')
        obj_hr_acct = self.pool.get('hr.account')
        
        payment_term_id = False
        address_contact_id, address_invoice_id = \
                    self._get_address_invoice(cr, uid, partner).values()
        invoice_vals = {
            'name': pago.payslip_id.name,
            'origin': (pago.payslip_id.name or ''),
            'type': type,
            'account_id': partner.property_account_payable.id,
            'partner_id': partner.id,
            'address_invoice_id': address_invoice_id,
            'address_contact_id': address_contact_id,
            'comment': 'Factura generada desde el asistente de nomina para pago de guarderias',
            'payment_term': payment_term_id,
            'fiscal_position': partner.property_account_position.id,
            'date_document': context.get('date_invoice',False),
            'reference': context.get('number_invoice',False),
            'company_id': pago.employee_id.company_id.id,
            'user_id': uid,
            'process_id': pago.payslip_id.hr_process_id.id,
            'pago_guarderia_id': pago.id,
        }
        
        if journal_id:
            invoice_vals['journal_id'] = journal_id
        invoice_id = invoice_obj.create(cr, uid, invoice_vals, context=context)
        
        param_tax_id = self.pool.get('hr.parameters').search(cr, uid, 
          [('type', '=', 'impuesto_pago_guarderias')])
        if not param_tax_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro \
              definido para Impuesto que aplica a pago de guarderias. \
              Ir a la pestaña Configuracion -> Configuraciones\
              de nomina -> Parametros de nomina y registrar un \
              parametro de tipo Impuesto que aplica a pago de guarderias'))
        
        tax_id = self.pool.get('hr.parameters').browse(cr, uid, 
          param_tax_id)[0].impuesto_guarderia_id
        
        for l in lines:
            concept = l.salary_rule_id.concept_id if l.salary_rule_id else l.attribute_id.concept_id
            department = l.employee_id.department_id
            payroll = l.employee_id.hr_payroll_id
            hr_account_id = obj_hr_acct.search(cr, uid, 
                      [('concept_id', '=', concept.id),
                      ('analytic_account_id', '=', department.departament_analytic_account_id.id),
                      ('payroll_id', '=', payroll.id)])
            account = obj_hr_acct.browse(cr, uid, hr_account_id)[0]
            
            name_line = str(l.process_id.name)+'/'+l.employee_family_id.name
            concept_id = self.pool.get('islr.wh.concept').search(cr, uid, 
              [('name','=', 'NO APLICA RETENCION')])
            
            invoice_line_dict = {
                'name': l.name,
                'origin': (pago.payslip_id.name or ''),
                'invoice_id': invoice_id,
                'uom_id': False,
                'product_id': False,
                'account_id': account.account_id.id,
                'partida_presupuesto': account.partida_presupuesto.id,
                'accion_especifica_id': pago.employee_id.accion_especifica_id.id,
                'price_unit': l.total,
                'discount': 0.0,
                'quantity': 1,
                'invoice_line_tax_id': False,
                'account_analytic_id': account.analytic_account_id.id,
                'invoice_line_tax_id': [(6,0,[tax_id.id])],
            }
            
            if concept_id:
                invoice_line_dict.update({'concept_id': concept_id[0],})
                
            invoice_line_id = invoice_line_obj.create(cr, uid, invoice_line_dict, context=context)

        invoice_obj.button_compute(cr, uid, [invoice_id], context=context,
                set_total=(type in ('in_invoice', 'in_refund')))
        
        return invoice_id
                    
    def action_invoice_create(self, cr, uid, pagos_brw, journal_id=False, type='in_invoice', context=None):
        """ Crea las facturas de para pago de guarderias
        @param journal_id: Id del diario
        @param type: Tipo de factura que debe crearse
        @return: Ids de la factura creada para la(s) hoja(s) de pago
        """
        obj_hr_acct = self.pool.get('hr.account')
        
        if context is None:
            context = {}

        res = {}
        invoices = {}
        inv_type = type
        
        for pago in pagos_brw:
            partners = {}
            payment_term_id = False
            for linea in pago.linea_pago_guarderias_ids:
                slip_line = linea.payslip_line_ids[0]
                """ Comprobar configuracion contable"""
                concept = slip_line.salary_rule_id.concept_id if slip_line.salary_rule_id else slip_line.attribute_id.concept_id
                department = slip_line.employee_id.department_id
                payroll = slip_line.employee_id.hr_payroll_id
                
                account_id = obj_hr_acct.search(cr, uid, 
                    [('concept_id', '=', concept.id),
                    ('analytic_account_id', '=', department.departament_analytic_account_id.id),
                    ('payroll_id', '=', payroll.id)])

                if not account_id:
                    raise osv.except_osv(_('Error!'), 
                        _('No existe una configuración contable para el concepto\
                        %s, cuenta analítica %s y nómina %s. Para verificar ir a\
                        Talento Humano -> Configuración -> Contabilidad nómina.')
                        % (concept.name, department.departament_analytic_account_id.complete_name, 
                        payroll.name))
                        
                if (pago, pago.guarderia_id.partner_id) in partners:
                    lista = partners[pago, pago.guarderia_id.partner_id] + [slip_line]
                    partners[pago, pago.guarderia_id.partner_id] =  lista
                else:
                    partners[pago, pago.guarderia_id.partner_id] = [slip_line]

            invoices[pago] = partners
            
        for s in invoices:
            pago = s
            for inv in invoices[s]:
                partner = inv[1]
                lines_inv = invoices[s][inv]
                new_inv = self.new_invoice(cr, uid, pago, partner, lines_inv, journal_id, type, context)
                res[pago.id, new_inv] = new_inv
        
        return res

guarderia_invoice()

class account_invoice(osv.osv):

    _inherit = "account.invoice"
    _description = "Account Invoice"

    _columns = {
        'process_id': fields.many2one('hr.process', 'Proceso de Nómina'),
        'pago_guarderia_id': fields.many2one('hr.pagos.guarderias', 'Pago de guarderias'),
    }
    
    def confirm_paid(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        inv_brw = self.browse(cr, uid, ids, context)
        if not isinstance(inv_brw, list):
            inv_brw = [inv_brw] 
        for inv in inv_brw:
            if not inv.pago_guarderia_id:
                break
            self.pool.get('hr.pagos.guarderias').set_to_paid(cr, uid, [inv.pago_guarderia_id.id])
            
        return super(account_invoice, self).confirm_paid(cr, uid, ids, context)
        
    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        if ('pago_guarderia_id' in vals) and (vals['pago_guarderia_id'] <> False):
            self.pool.get('hr.pagos.guarderias').set_to_invoiced(cr, uid, [vals['pago_guarderia_id']])
            
        return super(account_invoice, self).create(cr, uid, vals, context)
    
    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        res = self.browse(cr, uid, id)
        if res.pago_guarderia_id:
            raise osv.except_osv(_('Error!'), 
                _('No se permite duplicar facturas de guarderias.'))
        default.update({
            'process_id': False,
            'pago_guarderia_id': False,
        })        
        return super(account_invoice, self).copy(cr, uid, id, default, 
          context=context)
            
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        inv_brw = self.browse(cr, uid, ids, context)
        if not isinstance(inv_brw, list):
            inv_brw = [inv_brw] 
        for inv in inv_brw:
            if not inv.pago_guarderia_id:
                break
            self.pool.get('hr.pagos.guarderias').set_to_done(cr, uid, [inv.pago_guarderia_id.id])
            
        return super(account_invoice, self).unlink(cr, uid, ids, context)
        
account_invoice()

class hr_pagos_guarderias(osv.osv):
    
    _inherit = 'hr.pagos.guarderias'
    
    _columns = {
        'account_invoice_ids': fields.one2many('account.invoice', 
          'pago_guarderia_id', 'Factura de guarderia'),
    }

hr_pagos_guarderias()

