# -*- coding: utf-8 -*-

import time
from osv import fields, osv
from datetime import datetime
import pooler
import calendar
import logging
import netsvc

class wizard_maximum_passive_holiday(osv.osv_memory):
    _name = "wizard.maximum.passive.holiday"
    _logger = logging.getLogger('wizard.maximum.passive.holiday')
    _columns = {
        'type_report': fields.selection([('resumido', 'Resumido'),('detallado', 'Detallado')], 'Reporte', select=True, required=True),
        'sede_id': fields.many2one('hr.sede', 'Sede'),
        'for_sede': fields.boolean(''),
        'mes': fields.many2one('hr.month', 'Mes', required=True),
       }
    _defaults = {'for_sede': True}
    
    def _check_email_server(self, cr, uid, ids, context=None):
        smtp_client = self.pool.get('email.smtpclient')
        smtp_server = smtp_client.search(cr, uid, [('active','=',True),('state','=','confirm')], context=False)
        if not smtp_server:
            raise osv.except_osv(_('Acción Denegada!'), _('Debe configurar un servidor de correos para generar el reporte'))
        return True
      
    def _check_user_email_adress(self, cr, uid, ids, context=None):
        email = self.pool.get('res.users').browse(cr, uid, uid).user_email
        if not email:
            raise osv.except_osv(('Acción Denegada!'), ('El usuario debe tener una cuenta de correo asociada para recibir el reporte'))
        return email
    
    def _calcular_regla(self, cr, uid, rule, localdict, employee, context):
        obj_rule = self.pool.get('hr.salary.rule')
        amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)
        previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
        tot_rule = amount * qty * rate / 100.0
        return {
            'salary_rule_id': rule.id,
            'name': rule.name,
            'quantity': qty,
            'amount': amount,
        }

    def query_concept(self, cr):
        concept_ids = []
        sql = """
                select hr_concept.id, hr_concept.code, hr_concept.name  from hr_parameters
                inner join hr_concept on hr_concept.id = hr_parameters.concept_vac_ven_id
                where hr_parameters.type= 'concepto_vac_venc'
                union
                select hr_concept.id, hr_concept.code, hr_concept.name  from hr_parameters
                inner join hr_concept on hr_concept.id = hr_parameters.concepto_dias_vac_ven_id
                where hr_parameters.type= 'concepto_dias_vac_ven'
                union
                select hr_concept.id, hr_concept.code, hr_concept.name  from hr_parameters
                inner join hr_concept on hr_concept.id = hr_parameters.concepto_dias_adic_vac_ven_id
                where hr_parameters.type= 'concepto_dias_adic_vac_ven'
                union
                select hr_concept.id, hr_concept.code, hr_concept.name  from hr_parameters
                inner join hr_concept on hr_concept.id = hr_parameters.concept_vac_no_disf_id
                where hr_parameters.type= 'concepto_vac_no_disf'
            """
        cr.execute(sql)
        concepts = cr.fetchall()
        if concepts:
            for concept in concepts:
                concept_ids.append(concept[1])
        else:
            raise osv.except_osv(('ERROR!'), ('No existen conceptos de vacaciones'))
        return concept_ids
    
    
    def get_cal_concept(self, params, employee, code, tipo, num_periods=False, fechas_inicio_fin=[]):
        
        if not num_periods and not fechas_inicio_fin:
            return (0, False)
        total = 0
        exist = False
        query = """
            select coalesce(t1.total, 0)
            from hr_payslip hp
            join hr_process hpr on hpr.id = hp.hr_process_id
            join hr_employee he on he.id = hp.employee_id
            left outer join (select hpl.slip_id, 
            sum(case when hc.type_calc = 'allocation' then hpl.total
                else hpl.total*-1
                end) as total 
            from hr_payslip_line hpl
            join hr_concept hc on hc.id =  hpl.concept_id
            where hpl.code in %s
            and  hpl.employee_id = %s
            group by slip_id) as t1 on hp.id = t1.slip_id
            where he.id = %s
            and hpr.type in %s
            and hpr.state = 'done'
        """
        
        args = (tuple(code), employee.id, employee.id, tuple(tipo))
        if fechas_inicio_fin:
            query = query + """ and hp.date_to >= %s
              and hp.date_from <= %s
              """
            args = args + (fechas_inicio_fin[0], fechas_inicio_fin[1])
        query += " order by hp.date_from desc"
        params[0].execute(query, args)
        lines_total = params[0].fetchall()
        lines_total = [x[0] for x in lines_total]
        if num_periods:
            if isinstance(num_periods, tuple):
                lines_total = lines_total[num_periods[0]:num_periods[1]]
            else:
                lines_total = lines_total[:num_periods]
        if lines_total:
            total = reduce(lambda x,y: x+y, [x for x in lines_total])
            exist = True
        return (total, exist)
    
    def _sueldo(self, employee_id, date_start, cr, uid):
        if employee_id:
            query = """SELECT amount FROM hr_attribute WHERE employee_id = %s and date_start <= %s and concept_id = 59 
                    ORDER BY id"""
            cr.execute(query, (employee_id, date_start))
            salario = cr.fetchall()
            if salario:
                return float(salario[-1][-1])
        return ''
    
    def get_payslip_lines(self, cr, uid, structure_brw, employee, list_concept_codes, fecha_creada):
        
        obj_rule = self.pool.get('hr.salary.rule')
        obj_line_liq = self.pool.get('hr.linea.liquidacion')
        parameter_obj = self.pool.get('hr.parameters')
        
        fecha_liquidacion = datetime.strptime(fecha_creada,"%Y-%m-%d")
        
        period = self.pool.get('hr.period').search(cr, uid, [
          ('date_start','<=', fecha_liquidacion), 
          ('date_end','>=', fecha_liquidacion), 
          ('type','=',employee.hr_payroll_id.hr_payment_calendar_id.type)])
        
        period = self.pool.get('hr.period').browse(cr, uid, period[0])
        
        #se obtienen los conceptos de la estructura salarial
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, [structure_brw.id], employee, period.id, context=None)
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]
        
        #  vac_vencidas: Lista de tuplas. Cada tupla tiene la forma (vac_vencida, num_dias_adicionales)
        start_rel_lab = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d")
        vac_vencidas = []
        employee_hol_ids = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id','=',employee.id),('state','=','done')], order='corresp_to_start desc')
        employee_hol_brw = False
        
        if employee_hol_ids:
            employee_hol_brw = self.pool.get('hr.employee.holidays').browse(cr, uid, employee_hol_ids[0], context=None)
            employee_hol_date = datetime.strptime(employee_hol_brw.corresp_to_end,"%Y-%m-%d")
            #Se cancelan 1 dia adicional por año a partir del 2º año
            dias_adicionales = employee_hol_date.year-start_rel_lab.year-1 if \
              employee_hol_date.year-start_rel_lab.year > 1 else 0
            dias_adicionales = dias_adicionales if dias_adicionales < 15 \
              else 15
            num_vac_vencidas = (fecha_liquidacion.year - employee_hol_date.year) if \
              fecha_liquidacion.month >= employee_hol_date.month else \
              (fecha_liquidacion.year - employee_hol_date.year - 1)
            for x in range(num_vac_vencidas):
                vac_vencidas.append((employee_hol_date.year+x, dias_adicionales + \
                  1 if dias_adicionales < 15 else 15))
        else:
            new_date = datetime(start_rel_lab.year, start_rel_lab.month, start_rel_lab.day)
            num_vac_vencidas = (fecha_liquidacion.year - new_date.year) if \
              fecha_liquidacion.month > new_date.month else \
              (fecha_liquidacion.year - new_date.year - 1)
            
            for x in range(num_vac_vencidas):
                dias_adicionales = x
                vac_vencidas.append((new_date.year + x, dias_adicionales \
                  if dias_adicionales < 15 else 15))
    
        new_date = datetime(start_rel_lab.year, start_rel_lab.month, start_rel_lab.day)
        
        ultimo_anio_pagado = datetime.strptime(employee_hol_brw.corresp_to_end,"%Y-%m-%d") \
          if employee_hol_brw else new_date
        
        reg_vac_no_disf = obj_line_liq.vacaciones_no_disfrutadas(cr, uid, employee)
        
        # 1 Busqueda del concepto que se definio como concepto Vacaciones vencidas
        param_vac_ven_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_vac_venc')])
        concept_vac_ven_brw = parameter_obj.browse(cr, uid, param_vac_ven_id)[0].concept_vac_ven_id
        # 2 Busqueda del concepto que se definio como Concepto Dias Vacaciones vencidas
        param_dias_vac_ven_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_dias_vac_ven')])
        concept_dias_vac_ven_brw = parameter_obj.browse(cr, uid, param_dias_vac_ven_id)[0].concepto_dias_vac_ven_id
        # 3 Busqueda del concepto que se definio como Concepto Dias adicionales Vacaciones vencidas
        param_dias_adic_vac_ven_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_dias_adic_vac_ven')])
        concept_dias_adic_vac_ven_brw = parameter_obj.browse(cr, uid, param_dias_adic_vac_ven_id)[0].concepto_dias_adic_vac_ven_id
        # 4 Busqueda del concepto que se definio como conceptos de Vacaciones no disfrutadas
        param_vac_no_disf_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_vac_no_disf')])
        concept_vac_no_disf_brw = parameter_obj.browse(cr, uid, param_vac_no_disf_id)[0].concept_vac_no_disf_id
        
        get_method = getattr(self.pool.get('hr.employee'), '_get')
        get_concept = getattr(self, 'get_cal_concept')
        localdict = {
                'atributo': get_method,
                'historial': get_concept,
                'param': [cr, uid],
                'concepto': {},
                'cantidad_horas': {},
                'categoria_concepto': {},
                'datetime': datetime,
                'vac_vencidas': vac_vencidas,
                'ultimas_vac_pagadas': employee_hol_brw,
                'fecha_liquidacion': fecha_liquidacion,
                'calendar': calendar,
                'self': self,
            }
    
        localdict.update({'empleado': employee,})
        param_fp_id = parameter_obj.search(cr, uid, [('type', '=', 'fecha_prest')])

        parameters_dict = {}
        for parameter in employee.hr_payroll_id.parameters_ids:
            parameters_dict[parameter.type] = parameter.value or 0.0
        
        localdict.update({'parametros': parameters_dict,})
        localdict['periodo'] = period
        rules_dict = {}
        rules_dict['SUELDO'] = self._sueldo(employee.id, fecha_creada, cr, uid)
        localdict['concepto'] = rules_dict

        categories_concept_dict={}
        blacklist = {}
        result_dict = {}
        
        for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=None):
            key = rule.code
            if key in list_concept_codes:
                localdict['result'] = None
                localdict['result_qty'] = localdict['cantidad'] = 1.0
    
                if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=None) and rule.id not in blacklist:
                    inicial_dict = self._calcular_regla(cr, uid, rule, localdict, employee, context=None)
                    amount = inicial_dict['amount']
                    qty = localdict['cantidad'] = inicial_dict['quantity']
                    
                    if rule.code == concept_vac_ven_brw.code or rule.code == \
                      concept_dias_vac_ven_brw.code:

                        qty_concept = parameters_dict['vacaciones'] \
                          if rule.code == concept_vac_ven_brw.code else 15
    
                        desde = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d")
                        
                        num_vac = len(vac_vencidas) if (desde.month == fecha_liquidacion.month and 
                            desde.day == fecha_liquidacion.day) else len(vac_vencidas) + 1
                        
                        for c in range(num_vac):

                            if fecha_liquidacion.year > desde.year:
                                if fecha_liquidacion.month == desde.month:
                                    if fecha_liquidacion.day < desde.day:
                                        meses = 12 - desde.month + fecha_liquidacion.month - 1
                                    else:
                                        meses = fecha_liquidacion.month - desde.month
                                        
                                        if ultimo_anio_pagado.year < fecha_liquidacion.year:
                                            meses = 12
                                
                                elif fecha_liquidacion.month < desde.month:
                                    if fecha_liquidacion.day < desde.day:
                                        meses = 12 - desde.month + fecha_liquidacion.month - 1
                                    else:
                                        meses = 12 - desde.month + fecha_liquidacion.month
                                else:
                                    meses = fecha_liquidacion.month - desde.month
                                    if fecha_liquidacion.day < desde.day:
                                        meses = fecha_liquidacion.month - desde.month - 1
                                    else:
                                        meses = fecha_liquidacion.month - desde.month
                            else:
                                meses = fecha_liquidacion.month - desde.month
                                if fecha_liquidacion.day < desde.day:
                                    meses = fecha_liquidacion.month - desde.month - 1
                                else:
                                    meses = fecha_liquidacion.month - desde.month
                            
                            qty = qty_concept if ((ultimo_anio_pagado.year+c < fecha_liquidacion.year-1) or \
                              (ultimo_anio_pagado.year+c == fecha_liquidacion.year-1 and \
                              fecha_liquidacion.month > ultimo_anio_pagado.month)) \
                              else float(qty_concept)/12*meses
                            
                            key_ = key + '-' + str(ultimo_anio_pagado.year+c)
                            result_dict[key_] = inicial_dict.copy()
                            result_dict[key_].update({
                                'name': rule.name,
                                'periodo': 'Periodo ' + str(ultimo_anio_pagado.year+c) \
                                  + '-' + str(ultimo_anio_pagado.year+1+c)\
                                  if qty ==  qty_concept \
                                  else 'Fraccion periodo ' + \
                                  str(fecha_liquidacion.year) \
                                  + '-' + str(fecha_liquidacion.year+1) if \
                                  fecha_liquidacion.month > ultimo_anio_pagado.month else
                                  'Fraccion periodo ' + \
                                  str(fecha_liquidacion.year-1) \
                                  + '-' + str(fecha_liquidacion.year),
                                'quantity': qty,
                            })
                            rules_dict[key_] = amount * qty or 0.0
                            localdict['concepto'] = rules_dict
                            
                    elif rule.code == concept_dias_adic_vac_ven_brw.code:

                        for c in range(len(vac_vencidas)):
                            qty = vac_vencidas[c][1]
                            key_ = key + '-' + str(ultimo_anio_pagado.year+c)
                            result_dict[key_] = inicial_dict.copy()
                            result_dict[key_].update({
                                'name': rule.name,
                                'periodo':'Periodo ' + str(ultimo_anio_pagado.year+c) \
                                  + '-' + str(ultimo_anio_pagado.year+1+c)\
                                  if ((ultimo_anio_pagado.year+c < fecha_liquidacion.year-1) or \
                                  (ultimo_anio_pagado.year+c == fecha_liquidacion.year-1 and \
                                  fecha_liquidacion.month >= ultimo_anio_pagado.month)) \
                                  else 'Fracción periodo ' + \
                                  str(fecha_liquidacion.year) \
                                  + '-' + str(fecha_liquidacion.year+1) if \
                                  fecha_liquidacion.month > ultimo_anio_pagado.month else
                                  'Fraccion periodo ' + \
                                  str(fecha_liquidacion.year-1) \
                                  + '-' + str(fecha_liquidacion.year),
                                'quantity': qty,
                            })
                            rules_dict[key_] = amount*qty or 0.0
                            localdict['concepto'] = rules_dict
                            
                    elif rule.code == concept_vac_no_disf_brw.code:
                        for _vac in reg_vac_no_disf:
                            anio = datetime.strptime(_vac.employee_holidays_add_id.corresp_to_end,"%Y-%m-%d").year
                            tipo = _vac.name[_vac.name.find('/')+13:]
                            qty = reg_vac_no_disf[_vac]
                            key_ = key + '-' + str(_vac.name[_vac.name.find('/')+2:])
                            result_dict[key_] = inicial_dict.copy()
                            result_dict[key_].update({
                                'name': rule.name,
                                'periodo': 'Periodo ' + str(anio-1) + ' - ' + str(anio),
                                'quantity': qty,
                            })
                            rules_dict[key_] = amount*qty or 0.0
                            localdict['concepto'] = rules_dict
                        
                    else:
                        result_dict[key] = inicial_dict.copy()
                        
                        if rule.code == concept_util_liq_brw.code:
                            qty = parameters_dict['utilidades']
                            result_dict[key]['quantity'] = qty
                            
                        elif rule.code == concept_prest_liq_brw.code:
                            qty = 5
                            result_dict[key]['quantity'] = qty
                            
                        elif rule.code == concept_util_frac_liq_brw.code:
                            contract_date_start = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d")
                            fraccion_mes = fecha_liquidacion.day / \
                              calendar.monthrange(fecha_liquidacion.year,fecha_liquidacion.month)[1]
                            if contract_date_start.year < fecha_liquidacion.year:
                                qty = parameters_dict['utilidades']/12*(fecha_liquidacion.month \
                                  - 1 + fraccion_mes)
                                result_dict[key]['quantity'] = qty
                            elif contract_date_start.year == fecha_liquidacion.year:
                                qty = parameters_dict['utilidades']/12*(fecha_liquidacion.month - \
                                  contract_date_start.month+fraccion_mes)
                                result_dict[key]['quantity'] = qty
                            
                        rules_dict[key] = amount*qty or 0.0
                        localdict['concepto'] = rules_dict
                    
                    if rule.concept_id.category_id.code not in categories_concept_dict:
                        categories_concept_dict[rule.concept_id.category_id.code] = \
                          amount or 0.0
                    else:
                        categories_concept_dict[rule.concept_id.category_id.code] = \
                          categories_concept_dict[rule.concept_id.category_id.code] \
                          + amount
                    localdict['categoria_concepto'] = categories_concept_dict
                else:
                    blacklist += [id for id, seq in 
                      self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, 
                      uid, [rule], context=context)]
        
        result = [value for code, value in result_dict.items()]
        
        return result
        
  
    def _send_report_by_email(self, cr, uid, parameters, context=None):
        
        subject = parameters['subject']
        message = parameters['message']
        data = parameters['data']
        file_name = parameters['file_name']
        fecha_creada = parameters['data']['fecha_creada']

        smtp_client = self.pool.get('email.smtpclient')
        smtp_server = smtp_client.search(cr, uid, [('active','=',True),('state','=','confirm')], context=False)
        
        parameter_obj = self.pool.get('hr.parameters')
        payroll_obj = self.pool.get('hr.payroll')
        payroll_structure_obj = self.pool.get('hr.payroll.structure')
        obj_line_liq = self.pool.get('hr.linea.liquidacion')
        
        param_struct_id = parameter_obj.search(cr, uid, [('type', '=', 'estructuras_liq')])
        struct_list = parameter_obj.browse(cr, uid, param_struct_id)[0].estruc_liq_ids

        list_concept_codes = self.query_concept(cr)
        
        if data['for_sede']:
            employee_ids = self.pool.get('hr.employee').search(cr, uid, [('state','in',['vacaciones','activo','reposo']),('start_rel_lab','<=', fecha_creada)])
            title = "Todas las sedes"
        else:
            brw_sede = self.pool.get('hr.sede').browse(cr, uid, data['sede']) 
            employee_ids = self.pool.get('hr.employee').search(cr, uid, [('hr_sede_id','=',data['sede']),('state','in',['vacaciones','activo','reposo']),('start_rel_lab','<=', fecha_creada)])
            title = brw_sede.name
        
        list_val = []
        value = []
        
        dicc_department = {}
        
        for i in employee_ids:
            employee = self.pool.get('hr.employee').browse(cr, uid, [i])
            if employee[0].start_rel_lab <= fecha_creada:
                dicc_department[employee[0].department_id.id] = []
        
        
        for i in employee_ids:
            employee = self.pool.get('hr.employee').browse(cr, uid, [i])
            if employee[0].start_rel_lab <= fecha_creada:
                struct_brw = [x for x in struct_list if x.hr_payroll_id == employee[0].hr_payroll_id]
                lines = self.get_payslip_lines(cr, uid, struct_brw[0], employee[0], list_concept_codes, fecha_creada)
                val= []
                l=[]
                [l.append(key) for key in lines if key not in l]
                for line in l:
                    if line['amount'] > 0.0 and line['quantity'] > 0.0:
                        amount = line['amount'] * line['quantity']
                        val.append({'n':line['name'] + " " + line['periodo'], 'a':amount, 'q':line['quantity'], 'p':line['periodo']})
                if val:
                    dicc_department[employee[0].department_id.id].append((employee[0].cod_number, employee[0].name, sorted(val)))
        
        total_departamento={}
        for c in dicc_department:
            total_departamento[c] = 0.0
        
        for c in dicc_department:
            amount = 0.0
            for i in dicc_department[c]:
                for x in i[2]:
                    amount += x['a']
            total_departamento[c] = amount
        
        total_resumen = []
        for c in total_departamento:
            total_resumen.append(total_departamento[c])
        total_resumen = sum(total_resumen)
        
        total_empleado = {}
        for c in dicc_department:
            for i in dicc_department[c]:
                total_empleado[i[0]] = 0.0
                
        for c in dicc_department:
            for i in dicc_department[c]:
                for x in i[2]:
                    total_empleado[i[0]] += x['a']
        
        new = datetime.strptime(fecha_creada, '%Y-%m-%d')
         
        data['form'] = dicc_department
        data['total_resumen'] = total_resumen
        data['total_empleado'] = total_empleado
        data['sede'] = title
        data['al'] = str(new.strftime('%d-%m-%Y'))
        
        report = self._create_report(cr, uid, data['ids'], data['model'], file_name, data)
        attachments = report[0] and [report[1]] or []
        
        if attachments == []:
            message = 'Ocurrio un error al generar el reporte.'
        try:
            if smtp_server:
                email_to = self.pool.get('res.users').browse(cr, uid, parameters['uid']).user_email
                if email_to:
                    smtp_server_id = smtp_server[0]
                    server_name = smtp_client.browse(cr, uid, smtp_server_id).server    
                    email = {
                            'server_id': smtp_server_id,
                            'name': (subject),
                            'server': server_name,
                            'body': (message),
                            'to': email_to,
                            'from': uid,
                            'attachments': attachments
                            }
                            
                    smtp_client.send_email(cr, uid, smtp_server_id, email['to'], email['name'], email['body'], email['attachments'])
        finally:
            cron = self.pool.get('ir.cron')
            accion_planificada = cron.search(cr, uid, [('name','=', subject)])
            cron.unlink(cr, uid, accion_planificada)
    
    def _send_request(self, cr, uid, ids, subject, message, context=None):
        """
        Metodo que envia una notificacion interna de OpenERP
        """
        notificaciones = {
                'name': (subject),
                'act_to': uid,
                'act_from': 1,
                'body': (message),
                'active': True,
                'priority': '2',
                'state': 'waiting'
            }
        request_obj = self.pool.get('res.request')
        request_obj.create(cr, uid, notificaciones, context)
        return True
    
    def _create_report(self, cr, uid, res_ids, report_name=False, file_name=False, data=False):
        try:
            ret_file_name = '/tmp/'+file_name+'.pdf'
            service = netsvc.LocalService("report.maximum.passive");
            (result, format) = service.create(cr, uid, res_ids, data, {})
            fp = open(ret_file_name, 'wb+');
            fp.write(result);
            fp.close();
        except Exception,e:
            print 'Exception in create report:',e
            return (False, str(e))
        return (True, ret_file_name)
    
    def getObtenerDia(self, mes, dia="primer"):
        meses = {'ENERO':'01', 'FEBRERO':'02', 'MARZO':'03', 'ABRIL':'04', 'MAYO':'05', 'JUNIO':'06', 
        'JULIO':'07', 'AGOSTO':'08', 'SEPTIEMBRE':'09', 'OCTUBRE':'10', 'NOVIEMBRE':'11', 'DICIEMBRE':'12'}
        mes = mes.split(" ")
        anio = mes[1]
        mes =  meses[mes[0].upper()]
        calendario = calendar.monthrange(int(anio),int(mes))
        if dia == "ultimo":
            fecha_creada = str(anio)+"-"+ str(mes)+ "-" + str(calendario[1])
            return fecha_creada
        return str(anio)+"-"+ str(mes)+ "-" + "-01"
    
    def print_report_buton(self, cr, uid, ids, context=None):
        
        brw = self.browse(cr, uid, ids)[0]
        mes = brw.mes.name
        month_id = brw.mes.id
        fecha_creada = self.getObtenerDia(mes, 'ultimo')
        for_sede = brw.for_sede
        type_report = brw.type_report
         
        self._check_email_server(cr, uid, ids, context)
        email = self._check_user_email_adress(cr, uid, ids, context)
        
        parameter_obj = self.pool.get('hr.parameters')
        payroll_obj = self.pool.get('hr.payroll')
        payroll_structure_obj = self.pool.get('hr.payroll.structure')
        obj_line_liq = self.pool.get('hr.linea.liquidacion')
        obj_process = self.pool.get('hr.process')
        obj_period = self.pool.get('hr.period')
        
        param_struct_id = parameter_obj.search(cr, uid, [('type', '=', 'estructuras_liq')])
        if not param_struct_id:
            raise osv.except_osv(('ERROR!'), ('No hay un parametro \
              definido para Estructuras de liquidaciones. Ir a \
              la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y registrar un parametro de tipo Estructuras para liquidaciones'))
        
        # 1 Busqueda del concepto que se definio como concepto Vacaciones vencidas
        param_vac_ven_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_vac_venc')])
        if not param_vac_ven_id:
            raise osv.except_osv(('ERROR!'), ('No hay un parametro \
              definido para el Concepto Vacaciones vencidas. Ir a \
              la pestaña Configuracion -> Configuraciones de nomina -> \
              Parametros de nomina y registrar un parametro de tipo \
              Concepto Vacaciones vencidas'))
        
        # 2 Busqueda del concepto que se definio como Concepto Dias Vacaciones vencidas
        param_dias_vac_ven_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_dias_vac_ven')])
        if not param_dias_vac_ven_id:
            raise osv.except_osv(('ERROR!'), ('No hay un parametro \
              definido para el Concepto Dias Vacaciones vencidas. Ir a \
              la pestaña Configuracion -> Configuraciones de nomina -> \
              Parametros de nomina y registrar un parametro de tipo \
              Concepto Dias Vacaciones vencidas'))
        
        # 3 Busqueda del concepto que se definio como Concepto Dias adicionales Vacaciones vencidas
        param_dias_adic_vac_ven_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_dias_adic_vac_ven')])
        if not param_dias_adic_vac_ven_id:
            raise osv.except_osv(('ERROR!'), ('No hay un parametro \
              definido para el Concepto Dias adicionales Vacaciones vencidas. Ir a \
              la pestaña Configuracion -> Configuraciones de nomina -> \
              Parametros de nomina y registrar un parametro de tipo \
              Concepto Dias adicionales Vacaciones vencidas'))

        # 4 Busqueda del concepto que se definio como conceptos de Vacaciones no disfrutadas
        param_vac_no_disf_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_vac_no_disf')])
        if not param_vac_no_disf_id:
            raise osv.except_osv(('ERROR!'), ('No hay un parametro \
              definido para Conceptos Vacaciones no disfrutadas. Ir a \
              la pestaña Configuracion -> Configuraciones de nomina -> \
              Parametros de nomina y registrar un parametro de tipo \
              Conceptos Vacaciones no disfrutadas'))

        nomina_ids = payroll_obj.search(cr, uid, [])
        nomina_browse = payroll_obj.browse(cr, uid, nomina_ids)
        
        for i in nomina_browse:
            structura_id = payroll_structure_obj.search(cr, uid, [('type','=','liquidacion'),('hr_payroll_id','=',i.id)])
            if not structura_id:
                raise osv.except_osv(('ERROR!'), ('No hay una Estructuras de Liquidaciones para la nomina %s')%(i.name))
        
        for i in nomina_browse:
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),
            ('payroll_id', '=', i.id)])
        
            if not param_concept_salary_id:
                param_concept_salary_id = parameter_obj.search(cr, uid, 
                  [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
            
            if not param_concept_salary_id:
                raise osv.except_osv(('ERROR!'), ('No hay un parametro definido \
                  para el concepto salario para la nomina %s. Ir a \
                  la pestaña Configuracion -> Configuraciones de nomina -> \
                  Parametros de nomina y registrar un parametro de tipo \
                  Concepto Salario')%(i.name))
            
            fecha_liquidacion = datetime.strptime(fecha_creada,"%Y-%m-%d")
            
            period_id = obj_period.search(cr, uid, [('hr_month_id', '=', month_id), ('state', '=', 'close')])
            
            if not period_id:
                raise osv.except_osv(('ERROR!'), ('No hay un periodo asociado a la fecha %s para la %s')%(today, i.name))  
            
            otros_procesos_id = obj_process.search(cr, uid, [('hr_payroll_ids', '=', i.id), 
              ('type', 'not in', ('prestaciones','bono_alimentacion','guarderias')), 
              ('hr_period_ids', 'in', period_id), ('state', '=', 'done')])
            
            if otros_procesos_id:

                proceso_id = obj_process.search(cr, uid, [('hr_payroll_ids', '=', i.id), ('type', '=', 'prestaciones'), 
                      ('hr_period_ids', 'in', period_id), ('state', '=', 'done')])
                
                if not proceso_id:
                   raise osv.except_osv(('ERROR!'), ('La %s para el mes de %s no \
                     posee proceso de Prestaciones Sociales aprobado'%(i.name, mes)))
                
                proceso_nomina_id = obj_process.search(cr, uid, [('hr_payroll_ids', '=', i.id), ('type', '=', 'nomina'), 
                  ('hr_period_ids', 'in', period_id), ('state', '=', 'done')])
                
                if not proceso_nomina_id:
                   raise osv.except_osv(('ERROR!'), ('La %s para el mes de %s no \
                     posee proceso de Nomina aprobado'%(i.name, mes)))
            
        if for_sede:
            employee_ids = self.pool.get('hr.employee').search(cr, uid, [('state','in',['vacaciones','activo','reposo'])])
            title = "TODAS LAS SEDES"
        else:
            employee_ids = self.pool.get('hr.employee').search(cr, uid, [('hr_sede_id','=',brw.sede_id.id),('state','in',['vacaciones','activo','reposo'])])
            title = brw.sede_id.name
        
        
        if employee_ids:
            for i in employee_ids:
                employee = self.pool.get('hr.employee').browse(cr, uid, [i])
                if employee[0].start_rel_lab <= fecha_creada:
                    struct_list = parameter_obj.browse(cr, uid, param_struct_id)[0].estruc_liq_ids
                    struct_brw = [x for x in struct_list if x.hr_payroll_id == employee[0].hr_payroll_id]
                    if struct_brw:
                        reg_vac_no_disf = obj_line_liq.vacaciones_no_disfrutadas(cr, uid, employee[0])
                    else:
                        raise osv.except_osv(('ERROR!'), ('La %s no esta incluida en los parametros de la estructura para liquidacion'%(employee[0].hr_payroll_id.name)))
            data = {}
            data['form'] = {}
            data['ids'] = [1]
            data['model'] = "wizard.maximum.passive.holiday"
            data['sede'] = brw.sede_id.id
            data['type_report'] = brw.type_report
            data['for_sede'] = brw.for_sede
            data['fecha_creada'] = fecha_creada

            if data:
                subject = "Vacaciones por pagar %s %s"%(title, brw.type_report.capitalize())     
                request_message = "Se esta generando el reporte vacaciones por pagar para %s %s" % (title, brw.type_report.capitalize())
                email_message = "Se adjunta el reporte de vacaciones por pagar para %s %s" % (title, brw.type_report.capitalize())
                
                cron_parameters = dict()
                cron_parameters['data'] = data
                cron_parameters['subject'] = subject
                cron_parameters['message'] = email_message
                cron_parameters['file_name'] = subject
                cron_parameters['uid'] = uid
                vals = {
                    'name': subject,
                    'active': True,
                    'user_id': uid,
                    'interval_number': 1,
                    'interval_type':'minutes',
                    'numbercall': 1,
                    'doall': True,
                    'model': 'wizard.maximum.passive.holiday',
                    'function': '_send_report_by_email',
                    'args': '(' + str(cron_parameters) + ',)',
                }
                self.pool.get('ir.cron').create(cr, uid, vals, context)
                self._send_request(cr, uid, ids, subject, request_message, context)
                return {'type':'ir.actions.act_window_close'}
            else:
                raise osv.except_osv(('Accion Denegada!'), ('No existen datos disponibles para generar el reporte'))
        else:
            raise osv.except_osv(('Accion Denegada!'), ('%s no cuenta con trabajadores para poder generar este reporte')%(title))
    
    
    
wizard_maximum_passive_holiday()
