#-*- coding:utf-8 -*-
#!/usr/bin/env python

from osv import osv, fields
from tools.translate import _
import unicodedata

class wizard_level_department(osv.osv_memory):
    _name = "wizard.level.department"
    _description = 'REPORTE RESUMEN POR DEPARTAMENTO'
    _columns = {
        'sede': fields.many2one('hr.sede', 'Sede', required=True),
        'department': fields.many2one('hr.department', 'Departamento', required=True),
        'payroll': fields.many2one('hr.payroll', 'Nomina', required=True),
        'state_employee': fields.selection([('todos','Todos'),('activo', 'Activo'),('reposo', 'Reposo'),('vacaciones', 'Vacaciones'),], 'Estado trabajador', select=True, required=True),
        'type_report': fields.selection([('resumido','Resumido'),('detallado', 'Detallado')], 'Tipo reporte', select=True, required=True),
        'all_departments': fields.boolean('Todos los departamentos'),
        'all_payroll': fields.boolean('Todas las nominas'),
        'zero': fields.boolean('Mostrar resultados en cero'),
        'all_sede': fields.boolean('Todas las sedes'),
        'level': fields.char('Nivel', size=64, required=False, help='Ingrese el nivel de jerarquia en numero.'),
        'type_employee': fields.selection([('todos','Todos'),('obrero', 'Obrero'),('empleado', 'Empleado'),('vendedor', 'Vendedor'),], 'Tipo trabajador', select=True, required=True),
    }
    _defaults = { 'all_departments': True, 'all_payroll':True, 'all_sede':True, 'zero':True,}
    
    def minmax(self, lista):
        if lista:
            lista.sort()
        return lista[-1]
    
    def validate_input(self, char):
        """ METODO QUE VALIDA LA ENTRADA DE DATOS
            EN EL CAMPO DE ENTRADA LEVEL, VERIFICA SI EXISTE Y SI EXISTE 
            QUE SEA UN NUMERO """
        if char:
            if char.isdigit():
                return char
            return False
        return True
    
    def complete_name_department(self, cr, uid, department_id):
        query = """ select id, nombre_departamento(id) as departamento from hr_department where id = %s """ %(department_id)
        cr.execute(query)
        full_name_department = cr.fetchall()
        if full_name_department:
            return full_name_department[0][1]
        else:
            return department_id
    
    def search_department_children(self, cr, uid, department_id):
        lista_department=[]
        query = """ select id, nombre_departamento(id) as departamento from hr_department where id = %s """ %(department_id)
        cr.execute(query)
        full_name_department = cr.fetchall()
        if full_name_department:
            lista_department.append(full_name_department[0][0])
            """ ESTE CICLO SE ENCARGA DE BUSCAR LOS HIJOS DEL DEPARTAMENTO O GERENCIA PADRE. 
                EL DEPARTAMENTO O GERENCIA PADRE = DEPARTAMENTO O GERENCIA SELECCIONADA """
            for i in full_name_department:
                query_children = """ select id, nombre_departamento(id) as departamento from hr_department where parent_id = %s """ %(i[0])
                cr.execute(query_children)
                full_name_department_childrens = cr.fetchall()
                for i in full_name_department_childrens:
                    full_name_department.append(i)
                    lista_department.append(i[0])
        return lista_department
                            
    def generar_reporte(self, cr, uid, ids, context=None):
        
        brw = self.browse(cr, uid, ids)[0]
        sede = brw.sede
        state_employee = brw.state_employee
        department = brw.department
        level = brw.level
        level = self.validate_input(level)
        all_sede = brw.all_sede
        all_departments = brw.all_departments
        zero = brw.zero
        type_employee = brw.type_employee
        payroll = brw.payroll
        all_payroll = brw.all_payroll
        type_report = brw.type_report
        
        if not level:
            raise osv.except_osv(_('ERROR!'), _('Invalid Entry'))
        
        parametros = []
        
        if department:
            l=[]
            ids_department_children = self.search_department_children(cr, uid, department.id)
            for i in ids_department_children:
                l.append(i)
            parametros.append(('department_id','in',l))
        
        if sede:
            parametros.append(('hr_sede_id','=', brw.sede.id))
            sede = brw.sede.name
        else:
            sede = False
        
        if state_employee != "todos":
                parametros.append(('state','=', state_employee))
                
        else:
            parametros.append(('state','in',['activo','reposo','vacaciones']))
            
        
        if payroll:
            parametros.append(('hr_payroll_id','=', brw.payroll.id))
            
        
        if type_employee != "todos":
            parametros.append(('type_employee','=', type_employee))
            
        
        obj_employee = self.pool.get('hr.employee')
        
        employee_id =  obj_employee.search(cr, uid, parametros)
        
        if not employee_id:
        
            raise osv.except_osv(_('ERROR!'), _('The search did not return any results'))
        
        employee_brw = obj_employee.browse(cr, uid, employee_id)
        
        dic_structure = {}
        
        for i in employee_brw:
            dic_structure[i.hr_sede_id.name] = {}
            
        for i in employee_brw:
            dic_structure[i.hr_sede_id.name].update({self.complete_name_department(cr, uid, i.department_id.id):[]})
        
        for i in employee_brw:
            dic_structure[i.hr_sede_id.name][self.complete_name_department(cr, uid, i.department_id.id)].append(i.id)
            
        full_name_departments = []
        
        for s in dic_structure:
            for departamento in dic_structure[s]:
                full_name_departments.append(departamento)
            
        ######3 verifica si existe el nivel que el usuario solicita #################################################
        levels = []
        for i in full_name_departments:
            levels.append(len(i.split('/'))-1)
        for i in levels:
            if level == True:pass
            else:level = int(level)
            maximo = self.minmax(levels)
            if level > maximo:
                if level != 0: raise osv.except_osv(_('ERROR!'), _('No existe el nivel numero %s' %(str(level))))
        ##############################################################################################################
        
        
        dic={}
        final_list = []
        
        if all_departments:
            
            for s in dic_structure:
                for departamento in dic_structure[s]:
                    countLevel=0
                    depart = ''
                    for x in departamento.split('/'):
                        if countLevel <= level:
                            if depart == '':
                                depart += x
                            else:
                                depart += '/' + x
                        countLevel+=1
                    dic[depart.strip()]=[]
                
            for s in dic_structure:
                for departamento in dic_structure[s]:
                    countLevel=0
                    depart = ''
                    for x in departamento.split('/'):
                        if countLevel <= level:
                            if depart == '':
                                depart += x
                            else:
                                depart += '/' + x
                        countLevel+=1
                    dic[depart.strip()].append(dic_structure[s][departamento])
                    
            for i in dic:
                suma = []
                employee_ids=[]
                for x in dic[i]:
                    for ids in x:
                       employee_ids.append(ids) 
                    suma.append(len(x)) 
                    
                final_list.append((i, sum(suma), employee_ids))
        else:
        
            for i in ids_department_children:
                suma = []
                parametros_final=[]
                
                if state_employee != "todos":
                    if state_employee != "todos":parametros_final.append(('state','=', state_employee))
                else:
                    parametros_final.append(('state','in',['activo','reposo','vacaciones']))
                
                if payroll:
                    parametros_final.append(('hr_payroll_id','=', brw.payroll.id))
                
                if type_employee != "todos":
                    parametros_final.append(('type_employee','=', type_employee))
                
                parametros_final.append(('department_id','=', i))
                employee_ids =  obj_employee.search(cr, uid, parametros_final)
                if len(employee_ids) != 0:
                    suma.append(len(employee_ids)) 
                    final_list.append((self.complete_name_department(cr, uid, i), sum(suma), employee_ids))
        
        data={}
        
        data['form'] = final_list = sorted(final_list)
        data['type_report'] = type_report.upper()
        
        title = ''
        
        if sede:
            print sede
            if title == '': title += "Sede: " + sede
            else: title += ", Sede: " + sede
        
        if state_employee != 'todos':
            if title == '': title += "Estado trabajador: " + str(state_employee).upper() 
            else: title += ", Estado trabajador: " + str(state_employee).upper() 
        
        if department:
            if title == '': title +=  department.name
            else: title +=  ", " + department.name
        
        if payroll:
            if title == '': title += payroll.name
            else: title += ", " + payroll.name
        
        if type_employee != "todos":
            if title == '': title += "Tipo empleado: " + str(type_employee).upper()
            else: title += ", Tipo empleado: " + str(type_employee).upper()
        
        if all_departments:
            if title == '': title += "Nivel: " + str(level)
            else: title += ", Nivel: " + str(level)
            
        data['title'] = title + "."
                    
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'sisb.level.department.report',
            'datas':data, 
            }
                
wizard_level_department()
