#-*- coding:utf-8 -*-
##############################################################################
#
#    SISB, Sistema Integral Simón Bolivar
#    Copyright (C) 2012-2013 Industrias Diana, C.A. All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
from datetime import datetime
import logging


class hr_variable_concepts(osv.osv):
    """
    Herencia para:
    1) Incluir funcion para obtener el nombre de los conceptos
    variables
    """
    _inherit = 'hr.variable.concepts'

    def __init__(self, *args, **kargs):
        """ Corregida la manera en que se agregaba el campo 'tipo_de_proceso', para que
            se tome en cuenta si ya existe y se deba agregar el tipo al selection, o si
            se debe crear por primera vez
        """
        super(hr_variable_concepts, self).__init__(*args, **kargs)
        if 'tipo_de_proceso' in self._columns:
            if ('comisiones', 'Comisiones') not in self._columns['tipo_de_proceso'].selection:
                self._columns['tipo_de_proceso'].selection.append(('comisiones', 'Comisiones'))
        else:
            self._columns['tipo_de_proceso'] = fields.selection([('comisiones', 'Comisiones')], "Tipo de proceso")

    def unlink(self, cr, uid, ids, context=None):
        brw = self.browse(cr, uid, ids[0])
        commissiones_obj = self.pool.get('sisb.commission')
        variables_lines_obj = self.pool.get('hr.variable.concept.line')

        if brw.tipo_de_proceso == 'comisiones':
            if brw.state not in ['done', 'execute']:
                context.update({"eliminar_comision": True})
                commissiones_id = commissiones_obj.search(cr, uid, [('variable_concept_id', 'in', ids)])
                commissiones_obj.write(cr, uid, commissiones_id, {'state': 'confirmed', 'variable_concept_id': False})
                lines = [i.id for i in brw.variable_concept_line_ids]
                variables_lines_obj.unlink(cr, uid, lines, context=context)
        return super(hr_variable_concepts, self).unlink(cr, uid, ids)


hr_variable_concepts()


class hr_variable_concept_line(osv.osv):
    _inherit = 'hr.variable.concept.line'
    _description = 'Variable Concept Lines'

    def unlink(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        if ids:
            brw = self.browse(cr, uid, ids[0])
            if brw.variable_concept_id.tipo_de_proceso == 'comisiones' and not context.get('eliminar_comision', False):
                raise osv.except_osv(_('ERROR!'), _("""No se pueden borrar 
			   las lineas de conceptos variables tipo %s en estado %s""" % (brw.variable_concept_id.tipo_de_proceso, brw.variable_concept_id.state)))
        return super(hr_variable_concept_line, self).unlink(cr, uid, ids, context=context)


hr_variable_concept_line()
