#!/usr/bin/env python
# -*- coding:utf-8 -*-
##############################################################################
#
#    SISB, Sistema Integral Simon Bolivar
#    Copyright (C) 2012-2013 Industrias Diana, C.A. All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
{
    'name': 'Permisos de Trabajadores',
    'version': '1.0',
    "author": "Industrias Diana C.A.",
    "category": "Human Resources",
    "website": "http://www.industriasdiana.gob.ve",
    'description': """\
        Modulo para la carga de los permisos de ausencia de los trabajadores

        Configuración Inicial
        -------------------------------------------------------------------
        * Antes de poder hacer uso del módulo, se deben crear los "Movitos de
          Permisos", desde un usuario administrador o uno que pertenezca al
          grupo "Permisos / TH Global".

        * Por defecto, el límite de horas en permisos remunerados no puede
          exceder de 40 horas al mes. Esto se puede cambiar configurando el
          parámetro 'permits_limit' agregado a la vista de compañías.

        * Los motivos tienen 2 tipos de conceptos que se pueden asociar, de
          asignación y deducción, si el motivo no tiene ningún concepto
          asociado, este no será tomado en cuenta a la hora de crear los
          conceptos variables. El campo "Remunerado" que reflejan los motivos
          es un indicativo visual y no afecta al comportamiento de la carga
          de conceptos, es un campo funcional que se activará si al motivo
          se le asocia un concepto de asignación.

        * A excepcion del grupo "Permisos / TH Global", los usuarios deben
          tener una ficha asociada en Talento Humano, con el fin de aplicar
          las restricciones del departamento o sede.

        Casos de uso y restricciones:
        -------------------------------------------------------------------
        * Los permisos no deben exceder de 24 horas de duración, en caso de
          necesitar registrar un permiso mayor a 24 horas, se debe hacer uso
          del módulo de ausencias.
        * El permiso no está planteado para registrar la inasistencia de los
          trabajadores en su turno, sino para registrar retrasos en la hora
          de llegada, permisos de salida con retorno previsto el mismo día,
          y permisos para retirarse antes de cumplir el turno del día.
        * Los permisos podrán ser cargados para cualquier fecha, incluso hasta
          ser aprobados para cualquier fecha, pero a la hora de cargar los
          conceptos variables, solo se tomarán en cuenta aquellos que estén en
          estado Aprobado/Validado y que tengan fecha correspondiente al periodo
          que se está evaluando para el pago de la nómina.
        * Al momento de cargar los conceptos variables, el sistema verificará
          que las horas del permiso correspondan con el turno del trabajador, y
          ajustará las horas para que estén dentro del horario que debe cumplir
          el trabajador, cualquier permiso cuyas horas estén fuera del horario
          del trabajador será marcado como "Inválido" y no será tomado en cuenta.
        * Pueden ser marcados como "Inválido" aquellos permisos que al ser
          corregidos por el sistema se encuentren solapados con algún otro permiso
          ya existente.
        * Los cálculos se basan en los periodos anteriores a la nómina que se
          está pagando.
        * Las hojas de conceptos variables cargados automáticamente por el sistema
          no pueden ser editadas, si se necesita hacer un ajuste, se debe borrar
          la hoja y recalcular nuevamente la misma desde el asistente.
        * No pueden haber permisos cuyas fechas y horas se solapen para un mismo
          trabajador, incluso si el permiso aún se encuentra en estado Borrador.

        Sobre los grupos:
        -------------------------------------------------------------------
        * Permisos / Consultar: Sólo pueden ver los registros ya creados, y no
          poseen restriccion sobre departamento, podrán ver cualquier registro.
          No pueden crear, editar ni borrar los registros existentes. Solo tiene
          acceso al menú "Líneas de Permisos".

        * Permisos / Solicitar: Puede crear permisos y editarlos mientras estén
          en estado borrador. Puede ver los permisos que otros usuarios han creado
          siempre y cuando pertenezcan a su departamento y departamento por debajo
          del mismo. Tiene acceso a los menús "Líneas de Permisos" y "Carga masiva
          de permisos".

        * Permisos / Confirmar: Además de poder crear y ver los permisos de su
          departamento, también puede confirmar los permisos y hacer que estos ya
          no puedan ser editados. Puede enviar los permisos a estado Borrador para
          hacerlos editables. Tiene acceso a los menús "Líneas de Permisos" y "Carga
          masiva de permisos".

        * Permisos / Aprobar: Además de crear, editar y confirmar los permisos de su
          departamento, podrá aprobar los permisos que crea válidos. También puede
          devolver los permisos a estado Borrador. Tiene acceso a los menús "Líneas
          de Permisos", "Carga masiva de permisos" y "Aprobar Permisos".

        * Permisos / TH Local: Usuario destinado al uso de Talento Humano por sedes.
          Puede crear, editar, borrar, confirmar y aprobar permisos con la única
          restricción de poder hacerlo sobre permisos que tengan en común su sede.
          Esta distinción se hace mediante la cuenta analítica asociada a los
          departamentos. Este usuario podrá cargar los conceptos variables de tipo
          "Permisos", que serán reflejados automáticamente en las nóminas de los
          empleados de esa sede. Tiene acceso a los menús "Líneas de Permisos",
          "Carga masiva de permisos", "Aprobar Permisos" y "Cargar Conceptos Variables".

        * Permisos / TH Global: Usuario de Talento Humano global, sin restricciones
          de sede o departamento, puede crear, editar, borrar, confirmar y aprobar
          cualquier permiso. Este usuario, además del administrador, puede crear las
          configuraciones para los motivos de permisos. Tiene acceso a los menús
          "Líneas de Permisos", "Carga masiva de permisos", "Aprobar Permisos",
          "Cargar Conceptos Variables" y "Motivos de Permisos".
    """,
    'depends': ['base', 'sisb_nomina', 'sisb_turn_swap'],
    'external_dependencies': {},
    'init_xml': [],
    'update_xml': [
        'security/groups.xml',
        'security/ir.model.access.csv',
        'views/permits_view.xml',
        'wizards/permits_load_wizard_view.xml',
        'wizards/permits_approve_wizard_view.xml',
        'wizards/permits_concepts_wizard_view.xml',
        'wizards/permits_query_wizard_view.xml',
        'report/permits_report_wizard.xml',
    ],
    'active': True,
    'installable': True
}
