#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import fields, osv
from tools.translate import _
from lxml import etree


class hr_variable_concepts(osv.osv):
    _inherit = 'hr.variable.concepts'

    def __init__(self, *args, **kargs):
        super(hr_variable_concepts, self).__init__(*args, **kargs)
        if 'tipo_de_proceso' in self._columns:
            if ('permisos', 'Calculo de permisos') not in self._columns['tipo_de_proceso'].selection:
                self._columns['tipo_de_proceso'].selection.append(('permisos', 'Calculo de permisos'))
        else:
            self._columns['tipo_de_proceso'] = fields.selection([('permisos', 'Calculo de permisos')], "Tipo de proceso")

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        view = super(hr_variable_concepts, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar, submenu=submenu)
        if view_type == 'form':
            doc = etree.XML(view['arch'])
            # In case 'tipo_de_proceso' field is not in the view, add it to compare it
            node = doc.xpath("//field[@name='tipo_de_proceso']")
            if not node:
                tdp = etree.Element('field', name='tipo_de_proceso', invisible="1")
                if 'tipo_de_proceso' not in view['fields']:
                    view['fields'].update(self.pool.get('hr.variable.concepts').fields_get(cr, uid, fields=['tipo_de_proceso'], context=context))
                doc.append(tdp)
            # Now, we must iterate all fields to set them all readonly
            for n in doc.xpath("//field"):
                attrs = eval(n.get('attrs', '{}'))
                if attrs:
                    if 'readonly' in attrs:
                        attrs['readonly'].insert(0, '|')
                        for i, d in enumerate(attrs['readonly']):
                            f, o, v = d if type(d) == tuple else False, False, False
                            if f == 'tipo_de_proceso':
                                if o == 'in':
                                    v = v + ['permisos'] if type(v) == list else v + ('permisos',)
                                else:  # assuming '=' is the operator, 'not in' or another operator should not be used
                                    o, v = 'in', ['permisos', v]
                                attrs['readonly'][i] = (f, v, o)
                                break
                        else:
                            attrs['readonly'].append(('tipo_de_proceso', 'in', ('permisos',)))
                    else:
                        attrs.update({'readonly': [('tipo_de_proceso', 'in', ('permisos',))]})
                    n.set('attrs', str(attrs))
                else:
                    n.set('attrs', "{'readonly': [('tipo_de_proceso', 'in', ('permisos',))]}")
            view['arch'] = etree.tostring(doc)
        return view

    def unlink(self, cr, uid, ids, context=None):
        context = {} if context is None else context
        _sp_obj = self.pool.get('sisb.permits')
        vc_lines = []
        permits_ids = []
        vc_ids = []
        for vc in self.browse(cr, uid, ids, context=context):
            if vc.tipo_de_proceso == 'permisos' and vc.state not in ['done', 'execute']:
                context.update({"eliminar_permiso": True})
                self.pool.get('hr.variable.concept.line').unlink(cr, uid, map(int, vc.variable_concept_line_ids), context=context)
                vc_ids.append(vc.id)
        permits_ids = _sp_obj.search(cr, uid, [('variable_concept_id', 'in', vc_ids)], context=context)
        if permits_ids:
            _sp_obj.write(cr, uid, permits_ids, {'state': 'approved'}, context=context)
        return super(hr_variable_concepts, self).unlink(cr, uid, ids, context=context)


hr_variable_concepts()


class hr_variable_concept_line(osv.osv):
    _inherit = 'hr.variable.concept.line'

    def unlink(self, cr, uid, ids, context=None):
        ids = ids if isinstance(ids, list) else [ids]
        for line in self.browse(cr, uid, ids, context=context):
            if line.variable_concept_id.tipo_de_proceso == 'permisos' and not context.get('eliminar_permiso', False):
                raise osv.except_osv(_('ERROR'), _('Las líneas de conceptos variables de permisos no pueden ser borradas directamente'))
        return super(hr_variable_concept_line, self).unlink(cr, uid, ids, context=context)


hr_variable_concept_line()
