#!/usr/bin/env python
# -*- coding: utf-8 -*-
from report import report_sxw
from datetime import datetime, timedelta
from tools.translate import _


class report_sisb_permits(report_sxw.rml_parse):
    _name = 'report.sisb.permits'

    def __init__(self, cr, uid, name, context):
        super(report_sisb_permits, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'get_empresa': self.get_empresa,
            'get_final_date': self.get_final_date,
            'get_pretty_date': self.get_pretty_date,
            'pretty_date_start': self.pretty_date_start,
            'pretty_date_end': self.pretty_date_end,
            'pretty_duration': self.pretty_duration,
            'sort': self.sort
        })
        self.context = context

    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id = obj_addr.search(self.cr, self.uid, [('type', '=', 'invoice'), ('partner_id', '=', partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
                'street': row.street,
                'phone': row.phone,
                'fax': row.fax,
                'email': row.email,
                'city': row.city,
                'name': row.name,
                'country': row.country_id.name,
            }
        return res

    def get_final_date(self):
        return datetime.now().strftime('%d/%m/%Y %H:%M')

    def get_pretty_date(self):
        date = datetime.now().strftime('%A %d/%m/%Y')
        return date[0].upper() + date[1:]

    def sort(self, permits_ids):
        return sorted(permits_ids, key=lambda x: (x.employee_id.department_id.id, x.date_start, x.hour_start, x.employee_id.cod_number))

    def pretty_date_start(self, permit):
        return (datetime.strptime(permit.date_start, '%Y-%m-%d') + timedelta(hours=permit.hour_start)).strftime('%H:%M:%S %d/%m/%Y')

    def pretty_date_end(self, permit):
        return (datetime.strptime(permit.date_end, '%Y-%m-%d') + timedelta(hours=permit.hour_end)).strftime('%H:%M:%S %d/%m/%Y')

    def pretty_duration(self, permit):
        return str(timedelta(hours=permit.duration))

report_sxw.report_sxw(
    'report.sisb.permits',
    'sisb.permits.report.wizard',
    'sisb_permits/report/permits_report.rml',
    parser=report_sisb_permits, header=False)
