#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import osv, fields
from tools.translate import _


class sisb_permits_report_wizard(osv.osv_memory):
    _name = 'sisb.permits.report.wizard'
    _columns = {
        'department_id': fields.many2one('hr.department', 'Department', required=True),
        'date_start': fields.date('Date Start', required=True),
        'date_end': fields.date('Date End', required=True),
        'permits_ids': fields.many2many('sisb.permits', 'permits_report_rel', 'report_id', 'permit_id', 'Permits')
    }

    def print_report(self, cr, uid, ids, context=None):
        wiz_data = self.browse(cr, uid, ids[0], context=context)
        _sp_obj = self.pool.get('sisb.permits')
        perm_ids = _sp_obj.search(cr, uid, [('department_id', '=', wiz_data.department_id.id), ('date_start', '<=', wiz_data.date_end), ('date_end', '>=', wiz_data.date_start), ('state', 'in', ['approved', 'load', 'done'])], context=context)
        if not perm_ids:
            raise osv.except_osv(_('Error'), _('No permits were found during %s to %s for department %s') % (wiz_data.date_start, wiz_data.date_end, wiz_data.department_id.name))
        self.write(cr, uid, ids, {'permits_ids': [(6, 0, perm_ids)]}, context=context)
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'sisb.permits',
        }

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        _sp_obj = self.pool.get('sisb.permits')
        view = super(sisb_permits_report_wizard, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar, submenu=submenu)
        if view_type in ['form']:
            site_domain = _sp_obj.sites_domain(cr, uid, context=context)
            if site_domain:
                dept_domain = ('member_ids.hr_sede_id', 'in', site_domain)
            else:
                dept_domain = ('id', 'in', _sp_obj.depts_domain(cr, uid, context=context))
            view['fields']['department_id']['domain'] = [dept_domain]
        return view


sisb_permits_report_wizard()
