#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import osv, fields
from tools.translate import _


class sisb_permits_approve_wizard(osv.osv_memory):

    _name = 'sisb.permits.approve.wizard'
    _description = 'Wizard to approve employees permits'
    _columns = {
        'permits_ids': fields.many2many('sisb.permits', 'permits_approve_rel', 'approve_id', 'permit_id', 'Permits'),
        'reason_id': fields.many2one('sisb.permits.reasons', 'Reason'),
        'remunerated': fields.related('reason_id', 'remunerated', type='boolean', string="Remunerated?", readonly=True),
    }

    def reason_onchange(self, cr, uid, ids, reason_id, context):
        permits_domain = [('state', 'in', ['confirmed'])]
        if reason_id:
            permits_domain.append(('reason_id', '=', reason_id))
        return {
            'value': {
                'remunerated': self.pool.get('sisb.permits.reasons').browse(cr, uid, reason_id, context=context).remunerated if reason_id else None
            },
            'domain': {
                'permits_ids': permits_domain
            }
        }

    def approve_permits(self, cr, uid, ids, context=None):
        _sp_obj = self.pool.get('sisb.permits')
        for record in self.browse(cr, uid, ids, context=context):
            permits_ids = map(int, record.permits_ids)
            if not permits_ids:
                raise osv.except_osv(_('Error'), _('There are no permits loaded'))
            _sp_obj.write(cr, uid, permits_ids, {'state': 'approved', 'approved_by': uid}, context=context)
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'sisb.permits',
            'domain': [('id', 'in', permits_ids)],
            'target': 'self',
            'context': context,
        }

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        view = super(self.__class__, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar, submenu=submenu)
        if view_type == 'form':
            _sp_obj = self.pool.get('sisb.permits')
            site_domain = _sp_obj.sites_domain(cr, uid, context=context)
            emp_domain = ('employee_id.hr_sede_id', 'in', site_domain) if site_domain else ('employee_id.department_id', 'in', _sp_obj.depts_domain(cr, uid, context=context))
            view['fields']['permits_ids']['domain'] = [emp_domain, ('state', 'in', ['confirmed'])]
        return view


sisb_permits_approve_wizard()
