#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import osv, fields
from tools.translate import _
from datetime import datetime, timedelta
from logging import getLogger
from re import search as regex


def days_in_range(start, end):
    start, end = regex('\d{4}-\d{2}-\d{2}', start).group(0), regex('\d{4}-\d{2}-\d{2}', end).group(0)
    res = set()
    while start <= end:
        res.add(start)
        start = (datetime.strptime(start, '%Y-%m-%d') + timedelta(days=1)).strftime('%Y-%m-%d')
    return res


class sisb_permits_concepts_wizard(osv.osv_memory):
    _logger = getLogger('sisb.permits')
    _name = 'sisb.permits.concepts.wizard'
    _description = 'Wizard to load concepts related to employees permits'
    _columns = {
        'hr_process_id': fields.many2one('hr.process', 'Process', domain=[('type', '=', 'nomina'), ('state', 'in', ['draft', 'load', 'review'])], required=True),
    }

    def validate_permits(self, cr, uid, date_start, date_end, permit_brw, _sp_obj, context=None):
        daysnames = {0: 'monday', 1: 'tuesday', 2: 'wednesday', 3: 'thursday', 4: 'friday', 5: 'saturday', 6: 'sunday'}
        vals = {'state': 'load'}
        # Setting permit start and end datetimes
        p_start = datetime.strptime(permit_brw.date_start, '%Y-%m-%d') + timedelta(hours=permit_brw.hour_start)
        p_end = datetime.strptime(permit_brw.date_end, '%Y-%m-%d') + timedelta(hours=permit_brw.hour_end)
        # Finding the right shift_week
        found = False
        turn_start = permit_brw.date_start
        while not found:
            hsw_obj = max(filter(lambda x: x.period_id.date_start <= turn_start and x.period_id.date_end >= turn_start, permit_brw.employee_id.shift_week_ids))
            dt_permit, dt_turn = map(lambda x: datetime.strptime(x, '%Y-%m-%d'), [turn_start, hsw_obj.period_id.date_start])
            dayname = daysnames[(dt_permit - dt_turn).days]
            hour_start, hour_end = map(hsw_obj[dayname + '_hours_day_id'][dayname + '_hours_id'].__getitem__, ['hour_start', 'hour_end'])
            hh, mm, ss = map(float, hour_start.split(':'))
            if p_end < dt_permit + timedelta(hours=hh, minutes=mm, seconds=ss) or hour_start == hour_end:
                turn_start = str(dt_permit - timedelta(days=1))[:10]
            else:
                found = True
        turn_date_start = dt_permit
        turn_date_end = turn_date_start + timedelta(days=1) if hour_start > hour_end else turn_date_start
        # Setting turn start and end datetimes
        t_start = turn_date_start + timedelta(hours=float(hour_start[:2]), minutes=float(hour_start[3:5]), seconds=float(hour_start[6:8]))
        t_end = turn_date_end + timedelta(hours=float(hour_end[:2]), minutes=float(hour_end[3:5]), seconds=float(hour_end[6:8]))
        # Checking hour_start
        if p_start < t_start:
            hh, mm, ss = map(float, hour_start.split(':'))
            vals['hour_start'] = hh + mm / 60 + ss / 3600
        elif p_start > t_end:
            hh, mm, ss = map(float, hour_end.split(':'))
            vals['hour_start'] = hh + mm / 60 + ss / 3600
        # Checking hour_end
        if p_end > t_end:
            hh, mm, ss = map(float, hour_end.split(':'))
            vals['hour_end'] = hh + mm / 60 + ss / 3600
        elif p_end < t_start:
            hh, mm, ss = map(float, hour_start.split(':'))
            vals['hour_end'] = hh + mm / 60 + ss / 3600
        vals = {'state': 'invalid'} if vals.get('hour_start', permit_brw.hour_start) == vals.get('hour_end', permit_brw.hour_end) else vals
        try:
            return _sp_obj.write(cr, uid, permit_brw.id, vals, context=context)
        except:
            vals = {'state': 'invalid'}
            return _sp_obj.write(cr, uid, permit_brw.id, vals, context=context)

    def query_permits(self, date_start, date_end, employees_ids, context=None):
        return """
            SELECT sp.id, sp.employee_id, spr.asign_concept_id concept_id,
                SUM (EXTRACT(epoch FROM (sp.date_end + sp.hour_end * interval '1 hour') - (sp.date_start + sp.hour_start * interval '1 hour')) / 3600) duration
            FROM sisb_permits sp
            JOIN sisb_permits_reasons spr on spr.id = sp.reason_id
            WHERE ARRAY(SELECT(generate_series(date '%s', date '%s', interval '1 day'))) &&
                ARRAY(SELECT(generate_series(sp.date_start,sp.date_end, interval '1 day'))) AND
                sp.state = 'load' AND sp.employee_id in %s AND  spr.asign_concept_id > 0
            GROUP BY sp.id, sp.employee_id, concept_id

            UNION

            SELECT sp.id, sp.employee_id, spr.deduct_concept_id concept_id,
                SUM (EXTRACT(epoch FROM (sp.date_end + sp.hour_end * interval '1 hour') - (sp.date_start + sp.hour_start * interval '1 hour')) / 3600) duration
            FROM sisb_permits sp
            JOIN sisb_permits_reasons spr on spr.id = sp.reason_id
            WHERE ARRAY(SELECT(generate_series(date '%s', date '%s', interval '1 day'))) &&
                ARRAY(SELECT(generate_series(sp.date_start,sp.date_end, interval '1 day'))) AND
                sp.state = 'load' AND sp.employee_id in %s AND  spr.deduct_concept_id > 0
            GROUP BY sp.id, sp.employee_id, concept_id
        """ % (date_start, date_end, employees_ids, date_start, date_end, employees_ids)

    def load_concepts(self, cr, uid, ids, context=None):
        _sp_obj = self.pool.get('sisb.permits')
        _hvc_obj = self.pool.get('hr.variable.concepts')
        _hvcl_obj = self.pool.get('hr.variable.concept.line')
        _hpr_obj = self.pool.get('hr.process')
        _hpe_obj = self.pool.get('hr.period')
        vc_lines = {}

        for record in self.browse(cr, uid, ids, context=context):
            process_id = record.hr_process_id.id
            _hpr_brw = _hpr_obj.browse(cr, uid, process_id, context=context)
            # Checking if process has th_local site
            th_local = self.pool.get('ir.model.data').get_object_reference(cr, uid, 'sisb_permits', 'group_sisb_permits_th_local')[1]
            user_groups = map(int, self.pool.get('res.users').browse(cr, uid, uid, context=context).groups_id)
            if th_local in user_groups:
                emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)], context=context)
                if not emp_id and uid != 1:
                    raise osv.except_osv(_('Error'), _('%s has no employee profile configured') % (self.pool.get('res.users').browse(cr, uid, uid).name))
                site_depts_id = self.pool.get('hr.employee').browse(cr, uid, emp_id, context=context)[0].hr_sede_id.id if uid != 1 else 0
                if site_depts_id not in map(int, _hpr_brw.hr_sede_ids) and uid != 1:
                    raise osv.except_osv(_('Error'), _('This process does not include your site'))
            # Checking if there is already a sheet for permits created
            if _hvc_obj.search(cr, uid, [('hr_process_id', '=', process_id), ('tipo_de_proceso', '=', 'permisos'), ('state', 'in', ['draft', 'load', 'review', 'done'])], context=context):
                raise osv.except_osv(_('Error'), _('There is already a concept variable sheet for permits, delete it to recalculate'))
            sheet = _hvc_obj.create(cr, uid, {'hr_process_id': process_id, 'tipo_de_proceso': 'permisos', 'state': 'load'}, context=context)
            for period_brw in record.hr_process_id.hr_period_ids:
                # Getting start and end date per process for last period, not the current one
                # date_end of past period is date_start -1 from current period
                past_date_end = (datetime.strptime(period_brw.date_start, '%Y-%m-%d') - timedelta(days=1)).strftime('%Y-%m-%d')
                pde_id = _hpe_obj.search(cr, uid, [('date_end', '=', past_date_end), ('type', '=', period_brw.type)], context=context)
                try:
                    date_start, date_end = map(_hpe_obj.browse(cr, uid, pde_id[0], context=context).__getitem__, ['date_start', 'date_end'])
                except:
                    raise osv.except_osv(_('Error'), _('Past period not found for process %s') % _hpr_brw.name)
                # Getting employees_ids from current process and period
                employees_ids = list(set(map(int, _hpr_brw.hr_employee_ids) + map(lambda x: x.employee_id.id, _hpr_brw.hr_payslip_ids)))
                if not employees_ids:
                    raise osv.except_osv(_('Error'), _('No employees loaded on selected process'))
                if len(employees_ids) == 1:
                    employees_ids.append(0)
                # Validating permits
                for permit_brw in _sp_obj.browse(cr, uid, _sp_obj.search(cr, uid, [('date_start', '<=', date_end), ('date_end', '>=', date_start), ('employee_id', 'in', employees_ids), ('state', '=', 'approved')], context=context), context=context):
                    self.validate_permits(cr, uid, date_start, date_end, permit_brw, _sp_obj, context=context)
                # Querying permits from employees on the current period
                cr.execute(self.query_permits(date_start, date_end, tuple(employees_ids), context=context))
                permits_ids = []
                for permit in cr.dictfetchall():
                    permits_ids.append(permit['id'])
                    if str(permit['employee_id']) + ':' + str(permit['concept_id']) not in vc_lines:
                        vc_lines[str(permit['employee_id']) + ':' + str(permit['concept_id'])] = {
                            'employee_id': permit['employee_id'],
                            'concept_id': permit['concept_id'],
                            'type': 'qty',
                            'value': permit['duration'],
                            'variable_concept_id': sheet
                        }
                    else:
                        vc_lines[str(permit['employee_id']) + ':' + str(permit['concept_id'])]['value'] += permit['duration']
        if not vc_lines:
            raise osv.except_osv(_('Error'), _('No concepts were calculated for the process payrolls'))
        self._logger.info(_('On queue %d variable concept lines for creation') % len(vc_lines))
        for key, line in sorted(vc_lines.items()):
            _hvcl_obj.create(cr, uid, line, context=context)
        self.log(cr, uid, record.id, _('Permits concepts calculated for process %s') % record.hr_process_id.name)
        _sp_obj.write(cr, uid, permits_ids, {'variable_concept_id': sheet}, context=context)
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'hr.variable.concepts',
            'res_id': sheet,
            'target': 'self',
            'context': context,
        }


sisb_permits_concepts_wizard()
