#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import osv, fields
from tools.translate import _


class sisb_permits_load_wizard(osv.osv_memory):

    _name = 'sisb.permits.load.wizard'
    _description = 'Wizard to load employees permits'
    _columns = {
        'requester': fields.many2one('res.users', 'Requester', readonly=True),
        'employee_ids': fields.many2many('hr.employee', 'employee_permit_rel', 'permit_id', 'employee_id', 'Employee'),
        'department_id': fields.many2one('hr.department', 'Department'),
        'date_start': fields.date('Date start', required=True),
        'date_end': fields.date('Date end', required=True),
        'hour_start': fields.float('Hour start', required=True),
        'hour_end': fields.float('Hour end', required=True),
        'reason_id': fields.many2one('sisb.permits.reasons', 'Reason', required=True),
        'remunerated': fields.related('reason_id', 'remunerated', type='boolean', string="Remunerated?", readonly=True),
    }
    _defaults = {
        'requester': lambda self, cr, uid, context=None: uid
    }

    def reason_onchange(self, cr, uid, ids, reason_id, context):
        return {
            'value': {
                'remunerated': self.pool.get('sisb.permits.reasons').browse(cr, uid, reason_id, context=context).remunerated if reason_id else None
            }
        }

    def load_department(self, cr, uid, ids, context=None):
        for record in self.browse(cr, uid, ids, context=context):
            if not record.department_id:
                raise osv.except_osv(_('Error'), _('There is no department selected'))
            depts_ids = self.pool.get('sisb.permits').child_depts(cr, uid, record.department_id.id, context=context)
            emp_ids = list(set(self.pool.get('hr.employee').search(cr, uid, [('department_id', 'in', depts_ids), ('state', 'not in', ['egresado', 'borrador'])], context=context) + map(int, record.employee_ids)))
            self.write(cr, uid, [record.id], {'employee_ids': [(6, 0, emp_ids)], 'department_id': None}, context=context)
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'form,tree',
            'res_model': 'sisb.permits.load.wizard',
            'res_id': [record.id],
            'target': 'new',
            'ref': 'sisb_permits_load_wizard.sisb_permits_load_wizard_form_view',
            'context': context,
        }

    def request_permits(self, cr, uid, ids, context=None):
        _sp_obj = self.pool.get('sisb.permits')
        for record in self.browse(cr, uid, ids, context=context):
            if not record.employee_ids:
                raise osv.except_osv(_('Error'), _('There are no employees loaded'))
            vals = {
                'date_start': record.date_start,
                'date_end': record.date_end,
                'hour_start': record.hour_start,
                'hour_end': record.hour_end,
                'reason_id': record.reason_id.id,
                'remunerated': record.remunerated,
                'requester': record.requester.id
            }
            for employee in record.employee_ids:
                vals['employee_id'] = employee.id
                _sp_obj.create(cr, uid, vals, context=context)
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'sisb.permits',
            'target': 'self',
            'context': context,
        }

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        view = super(self.__class__, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar, submenu=submenu)
        if view_type == 'form':
            _sp_obj = self.pool.get('sisb.permits')

            site_domain = _sp_obj.sites_domain(cr, uid, context=context)
            if site_domain:
                emp_domain = ('hr_sede_id', 'in', site_domain)
                dept_domain = ('member_ids.hr_sede_id', 'in', site_domain)
            else:
                emp_domain = ('department_id', 'in', _sp_obj.depts_domain(cr, uid, context=context))
                dept_domain = ('id', 'in', _sp_obj.depts_domain(cr, uid, context=context))

            view['fields']['employee_ids']['domain'] = [emp_domain, ('state', 'not in', ['egresado', 'borrador'])]
            view['fields']['department_id']['domain'] = [dept_domain]
        return view


sisb_permits_load_wizard()
