#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import osv, fields
from tools.translate import _
from datetime import datetime


class sisb_permits_query_wizard(osv.osv_memory):

    _name = 'sisb.permits.query.wizard'
    _description = 'Wizard to query an employee permits on current day'
    _columns = {
        'employee_id': fields.many2one('hr.employee', 'Employee', required=True, help="Indique el codigo, nombre o cedula del empleado para consultar sus permisos en este dia")
    }

    def query_permits(self, cr, uid, ids, context=None):
        ids = ids if isinstance(ids, list) else [ids]
        employee_brw = self.browse(cr, uid, ids, context=context)[0].employee_id
        today = datetime.now()
        permit_ids = self.pool.get('sisb.permits').search(cr, uid, [('employee_id', '=', employee_brw.id), ('date_start', '<=', today), ('date_end', '>=', today)], context=context)
        if not permit_ids:
            raise osv.except_osv(_('ERROR'), _('No permits found for %s in this day') % employee_brw.name)
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'sisb.permits',
            'res_id': permit_ids,
            'domain': [('employee_id', '=', employee_brw.id), ('date_start', '<=', today.strftime('%Y-%m-%d')), ('date_end', '>=', today.strftime('%Y-%m-%d'))],
            'target': 'self',
            'context': context,
        }


sisb_permits_query_wizard()
