#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits########################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

import time
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from osv import osv, fields
from tools.translate import _
import decimal_precision as dp


class sisb_report_z(osv.osv):
    _name = "sisb.report.z"
sisb_report_z()


class maquina_fiscal(osv.osv):
    _name = "maquina.fiscal"
    _description = "Maquina Fiscal"
    
    #Sucursal o Direccion
    _columns = {
        'name': fields.char('Numero de Maquina', size=16, required=True),
        'description': fields.char('Descripcion', size=128),
        'direccion_asoc_id': fields.many2one('res.partner.address', 'Ubicación de Maquina', required=True),
        'company_id': fields.many2one('res.company', 'Company'),
        'partner_id': fields.related('company_id', 'partner_id', type='many2one', relation='res.partner', string="Partner"),
        'state': fields.selection([
            ('active','Active'),
            ('inactive','Inactive')
            ],'State', readonly=True),
    }
    
    _defaults = {
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'partner_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.partner_id.id,
        'state': 'active',
    }
    
    _sql_constraints = [
        ('maquina_fiscal_unique', 'unique (name)', 'El numero de la maquina fiscal debe ser unico'),
    ]
    
    
    def set_state_maquina_fiscal(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        state = 'active'
        if self.browse(cr, uid, ids)[0].state == 'active':
            state = 'inactive'
        
        self.write(cr, uid, ids, {'state': state}, context=context)
        
        return True


maquina_fiscal()


class sisb_report_z_tax(osv.osv):
    _name = "sisb.report.z.tax"
    _description = "Impuestos del Reporte Z"
    
    
    def _get_amount_tax(self, cr, uid, ids, name, args, context=None):
        res = {}
        for tax_z in self.browse(cr, uid, ids, context):
            res[tax_z.id] = {'amount_tax': 0.0,
                'amount_untaxed': 0.0
            } 
            if tax_z.name and tax_z.amount_bi_tax:
                impuesto = tax_z.amount_bi_tax * tax_z.name.amount
                res[tax_z.id] = {'amount_tax': impuesto,
                    'amount_untaxed': tax_z.amount_bi_tax + impuesto
                }
                
        return res
    
    
    _columns = {
        'name': fields.many2one('account.tax', 'Impuesto Iva', domain=[('type_tax_use','=','sale')], required=True),
        'amount_bi_tax': fields.float('Base', digits_compute=dp.get_precision('Account'), required=True),
        'amount_tax': fields.function(_get_amount_tax, method=True, digits_compute=dp.get_precision('Account'), string='Importe', store=True, multi="taxes"),
        'amount_untaxed': fields.function(_get_amount_tax, method=True, digits_compute=dp.get_precision('Account'), string='Total', store=False, multi="taxes"),
        'type': fields.selection([('inv','Invoice'),
                ('cr','Nota de Credito')], 'Tipo', required=True),
        'report_z_id': fields.many2one('sisb.report.z', 'Reporte Z', ondelete='cascade', select=True),
    }

    _sql_constraints = [
        ('report_z_tax_uniq', 'unique (name, report_z_id, type)', 'Solo puede existir un tipo de impuesto por Reporte Z'),
        ('amount_bi_tax_positive', 'CHECK (amount_bi_tax>=0)', 'Error el monto debe ser mayor a 0!'),
    ]

sisb_report_z_tax()


class sisb_report_z(osv.osv):
    _name = "sisb.report.z"
    _description = "Reporte Z"

    
    _order = "date_report desc"
    
    def _get_first_invoice(self, cr, uid, ids, name, args, context=None):
        res = {}
        for report_z in self.browse(cr, uid, ids, context):
            res[report_z.id] = {
                'pri_invoice': report_z.ult_invoice and report_z.nro_invoice and (report_z.ult_invoice - report_z.nro_invoice + 1) or 0,
                'pri_invoice_credit': report_z.ult_invoice_credit and report_z.nro_invoice_credit and (report_z.ult_invoice_credit - report_z.nro_invoice_credit + 1) or 0
            }
            if res[report_z.id]['pri_invoice'] < 0:
                raise osv.except_osv(_('Error'),_('En el Reporte Z %s, exite una diferencia negativa entre la ultima factura y el numero de facturas'%(report_z.name)))
            elif res[report_z.id]['pri_invoice_credit'] < 0:
                raise osv.except_osv(_('Error'),_('En el Reporte Z %s, exite una diferencia negativa entre la ultima nota de debito y el numero de facturas'%(report_z.name)))

        return res
    
    
    def _get_amount_total(self, cr, uid, ids, name, args, context=None):
        res = {}
        for report_z in self.browse(cr, uid, ids, context):
            res[report_z.id] = {
                'total_vta': 0.0,
                'total_iva_vta': 0.0
            }
            for tax_z in report_z.report_tax_ids:
                res[report_z.id] = {
                    'total_vta': res[report_z.id]['total_vta'] + tax_z.amount_bi_tax,
                    'total_iva_vta': res[report_z.id]['total_iva_vta'] + tax_z.amount_tax,
                }
                
        return res
    
    
    def _get_amount_total_credit(self, cr, uid, ids, name, args, context=None):
        res = {}
        for report_z in self.browse(cr, uid, ids, context):
            res[report_z.id] = {
                'total_vta_credit': 0.0,
                'total_iva_vta_credit': 0.0
            }
            for tax_z in report_z.report_tax_nc_ids:
                res[report_z.id] = {
                    'total_vta_credit': res[report_z.id]['total_vta_credit'] + tax_z.amount_bi_tax,
                    'total_iva_vta_credit': res[report_z.id]['total_iva_vta_credit'] + tax_z.amount_tax,
                }
                
        return res
    

    _columns = {
        'name': fields.char('Reporte Z', size=16, required=True),
        'maquina_fiscal_id': fields.many2one('maquina.fiscal', 'Maquina Fiscal', required=True),
        'report_tax_ids': fields.one2many('sisb.report.z.tax', 'report_z_id', 'IVA Factura', domain=[('type','=','inv')], context={'default_type':'inv'}),
        'report_tax_nc_ids': fields.one2many('sisb.report.z.tax', 'report_z_id', 'IVA Nota de Credito', domain=[('type','=','cr')], context={'default_type':'cr'}),
        'date_report': fields.datetime('Fecha', required=True),
        'pri_invoice': fields.function(_get_first_invoice, method=True, type='integer', string='Primera Factura', multi='invoice_cal', store=True),
        'ult_invoice': fields.integer('Ultima Factura', required=True),
        'nro_invoice': fields.integer('Numero de Facturas', required=True),
        'pri_invoice_credit': fields.function(_get_first_invoice, method=True, type='integer', string='Primera Nota de Credito', multi='invoice_cal', store=True),
        'ult_invoice_credit': fields.integer('Ultima Nota de Credito'),
        'nro_invoice_credit': fields.integer('Numero de Notas de Credito'),
        'total_vta': fields.function(_get_amount_total, method=True, digits_compute=dp.get_precision('Account'), string='Total Venta', 
            #~ store={
                    #~ 'sisb.report.z': (lambda self, cr, uid, ids, c={}: ids, ['report_tax_ids'], 20),
                #~ },
            multi="total"),
        'total_iva_vta': fields.function(_get_amount_total, method=True, digits_compute=dp.get_precision('Account'), string='Total Iva Venta',
            #~ store={
                    #~ 'sisb.report.z': (lambda self, cr, uid, ids, c={}: ids, ['report_tax_ids'], 20),
                #~ },
            multi="total"),
        'total_vta_credit': fields.function(_get_amount_total_credit, method=True, digits_compute=dp.get_precision('Account'), string='Total Notas de Credito', 
            #~ store={
                    #~ 'sisb.report.z': (lambda self, cr, uid, ids, c={}: ids, ['report_tax_nc_ids'], 1),
                #~ },
            multi="total_credit"),
        'total_iva_vta_credit': fields.function(_get_amount_total_credit, method=True, digits_compute=dp.get_precision('Account'), string='Total Iva Notas de Credito', 
            #~ store={
                    #~ 'sisb.report.z': (lambda self, cr, uid, ids, c={}: ids, ['report_tax_nc_ids'], 1),
                #~ },
            multi="total_credit"),
        'company_id': fields.many2one('res.company', 'Company'),
        'address_id': fields.many2one('res.partner.address', 'Direccion'),
        'state': fields.selection([
            ('draft','Draft'),
            ('done','Done'),
            ('cancel','Cancelled')
            ],'State', readonly=True),
    }
    
    _defaults = {
        'date_report': lambda *args: time.strftime('%Y-%m-%d %H:%M:%S'),
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'state': 'draft',
    }
    
    _sql_constraints = [
        ('nro_invoice_positive', 'CHECK (nro_invoice>=0)', 'El numero de facturas debe ser mayor a 0!'),
        ('ult_invoice_positive', 'CHECK (ult_invoice>=0)', 'La ultima factura debe ser mayor a 0!'),
        ('nro_invoice_credit_positive', 'CHECK (nro_invoice_credit>=0)', 'El numero de la nota de credito debe ser mayor a 0!'),
        ('ult_invoice_credit_positive', 'CHECK (ult_invoice_credit>=0)', 'La ultima nota de credito debe ser mayor a 0!'),
    ]
    
    
    def _check_repeat_report_z(self, cr, uid, ids, context=None):
        """ Verificar exista solo un reporte z por maquina fiscal y por dia.
        @return: True (bien) or False (mal)
        """
        if context is None:
            context = {}
        
        repo_obj = self.browse(cr, uid, ids)[0]
        name = repo_obj.name
        maquina_fiscal_id = repo_obj.maquina_fiscal_id.id
        new = datetime.strptime(repo_obj.date_report, '%Y-%m-%d %H:%M:%S')
        date_report = str(new.strftime('%Y-%m-%d'))
        
        SQL = """select count(*) as cantidad
            from sisb_report_z
            where maquina_fiscal_id=%s and date_report::date='%s' and name='%s';"""%(maquina_fiscal_id, date_report, name)
        cr.execute(SQL)
        res = cr.dictfetchone()
        
        if res['cantidad'] > 1:
            return False
        
        return True
    
    
    _constraints = [(_check_repeat_report_z, 'Debe existir solo un Reporte Z por Maquina Fiscal. Verifique que no este intentando crear un Reporte Z ya existente.', ['maquina_fiscal_id, name, date_report'])]
    
    
    def create(self, cr, uid, vals, context=None, check=True):
        if not context:
            context={}
            
        maquina = self.pool.get('maquina.fiscal')
        
        if vals.get('maquina_fiscal_id', False):
            vals['address_id'] = maquina.browse(cr, uid, vals['maquina_fiscal_id']).direccion_asoc_id.id

        return super(sisb_report_z, self).create(cr, uid, vals, context)
        
    
    def write(self, cr, uid, ids, vals, context=None):
        if not context:
            context={}
            
        maquina = self.pool.get('maquina.fiscal')
        
        if vals.get('maquina_fiscal_id', False):
            vals['address_id'] = maquina.browse(cr, uid, vals['maquina_fiscal_id']).direccion_asoc_id.id

        return super(sisb_report_z, self).write(cr, uid, ids, vals, context)

sisb_report_z()
