# -*- encoding: utf-8 -*-
###############Credits#########################################################	
#    Finance by: Industrias Diana, C.A. http://www.industriasdiana.gob.ve	
###############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
from osv import osv, fields
import decimal_precision as dp
import netsvc
import pooler
from tools.sql import drop_view_if_exists

class fiscal_reports_sale(osv.osv):
    '''
    Modifying the object fiscal.reports.sales
    '''
    _inherit = "fiscal.reports.sale"

    _columns = {
        'ai_date_document': fields.date('Date'),
        'ai_date_invoice': fields.date('Date'),
        'ar_date_document': fields.date('Date Account Retencion'),
        'rp_vat':fields.char('VAT Number', size=64, required=False, readonly=False),
        'rp_id':fields.many2one('res.partner', 'Partner Name', required=True),
        'ai_reference':fields.char('Invoice Number', size=64, required=False, readonly=False),
        'ai_nro_ctrl':fields.char('Control No.', size=64, required=False, readonly=False),
        'ai_amount_total': fields.float('Amount Total', digits_compute= dp.get_precision('Fiscal Report')),
        'ai_amount_untaxed': fields.float('Untaxed amount', digits_compute= dp.get_precision('Fiscal Report')),
        'ai_amount_tax': fields.float('Tax Amount', digits_compute= dp.get_precision('Fiscal Report')),
        'ai_type':fields.char('Type', size=64, required=False, readonly=False),
        'rp_retention': fields.float('Withholding', digits_compute= dp.get_precision('Fiscal Report')),
        'ai_id':fields.many2one('account.invoice', 'Invoice Description', required=False),
        'ar_line_id':fields.many2one('account.wh.iva.line', 'Account Retention', readonly=True),
        'ar_id':fields.many2one('account.wh.iva', 'Account Retention', readonly=True),
        'report_type':fields.selection([('invoice', 'Invoice'),('report','Report Z')], 'Report Type', readonly=True),
    }
    
    def init(self, cr):
        '''
        Create or replace view fiscal_reports_sale
        '''   
        drop_view_if_exists(cr, 'fiscal_reports_sale') 
        cr.execute("""
            create or replace view fiscal_reports_sale as (
                SELECT row_number() OVER(ORDER BY total.ai_date_document) as id, *
                FROM (
                    SELECT
                        ai."date_document" AS ai_date_document,
                        ai."date_invoice" AS ai_date_invoice,
                        rp."vat" AS rp_vat,
                        rp."id" AS rp_id,
                        ai."number" AS ai_reference,
                        ai."nro_ctrl" AS ai_nro_ctrl,
                        ar."date_ret" AS ar_date_document,
                        case when ai.type='out_refund'
                        then
                            ai."amount_total"*(-1)
                        else
                            ai."amount_total" 
                        end as ai_amount_total,
                        case when ai.type='out_refund'
                        then
                            ai."amount_untaxed"*(-1)
                        else
                            ai."amount_untaxed" 
                        end as ai_amount_untaxed,
                        case when ai.type='out_refund'
                        then
                            ai."amount_tax"*(-1)
                        else
                            ai."amount_tax" 
                        end as ai_amount_tax,
                        ai."type" AS ai_type,
                        rp."wh_iva_rate" AS rp_retention,
                        ai."id" AS ai_id,
                        ar_line."id" AS ar_line_id,
                        ar_line."retention_id" AS ar_id,
                        'invoice' AS report_type
                    FROM
                        "res_partner" rp 
                        INNER JOIN "account_invoice" ai ON rp."id" = ai."partner_id"
                        LEFT JOIN "account_wh_iva_line" ar_line ON ar_line."invoice_id" = ai."id"
                        LEFT JOIN "account_wh_iva" ar ON ar_line."retention_id" = ar."id"
                    WHERE
                        (ai.type = 'out_refund'
                        OR ai.type = 'out_invoice')
                        AND (ai.state = 'open'
                        OR ai.state = 'paid'
                        OR ai.state = 'done')
                    UNION
                    SELECT 
                        rz."date_report"::date AS ai_date_document,
                        rz."date_report"::date AS ai_date_invoice,
                        rp."vat" AS rp_vat,
                        rp."id" AS rp_id,
                        case when rzt."type"='cr'
                        then
                            rz."pri_invoice_credit"||'..'||rz."ult_invoice_credit"
                        else
                            rz."pri_invoice"||'..'||rz."ult_invoice"
                        end AS ai_reference,
                        rz."name" AS ai_nro_ctrl,
                        Null AS ar_date_document,
                        sum(case when rzt."type"='cr'
                        then
                            (rzt."amount_bi_tax"+rzt."amount_tax")*(-1)
                        else
                            rzt."amount_bi_tax"+rzt."amount_tax" 
                        end) AS ai_amount_total,
                        sum(case when rzt."type"='cr'
                        then
                            rzt."amount_bi_tax"*(-1)
                        else
                            rzt."amount_bi_tax" 
                        end) AS ai_amount_untaxed,
                        sum(case when rzt."type"='cr'
                        then
                            rzt."amount_tax"*(-1)
                        else
                            rzt."amount_tax" 
                        end) AS ai_amount_tax,
                        rzt."type" AS ai_type,
                        rp."wh_iva_rate" AS rp_retention,
                        rz."id" AS ai_id,
                        Null AS ar_line_id,
                        Null AS ar_id,
                        'report' AS report_type
                    FROM 
                        sisb_report_z rz
                        INNER JOIN sisb_report_z_tax rzt ON rzt.report_z_id = rz.id
                        INNER JOIN maquina_fiscal mf ON rz.maquina_fiscal_id = mf.id
                        LEFT JOIN res_company rc ON rz.company_id = rc.id
                        LEFT JOIN res_partner rp ON rc.partner_id = rp.id
                    WHERE
                        rz.state = 'done'
                    GROUP BY rz."date_report",
                        rz."date_report",
                        rp."vat",
                        rp."id",
                        mf."name",
                        rz."name",
                        rzt."type",
                        rp."wh_iva_rate",
                        rz."id"
                    ) AS total
                )
        """)
        
fiscal_reports_sale()


class sales_book_wizard(osv.osv_memory):
    """
    Sales book wizard implemented using the osv_memory wizard system
    """
    _name = "sales.book.wizard"
    _inherit = "sales.book.wizard"

    def _print_report(self, cr, uid, ids, data, context=None):
        return { 'type': 'ir.actions.report.xml', 'report_name': 'fiscal.reports.sale.book.seniat', 'datas': data}
    
    
    def create_scheduled_report(self, cr, uid, ids, context=None):
        """
        Crea una accion planificada que ejecutara un metodo para generar
        el reporte y enviarlo por correo electronico. Se envia una notificacion
        interna para indicar al usuario que se generara el reporte. Solo se
        permite crear la accion planificada si existe un servidor de correos
        configurado y el usuario posee un email asociado.
        """
        
        self._check_email_server(cr, uid, ids, context)
        email = self._check_user_email_adress(cr, uid, ids, context)
        
        data = {}
        data['ids'] = [1]
        data['form'] = self.read(cr, uid, ids, ['date_start',  'date_end',  'control_start', 'control_end', 'type'])[0]    
        book_type = 'compras' if data['form']['type'] == 'purchase' else 'ventas'
        data['model'] = "fiscal.reports.sale.book.seniat"
        
        if self._valid_report(cr, uid, ids, book_type, data['form']['date_start'], data['form']['date_end'], context):
            subject = "Libro de %s SENIAT desde %s hasta %s" % (book_type, data['form']['date_start'], data['form']['date_end'])        
            request_message = "Se esta generando el reporte de libro de %s. Al finalizar se enviara a su direccion de correo '%s'" % (book_type, email)
            email_message = "Se adjunta el reporte de libro de %s" % (book_type)
            
            cron_parameters = dict()
            cron_parameters['data'] = data
            cron_parameters['subject'] = subject
            cron_parameters['message'] = email_message
            cron_parameters['file_name'] = subject
            cron_parameters['uid'] = uid
            vals = {
                'name': subject,
                'active': True,
                'user_id': uid,
                'interval_number': 1,
                'interval_type':'minutes',
                'numbercall': 1,
                'doall': True,
                'model': 'sales.book.wizard',
                'function': '_send_report_by_email',
                'args': '(' + str(cron_parameters) + ',)',
            }
            self.pool.get('ir.cron').create(cr, uid, vals, context)
            self._send_request(cr, uid, ids, subject, request_message, context)
            return {'type':'ir.actions.act_window_close'}
        else:
            raise osv.except_osv(_('Acción Denegada!'), _('No existen datos para el reporte en el rango de fechas seleccionado'))
    
sales_book_wizard()
