#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits########################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

'''
Fiscal Report with Report Z For Venezuela
'''

import time
from report import report_sxw
from osv import osv
import pooler

class pur_sal_book(report_sxw.rml_parse):
    '''
    Book generates purchase and sale
    '''

    def __init__(self, cr, uid, name, context):
        '''
        Reference to the current instance
        '''
        super(pur_sal_book, self).__init__(cr, uid, name, context)    
        self.localcontext.update({
            'time': time,
            'get_data':self._get_data,
            'get_partner_addr': self._get_partner_addr,
            'get_p_country': self._get_p_country,
            'get_rif': self._get_rif,
            'get_month':self._get_month,
            'get_doc':self._get_doc,
            'get_ret':self._get_ret,
            'get_data_adjustment': self._get_data_adjustment,
            'validation': self._validation,
            'get_data_wh': self._get_data_wh,
            'get_amount_withheld': self._get_amount_withheld,
            'get_date_wh': self._get_date_wh,
            'get_v_sdcf': self._get_v_sdcf,
            'get_tax_line': self._get_tax_line,
            'get_total_wh': self._get_total_wh,
            'get_total_iva': self._get_total_iva,
            'get_amount_untaxed_tax': self._get_amount_untaxed_tax,
            'get_taxes': self._get_taxes,
            'get_wh_actual': self._get_wh_actual,
            'get_id': self._get_id,
            'get_papel_anulado': self._get_papel_anulado,
            'get_nro_ctrl': self._get_nro_ctrl,
            'get_doc_afectado': self._get_doc_afectado,
        })
    
    
    def _get_nro_ctrl(self, l):
        #~ print "creado nuevo, _get_nro_ctrl",l
        if l.ai_type in ['inv','cr']:
            return l.ai_nro_ctrl
            
        return l.ai_id.nro_ctrl
    
    
    def _get_doc_afectado(self, l):
        #~ print "creado nuevo, _get_doc_afectado",l
        if l.ai_type in ['inv','cr']:
            #~ return l.ai_reference
            ref = ''
            return ref
        
        return l.ai_id.parent_id.number
            
    
    
    def _get_id(self,form,idh,type=None):
        x=1
        ids =None
        
        if form['type']=='sale':
            ids = self._get_data_wh(form)
        if form['type']=='sale' and type:
            x+=len(ids)
        if type=='book':
            ids = self._get_data(form)

        for a in ids:
            if a.id != idh:
                x+=1
            else:
                return x

    def _get_book_type(self,form):
        book_type=None
        book_type='fiscal.reports.sale'
        if form['type']=='purchase':
            book_type='fiscal.reports.purchase'
        
        return book_type

    def _get_book_type_wh(self,form):
        book_type=None
        book_type='fiscal.reports.whs'
        if form['type']=='purchase':
            book_type='fiscal.reports.whp'
        return book_type

    def _validation(self,form):
        type_doc = 'sale'
        if form['type']=='purchase':
            type_doc = 'purchase'
        period_obj=self.pool.get('account.period')
        adjust_obj = self.pool.get('adjustment.book')
        period_ids = period_obj.search(self.cr,self.uid,[('date_start','<=',form['date_start']),('date_stop','>=',form['date_end'])])
        if len(period_ids)<=0:
            return False
        fr_ids = adjust_obj.search(self.cr,self.uid,[('period_id','in',period_ids),('type','=',type_doc)])
        if len(fr_ids)<=0:
            return False
        return True
        
    def _get_data_adjustment(self,form):

        type_doc = 'sale'
        if form['type']=='purchase':
            type_doc = 'purchase'
        adjust_obj = self.pool.get('adjustment.book')
        adjust_line_obj = self.pool.get('adjustment.book.line')
        period_obj=self.pool.get('account.period')
        data=[]
        data_line=[]
        period_ids = period_obj.search(self.cr,self.uid,[('date_start','<=',form['date_start']),('date_stop','>=',form['date_end'])])
        
        if len(period_ids)>0:
            fr_ids = adjust_obj.search(self.cr,self.uid,[('period_id', 'in',period_ids),('type','=',type_doc)])
            if len(fr_ids)>0:
                adj_ids = adjust_line_obj.search(self.cr,self.uid,[('adjustment_id','=',fr_ids[0])],order='date_admin')
                data = adjust_obj.browse(self.cr,self.uid, fr_ids)
                data_line = adjust_line_obj.browse(self.cr,self.uid, adj_ids)
        return (data,data_line)

    def _get_date_wh(self,form, l):
        if l.ar_date_document> form['date_end']:
            return False
        return True

    def _get_papel_anulado(self,data,l,tipo=None,campo='rif'):
        #~ print "modificado, agregado variable campo, _get_papel_anulado"
        report_z_obj = self.pool.get('sisb.report.z')
        invoice_obj = self.pool.get('account.invoice')
        if l.ai_type in ['inv','cr']:
            # En este condicional se debe agregar los cambios para trabajar el reporte z
            # Se debe cambiar el campo ai_reference en el query, para colocar primera y ultima factura
            # Agregar otro campo para el id de la maquina fiscal o usar alguno de los existentes.
            report_z_brw = report_z_obj.browse(self.cr, self.uid, l.ai_id.id)
            if campo == 'name':
                return 'S/N: '+report_z_brw.maquina_fiscal_id.name+' Rep. Z: '+report_z_brw.name
                """elif campo == 'reference':
                    return 'algo'
                """
            else:
                return data
        else:
            invoice_brw = invoice_obj.browse(self.cr, self.uid, l.ai_id.id)
            if invoice_brw.name:
                if invoice_brw.name.find('PAPELANULADO')>=0: 
                    return tipo
                else: 
                    return data
            else:
                return data
            return tipo
        

    def _get_v_sdcf(self,l):
        #~ print "modificado, _get_v_sdcf"
        report_z_obj = self.pool.get('sisb.report.z')
        invoice_obj = self.pool.get('account.invoice')
        amount = 0.0
        if not l:
            return 0.0
        if l.ai_type in ['inv','cr']:
            #Agrega calculo de monto sin iva para reporte z
            report_z_brw = report_z_obj.browse(self.cr, self.uid, l.ai_id.id)
            type_report = []
            tipo = 1
            if l.ai_type == 'inv':
                type_report = report_z_brw.report_tax_ids
            elif l.ai_type == 'cr':
                type_report = report_z_brw.report_tax_nc_ids
                tipo = -1
                
            for tax in type_report:
                if tax.amount_tax == 0.0:
                    amount =+ tax.amount_bi_tax * tipo
        else:
            invoice_brw = invoice_obj.browse(self.cr, self.uid, l.ai_id.id)
            for tax in invoice_brw.tax_line:
                name=tax.name
                if name.find('SDCF')>=0:
                    amount = tax.base+amount
                    if invoice_brw.type in ['in_refund', 'out_refund']:
                        amount = amount * (-1)
        
        return (amount)


    def _get_tax_line(self,s):
        name = s.name
        cont = 0
        if name.find('SDCF')>=0:
            if cont==0:
                return 0
        else:
            cont = cont + 1
        return s.base_amount
    
    def _get_data(self,form):
        #~ print "_get_data",form
        data=[]
        #~ orden='ai_nro_ctrl'
        orden='ai_date_document'
        
        book_type = self._get_book_type(form)
        fr_obj = self.pool.get(book_type)
        
        if book_type=='fiscal.reports.purchase':
            orden='ai_date_document'
        fr_ids = fr_obj.search(self.cr,self.uid,[('ai_date_invoice', '<=', form['date_end']), ('ai_date_invoice', '>=', form['date_start'])], order=orden)
        
        if len(fr_ids)<=0:
            return False
        data = fr_obj.browse(self.cr,self.uid, fr_ids)
        return data

    def _get_data_wh(self,form):
        data=[]
        d1=form['date_start']
        d2=form['date_end']
        fr_obj = self.pool.get('fiscal.reports.whs')
        fr_ids = fr_obj.search(self.cr,self.uid,[('ar_date_ret', '<=', d2), ('ar_date_ret', '>=',d1),('ar_date_document','<=',d1)], order='ar_date_ret')
        data = fr_obj.browse(self.cr,self.uid, fr_ids)
        return data

    def _get_total_wh(self,form,actual=None):
        total=0
        data=[]
        book_type= self._get_book_type_wh(form)
        fr_obj = self.pool.get(book_type)
        
        fr_ids = fr_obj.search(self.cr,self.uid,[('ar_date_ret', '<=', form['date_end']), ('ar_date_ret', '>=', form['date_start'])])
        
        data = fr_obj.browse(self.cr,self.uid, fr_ids)
        for wh in data:
            if wh.ai_id.type in ['in_refund', 'out_refund']:
                total+= wh.ar_line_id.amount_tax_ret * (-1)
            else:
                total+= wh.ar_line_id.amount_tax_ret
        return total


    def _get_wh_actual(self,form):
        total=0
        data=[]
        book_type= self._get_book_type_wh(form)
        fr_obj = self.pool.get(book_type)
        
        fr_ids = fr_obj.search(self.cr,self.uid,[('ar_date_ret', '<=', form['date_end']),('ar_date_ret', '>=', form['date_start']),('ai_date_inv','>=',form['date_start']),('ai_date_inv','<=',form['date_end'])])
        data = fr_obj.browse(self.cr,self.uid, fr_ids)
        for wh in data:
            if wh.ai_id.type in ['in_refund', 'out_refund']:
                total+= wh.ar_line_id.amount_tax_ret * (-1)
            else:
                total+= wh.ar_line_id.amount_tax_ret
        return total



    def _get_ret(self,form,ret_id=None):
        '''
            Ensure that Withholding date is inside period specified on form.
        '''
        d1=form['date_start']
        d2=form['date_end']
        if form['type']=='purchase':
            if ret_id:
                ret_obj = self.pool.get('account.wh.iva')
                rets = ret_obj.browse(self.cr,self.uid,[ret_id])
                return rets[0].number
        if ret_id:
            ret_obj = self.pool.get('account.wh.iva')
            rets = ret_obj.browse(self.cr,self.uid,[ret_id])
            if rets:
                if time.strptime(rets[0].date, '%Y-%m-%d') >= time.strptime(d1, '%Y-%m-%d') \
                and time.strptime(rets[0].date, '%Y-%m-%d') <=  time.strptime(d2, '%Y-%m-%d'):
                    return rets[0].number
                else:
                    return False
            else:
                return False
        else:
            return False

    def _get_amount_withheld(self,wh_line_id):
        wh_obj = self.pool.get('account.wh.iva.line')
        data = wh_obj.browse(self.cr,self.uid, [wh_line_id])[0]
        return data.amount_tax_ret

    def _get_partner_addr(self, idp=None):
        '''
        Obtains the address of partner
        '''
        if not idp:
            return []

        addr_obj = self.pool.get('res.partner.address')
        addr_inv = 'NO HAY DIRECCION FISCAL DEFINIDA'
        addr_ids = addr_obj.search(self.cr,self.uid,[('partner_id','=',idp), ('type','=','invoice')])
        if addr_ids:
            addr = addr_obj.browse(self.cr,self.uid, addr_ids[0])
            addr_inv = (addr.street or '')+' '+(addr.street2 or '')+' '+(addr.zip or '')+ ' '+(addr.city or '')+ ' '+ (addr.country_id and addr.country_id.name or '')+ ', TELF.:'+(addr.phone or '')
        return addr_inv
    
    def _get_p_country(self, idp=None):
        '''
        Obtains the address of partner
        '''
        if not idp:
            return []

        addr_obj = self.pool.get('res.partner.address')
        a_id = 1000
        a_ids = addr_obj.search(self.cr,self.uid,[('partner_id','=',idp), ('type','=','invoice')])
        if a_ids:
            a = addr_obj.browse(self.cr,self.uid, a_ids[0])
            a_id = a.country_id.id
        return a_id 

    def _get_doc(self, l):
        '''
        Get String Document Type
        '''
        #~ print "modificado, _get_doc"
        inv_obj = self.pool.get('account.invoice')
        
        if l.ai_type in ['inv','cr']:
            if l.ai_type == 'inv':
                doc_type = "F"
            elif l.ai_type == 'cr':
                doc_type = "NC"
        else:
            inv_id = l.ai_id.id
            inv_ids = inv_obj.search(self.cr,self.uid,[('id', '=', inv_id)])
            inv = inv_obj.browse(self.cr,self.uid, inv_ids)[0]        
            doc_type = ''
            if (inv.type=="in_invoice" or inv.type=="out_invoice") and inv.parent_id:
                doc_type = "ND"
            elif (inv.type=="in_invoice" or inv.type=="in_refund") and inv.expedient:
                doc_type="E"
            elif inv.type=='in_refund' or inv.type=='out_refund':
                doc_type = "NC"
            elif inv.type=="in_invoice" or inv.type=="out_invoice":
                doc_type = "F"
        return doc_type

    def _get_rif(self, vat=''):
        '''
        Get R.I.F.
        '''
        if not vat:
            return []
        return vat[2:].replace(' ', '')

    def _get_month(self, form):
        '''
        return year and month
        '''
        months=["Enero","Febrero","Marzo","Abril","Mayo","Junio","Julio","Agosto","Septiembre","Octubre","Noviembre","Diciembre"]
        res = ['',0]
        res[0] = months[time.strptime(form['date_start'],"%Y-%m-%d")[1]-1]
        res[1] = time.strptime(form['date_start'],"%Y-%m-%d")[0]
        return res

    def _get_total_iva(self,form):
        '''
        Return Amount Total of each invoice at Withholding Vat
        '''
        book_type= self._get_book_type(form)
        
        sql =   ''' select sum(ai_amount_total) as total 
                    from %s 
                    where ai_date_invoice>= '%s' and ai_date_invoice<='%s' 
                ''' % (book_type.replace('.','_'),form['date_start'],form['date_end'])
        self.cr.execute(sql)
        
        res = self.cr.dictfetchone()
        return res['total']

    def _get_amount_untaxed_tax(self,form,percent,nationality=''):
        #~ print "_get_amount_untaxed_tax"
        '''
        Return Amount Untaxed and Amount Tax, accorded percent of withholding vat
        '''
        amount_untaxed=0.0
        amount_tax=0.0

        book_type = self._get_book_type(form)
    
        fr_obj = self.pool.get(book_type)
        user_obj = self.pool.get('res.users')
        report_z_obj = self.pool.get('sisb.report.z')
        invoice_obj = self.pool.get('account.invoice')
        
        #~ user_ids = user_obj.search(self.cr,self.uid,[('id', '=', self.uid)])
        fr_ids = fr_obj.search(self.cr,self.uid,[('ai_date_invoice', '<=', form['date_end']), ('ai_date_invoice', '>=', form['date_start'])])

        user=user_obj.browse(self.cr,self.uid, [self.uid])
        
        for d in fr_obj.browse(self.cr,self.uid, fr_ids):
            #~ print "D",d
            if d.ai_type in ['inv','cr']:
                if nationality=='nacional':
                    report_z_brw = report_z_obj.browse(self.cr, self.uid, d.ai_id.id)
                    #~ print "report_z_brw",report_z_brw
                    type_report = []
                    tipo = 1
                    if d.ai_type == 'inv':
                        type_report = report_z_brw.report_tax_ids
                    elif d.ai_type == 'cr':
                        type_report = report_z_brw.report_tax_nc_ids
                        tipo = -1
                    
                    for tax in type_report:
                        #~ print "tax",tax
                        #~ print "tax.name.name",tax.name.name
                        if percent in tax.name.name:
                            amount_untaxed += tax.amount_bi_tax * tipo
                            amount_tax += tax.amount_tax * tipo
            else:
                invoice_brw = invoice_obj.browse(self.cr, self.uid, d.ai_id.id)
                for tax in invoice_brw.tax_line:
                    if percent in tax.name:
                        if nationality=='nacional':
                            if self._get_p_country(user[0].company_id.partner_id.id)==self._get_p_country(invoice_brw.partner_id.id):
                                #~ amount_untaxed+= self._get_amount_untaxed_tax2(d.ai_id.type,tax,'invoice')[0]
                                #~ amount_tax+= self._get_amount_untaxed_tax2(d.ai_id.type,tax,'invoice')[1]
                                amount_untaxed+= self._get_amount_untaxed_tax2(invoice_brw.type,tax)[0]
                                amount_tax+= self._get_amount_untaxed_tax2(invoice_brw.type,tax)[1]
                        else:
                            if self._get_p_country(user[0].company_id.partner_id.id)!=self._get_p_country(invoice_brw.partner_id.id):
                                #~ amount_untaxed+= self._get_amount_untaxed_tax2(d.ai_id.type,tax,'invoice')[0]
                                #~ amount_tax+= self._get_amount_untaxed_tax2(d.ai_id.type,tax,'invoice')[1]
                                amount_untaxed+= self._get_amount_untaxed_tax2(invoice_brw.type,tax)[0]
                                amount_tax+= self._get_amount_untaxed_tax2(invoice_brw.type,tax)[1]
            
        #~ print "TOTAL amount_untaxed: %s, amount_tax: %s"%(amount_untaxed,amount_tax)
        return (amount_untaxed,amount_tax)


    #~ def _get_amount_untaxed_tax2(self,type,tax,formato):
    def _get_amount_untaxed_tax2(self,type,tax):
        amount_untaxed=0.0
        amount_tax=0.0
                
        if type in ['in_refund', 'out_refund']:
            amount_untaxed= tax.base * (-1)
            amount_tax= tax.amount * (-1)
        else:
            amount_untaxed= tax.base
            amount_tax= tax.amount
            
        return [amount_untaxed,amount_tax]


    def _get_taxes(self,l):
        #~ print "modificado, _get_taxes, en proceso"
        tax_obj = self.pool.get('account.invoice.tax')
        report_z_obj = self.pool.get('sisb.report.z')
        
        taxes = []
        if l.ai_type in ['inv','cr']:
            report_z_brw = report_z_obj.browse(self.cr, self.uid, l.ai_id.id)
            type_report = []
            tipo = 1
            if l.ai_type == 'inv':
                type_report = report_z_brw.report_tax_ids
            elif l.ai_type == 'cr':
                type_report = report_z_brw.report_tax_nc_ids
                tipo = -1
                
            for tax in type_report:
                if tax.amount_tax > 0.0:
                    dict_tax = {
                        #~ 'partner_id': l.l.rp_id.id,
                        'base': tax.amount_bi_tax * tipo,
                        'porcentaje': round(tax.name.amount * 100.0),
                        'impuesto': tax.amount_tax * tipo,
                    }
                    taxes.append(dict_tax)
                    
            if not taxes:
                dict_tax = {
                    'base': 0.0,
                    'porcentaje': 0.0,
                    'impuesto': 0.0,
                }
                taxes.append(dict_tax)
                
            return taxes
            
        else:
            tax_ids = tax_obj.search(self.cr,self.uid,[('invoice_id', '=', l.ai_id.id)])
            
            tam = len(tax_ids)
            data = tax_obj.browse(self.cr,self.uid, tax_ids)
            
            for tax in data:
                dict_tax = {}
                if 'SDCF' in tax.name and tam>=2:
                    tax_ids.remove(tax.id)
                else:
                    tipo = 1
                    if tax.invoice_id.type in ('in_refund', 'out_refund'):
                        tipo = -1
                    dict_tax = {
                        #~ 'partner_id': l.l.rp_id.id,
                        'base': self._get_tax_line(tax) * tipo,
                        'porcentaje': round(tax.tax_amount/(tax.base_amount and tax.base_amount or 1)*100.0),
                        'impuesto': tax.tax_amount * tipo,
                    }
                    taxes.append(dict_tax)
            
            data = tax_obj.browse(self.cr,self.uid, tax_ids)
            if len(data)<=0:
                return False
            return taxes

report_sxw.report_sxw(
    'report.fiscal.reports.sale.book.seniat',
    'fiscal.reports.purchase',
    'sisb_reporte_z/report/book_seniat.rml',
    parser=pur_sal_book,
    header=False
)      
