#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############Credits########################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

import time
from report import report_sxw
from osv import osv
import pooler
from datetime import datetime, timedelta
import datetime as dt

class report_z_parser(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(report_z_parser, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
            'time': time,
            'user': self.pool.get('res.users').browse(cr, uid, uid, context),
            'get_empresa': self.get_empresa,
            'get_final_date': self.get_final_date,
            'get_fecha': self.get_fecha,
            'get_maquinas': self.get_maquinas,
            'get_lines_detal': self.get_lines_detal,
            'get_total_impuesto': self.get_total_impuesto,
            'get_name': self.get_name,
            'get_total_detal': self.get_total_detal,
        })

        
    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res
        
    
    def get_final_date(self):
        return datetime.now().strftime('%d-%m-%Y')
    
    
    def get_fecha(self, date):
        if str(date) == '':
            return False
        new = datetime.strptime(date, '%Y-%m-%d %H:%M:%S')
        return str(new.strftime('%d-%m-%Y'))
    
    
    def get_total_impuesto(self, data):
        result_impuesto = []
        ids = data['ids']
        ids_sql = str(ids).replace("[","(").replace("]",")")
        sql_impuesto = """
            SELECT atax.name AS tipo_impuesto,
                srzt.type AS document,
                SUM(srzt.amount_bi_tax+srzt.amount_tax) AS total,
                SUM(srzt.amount_tax) AS impuesto,
                SUM(srzt.amount_bi_tax) AS base
            FROM sisb_report_z_tax AS srzt
            INNER JOIN account_tax atax ON atax.id = srzt.name
            WHERE srzt.report_z_id in %s
            GROUP BY atax.name, srzt.type
            ORDER BY srzt.type desc, atax.name
        """%(ids_sql)
        self.cr.execute(sql_impuesto)
        result_impuesto = self.cr.dictfetchall()
        #~ print "impuesto",result_impuesto
        #~ return result_impuesto

    
    def get_maquinas(self, data):
        result_maquina = []
        ids = data['ids']
        ids_sql = str(ids).replace("[","(").replace("]",")")
        report_z = self.pool.get('sisb.report.z')
        repo_brw = report_z.browse(self.cr, self.uid, ids)
        sql_maquinas = """
            SELECT mf.id as name
            FROM sisb_report_z srz
            INNER JOIN maquina_fiscal mf ON mf.id = srz.maquina_fiscal_id
            WHERE srz.id in %s
            GROUP BY mf.id
            ORDER BY mf.name, mf.direccion_asoc_id
        """%(ids_sql)
        self.cr.execute(sql_maquinas)
        result_maquina = self.cr.dictfetchall()
        list_fin = []
        for detal in result_maquina:
            dict_detal = {}
            dict_detal[detal['name']] = {}
            inv, impuesto, base, total = self.get_lines_detal(detal,'inv',ids_sql)
            if inv:
                dict_detal[detal['name']]['inv'] = inv
                dict_detal[detal['name']]['impuesto_inv'] = impuesto
                dict_detal[detal['name']]['total_inv'] = total
                dict_detal[detal['name']]['base_inv'] = base
            cr, impuesto, base, total = self.get_lines_detal(detal,'cr',ids_sql)
            if cr:
                dict_detal[detal['name']]['cr'] = cr
                dict_detal[detal['name']]['impuesto_cr'] = impuesto
                dict_detal[detal['name']]['total_cr'] = total
                dict_detal[detal['name']]['base_cr'] = base
            list_fin.append(dict_detal)
        #~ print "Detal",list_fin
        #~ print "----------------------------------"
        self.get_total_impuesto(data)
        return list_fin
    
    
    def get_lines_detal(self, maquina, tipo, ids_sql):
        result = []
        sql_detalle = """
            SELECT atax.name AS tipo_impuesto,
                SUM(srzt.amount_bi_tax+srzt.amount_tax) AS total,
                SUM(srzt.amount_tax) AS impuesto,
                SUM(srzt.amount_bi_tax) AS base
            FROM sisb_report_z_tax AS srzt
            INNER JOIN sisb_report_z srz ON srzt.report_z_id = srz.id
            INNER JOIN maquina_fiscal mf ON mf.id = srz.maquina_fiscal_id
            INNER JOIN account_tax atax ON atax.id = srzt.name
            WHERE mf.id = %s AND srzt.type = '%s' and srz.id in %s
            GROUP BY atax.name
        """%(maquina['name'],tipo, ids_sql)
        
        self.cr.execute(sql_detalle)
        result = self.cr.dictfetchall()
        total_imp = 0
        total_total = 0
        total_base = 0
        for tol in result:
            #~ print tol
            #total_imp = total_imp + (tol['impuesto'] or 0)
            total_imp = total_imp + tol['impuesto']
            #total_total = total_total + (tol['total'] or 0)
            total_total = total_total + tol['total']
            #total_base = total_base + (tol['base'] or 0)
            total_base = total_base + tol['base']
        return result, total_imp, total_base, total_total
        #~ return result
        
        
    def get_name(self, id):
        maquina = self.pool.get('maquina.fiscal')
        return maquina.browse(self.cr, self.uid, id).name
    
    
    def get_total_detal(self, repo, string):
        #~ print repo
        #~ print repo.keys()
        #~ print string
        return repo[repo.keys()[0]][string]
        #~ return string
    
        
report_sxw.report_sxw('report.sisb.report.list.month',
    'sisb.report.z',
    'sisb_reporte_z/report/list_month.rml',
    parser=report_z_parser)
