# -*- coding: utf-8 -*-


import time
from report import report_sxw
import datetime
from datetime import datetime


class analisis_factura_report_impagada(report_sxw.rml_parse):
    
    _name = 'analisis.facturas.report.impagada'
    
    def __init__(self, cr, uid, name, context=None):
        super(analisis_factura_report_impagada, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
        'get_empresa':self.get_empresa,
        'get_final_date':self.get_final_date,
        'cvr_date':self.cvr_date,
        'get_total':self.get_total,
        'get_subtotal':self.get_subtotal,
        'get_impuesto':self.get_impuesto,
        'get_pendiente':self.get_pendiente,
        'sumador_parcial':self.sumador_parcial,
        'get_fecha':self.get_fecha,
        'generate_query':self.generate_query,})
        self.context = context
        self.cr = cr
        self.uid = uid
        self.sub_total = 0.0
        self.pendiente = 0.0        
        self.impuesto = 0.0
        self.total = 0.0
        self.salida_query = []
    
    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res
    
    def get_final_date(self):
        return datetime.now().strftime('%d/%m/%Y %H:%M')    
    
    def cvr_date(self, fecha):
        d = datetime.datetime.strptime(fecha, '%Y-%m-%d').strftime('%Y/%m/%d')
        return d 
    
    def generate_query(self, value):
        """
        Funcion llamada en los rml para generar la consulta dependiendo
        del tipo de opcion que traiga la variable type
        parametros value: contiene el diccionario que carga 
        todas las variables necesarias utilizadas en las funciones   
        retorna una funcion dependiendo del tipo de reporte
        """
        period_id = value['form']['period_id']
        journal_id = value['form']['journal_id']
        user_id = value['form']['user_id']
        journal_access = [int(i) for i in value['form']['journal_access']]
        return self.report_sale_diario(period_id, journal_id, user_id)
                
    def report_sale_diario(self, period_id, journal_id, user_id):
        """
        Funcion que genera el reporte detallado de facturas impagadas por diario, 
        esta funcion ejecuta un query que es modelado dependiendo de las opciones
        seleccionadas
        @param period_id: variable que posee el id del periodo seleccionado 
        @param journal_id: variable que posee el id del diario seleccionado 
        @return [{id_factura:{
                ('cliente','referencias','nro_documento',
                'fecha_de_documento','fecha_origen','origen'):
                 [('descripcion_producto','cuenta','cantidad',
                 'unidad','impuesto','precio_uni','descuento',                                          
                 'sub_total','categoria')]
                }}] : retorna una lista con un diccionario con los que 
                     indican los campos del rml  
        """
        sql = """
                SELECT invoice.id, partner.name, invoice.reference, invoice.number,
                invoice.date_document, invoice.date_invoice, invoice.origin, 
                invoice.amount_total, invoice.amount_tax, invoice.residual, users.name 
                FROM account_invoice as invoice
                INNER JOIN res_partner as partner ON (invoice.partner_id = partner.id)
                LEFT JOIN sale_order_invoice_rel as sale_rel ON (invoice.id=sale_rel.invoice_id)
                LEFT JOIN sale_order as sale ON (sale_rel.order_id=sale.id)
                LEFT JOIN res_users as users ON (invoice.user_id=users.id)
                WHERE invoice.state IN ('open')
                and (invoice.nro_ctrl != 'saldo elite' or invoice.nro_ctrl is null)
            """
        
        sql = sql + "\n and invoice.type = 'out_invoice' "   
        sql = sql + '\nand invoice.journal_id = %s'%journal_id
        
        if user_id:
            sql = sql + '\nand invoice.user_id = %s'%user_id
        
        sql = sql + '\n and invoice.period_id = %s'%period_id 
        sql = sql + "\nGROUP BY 1,2,3,4,5,6,7,8,9,10,11\n ORDER BY 1, 5;" 
 
        agrupador = {}
        self.cr.execute(sql)
        consulta = self.cr.fetchall()
        
        for i in consulta:
            agrupador[i[10]] = []

        for i in consulta:
            agrupador[i[10]].append(i[1:10])
            self.total += i[7]
            self.impuesto += i[8]
            self.pendiente += i[9]

        return [agrupador]
    
    def get_total(self):
        return self.total
    
    def get_subtotal(self):
        return self.sub_total
        
    def get_impuesto(self):
        return self.impuesto
      
    def get_pendiente(self):
        return self.pendiente
         
    def sumador_parcial(self, k, data):
        tipo = {'subtotal':0.0, 'impuesto':0.0, 'pendiente':0.0}
        for i in data[k]:
            tipo['subtotal'] +=i[6]
            tipo['impuesto'] +=i[7]
            tipo['pendiente'] +=i[8]
        return tipo

    def cvr_fecha(self, f):
        d = datetime.strptime(f, '%Y-%m-%d').strftime('%d/%m/%Y')
        return d 
        
    def list_2_typle(self, list):
            list = [str(i) for i in list]    
            t = ()
            if len(list) == 1:
               t = "(%s)"%"".join(list)
            else:
                t = tuple(list)
            return t    

    def get_fecha(self, value):
        salida = ''
        if value['is_periodo']:
           salida = "Periodo: %s" %value['periodo']
        else:
            salida = 'Desde: %s Hasta: %s'%(self.cvr_fecha(value['fecha_desde']), self.cvr_fecha(value['fecha_hasta']))
        return salida 
    
report_sxw = report_sxw.report_sxw    
class RMLParser(report_sxw):
    """Custom class that use webkit to render HTML reports
       Code partially taken from report openoffice. Thanks guys :)
    """
    def __init__(self, name, table, rml=False, parser=False, header=True, store=False):
        self.parser_instance = False
        self.localcontext={}
        report_sxw.__init__(self, name, table, rml, parser,
            header, store)
       
    
    
    def create_single_pdf(self, cursor, uid, ids, data, report_xml, context=None):
        """generate the PDF"""
        
        if context is None:
            context={}
        
        if report_xml.report_type == 'pdf':
            return super(RMLParser,self).create_single_pdf(cursor, uid, ids, data, report_xml, context=context)


RMLParser('report.analisis.facturas.impagadas',
          'wizard.analisis.factura.impagada', 
          'sisb_invoice_report/report/analisis_factura_report_impagada.rml',
          parser=analisis_factura_report_impagada)
