# -*- coding: utf-8 -*-


import time
from report import report_sxw
import datetime
from datetime import datetime


class analisis_origen_asiento_report(report_sxw.rml_parse):
    
    _name = 'analisis.origen.asiento.report'
    
    def __init__(self, cr, uid, name, context=None):
        super(analisis_origen_asiento_report, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
        'generate_query':self.generate_query,
        'producto_detalle':self.producto_detalle,
        'get_empresa':self.get_empresa,
        'get_fecha':self.get_fecha,
        'titulos_rml':self.titulos_rml,
        'sumar':self.sumar,
        'get_totales':self.get_totales,
        })
        self.context = context
        self.cr = cr
        self.uid = uid
        self.sub_total = 0.0
        self.impuesto = 0.0
        self.total = 0.0
        self.salida_query = []
        self.totales = {}
        self.producto = {}
        
    
    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res
    
    def generate_query(self, value):
        value = value['form']
        account = value['account_id']
        all_account = value['all_account']
        is_periodo = value['is_periodo']
        period = value['period_id']
        fecha_desde = value['fecha_desde']
        fecha_hasta = value['fecha_hasta']
        product = value['product_id']
        all_product = value['all_product']
        formato = 'detail'
        salida = self.detallado_productos_origen_asiento(account, all_account, is_periodo, period, fecha_desde, fecha_hasta, product, all_product)
        return [salida]
   
    def filter_resumido_detallado_sql(self, sql, account, all_account, is_periodo, period, fecha_desde, fecha_hasta, product, all_product):
        if not all_account:
           sql = sql + '\nAND (aml.account_id = %s)'%(account)		
        if not all_product:
           sql = sql + '\nAND m.product_id = %s'%product
        if is_periodo:
           sql = sql + """\nand aml.date_created :: date >= (SELECT date_start FROM account_period WHERE id = %s) 
                            \nand aml.date_created :: date <= (SELECT date_stop FROM account_period WHERE id = %s)"""%(period,period)
        else:
            sql = sql + "\n AND aml.date_created :: date >= '%s' and aml.date_created :: date <= '%s'" %(fecha_desde, fecha_hasta)
        sql += 'ORDER BY aml.ref, m.id'
        return sql 
        
    def producto_detalle(self, k):
        producto_detail = self.producto[k]
        return producto_detail
            
    def detallado_productos_origen_asiento(self,  account, all_account, is_periodo, period, fecha_desde, fecha_hasta, product, all_product):
        
        move_obj = self.pool.get('stock.move')
        context = {}
        agrupador = {}
        valorado = {}
        states = {'done':'Realizado'}
        total = 0.0        
        sql = """
        SELECT DISTINCT aml.ref, m.id
        FROM account_move_line as aml
        INNER JOIN stock_picking as p ON p.name = aml.ref 
        INNER JOIN stock_move as m ON m.picking_id = p.id AND m.product_id = aml.product_id       
        WHERE m.state = 'done' """       
        sql = self.filter_resumido_detallado_sql(sql, account, all_account, is_periodo, period, fecha_desde, fecha_hasta, product, all_product)
        salida = self.execute_sql(sql) 
        for i in salida:
            sql = """
            SELECT  m.date, l.name as origen, l2.name as destino, 
            CASE WHEN t_unidad_deter.name_puom is null THEN 
            pu.name ELSE t_unidad_deter.name_puom 
                END as name_unidad,
            CASE WHEN m.product_uom != pt.uom_id THEN
                COALESCE(round((m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3), 0.0)
            ELSE COALESCE(m.product_qty, 0.0)
            END as cant_convertida,
            CASE WHEN m.product_uom != pt.uom_id THEN
                COALESCE(round((m.price_unit :: numeric/pu.factor :: numeric) * u.factor :: numeric ,5), 0.0)
            ELSE COALESCE(m.price_unit, 0.0)
            END as price_unit, 
            m.product_id,
                pt.name, 
                pp.default_code,
            m.product_qty as qty_orig   
            FROM stock_move as m 
            INNER JOIN stock_picking as p ON m.picking_id = p.id
                INNER JOIN stock_location l ON m.location_id = l.id
                INNER JOIN stock_location l2 ON m.location_dest_id = l2.id 
            INNER JOIN (select distinct p.id, p.product_tmpl_id, p.default_code from product_product p inner join product_template pt on p.product_tmpl_id = pt.id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_location_rel dcr on dcr.category_id =slp.id
                inner join diana_groups_location_rel dlr on dlr.location_id = dcr.location_id
                where dlr.groups_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld on pt.categ_id = tbld.child_id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld2 on pt.categ_id = tbld2.child_id ) pp ON m.product_id = pp.id
			INNER JOIN product_template pt ON pp.product_tmpl_id = pt.id
			INNER JOIN product_uom pu ON pt.uom_id = pu.id
			INNER JOIN product_uom u ON m.product_uom = u.id
			LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom 
			FROM ir_translation it
			INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
			WHERE it.name = 'product.uom,name' ) as t_unidad_deter ON pu.id = t_unidad_deter.p_uom_id         
			WHERE m.id = %s 
            """%(self.uid, self.uid, i[1])

            move_sql = self.execute_sql(sql)
            total = 0.0
            if len (move_sql) > 0:
                move_sql_line = move_sql[0]
                if move_sql_line[5] is not None:
                    total = move_sql_line[4]*move_sql_line[5]
                valorado = {'fecha':move_sql_line[0][0:10], 'ref':i[0], 'ubicacion_origen': move_sql_line[1], 'ubicacion_destino': move_sql_line[2],
                            'udm':move_sql_line[3], 
                            'cantidad':move_sql_line[4], 
                            'costo_unit':move_sql_line[5], 'total': total}
                move = move_obj.browse(self.cr, self.uid, i[1], context)
                journal_id, acc_src, acc_dest, acc_variation = move_obj._get_accounting_data_for_valuation(self.cr, self.uid, move, context)
                
                #~ print 'Ref', i[0]
                #~ print 'Cantidad', move_sql_line[4]
                #~ print 'Producto', move_sql_line[6]
                #~ print 'Cuenta Origen:', acc_src
                #~ print 'Cuenta Destino:', acc_dest 
                #~ print 'Cuenta Variacion:', acc_variation
                #~ print '---------------------------------'
                
                # Outgoing moves (or cross-company output part)
                if move.location_id.company_id \
                    and (move.location_id.usage == 'internal' and move.location_dest_id.usage != 'internal'\
                         or move.location_id.company_id != move.location_dest_id.company_id):
                             valorado.update(self.detallado_movimiento_contable(i[0], move_sql_line[6], move_sql_line[9], acc_variation, acc_dest, account, all_account))        
                
                
                # Incoming moves (or cross-company input part)
                if move.location_dest_id.company_id \
                    and (move.location_id.usage != 'internal' and move.location_dest_id.usage == 'internal'\
                         or move.location_id.company_id != move.location_dest_id.company_id):
                             valorado.update(self.detallado_movimiento_contable(i[0], move_sql_line[6], move_sql_line[9], acc_src, acc_variation, account, all_account))        
                
                if move_sql_line[6] not in self.producto:
                    self.producto[move_sql_line[6]] = {'name':move_sql_line[7], 'codigo':move_sql_line[8]}            
                
                if move_sql_line[6] not in agrupador:
                    agrupador[move_sql_line[6]] = []
                agrupador[move_sql_line[6]].append(valorado)
                
                if valorado['cta_debito_name'] not in self.totales and valorado['cta_debito_name'] is not None:
                    self.totales[valorado['cta_debito_name']] = {'debito':0.0, 'credito':0.0}
                if valorado['cta_credito_name'] not in self.totales and valorado['cta_credito_name'] is not None:
                    self.totales[valorado['cta_credito_name']] = {'debito':0.0, 'credito':0.0}
                            
                if valorado['cta_credito_name'] in self.totales and valorado['cta_credito_name'] is not None:   
                    self.totales[valorado['cta_credito_name']]['credito'] += valorado['credito']
                if valorado['cta_debito_name'] in self.totales and valorado['cta_debito_name'] is not None:      
                    self.totales[valorado['cta_debito_name']]['debito'] += valorado['debito']
        return agrupador
        
    def detallado_movimiento_contable(self, ref, product, quantity, origin, destin,  account, all_account):
        sql = """
        SELECT aa.code as cta_debito, aml.debit as debito, aa_aux.code as cta_credito, aml_aux.credit as credito, aa.name, aa_aux.name,
        aa.id, aa_aux.id 
        FROM account_move_line as aml
        INNER JOIN account_move_line aml_aux ON aml.move_id = aml_aux.move_id
        INNER JOIN account_account as aa ON aa.id = aml.account_id
        INNER JOIN account_account as aa_aux ON aa_aux.id = aml_aux.account_id
        WHERE aml_aux.credit is not null AND aml.debit is not null AND aml.ref = '%s' AND aml.product_id = %s
        AND aml.quantity = %s AND aml.account_id = %s      
        """ %(ref, product, quantity, destin)
        salida = self.execute_sql(sql)
        if not salida:
            sql = """
            SELECT aa.code as cta_debito, aml.debit as debito, aa_aux.code as cta_credito, aml_aux.credit as credito, aa.name, aa_aux.name,
            aa.id, aa_aux.id 
            FROM account_move_line as aml
            INNER JOIN account_move_line aml_aux ON aml.move_id = aml_aux.move_id
            INNER JOIN account_account as aa ON aa.id = aml.account_id
            INNER JOIN account_account as aa_aux ON aa_aux.id = aml_aux.account_id
            WHERE aml_aux.credit is not null AND aml.debit is not null AND aml.ref = '%s' AND aml.product_id = %s
            AND aml.quantity = %s AND aml_aux.account_id = %s      
            """ %(ref, product, quantity, origin)
            salida = self.execute_sql(sql)
            
        valor = {'cta_debito':None, 'debito':0.0, 'cta_credito':None, 'credito':0.0, 'cta_debito_name':None, 'cta_credito_name':None}
        for i in salida:
            valor['cta_debito'] = self.cuenta_excluida(i[0], i[6], account, all_account)
            valor['debito'] = self.cuenta_excluida_monto(i[1], i[6], account, all_account)
            valor['cta_credito'] = self.cuenta_excluida(i[2], i[7], account, all_account)
            valor['credito'] = self.cuenta_excluida_monto(i[3], i[7], account, all_account)
            valor['cta_debito_name'] = self.cuenta_excluida(i[0] + ' - ' + i[4], i[6], account, all_account)
            valor['cta_credito_name'] = self.cuenta_excluida( i[2] + ' - ' + i[5], i[7], account, all_account)
        
        return valor         
                             
    def execute_sql(self, sql):
        self.cr.execute(sql)
        consulta = self.cr.fetchall()
        return consulta
		
    def cvr_fecha(self, f):
        d = datetime.strptime(f, '%Y-%m-%d').strftime('%d/%m/%Y')
        return d 
        
    def list_2_typle(self, list):
            list = [str(i) for i in list]    
            t = ()
            if len(list) == 1:
               t = "(%s)"%"".join(list)
            else:
                t = tuple(list)
            return t   
    
    def get_final_date(self):
        return datetime.now().strftime('%d/%m/%Y %H:%M')    
    
    def cvr_date(self, fecha):
        d = datetime.datetime.strptime(fecha, '%Y-%m-%d').strftime('%Y/%m/%d')
        return d 
    
    def titulos_rml(self, data):
        d = {'detail':'DETALLADO',
            'resume':'RESUMIDO',
            'total':'TOTAL'
            }
        return d[data['form']['formato']]

    def get_fecha(self, value):
        salida = ''
        if value['is_periodo']:
           salida = "Periodo: %s" %value['periodo_name']
        else:
            salida = 'Desde: %s Hasta: %s'%(self.cvr_fecha(value['fecha_desde']), self.cvr_fecha(value['fecha_hasta']))
        return salida
    
    def sumar(self, k, d):
        calculo = {'total':0.00, 'debito':0.00, 'credito':0.00}
        for i in d[k]:
            calculo['total'] += i['total']
            calculo['debito'] += i['debito']
            calculo['credito'] += i['credito']                           
        return calculo
            
    def get_totales(self):
        return [self.totales]
        
    def cuenta_excluida(self, code, valor, account_id, all_account):
        valor_aux = code
        if not all_account:
            if valor != account_id:
                valor_aux = None
        return valor_aux
        
    def cuenta_excluida_monto(self, monto, valor, account_id, all_account):
        valor_aux = monto
        if not all_account:
            if valor != account_id:
                valor_aux = 0.0
        return valor_aux        
           
report_sxw = report_sxw.report_sxw    
class RMLParser(report_sxw):
    """Custom class that use webkit to render HTML reports
       Code partially taken from report openoffice. Thanks guys :)
    """
    def __init__(self, name, table, rml=False, parser=False, header=True, store=False):
        self.parser_instance = False
        self.localcontext={}
        report_sxw.__init__(self, name, table, rml, parser,
            header, store)    
    
    def create_single_pdf(self, cursor, uid, ids, data, report_xml, context=None):
        """generate the PDF"""
        
        if context is None:
            context={}
        
        if report_xml.report_type == 'pdf':
            return super(RMLParser,self).create_single_pdf(cursor, uid, ids, data, report_xml, context=context)


RMLParser('report.analisis.origen.asiento',
          'wizard.analisis.origen.asiento', 
          'sisb_invoice_report/report/analisis_origen_asiento.rml',
          parser=analisis_origen_asiento_report)
