#!/usr/bin/python 
# -*- encoding: utf-8 -*- 
 
from datetime import datetime
from report import report_sxw 
import time 

class report_inventario_producto_terminado(report_sxw.rml_parse): 
        
        def __init__(self, cr, uid, name, context): 
            super(report_inventario_producto_terminado, self).__init__(cr, uid, name, context) 
            self.localcontext.update({ 
              'time': time, 
              'user': self.pool.get('res.users').browse(cr, uid, uid, context),  
              'get_empresa': self.get_empresa, 
              'get_final_date': self.get_final_date, 
              'get_fecha': self.get_fecha, 
              'get_hour_date': self.get_hour_date, 
              'generar_data': self.generar_data, 
              'child_cat': self.child_cat, 
              'obtener_data': self.obtener_data, 
              'obtener_total': self.obtener_total, 
              'obtener_fecha': self.obtener_fecha, 
              'contempla_cero': self.contempla_cero, 
              'obtener_titulo_productos': self.obtener_titulo_productos, 
            }) 
            
            self.salida = []
            
            self.existencia = {}
            self.producto = {}
            self.dicc = {}
            self.total = {}
            
            self.fin = {}
        
        def get_empresa(self, partner_id): 
            obj_addr = self.pool.get('res.partner.address') 
            addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)]) 
            res = {} 
            for row in obj_addr.browse(self.cr, self.uid, addr_id): 
              res = { 
                  'street':row.street, 
                  'phone':row.phone, 
                  'fax':row.fax, 
                  'email':row.email, 
                  'city':row.city, 
                  'name':row.name, 
                  'country':row.country_id.name, 
               } 

            return res 

        def get_hour_date(self): 
            current_dt = datetime.now() 
            a = ['AM','PM'][current_dt.hour>=12] 
            return datetime.now().strftime('%H:%M:%S') +  + a  

        def get_final_date(self): 
            return datetime.now().strftime('%d-%m-%Y') 

        def get_fecha(self, date): 
            if str(date) == '': 
              return False 
            new = datetime.strptime(date, '%Y-%m-%d') 
            return str(new.strftime('%d/%m/%Y')) 
        
        def child_cat(self, cr, uid, categoria_id, context=None):
            child_ids = [c.id for c in self.pool.get('product.category').browse(cr, uid, categoria_id, context=context).child_id]
            return [categoria_id] + reduce(lambda x, y: x + y, [self.child_cat(cr, uid, d, context=context) for d in child_ids], []) if child_ids else [categoria_id]
            
        def obtener_fecha(self, o):
            if o.is_periodo:
                return "Perido: " + o.period_id.name
            else:
                return " Hasta: " + self.get_fecha(o.fecha_hasta)
                
        def contempla_cero(self, o):
            if o.mostrarcero:
                return "Contempla Productos con Existencia en 0.0"
            return "No Contempla Productos con Existencia en 0.0"
        
        def generar_data(self, o, context=None):

            dicc={}
            
            filtros = []
            
            is_periodo = o.is_periodo
            mostrarcero = o.mostrarcero
            fecha_desde = o.fecha_hasta 
            fecha_hasta = o.fecha_hasta
            product_id = o.product_id.id
            location_id = o.location_id.id
            period_id = o.period_id.id
            formato = o.formato
            categ_id = o.categ_id.categ_id.id
            
            if not o.all_product:
                filtros.append(('id','=',product_id))
            
            categorias_ids = self.child_cat(self.cr, self.uid, categ_id, context)
            filtros.append(('categ_id','in',categorias_ids))
            
            obj_product = self.pool.get('product.product') 
            product_ids = obj_product.search(self.cr, self.uid, filtros)

            if formato == 'ubicacion':
                
                productos_ids = product_ids
                values = self.get_existencia_ubicacion(location_id, is_periodo, period_id, fecha_desde, fecha_hasta, productos_ids, mostrarcero)
                
                for cat in values:
                    
                    if not cat in self.total:
                        self.total[cat] = {'existencia':0.0,'peso_neto':0.0,'peso_bruto':0.0}
                    
                    for p in values[cat]:
                        
                        existencia = values[cat][p]['existencia']
                        peso_neto = values[cat][p]['peso_neto']
                        peso_bruto = values[cat][p]['peso_bruto']
                    
                        self.total[cat]['existencia'] += existencia
                        self.total[cat]['peso_neto'] += peso_neto
                        self.total[cat]['peso_bruto'] += peso_bruto
                    
                ordenamiento = []
                for x in sorted(values.iterkeys()):
                    ordenamiento.append({x:values[x]})
            
                self.dicc = ordenamiento
            
            return ''
            
        def obtener_data(self):
            return self.dicc
        
        def obtener_titulo_productos(self, o):
            if o.all_product:
                return 'Todos los Productos'
            return o.product_id.name
        
        
        
        def obtener_total(self, cat, tipo):
            return self.total[cat][tipo]
        
            
        def filter_sql(self, sql, is_periodo, period, fecha_desde, fecha_hasta, product_ids, mostrarcero):
            
            sql = sql + """\nAND  db_tbl.product_id in %s"""%(str(product_ids).replace('[','(').replace(']',')'))
            
            if is_periodo:
                sql = sql + """\nAND db_tbl.date :: date <= (SELECT date_stop FROM account_period WHERE id = %s)"""%period
            else:
                sql = sql + """ AND db_tbl.date :: date <= '%s' """%fecha_hasta
            
            sql = sql + """ group by db_tbl.product_id, db_tbl.default_code, db_tbl.name, db_tbl.categoria,
			db_tbl.peso_neto, db_tbl.peso_bruto, db_tbl.unidad_medida order by db_tbl.name ) as t  """
            
            if not mostrarcero:sql = sql + """ where existencia > 0.0 """
            
            return sql 
        
        
        def execute_sql(self, sql):
            self.cr.execute(sql)
            consulta = self.cr.fetchall()
            return consulta
        
        def get_existencia_ubicacion(self, location, is_periodo, period, fecha_desde, fecha_hasta, product_ids, mostrarcero):
            
            sql = """
            
            select * from (
            
            SELECT product_id, default_code, name, SUM(cant_convertida) as existencia, categoria,
            (peso_neto * SUM(cant_convertida) / 1000) as peso_neto, (peso_bruto * SUM(cant_convertida) / 1000) as peso_bruto, unidad_medida from (
            SELECT
            min(m.id) AS id, m.date,
            m.location_id,
            l.usage,
            m.product_id,
            pp.default_code,
            pt.name,
            m.state,
            pt.weight_net as peso_neto,
            pt.weight as peso_bruto,
            cat.name as categoria,
            pun.name as unidad_medida,
            CASE WHEN m.product_uom != pt.uom_id THEN
            COALESCE(sum(round((- m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
            ELSE COALESCE(sum(- m.product_qty), 0.0)
            END as cant_convertida
            FROM
            stock_move m
            LEFT JOIN stock_picking p ON m.picking_id = p.id
            LEFT JOIN product_product pp ON m.product_id = pp.id
            LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
            LEFT JOIN product_uom pu ON pt.uom_id = pu.id
            LEFT JOIN product_uom u ON m.product_uom = u.id
            LEFT JOIN stock_location l ON m.location_id = l.id
            LEFT JOIN product_category cat ON cat.id = pt.categ_id
            LEFT JOIN product_uom pun ON pun.id = pt.uom_id
            GROUP BY
            pun.name, pt.weight, pt.weight_net, cat.name, m.id, m.product_id, pp.default_code, m.product_uom, pt.name, pt.categ_id, m.address_id, m.location_id, m.location_dest_id,
            m.prodlot_id, m.date, m.state, l.usage, m.company_id, pt.uom_id
            UNION ALL
            SELECT
            - m.id AS id, m.date,
            m.location_dest_id AS location_id,
            l.usage,
            m.product_id,
            pp.default_code,
            pt.name,
            m.state,
            pt.weight_net as peso_neto,
            pt.weight as peso_bruto,
            cat.name as categoria,
            pun.name as unidad_medida,
            CASE WHEN m.product_uom != pt.uom_id THEN
            COALESCE(sum(round((m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
            ELSE COALESCE(sum(m.product_qty), 0.0)
            END as cant_convertida
            FROM
            stock_move m
            LEFT JOIN stock_picking p ON m.picking_id = p.id
            LEFT JOIN product_product pp ON m.product_id = pp.id
            LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
            LEFT JOIN product_uom pu ON pt.uom_id = pu.id
            LEFT JOIN product_uom u ON m.product_uom = u.id
            LEFT JOIN stock_location l ON m.location_dest_id = l.id
            LEFT JOIN product_category cat ON cat.id = pt.categ_id
            LEFT JOIN product_uom pun ON pun.id = pt.uom_id
            GROUP BY
            pun.name, pt.weight, pt.weight_net, cat.name, m.id, m.product_id, pp.default_code, m.product_uom, pt.name, pt.categ_id, m.address_id, m.location_id, m.location_dest_id,
            m.prodlot_id, m.date, m.state, l.usage, m.company_id, pt.uom_id
            ) db_tbl
            where db_tbl.location_id = %s
            and db_tbl.state = 'done' and db_tbl.usage = 'internal'
            """ %(location)
            
            sql = self.filter_sql(sql, is_periodo, period, fecha_desde, fecha_hasta, product_ids, mostrarcero)
            salida = self.execute_sql(sql)
            agrupar = {}
            
            for i in salida:
            
                product_id = i[0] 
                default_code = i[1]
                name = i[2] 
                existencia = i[3]
                categoria = i[4]
                peso_neto = i[5]
                peso_bruto = i[6]
                
                if not categoria in agrupar:
                    agrupar[categoria] = {}
                
                if not name in agrupar[categoria]:
                    agrupar[categoria][name] = {}
                
                agrupar[categoria][name] = {'udm':i[7], 'product_id':product_id, 'codigo':default_code, 'existencia':existencia, 'peso_neto':peso_neto, 'peso_bruto':peso_bruto}
                
            return agrupar 
            
             
report_sxw.report_sxw('report.wizard.inventario.producto.terminado.xml.id',
                    'wizard.inventario.producto.terminado',
                    'sisb_reportes_especiales/report/report_inventario_producto_terminado.rml',
                    parser=report_inventario_producto_terminado) 
