#!/usr/bin/python 
# -*- encoding: utf-8 -*- 
 
from datetime import datetime
from report import report_sxw 
import time 

class report_inventario_producto_terminado_detallado(report_sxw.rml_parse): 
        
        def __init__(self, cr, uid, name, context): 
            super(report_inventario_producto_terminado_detallado, self).__init__(cr, uid, name, context) 
            self.localcontext.update({ 
              'time': time, 
              'user': self.pool.get('res.users').browse(cr, uid, uid, context),  
              'get_empresa': self.get_empresa, 
              'get_final_date': self.get_final_date, 
              'get_fecha': self.get_fecha, 
              'get_hour_date': self.get_hour_date, 
              'generar_data': self.generar_data, 
              'child_cat': self.child_cat, 
              'obtener_data': self.obtener_data, 
              'obtener_total_general_resumen': self.obtener_total_general_resumen, 
              'obtener_fecha': self.obtener_fecha, 
              'retornar_data': self.retornar_data, 
              'obtener_total_centro': self.obtener_total_centro, 
              'retornar_resumen': self.retornar_resumen, 
              'obtener_total_categoria_resumen': self.obtener_total_categoria_resumen, 
              'obtener_titulo_productos': self.obtener_titulo_productos, 
              'obtener_titulo_ceros': self.obtener_titulo_ceros, 
              
            }) 
            
            self.salida = []
            self.producto = {}
            self.dicc = {}
            self.total_total = {}
            self.sub_total = {}
            self.data = ''
            self.data_resumen = {}
            
            self.total_total_categoria={}
            self.total_por_categoria = {}
            self.total_por_producto = {}
            self.total_general = {'existencia':0.0, 'peso_neto':0.0, 'peso_bruto':0.0}
            self.dicc_resumen = {}
        
        
        def obtener_total_general_resumen(self, tipo):
            return self.total_general[tipo]
        
        def obtener_total_categoria_resumen(self, cat, tipo):
            return self.total_total_categoria[cat][tipo]
        
        def obtener_titulo_productos(self, o):
            if o.all_product:
                return 'Todos los Productos'
            return o.product_id.name
        
        def obtener_total_centro(self, cd, cat, tipo):
            return self.total_por_categoria[cd][cat][tipo]
        
        def retornar_data(self):
            return self.data
        
        def get_empresa(self, partner_id): 
            obj_addr = self.pool.get('res.partner.address') 
            addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)]) 
            res = {} 
            for row in obj_addr.browse(self.cr, self.uid, addr_id): 
              res = { 
                  'street':row.street, 
                  'phone':row.phone, 
                  'fax':row.fax, 
                  'email':row.email, 
                  'city':row.city, 
                  'name':row.name, 
                  'country':row.country_id.name, 
               } 

            return res 
        
        def obtener_titulo_ceros(self, o):
            
            if not o.mostrarcero:
                return "No contempla productos con existencia en 0.0"
            return "Contempla productos con existencia en 0.0"
             
        def retornar_resumen(self):
            return self.data_resumen
        
        def get_hour_date(self): 
            current_dt = datetime.now() 
            a = ['AM','PM'][current_dt.hour>=12] 
            return datetime.now().strftime('%H:%M:%S') +  + a  

        def get_final_date(self): 
            return datetime.now().strftime('%d-%m-%Y') 

        def get_fecha(self, date): 
            if str(date) == '': 
              return False 
            new = datetime.strptime(date, '%Y-%m-%d') 
            return str(new.strftime('%d/%m/%Y')) 
        
       
        def child_cat(self, cr, uid, categoria_id, context=None):
            
            child_ids = [c.id for c in self.pool.get('product.category').browse(cr, uid, categoria_id, context=context).child_id]
            return [categoria_id] + reduce(lambda x, y: x + y, [self.child_cat(cr, uid, d, context=context) for d in child_ids], []) if child_ids else [categoria_id]
            
        
        def obtener_fecha(self, o):
            if o.is_periodo:
                return "Perido: " + o.period_id.name
            else:
                return " Hasta: " + self.get_fecha(o.fecha_hasta)
        
        def generar_data(self, o, context=None):
            
            filtros = []
            self.data = o
            
            mostrarcero = o.mostrarcero
            is_periodo = o.is_periodo
            fecha_desde = o.fecha_hasta 
            fecha_hasta = o.fecha_hasta 
            all_product = o.all_product
            product_id = o.product_id.id

            period_id = o.period_id.id
            formato = o.formato
            categ_id = o.categ_id.categ_id.id
        
            categorias_ids = self.child_cat(self.cr, self.uid, categ_id, context)
            
            if not all_product:
                filtros.append(('id','=',product_id))
            
            filtros.append(('categ_id','in',categorias_ids))
            
            obj_product = self.pool.get('product.product') 
            product_ids = obj_product.search(self.cr, self.uid, filtros)

            if formato == 'todas_ubicaciones':
                
                values = self.get_existencia_(is_periodo, period_id, fecha_desde, fecha_hasta, product_ids, mostrarcero)
                
                for cd in values:
                    
                    if not cd in self.total_por_categoria:
                        self.total_por_categoria[cd] = {}
                    
                    for cat in values[cd]:

                        if not cat in self.total_por_categoria[cd]:
                            self.total_por_categoria[cd][cat] = {'existencia':0.0, 'peso_neto':0.0, 'peso_bruto':0.0}
                        
                        if not cat in self.dicc_resumen:
                            self.dicc_resumen[cat] ={}
                        
                        if not cat in self.total_total_categoria:
                            self.total_total_categoria[cat] ={'existencia':0.0, 'peso_neto':0.0, 'peso_bruto':0.0}
                            
                        for p in values[cd][cat]:
                            
                            self.total_total_categoria[cat]['existencia'] += values[cd][cat][p]['existencia']
                            self.total_total_categoria[cat]['peso_neto'] += values[cd][cat][p]['peso_neto']
                            self.total_total_categoria[cat]['peso_bruto'] += values[cd][cat][p]['peso_bruto']
                            
                            self.total_por_categoria[cd][cat]['existencia'] += values[cd][cat][p]['existencia']
                            self.total_por_categoria[cd][cat]['peso_neto'] += values[cd][cat][p]['peso_neto']
                            self.total_por_categoria[cd][cat]['peso_bruto'] += values[cd][cat][p]['peso_bruto']
                            
                            if not p in self.dicc_resumen[cat]:
                                self.dicc_resumen[cat][p] = {'existencia':0.0, 'peso_neto':0.0, 'peso_bruto':0.0, 'udm':values[cd][cat][p]['udm'], 'codigo':values[cd][cat][p]['codigo']}
                            
                            self.dicc_resumen[cat][p]['existencia'] += values[cd][cat][p]['existencia']
                            self.dicc_resumen[cat][p]['peso_neto'] += values[cd][cat][p]['peso_neto']
                            self.dicc_resumen[cat][p]['peso_bruto'] += values[cd][cat][p]['peso_bruto']
                
                            self.total_general['existencia'] += values[cd][cat][p]['existencia']
                            self.total_general['peso_neto'] += values[cd][cat][p]['peso_neto']
                            self.total_general['peso_bruto'] += values[cd][cat][p]['peso_bruto']
                            
                        
                ordenamiento = []
                for x in sorted(values.iterkeys()):
                    ordenamiento.append({x:values[x]})
    
                ordenamiento_resumen = []
                for x in sorted(self.dicc_resumen.iterkeys()):
                    ordenamiento_resumen.append({x:self.dicc_resumen[x]})
                
                self.dicc = ordenamiento
                self.data_resumen = ordenamiento_resumen

                return ''
            
        
        def obtener_data(self):
            return self.dicc
            
        
        def filter_sql(self, sql, is_periodo, period, fecha_desde, fecha_hasta, product_ids, no_mostrar_cero):
            
            sql = sql + '''\n AND db_tbl.product_id in %s'''%(str(product_ids).replace('[','(').replace(']',')'))
            
            if is_periodo:
                sql = sql + '''\n AND db_tbl.date :: date <= (SELECT date_stop FROM account_period WHERE id = %s)'''%(period)
            else:
                sql = sql + """\n AND db_tbl.date :: date <= '%s' """%(fecha_hasta)
            
            sql = sql + """group by 
            
                                db_tbl.product_id, 
                                db_tbl.default_code, 
                                db_tbl.name, 
                                db_tbl.name_ubicacion, 
                                db_tbl.weight_net, 
                                categoria, 
                                unidad_medida,
                                db_tbl.weight
                                
                            order by db_tbl.name ) as t  """
            
            if not no_mostrar_cero:
                sql = sql + """  where existencia_inicial > 0.0 """
            
            return sql 
        
        
        def execute_sql(self, sql):
            self.cr.execute(sql)
            consulta = self.cr.fetchall()
            return consulta
        
        def get_existencia_(self, is_periodo, period, fecha_desde, fecha_hasta, product_ids, no_mostrar_cero):
            sql = """
            
                SELECT * FROM (
            
                SELECT product_id, default_code, name, name_ubicacion, SUM(cant_convertida) as existencia_inicial, (weight_net * SUM(cant_convertida) / 1000) as peso_neto, categoria, unidad_medida, (weight * SUM(cant_convertida) /1000) as peso_bruto from (
                SELECT
                min(m.id) AS id, m.date,
                m.location_id,
                l.name as name_ubicacion,
                l.usage,
                m.product_id,
                pp.default_code,
                pt.name,
                pt.weight_net,
                pt.weight,
                cat.name as categoria,
                pun.name as unidad_medida,
                m.state,
                CASE WHEN m.product_uom != pt.uom_id THEN
                COALESCE(sum(round((- m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
                ELSE COALESCE(sum(- m.product_qty), 0.0)
                END as cant_convertida
                FROM
                stock_move m
                LEFT JOIN stock_picking p ON m.picking_id = p.id
                LEFT JOIN product_product pp ON m.product_id = pp.id
                LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
                LEFT JOIN product_uom pu ON pt.uom_id = pu.id
                LEFT JOIN product_uom u ON m.product_uom = u.id
                LEFT JOIN stock_location l ON m.location_id = l.id
                
                LEFT JOIN product_category cat ON cat.id = pt.categ_id
                LEFT JOIN product_uom pun ON pun.id = pt.uom_id
                
                GROUP BY
                pt.weight, pun.name, cat.name, pt.weight_net, l.name, m.id, m.product_id, pp.default_code, m.product_uom, pt.name, pt.categ_id, m.address_id, m.location_id, m.location_dest_id,
                m.prodlot_id, m.date, m.state, l.usage, m.company_id, pt.uom_id
                UNION ALL
                SELECT
                - m.id AS id, m.date,
                m.location_dest_id AS location_id,
                l.name as name_ubicacion,
                l.usage,
                m.product_id,
                pp.default_code,
                pt.name,
                pt.weight_net,
                pt.weight,
                cat.name as categoria,
                pun.name as unidad_medida,
                m.state,
                CASE WHEN m.product_uom != pt.uom_id THEN
                COALESCE(sum(round((m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
                ELSE COALESCE(sum(m.product_qty), 0.0)
                END as cant_convertida
                FROM
                stock_move m
                LEFT JOIN stock_picking p ON m.picking_id = p.id
                LEFT JOIN product_product pp ON m.product_id = pp.id
                LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
                LEFT JOIN product_uom pu ON pt.uom_id = pu.id
                LEFT JOIN product_uom u ON m.product_uom = u.id
                LEFT JOIN stock_location l ON m.location_dest_id = l.id
                
                LEFT JOIN product_category cat ON cat.id = pt.categ_id
                LEFT JOIN product_uom pun ON pun.id = pt.uom_id
                
                GROUP BY
                pt.weight, pun.name, cat.name, pt.weight_net, l.name, m.id, m.product_id, pp.default_code, m.product_uom, pt.name, pt.categ_id, m.address_id, m.location_id, m.location_dest_id,
                m.prodlot_id, m.date, m.state, l.usage, m.company_id, pt.uom_id
                ) db_tbl
                where db_tbl.state = 'done' and db_tbl.usage = 'internal'
                
            """

            sql = self.filter_sql(sql, is_periodo, period, fecha_desde, fecha_hasta, product_ids, no_mostrar_cero)
            salida = self.execute_sql(sql)
            agrupar = {}
            
            for i in salida:
                product_id = i[0]
                default_code = i[1]
                name = i[2]
                name_ubicacion = i[3]
                existencia_inicial = i[4] 
                peso_neto = i[5]
                categoria = i[6]
                unidad_medida = i[7]
                peso_bruto = i[8]
                
                if not name_ubicacion in agrupar:
                    agrupar[name_ubicacion] = {}
                if not categoria in agrupar[name_ubicacion]:
                    agrupar[name_ubicacion][categoria] = {}
                if not name in agrupar[name_ubicacion][categoria]:
                    agrupar[name_ubicacion][categoria][name] = {'codigo':default_code, 
                                                                'product_id':product_id,
                                                                'udm':unidad_medida, 
                                                                'existencia':existencia_inicial,
                                                                'peso_bruto':peso_bruto,
                                                                'peso_neto':peso_neto}
            return agrupar
             
report_sxw.report_sxw('report.wizard.inventario.producto.terminado.xml.id.detallado',
                    'wizard.inventario.producto.terminado',
                    'sisb_reportes_especiales/report/report_inventario_producto_terminado_detallado.rml',
                    parser=report_inventario_producto_terminado_detallado) 
