#!/usr/bin/python 
# -*- encoding: utf-8 -*- 
 
from report import report_sxw 
import datetime 
import time 

class report_account_invoice_report_expenses(report_sxw.rml_parse): 
    
    def __init__(self, cr, uid, name, context): 
        super(report_account_invoice_report_expenses, self).__init__(cr, uid, name, context) 
        self.localcontext.update({ 
              'time': time, 
              'user': self.pool.get('res.users').browse(cr, uid, uid, context),  
              'get_empresa': self.get_empresa, 
              'get_final_date': self.get_final_date, 
              'get_fecha': self.get_fecha, 
              'get_hour_date': self.get_hour_date, 
              'generate': self.generate, 
              'get_total': self.get_total, 
              'table_tax': self.table_tax, 
              'get_amount_total': self.get_amount_total, 
              'get_result': self.get_result, 
              'get_sum': self.get_sum,  
              'generate_account_not': self.generate_account_not, 
              'generate_context': self.generate_context, 
              'convert': self.convert, 
              'codes': self.codes, 
          }) 
        
        self.dictionary_result={}
        self.total = 0.0
        self.amount_total = 0.0
        self.amount_base = 0.0
        self.amount_monto = 0.0
        self.invoice_list = []
        self.invoice_account = []
        self.account_codes = ''
        self.list_accounts()
        
    def get_empresa(self, partner_id): 
        obj_addr = self.pool.get('res.partner.address') 
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)]) 
        res = {} 
        for row in obj_addr.browse(self.cr, self.uid, addr_id): 
              res = { 
                  'street':row.street, 
                  'phone':row.phone, 
                  'fax':row.fax, 
                  'email':row.email, 
                  'city':row.city, 
                  'name':row.name, 
                  'country':row.country_id.name,} 
        return res 
    
    def list_accounts(self):
        query = """ SELECT aa.code, aa.name FROM configuration_account_invoice_report_expenses as caire
                    join account_account as aa ON aa.id = caire.account_id ORDER BY aa.name """
        self.cr.execute(query)
        self.invoice_account = self.cr.fetchall()
        invoice_account_code = [code[0] for code in self.invoice_account]
        if invoice_account_code:
            s= ', '
            self.account_codes = s.join(invoice_account_code)
        else:
            self.account_codes = 'No se encontraron cuentas en la configuración'
    
    def codes(self):
        return self.account_codes
    
    def get_hour_date(self): 
        current_dt = datetime.now() 
        a = ['AM','PM'][current_dt.hour>=12] 
        return datetime.now().strftime('%H:%M:%S') +  + a  
 
    def get_final_date(self): 
        return datetime.now().strftime('%d-%m-%Y') 
 
    def get_fecha(self, date): 
        if str(date) == '': 
            return False 
        new = datetime.strptime(date, '%Y-%m-%d') 
        return str(new.strftime('%d-%m-%Y')) 
    
    def taxes(self, invoice_id, tipo):
        if tipo == 'in_refund':
            sql_Post = """
                SELECT att.amount*-1, ait.base*-1, ait.amount*-1 from 
                account_invoice_tax as ait 
                join account_tax as att on ait.tax_id = att.id 
                where invoice_id = %s
            """%(invoice_id)
        else:
            sql_Post = """
                SELECT att.amount, ait.base, ait.amount from 
                account_invoice_tax as ait 
                join account_tax as att on ait.tax_id = att.id 
                where invoice_id = %s
            """%(invoice_id)
        self.cr.execute(sql_Post)
        resultSet = self.cr.fetchall()
        return sorted(resultSet)
        
    def convert_tuple(self, type_account_invoice):
        if len(type_account_invoice)>1:
            return str(tuple(type_account_invoice))
        return str(tuple(type_account_invoice)).replace(',', '')
    
    def generate(self, data):
        """ Se establecen los filtros """
        form = data['form']
        is_periodo = form['is_period']
        period_id = form['period_id']
        all_partner = form['all_partner']
        credit_note = form['credit_note']
        is_configuration = form['not_configuration']
        
        if is_periodo:
            """ Se obtiene el rango de fecha segun el periodo """
            date_from = self.pool.get('account.period').browse(self.cr, self.uid, period_id).date_start
            date_to = self.pool.get('account.period').browse(self.cr, self.uid, period_id).date_stop
        else:
            date_from = form['date_from']
            date_to = form['date_to']
        
        """ Se crea un parametro de busqueda que se utilizara en el query """
        type_account_invoice = ['in_invoice']
        if credit_note:
            type_account_invoice.append('in_refund')

        """ Se obtiene la lista de las cuentas que no se contemplaran en el reporte """
        configuration_list=[]
        configuration_ids = self.pool.get('configuration.account.invoice.report.expenses').search(self.cr, self.uid, [])
        configuration_browse = self.pool.get('configuration.account.invoice.report.expenses').browse(self.cr, self.uid, configuration_ids)
        
        for c in configuration_browse:
            configuration_list.append(c.account_id.id)
        
        sql_postgres = """
                SELECT  
                        ac.id,
                        ac.date_invoice,
                        ac.number,
                        ac.date_document,
                        ac.reference,
                        ac.partner_id,
                        CASE WHEN ac.type = 'in_refund' THEN ac.amount_tax*-1 ELSE ac.amount_tax END,
                        CASE WHEN ac.type = 'in_refund' THEN ac.amount_total*-1 ELSE ac.amount_total END,
                        rp.name as partner_id,
                        ac.type
                FROM 
                        account_invoice ac, res_partner rp
                WHERE 
                        rp.id = ac.partner_id
                        and ac.date_invoice >= '%s' 
                        and ac.date_invoice <= '%s' 
                        and ac.type in %s
                        and ac.state not in ('draft','cancel') """%(date_from, date_to, self.convert_tuple(type_account_invoice))
        
        if not all_partner:
            partner_id = form['partner_id']

            sql_postgres += """ and ac.partner_id = %s """%(partner_id)
                
        sql_postgres += """
                    order by date_invoice
                """
        self.cr.execute(sql_postgres)
        resultSet = self.cr.dictfetchall()
        
        for invoice in resultSet:
            """ Se recorre cada una de las facturas que nos proporciono el query"""
            tax_list=[]
            
            """ Se condiciona segun el filtro del wizard """
            """ Si no omitio la configuracion """
            if not is_configuration:
                
                """ Se consultan las lineas directas de la factura """                
                query = """ select distinct account_id as cuenta from account_invoice_line where invoice_id = %s """%(invoice['id'])
                self.cr.execute(query)
                resultList = self.cr.fetchall()
                
                """ Se obtiene la lista de las cuentas de la factura """
                """ Se verifica si alguna cuenta de la factura existe en la lista de cuentas de la configuracion """
                check = set([account_id[0] for account_id in resultList]) & set(configuration_list)

                if not check:
                    """ verifico si la factura viene de una albaran """
                    """ Si no aparece en el reporte """
                    sql_albaran = """
                        SELECT  
                            CASE WHEN ail.origin is null THEN 'not'
                            ELSE 'yes'
                            END as proviene_de_albaran,
                            split_part(ail.origin, ':', 1) as ref_albaran
                        FROM 
                            account_invoice as ac
                            join account_invoice_line as ail ON ac.id = ail.invoice_id
                        WHERE 
                            ac.id = %s """%(invoice['id'])
                    
                    self.cr.execute(sql_albaran)
                    resultAlbaran = self.cr.fetchall()
                    
                    if resultAlbaran[0][0] == 'yes':
                        
                        query = """ select distinct account_id as cuenta from account_move_line where ref = '%s' """%(resultAlbaran[0][1])
                        self.cr.execute(query)
                        resultList = self.cr.fetchall()
                    
                        check = set([account_id[0] for account_id in resultList]) & set(configuration_list)
                        """ Si no existen cuentas en ambas listas """
                        """ Aparece en el reporte """
                        if not check:
                            tax_list = self.taxes(invoice['id'], invoice['type'])
                            self.invoice_list.append((invoice['date_invoice'], invoice['date_document'], invoice['reference'], invoice['partner_id'], sorted(tax_list), invoice['amount_total']))
                            self.total += invoice['amount_total']
                    else:
                        tax_list = self.taxes(invoice['id'], invoice['type'])
                        self.invoice_list.append((invoice['date_invoice'], invoice['date_document'], invoice['reference'], invoice['partner_id'], sorted(tax_list), invoice['amount_total']))
                        self.total += invoice['amount_total']
            else:
                """ Sin configuracion """
                tax_list = self.taxes(invoice['id'], invoice['type'])
                self.invoice_list.append((invoice['date_invoice'], invoice['date_document'], invoice['reference'], invoice['partner_id'], sorted(tax_list), invoice['amount_total']))
                self.total += invoice['amount_total']

            """ Proceso para el cuadro de totales por iva """
            """ El cual esta al final del reporte """
            if tax_list:
                for tax in tax_list:
                    base = tax[1]
                    monto = tax[2]
                    if not tax[0] in self.dictionary_result:
                        self.dictionary_result[tax[0]] = {}
                        self.dictionary_result[tax[0]]['base'] = base
                        self.dictionary_result[tax[0]]['monto'] = monto
                    else:
                        self.dictionary_result[tax[0]]['base'] += base
                        self.dictionary_result[tax[0]]['monto'] += monto
        
    def get_sum(self, base, amount):
        return base + amount
        
    def get_total(self):
        total_base = 0.0
        total_monto = 0.0
        for i in self.dictionary_result:
            total_base += self.dictionary_result[i]['base']
            total_monto += self.dictionary_result[i]['monto']
        return [total_base, total_monto, self.total]
    
    def get_amount_total(self):
        for tx in self.dictionary_result:
            self.amount_total += self.dictionary_result[tx]
        return self.amount_total
        
    def get_result(self, key):
        return self.dictionary_result[key]
    
    def table_tax(self):
        return self.dictionary_result
        
    def generate_account_not(self):
        return self.invoice_account
        
    def generate_context(self):
        return sorted(self.invoice_list)
    
    def convert(self, iva):
        return str((iva * 100)) + ' %' 
        
report_sxw.report_sxw('report.account.invoice.report.expenses.id',
                      'wizard.account.invoice.report.expenses',
                      'sisb_reportes_especiales/report/reporte_de_gastos.rml',
                      parser=report_account_invoice_report_expenses) 
