#-*- coding:utf-8 -*- 
#!/usr/bin/env python 
 
from osv import osv, fields
from datetime import datetime

class wizard_account_invoice_report_expenses(osv.osv_memory): 
    _name = 'wizard.account.invoice.report.expenses' 
    _description = 'Wizard Report Expenses' 
    _columns = { 
        'is_period': fields.boolean('Por Periodo'),
        'date_from': fields.date('Fecha desde'),
        'date_to': fields.date('Fecha hasta'),
        'period_id': fields.many2one('account.period', 'Periodo'),
        'partner_id': fields.many2one('res.partner', 'Proveedor'),
        'all_partner': fields.boolean('Todos los proveedores'),
        'credit_note': fields.boolean('Nota de crédito'),
        'not_configuration': fields.boolean('Omitir la configuración'),
        'print_not_contemplated': fields.boolean('Mostrar cuentas no contempladas'),
      }
       
    _defaults = {'is_period':True,
                 'all_partner':True,
                 'not_configuration':False,
                 'credit_note':False,}
   
    def elimina_tildes(self, s): 
        import unicodedata 
        return ''.join((c for c in unicodedata.normalize('NFD', unicode(s)) if unicodedata.category(c) != 'Mn')) 
   
    def normalizar_string(self, string): 
        from unicodedata import normalize 
        unicode_string=unicode(string) 
        return normalize('NFKD', unicode_string).encode('ascii','ignore') 
   
    def generate_report(self, cr, uid, ids, context=None): 
        agrupador = {}
        wizard_brw = self.browse(cr, uid, ids)[0] 
   
        for i in self._columns: 
            if type(getattr(wizard_brw, i)) in [osv.orm.browse_record, osv.orm.browse_null]: 
                agrupador[i] = getattr(getattr(wizard_brw, i),'id') 
            else: 
                agrupador[i] =  getattr(wizard_brw, i) 
        
        if not agrupador['is_period']:
            fecha_desde_date = datetime.strptime(agrupador['date_from'], "%Y-%m-%d")
            fecha_hasta_date = datetime.strptime(agrupador['date_to'], "%Y-%m-%d")
            dif = fecha_desde_date - fecha_hasta_date
            dif = abs(dif)
            if dif.days > 31:
                raise osv.except_osv('ERROR', "Las fechas no pueden exceder de 31 dias") 
        
        if not agrupador['all_partner']:
            agrupador['partner_name'] =  wizard_brw.partner_id.name
        
        agrupador['period_name'] =  wizard_brw.period_id.name

        datas = {'form':agrupador} 
        
        return { 
           'type': 'ir.actions.report.xml',
           'report_name': 'account.invoice.report.expenses.id', 
           'datas': datas, 
        } 
   
wizard_account_invoice_report_expenses() 
