#!/usr/bin/env python
# -*- coding: utf-8 -*--
from dateutil import relativedelta
from osv import fields, osv
import tools
from tools.translate import _
import logging
import calendar
import unicodedata
from report import report_sxw
from datetime import *
import pooler

import xlwt
from xlwt import Workbook
from xlwt import Font
from xlwt import XFStyle
from xlwt import Borders

class report_reposo_medico_cal(report_sxw.rml_parse):
    _name =  'report.reposo.medico.cal'    
    def __init__(self, cr, uid, name, context):
        super(report_reposo_medico_cal, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'get_empresa':self.get_empresa,
            'get_final_date':self.get_final_date,
            'cvr_date':self.cvr_date,
        })
        self.context = context

    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res
    
    def get_final_date(self):
        return datetime.now().strftime('%d/%m/%Y %H:%M')    
    
    def cvr_date(self, fecha):
        return  datetime.datetime.strptime(fecha, '%Y/%m/%d').strftime('%Y%m%d')    
    
report_sxw = report_sxw.report_sxw     
   
class TxtParser(report_sxw):
    """Custom class that use webkit to render HTML reports
       Code partially taken from report openoffice. Thanks guys :)
    """
    def __init__(self, name, table, rml=False, parser=False,
        header=True, store=False):
        self.parser_instance = False
        self.localcontext={}
        
        report_sxw.__init__(self, name, table, rml, parser,
            header, store)
            
    def cvr_date(self, fecha):
        if fecha != '':
            new = datetime.strptime(fecha, '%Y-%m-%d')
            return str(new.strftime('%d/%m/%Y'))
        return False
    
    def query_sql(self, lunes, domingo, cr, uid):
        
            query = """ select id 
                        from sisb_reposos_medicos  
                        where fecha_inicio < '"""+str(lunes)+"""' and fecha_fin > '"""+str(domingo)+"""'
                        or fecha_inicio >= '"""+str(lunes)+"""' and fecha_fin <= '"""+str(domingo)+"""'
                        or fecha_inicio BETWEEN '"""+str(lunes)+"""' AND '"""+str(domingo)+"""' and fecha_fin > '"""+str(domingo)+"""'
                        or fecha_inicio < '"""+str(lunes)+"""' and fecha_fin BETWEEN '"""+str(lunes)+"""' AND '"""+str(domingo)+"""'"""
            try:
                cr.execute(query)
                results = cr.fetchall()
                return results
            except:
                print sys.exc_info()[0]
            
            
    
    def obtener_datos(self, data, cr, uid):
        lunes = data['form']
        domingo = str(data['domingo']).split(' ')
        domingo = domingo[0]
        sede_id = data['sede_id']
        nomina_id = data['nomina_id']
        pool= pooler.get_pool(cr.dbname)   
        employee_obj = pool.get('hr.employee')
        reposos_obj = pool.get('sisb.reposos.medicos')
        dic = {}
        lista_new = []
        lista_privada = []
        lista_ivss = []
        consulta = self.query_sql(lunes, domingo, cr, uid)
        for i in consulta:
            lista_new.append(i[0])
        lista_new = list(set(lista_new))
        if lista_new:
            reposos_browse = reposos_obj.browse(cr, uid, lista_new)
            for ids in reposos_browse:
                filtros = []
                filtros.append(('id','=',ids.trabajador_id.id))
                if sede_id:
                    filtros.append(('hr_sede_id','=', sede_id))
                if nomina_id:
                    filtros.append(('hr_payroll_id','=', nomina_id))
                employee_ids = employee_obj.search(cr, uid, filtros)
                if employee_ids:
                    employee_browse = employee_obj.browse(cr, uid, employee_ids)
                    for trabajador in employee_browse:
                        if ids.tipo_reposo_medico == 'privado':
                            lista_privada.append((trabajador.name, trabajador.identification_id, trabajador.cod_number, trabajador.mobile_phone, ids.tipo_reposo_servicio_medico_id.name, ids.codigo_diagnostico_id.name,ids.fecha_inicio, ids.fecha_fin, ids.fecha_reintegro, trabajador.hr_payroll_id.name, ids.centro_salud_privado_id.name, ids.medico_tratante))
                        else:
                            lista_ivss.append((trabajador.name, trabajador.identification_id, trabajador.cod_number, trabajador.mobile_phone, ids.tipo_reposo_servicio_medico_id.name, ids.codigo_diagnostico_id.name, ids.fecha_inicio, ids.fecha_fin, ids.fecha_reintegro, trabajador.hr_payroll_id.name, ids.centro_salud_ivss_id.name, ids.medico_tratante))
        else:
            raise osv.except_osv(_('ERROR!'), _('La combinación de filtros para generar dicho reporte no arrojo ningun resultado!!!')) 
        
        dic['privado'] = sorted(lista_privada)
        dic['ivss'] = sorted(lista_ivss)
        return dic
    
    
    def crear_xls(self, data, lunes, domingo, sede, nomina):
        import StringIO
        import base64
        try:
            import xlwt
        except:
            raise osv.except_osv('Warning !','Please download python xlwt module from\nhttp://pypi.python.org/packages/source/x/xlwt/xlwt-0.7.2.tar.gz\nand install it')
        domingo = str(domingo).split(' ')
        domingo = self.cvr_date(domingo[0])
        lunes = self.cvr_date(lunes)
        cabezera = ['Nombre y Apellido', 'Cedula', 'Codigo', 'Telefono', 'Patologia', 'Codigo diagnostico', 'Inicio de reposo', 'Fin de reposo', 'Reintegro', 'Tipo de nomina', 'Centro de salud', 'Medico tratante']
        wbk = xlwt.Workbook()
        title_left = xlwt.easyxf('align: horiz left;' 'font: name Times New Roman, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;')
        title_center = xlwt.easyxf('align: horiz center;' 'font: name Times New Roman, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;')
        title_center_hora = xlwt.easyxf('align: horiz center;' 'font: name Times New Roman, colour black, bold on, height 160; pattern: pattern solid, fore_colour white;')
        title2 = xlwt.easyxf('font: bold on, height 140;' 'align: horiz center;''borders: left thin, right thin, top thin')
        style1 = xlwt.easyxf('font: bold off, height 140;' 'align: horiz left;''borders: left thin, right thin, top thin')
        style3 = xlwt.easyxf('font: bold off, height 140;' 'align: horiz center;''borders: left thin, right thin, top thin')
        numbers = xlwt.easyxf('font: bold off, height 140;' 'align: horiz right;''borders: left thin, right thin, top thin', num_format_str='#,##0.00')
        numbers2 = xlwt.easyxf('font: bold on, height 140;' 'align: horiz right;''borders: left thin, right thin, top thin', num_format_str='#,##0.00')
        ws = wbk.add_sheet('Reposos medicos')
        
        filas = 12
        col = 0
        
        ws.col(0).width = 10000
        ws.write(0, 0, 'INDUSTRIAS DIANA C.A.', title_left)
        
        ws.col(0).width = 10000
        ws.write(1, 0, 'G200096144', title_left)
        
        ws.col(0).width = 10000
        ws.write(2, 0, 'AV Henry Ford Local Nro S/N Urb La Isabelica Sector Zona Industrial Sur II', title_left)
        
        ws.col(0).width = 10000
        ws.write(3, 0, 'Valencia-Venezuela', title_left)
        
        
        ws.col(0).width = 10000
        ws.write_merge(4, 4, 0, filas, 'RELACION DE REPOSOS DE LA SEMANA DEL LUNES ' + lunes + ' HASTA EL DOMINGO ' + domingo + '', title_center)
        
        
        ws.col(0).width = 10000
        ws.write_merge(5, 5, 0, filas, 'SEDE: ' + sede.upper() + ', NOMINA: ' + nomina.upper(), title_center)
        
        col = 6
        
        if data['ivss'] != []:
            ws.col(0).width = 10000
            ws.write_merge(col, col, 0, filas, 'REPOSOS IVSS', title_center)
            col += 1

            b = 0
            for i in cabezera:
                ws.write(col, b, i, title2)
                b += 1
            col += 1
            
            for ivss in data['ivss']:
                ws.write(col, 0, ivss[0] or '', title2)
                ws.write(col, 1, ivss[1] or '', title2)
                ws.write(col, 2, ivss[2] or '', title2)
                ws.write(col, 3, ivss[3] or '', title2)
                ws.write(col, 4, ivss[4] or '', title2)
                ws.write(col, 5, ivss[5] or '', title2)
                ws.write(col, 6, self.cvr_date(ivss[6]) or '', title2)
                ws.write(col, 7, self.cvr_date(ivss[7]) or '', title2)
                ws.write(col, 8, self.cvr_date(ivss[8]) or '', title2)
                ws.write(col, 9, ivss[9] or '', title2)
                ws.write(col, 10, ivss[10] or '', title2)
                ws.write(col, 11, ivss[11] or '', title2)
                col += 1
                
        ws.col(0).width = 10000
        ws.write_merge(col, col, 0, filas, '', title_center)
        col += 1
        if data['privado'] != []:
        
            ws.col(0).width = 10000
            ws.write_merge(col, col, 0, filas, 'REPOSOS DE OTROS CENTROS DE SALUD', title_center)
            col += 1
            
            b = 0
            for i in cabezera:
                ws.write(col, b, i, title2)
                b += 1
            col += 1
            
            for ivss in data['privado']:
                ws.write(col, 0, ivss[0] or '', title2)
                ws.write(col, 1, ivss[1] or '', title2)
                ws.write(col, 2, ivss[2] or '', title2)
                ws.write(col, 3, ivss[3] or '', title2)
                ws.write(col, 4, ivss[4] or '', title2)
                ws.write(col, 5, ivss[5] or '', title2)
                ws.write(col, 6, self.cvr_date(ivss[6]) or '', title2)
                ws.write(col, 7, self.cvr_date(ivss[7]) or '', title2)
                ws.write(col, 8, self.cvr_date(ivss[8]) or '', title2)
                ws.write(col, 9, ivss[9] or '', title2)
                ws.write(col, 10, ivss[10] or '', title2)
                ws.write(col, 11, ivss[11] or '', title2)
                col += 1
                
        ws.col(0).width = 10000
        ws.write_merge(col, col, 0, filas, '', title_center)
        
        col += 1
        
        ws.write(col, 0, 'TOTAL REPOSOS IVSS', title2)
        ws.write(col, 1, str(len(data['ivss'])), title2)
        col += 1
        ws.write(col, 0, 'TOTAL REPOSOS DE OTROS CENTROS DE SALUD', title2)
        ws.write(col, 1, str(len(data['privado'])), title2)
            
        try:
            file_data=StringIO.StringIO()
            o=wbk.save(file_data)
            return file_data.getvalue()
        except:
            raise osv.except_osv('Warning !','Error al crear el archivo')
    
    def create_single_pdf(self, cursor, uid, ids, data, report_xml, context=None):
        """generate the csv"""
        
        if context is None:
            context={}

        if report_xml.report_type != 'txt':
            return super(TxtParser,self).create_single_pdf(cursor, uid, ids, data, report_xml, context=context)

        self.parser_instance = self.parser(cursor, uid, self.name2, context=context)
        
        if report_xml.report_type == 'txt':
            if data:
                datos = self.obtener_datos(data, cursor, uid)
                if datos['ivss'] == [] and datos['privado'] == []:
                    raise osv.except_osv('Error !','La combinacion de filtros no arrojo ningun resultado')
                else:
                    xls_report = self.crear_xls(datos, data['form'], data['domingo'], data['sede_titulo'], data['nomina_titulo'])
                    return (xls_report, 'ods')
               
TxtParser(
    'report.reposos.medicos.report',
    'report.reposo.medico.cal',
    'sisb_reposos_medicos/report/report_reposo_medico_cal.rml',
    parser=report_reposo_medico_cal, header=False)    
                
