# -*- coding: utf-8 -*-
"""
Módulo utilizado para el registro de la recepción de cada buque y 
el registro del destino final del producto proveniente del buque.
"""

import time
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp
import toolssearch


#----------------------------------------------------------
#  Maestro de Buques
#----------------------------------------------------------

class sisb_ship_master(osv.osv):
    _name = "sisb.ship.master"
    _description = "Maestro de Buques"

    def _get_configuracion(self, cr, uid, context=None):
        """
        Funcion para extraer el proximo numero del buque
        """
        if context is None:
            context = {}
        
        qry_buque = """ select 'BQ-' || lpad(cast(cast(substring(code from '\d{' || cast( length(substr(code, strpos(code, '-')+1)) as text)||'}') as int) + 1 as text) , 4, '0') as codigo
                        from sisb_ship_master 
                        order by id desc limit 1  """
        cr.execute(qry_buque)
        datos = cr.fetchall()
        if datos[0][0]:
            return datos[0][0]
        return 'BQ-0001'
    

    _columns = {
        'code': fields.char('Code', size=14, select=True, required=True, readonly=True, help="The code of the product master."),
        'name': fields.char('Name', size=64, select=True, required=True),
        'permitted_location_dest_id': fields.many2many('stock.location', 'sisb_ship_master_stock_location_rel', 'sisb_ship_master_id', 'location_id', 'Location allowed', domain="[('usage','=','internal')]"),
        'mmsi': fields.char('MMSI', size=14, select=True, help="The MMSI of the product master."),
        'imo': fields.char('IMO', size=14, select=True, help="The IMO of the product master."),
    }
    
    _defaults = {
        'code': _get_configuracion,
    }
    
    _sql_constraints = [('unique_sisb_ship_master_code','unique(code)', _('The Code is unique')),
    ('unique_sisb_ship_master_name','unique(name)', _('The Name is unique'))]

    
    
    def onchange_ship(self, cr, uid, ids, name, context=None):
        if context is None:
            context = {}

        res=[]
        resultado = {}

        if name:
            lista_ship= []
            ship_master_ids = self.search(cr, uid, [], context=context)
            for ship_master in self.browse(cr, uid, ship_master_ids, context=context):
                lista_ship.append(ship_master.name)
                res = toolssearch.cadena_coincidencias(name,lista_ship)
        if res:
            texto = ''
            for x in res:
                texto+= str(x)+"\n"
            messaje_ship = _("The name of the ship can not be created, it is coincidence ") + texto
            resultado.update({'warning': {'message': messaje_ship}})
        return resultado
        
   
sisb_ship_master()


#----------------------------------------------------------
#  Maestro de Mercacias
#----------------------------------------------------------


class sisb_product_master(osv.osv):
    _name = "sisb.product.master"
    _description = "Maestro de Productos"

    _columns = {
        'product_id': fields.many2one('product.product', 'Product', required=True),
        'code' : fields.char('Code', size=14,help="The code of the product master.", required=True),
        'name': fields.char('Name', size=64, select=True, required=True),
        'product_uom': fields.many2one('product.uom', 'Unit of Measure', required=True),
    }
    
    _sql_constraints = [('unique_sisb_product_master_product_id','unique(product_id)', _('The Product is unique'))]
    
    
    def product_id_change(self, cr, uid, ids, product_id, context=None):
        """ Finds UoM of changed product.
        @param product_id: Id of changed product.
        @return: Dictionary of values.
        """
        
        resultado= {}
        if not product_id:
            return {'value': {
                'code': False,
                'product_uom': False,
                'name': False,
            }}
            
        product = self.pool.get('product.product').browse(cr, uid, product_id, context=context)
        result = {
            'product_uom': product.uom_id and product.uom_id.id or False,
            'code': product.default_code or False,
            'name': product.name_template or False,
        }
        
        resultado.update({'value': result})
        return resultado
        
    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        if not args:
            args=[]
        if name:
            ids = self.search(cr, user, [('code','=',name)]+ args, limit=limit, context=context)
            if not len(ids):
                ids = self.search(cr, user, ['|',('name',operator,name),('code',operator,name)] + args, limit=limit, context=context)
            if not len(ids):
               ptrn=re.compile('(\[(.*?)\])')
               res = ptrn.search(name)
               if res:
                   ids = self.search(cr, user, [('code','=', res.group(2))] + args, limit=limit, context=context)
        else:
            ids = self.search(cr, user, args, limit=limit, context=context)
        result = self.name_get(cr, user, ids, context=context)
        return result


sisb_product_master()



#----------------------------------------------------------
#  Maestro de Tipos
#----------------------------------------------------------



class sisb_transaction_type(osv.osv):
    _name = "sisb.transaction.type"
    _description = "Tipos de transacciones"
    
    
    _columns = {
        'code' : fields.char('Code', size=14, required=True, help="The code of the transaction type master."),
        'name': fields.char('Name', size=64, select=True, required=True),
        'location_dest_id': fields.many2one('stock.location', 'Destination Location', domain="[('usage','=','internal')]", select=True, required=True, help="Location where the system will stock the product to do transaction."),
        'transfer_between_warehousing': fields.boolean('Transfers between warehouse', help="By checking the active field, the location belong to warehouse of the company."),
        'is_own': fields.boolean('Is own'),
        'is_invoiced': fields.boolean('Is invoiced'),
        'type_invoice_transaction': fields.selection([('mark_to_invoice', 'Mark to invoice'), ('is_invoiced_auto', 'Is invoiced auto')], 'Type invoice of transaction', select=True,
            help='To select \'Mark to invoice\', must be active the field Is invoiced. Used for manual invoice.\
            \nTo select \'Is invoiced auto\', must be active the field Is invoiced. To allow invoice automatic.'),
    }
    
    _defaults = {
        'transfer_between_warehousing': False,
        'is_own': False,
        'is_invoiced': False,
        'type_invoice_transaction':  False,
    }
    
    
    def onchange_is_own(self, cr, uid, ids, is_own):
        result = {}
        if not is_own:
            result = {'value': {
                    'is_invoiced': False,
                    'type_invoice_transaction': False,
                    }
                }
            
        return result
    
    def onchange_is_invoiced(self, cr, uid, ids, is_invoiced):
        result = {}
        if not is_invoiced:
            result = {'value': {
                    'type_invoice_transaction': False,
                    }
                }
            
        return result
    
    
sisb_transaction_type()
