# -*- encoding: utf-8 -*-
##############################################################################
# Copyright (c) 2011 OpenERP Venezuela (http://openerp.com.ve)
# All Rights Reserved.
# Programmed by: Equipo SISB - Industrias Diana C.A.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################
from osv import osv
from osv import fields
import sys
from tools.translate import _
import decimal_precision as dp
import netsvc
from datetime import datetime

class sisb_assignment_cost_wizard(osv.osv_memory):
    """
    Assignment Cost
    """
    _name = "sisb.assignment.cost.wizard"
    
    def _get_product_uom_id(self, cr, uid, context=None):
        ship_recept_obj = self.pool.get('sisb.ship.reception')
        s_id = context.get('active_id',False)
        if s_id:
            product_uom_id = False
            product_uom_id = ship_recept_obj.browse(cr, uid, s_id, context=context).product_uom.id
            
        return product_uom_id 
    
    def _get_product_id(self, cr, uid, context=None):
        ship_recept_obj = self.pool.get('sisb.ship.reception')
        s_id = context.get('active_id',False)
        if s_id:
            product_id = False
            product_id = ship_recept_obj.browse(cr, uid, s_id, context=context).product_id.id
            
        return product_id 
        
    def _get_ship_reference(self, cr, uid, context=None):
        ship_recept_obj = self.pool.get('sisb.ship.reception')
        s_id = context.get('active_id',False)
        if s_id:
            reference = ''
            reference = ship_recept_obj.browse(cr, uid, s_id, context=context).reference
            
        return reference 
    
    
    _description = "Assignment Cost"
    _columns = {
       'cost_unit': fields.float('Unit Cost', digits_compute= dp.get_precision('Account'), required=True),
       'product_uom': fields.many2one('product.uom', 'Unit of Measure', required=True, readonly=True),
       'product_id': fields.many2one('sisb.product.master', 'Product Master', required=True, readonly=True),
       'ship_reference': fields.char('Reference', size=64, required=True, readonly=True),
    }
    
    _defaults = {
        'product_uom': _get_product_uom_id,
        'product_id': _get_product_id,
        'ship_reference': _get_ship_reference,
    }
    
    def assignment_cost(self, cr, uid, ids, context=None):
        data_unloading_cost = self.read(cr, uid, ids, [],context=context)[0]
        
        if data_unloading_cost['cost_unit'] <= 0.00:
            raise osv.except_osv(_('mensaje'),_('the cost assigned can not be zero.'))
        else:
            
            reception_ship_obj = self.pool.get('sisb.ship.reception')
            #preguntar por el estado
            if not reception_ship_obj.browse(cr, uid, context['active_id'], context={}).is_cost:
                result_process = reception_ship_obj.action_cost_assigned(cr, uid, context['active_id'], data_unloading_cost['cost_unit'], context=None)
                if result_process:
                    return {'type': 'ir.actions.act_window_close'}
                else:
                    raise osv.except_osv(_('mensaje'),_('There were errors in the process, contact the administrator.')) 
            else:
                raise osv.except_osv(_('mensaje'),_('Already the ship set cost.'))

sisb_assignment_cost_wizard()
