# -*- encoding: utf-8 -*-
##############################################################################
# Copyright (c) 2011 OpenERP Venezuela (http://openerp.com.ve)
# All Rights Reserved.
# Programmed by: Equipo SISB - Industrias Diana C.A.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################
from osv import osv
from osv import fields
import sys
from tools.translate import _
import decimal_precision as dp
import netsvc
from datetime import datetime
import random

class sisb_assignment_ship_wizard(osv.osv_memory):
    """
    Assignment
    """
    _name = "sisb.assignment.ship.wizard"
    
    _description = "Assignment Ship"
    _columns = {
       'partner_id': fields.many2one('res.partner', 'Customer', required=True, change_default=True, select=True),
       'product_qty': fields.float('Quantity', digits_compute=dp.get_precision('Account'), required=True),
       'transaction_type_id': fields.many2one('sisb.transaction.type', 'Transaction Type', select=True, required=True),
       'note': fields.text('Notes', required=True),
       'valid_item':fields.integer('Valid Item'),
    }
    
    _defaults = {
        'valid_item':lambda self,cr,uid,context : random.randrange(1, 100000000, 1)
    }
    
    def assignment_ship(self, cr, uid, ids, context=None):
        
        data_unloading_ship = self.read(cr, uid, ids, [],context=context)[0]
        
        #Consultar existencia
        reception_ship_brw = self.pool.get('sisb.ship.reception').browse(cr, uid, context['active_id'], context=context)
        
        product_exist = reception_ship_brw.product_available
        
        if data_unloading_ship['product_qty'] <= 0.00 or data_unloading_ship['product_qty'] > product_exist:
            raise osv.except_osv(_('mensaje'),_('The quantity of assignment can not be more than the existence.'))
        else:
            addr = self.pool.get('res.partner').address_get(cr, uid, [data_unloading_ship['partner_id']], ['invoice'])
            type_trans_brw = self.pool.get('sisb.transaction.type').browse(cr, uid, data_unloading_ship['transaction_type_id'], context=context)
            #Control factura
            if type_trans_brw.type_invoice_transaction == 'is_invoiced_auto':
                invoice_state_value = '2binvoiced'
            else:
                invoice_state_value = 'none'
            
            values = {
                'ship_reception_id': context['active_id'],
                'partner_id': data_unloading_ship['partner_id'],
                'partner_invoice_id': addr['invoice'],
                'product_qty': data_unloading_ship['product_qty'],
                'transaction_type_id': data_unloading_ship['transaction_type_id'],
                'location_id': reception_ship_brw.location_dest_id.id,
                'location_dest_id': type_trans_brw.location_dest_id.id,
                'date': datetime.now(),
                'note': data_unloading_ship['note'],
                'invoice_state': invoice_state_value,
                'valid_item': data_unloading_ship['valid_item'],
            }
            res = self.pool.get('sisb.ship.reception.detail').create(cr, uid, values, context)
            resultado = self.pool.get('sisb.ship.reception').action_assigned(cr,uid,[context['active_id']],context=context)

            if res and resultado:
                if reception_ship_brw.is_cost:
                    reception_ship_detail_obj = self.pool.get('sisb.ship.reception.detail')
                    #CREACION DE LOS ALBARANES INTERNOS
                    #***Para procesar alabaranes internos debe haber una condicion de que el albaran de entrada esta procesado***
                    if reception_ship_detail_obj.action_picking_create_from_detail(cr, uid, [res], context=context):
                        return {'type': 'ir.actions.act_window_close'}
                    else:
                        raise osv.except_osv(_('mensaje'),_('There were errors in the process, contact the administrator.')) 
                else:
                    return {'type': 'ir.actions.act_window_close'}
            else:
                raise osv.except_osv(_('mensaje'),_('There were errors in the process, contact the administrator.')) 
                
    def onchange_transaction_type_id(self, cr, uid, ids, transaction_type_id):
        result = {}
        valor02 = ''
        if transaction_type_id:
            valor01 = self.pool.get('sisb.transaction.type').browse(cr, uid, transaction_type_id).is_own
            if valor01:
                valor02 = self.pool.get('res.users').browse(cr, uid, uid).company_id.partner_id.id
        
        result = {'value': {
                'partner_id': valor02,
                }
            }
            
        return result

sisb_assignment_ship_wizard()
