# -*- encoding: utf-8 -*-
##############################################################################
# Copyright (c) 2011 OpenERP Venezuela (http://openerp.com.ve)
# All Rights Reserved.
# Programmed by:  Equipo SISB - Industrias Diana C.A.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################
from osv import osv
from osv import fields
import sys
from tools.translate import _
import decimal_precision as dp
import netsvc
from datetime import datetime

class sisb_confirmed_unloading_ship_wizard(osv.osv_memory):
    """
    Confirm landing ship
    """
    _name = "sisb.confirmed.unloading.ship.wizard"
    
    def _get_ship_locations_ids(self, cr, uid, context=None):
        ship_recept_obj = self.pool.get('sisb.ship.reception')
        s_id = context.get('active_id',False)
        if s_id:
            location_list = []
            ship_reception = ship_recept_obj.browse(cr, uid, s_id, context=context)
            for x in ship_reception.ship_id.permitted_location_dest_id:
                location_list.append(x.id)
            
        return str(location_list)
        
    def _get_product_uom_id(self, cr, uid, context=None):
        ship_recept_obj = self.pool.get('sisb.ship.reception')
        s_id = context.get('active_id',False)
        if s_id:
            product_uom_id = False
            product_uom_id = ship_recept_obj.browse(cr, uid, s_id, context=context).product_uom.id
            
        return product_uom_id 
    
    def _get_product_id(self, cr, uid, context=None):
        ship_recept_obj = self.pool.get('sisb.ship.reception')
        s_id = context.get('active_id',False)
        if s_id:
            product_id = False
            product_id = ship_recept_obj.browse(cr, uid, s_id, context=context).product_id.id
            
        return product_id 
        
    def _get_ship_reference(self, cr, uid, context=None):
        ship_recept_obj = self.pool.get('sisb.ship.reception')
        s_id = context.get('active_id',False)
        if s_id:
            reference = ''
            reference = ship_recept_obj.browse(cr, uid, s_id, context=context).reference
            
        return reference 
    
    _description = "Confirmed Unloading Ship"
    _columns = {
       'location_dest_id': fields.many2one('stock.location', 'Destination Location', required=True),
       'product_qty': fields.float('Real Quantity', digits_compute=dp.get_precision('Account'), required=True),
       'locations_unload': fields.text('Locations Unload'),
       'product_uom': fields.many2one('product.uom', 'Unit of Measure', required=True, readonly=True),
       'product_id': fields.many2one('sisb.product.master', 'Product Master', required=True, readonly=True),
       'ship_reference': fields.char('Reference', size=64, required=True, readonly=True),
       'note': fields.text('Notes', required=True),
    }
    
    _defaults = {
        'locations_unload': _get_ship_locations_ids,
        'product_uom': _get_product_uom_id,
        'product_id': _get_product_id,
        'ship_reference': _get_ship_reference,
    }
    
    
    def unloading_ship(self, cr, uid, ids, context=None):
        ship_recept_obj = self.pool.get('sisb.ship.reception')
        data_unloading_ship = self.read(cr, uid, ids, [],context=context)[0]
        
        ship_recept_brw = ship_recept_obj.browse(cr, uid, context['active_id'], context=context)
        
        if ship_recept_brw.state == 'waiting':
            
            if  data_unloading_ship['product_qty'] < 1:
                raise osv.except_osv(_('mensaje'),_('Set real quantity of unloading ship.'))
            elif round(abs(ship_recept_brw.product_uom_qty-data_unloading_ship['product_qty'])/ship_recept_brw.product_uom_qty,3) > 0.002:
                raise osv.except_osv(_('mensaje'),_('Exceeded the limit of decrease.'))
            else:
                data_update = {
                    'location_dest_id':data_unloading_ship['location_dest_id'],
                    'product_qty':data_unloading_ship['product_qty'],
                    'real_arrival_date': datetime.now(),
                    'note_discharged': data_unloading_ship['note'],
                    
                }
                
                resultado01 = ship_recept_obj.write(cr,uid,context['active_id'],data_update,context=context)
                resultado02 = ship_recept_obj.action_confirmed(cr,uid,[context['active_id']],context=context)
                
                
                if resultado01 and resultado02:
                    return {'type': 'ir.actions.act_window_close'}
                else:
                    raise osv.except_osv(_('mensaje'),_('There were errors in the process, contact the administrator.')) 
                    
        else:
            return {'type': 'ir.actions.act_window_close'}

sisb_confirmed_unloading_ship_wizard()
