# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv

from tools.translate import _

class sisb_ship_invoice_from_detail(osv.osv_memory):
    
    def lista_journal_permitidos(self, cr, uid):
            
        cr.execute("""select distinct aj.id from stock_location slp
            left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
            inner join diana_groups_location_rel dlr on dlr.location_id =slp.id
            inner join stock_warehouse sw on sw.lot_stock_id = slh.id
            inner join sale_shop ss on ss.warehouse_id = sw.id
            inner join account_journal aj on aj.sale_shop_id = ss.id 
            where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)""", (uid,))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
        
    def _get_journal_id(self, cr, uid, context=None):
        if context is None:
            context = {}
            
        model = context.get('active_model')
        if not model or model != 'sisb.ship.reception.detail':
            return []
        
        vals = []
        
        #journal de venta
        journal_obj = self.pool.get('account.journal')
        journal_type = ('sale',)
        value = journal_obj.search(cr, uid, [('type','in',journal_type )])
        
        #Filtrar journal de acuerdo al usuario conectado y sus permisos en ubicaciones
        jour_user = self.lista_journal_permitidos(cr, uid)
        
        #Filtrar listas
        res = list(set(jour_user) & set(value))
        res = res or []
        
        value_journal = journal_obj.search(cr, uid, [('id','in',res)])
        for jr_type in journal_obj.browse(cr, uid, value_journal, context=context):
            t1 = jr_type.id,jr_type.name
            if t1 not in vals:
                vals.append(t1)
        
        if not vals:
            raise osv.except_osv(_('Warning !'), _('Accounting Journals are misconfigured!'))
        
        return vals

    _name = "sisb.ship.invoice.from.detail"
    _description = "Create invoice from detail"

    _columns = {
        'journal_id': fields.selection(_get_journal_id, 'Destination Journal',required=True),
        'invoice_date': fields.date('Invoiced date'),
    }

    def view_init(self, cr, uid, fields_list, context=None):
        if context is None:
            context = {}
        res = super(sisb_ship_invoice_from_detail, self).view_init(cr, uid, fields_list, context=context)
        detail_obj = self.pool.get('sisb.ship.reception.detail')
        count = 0
        active_ids = context.get('active_ids',[])
        for detail in detail_obj.browse(cr, uid, active_ids, context=context):
            if detail.invoice_state != '2binvoiced':
                count += 1
        if len(active_ids) == 1 and count:
            raise osv.except_osv(_('Warning !'), _('This detail list does not require invoicing.'))
        if len(active_ids) == count:
            raise osv.except_osv(_('Warning !'), _('None of these detail lists require invoicing.'))
        return res

    def open_invoice(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        invoice_ids = []
        data_pool = self.pool.get('ir.model.data')
        res = self.create_invoice(cr, uid, ids, context=context)
        invoice_ids += res.values()
        inv_type = context.get('inv_type', False)
        action_model = False
        action = {}
        if not invoice_ids:
            raise osv.except_osv(_('Error'), _('No Invoices were created'))
        
        action_model,action_id = data_pool.get_object_reference(cr, uid, 'account', "action_invoice_tree1")
        
        if action_model:
            action_pool = self.pool.get(action_model)
            action = action_pool.read(cr, uid, action_id, context=context)
            action['domain'] = "[('id','in', ["+','.join(map(str,invoice_ids))+"])]"
        return action

    def create_invoice(self, cr, uid, ids, context=None):
        journal_mapping_type_inv = {
            'sale': 'out_invoice',
            'purchase': 'in_invoice',
            'sale_refund': 'out_refund',
            'purchase_refund': 'in_refund',
        }
        if context is None:
            context = {}
        detail_pool = self.pool.get('sisb.ship.reception.detail')
        onshipdata_obj = self.read(cr, uid, ids, ['journal_id', 'invoice_date'])
        journal_pool = self.pool.get('account.journal')
        
        context['date_inv'] = onshipdata_obj[0]['invoice_date']
        active_ids = context.get('active_ids', [])
        active_detail = detail_pool.browse(cr, uid, context.get('active_id',False), context=context)
        journal_type = journal_pool.browse(cr, uid, onshipdata_obj[0]['journal_id']).type
        inv_type = journal_mapping_type_inv[journal_type]
        context['inv_type'] = inv_type
        
        res = detail_pool.action_create_invoice_from_detail(cr, uid, active_ids,
              journal_id = onshipdata_obj[0]['journal_id'],
              invoice_type = inv_type,
              context=context)
        return res

sisb_ship_invoice_from_detail()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
