# -*- coding: utf-8 -*-
"""
Módulo utilizado para el registro de la recepción de cada buque y 
el registro del destino final del producto proveniente del buque.
"""

import time
from datetime import datetime, timedelta
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp

#----------------------------------------------------------
#  Datos de la recepción del Buque
#----------------------------------------------------------


class sisb_ship_reception(osv.osv):
    _inherit = "sisb.ship.reception"
    
    #Para saber la disponibilidad del diferencial descargado
    def _product_differential_available(self, cr, uid, ids, name, arg, context=None):
        if context is None:
            context = {}
            
        res = {}
        
        for ship_reception in self.browse(cr, uid, ids, context=context):
            s_id = ship_reception.id
            res[s_id] = ship_reception.product_qty_diff
            for mov_det in ship_reception.ship_reception_lines_differential:
                res[s_id] -= mov_det.product_qty
        return res
    
    _columns = {
        'is_differential': fields.boolean('Is Differential', select=True, readonly=True, states={'draft': [('readonly', False)]}),
        'diferential_partner_id': fields.many2one('res.partner', 'Partner Responsible', select=True, readonly=True, states={'draft': [('readonly', False)]}, help="Partner responsible of differential."),
        'product_qty_diff': fields.float('Quantity Differential', digits_compute=dp.get_precision('Account'), readonly=True),
        'product_differential_available': fields.function(_product_differential_available, method=True, type='float', string='Product Differential Available', digits_compute=dp.get_precision('Account'), store = False), 
        'ship_reception_lines_differential': fields.one2many('sisb.ship.reception.detail.diferential', 'ship_reception_id', 'Detail Moves Differential', select=True, readonly=True),
    }
    
    _defaults = {
        'is_differential': False,
        'product_qty_diff': 0.00,
    }
    
    def onchange_is_differential(self, cr, uid, ids, is_differential):
        result = {}
        if not is_differential:
            result = {'value': {
                    'diferential_partner_id': False,
                    }
                }
            
        return result
    
sisb_ship_reception()



#----------------------------------------------------------
#  Datos del envío del destino final del producto
#----------------------------------------------------------

class sisb_ship_reception_detail_diferential(osv.osv):
    _name = "sisb.ship.reception.detail.diferential"
    _order = "date desc"
    
    _columns = {
        'name': fields.related('ship_reception_id', 'reference', type='char', size=64, string='Name', store=True, select=True),
        'partner_id': fields.many2one('res.partner', 'Customer', required=True, select=True),
        'partner_invoice_id': fields.many2one('res.partner.address', 'Invoice Address', required=True, help="Invoice address for current partner."),
        'product_id': fields.related('ship_reception_id','product_id',type='many2one',relation='sisb.product.master',string='Product', store=True, select=True),
        'product_qty': fields.float('Quantity', digits_compute=dp.get_precision('Account'), required=True),
        'product_uom': fields.related('ship_reception_id','product_uom',type='many2one',relation='product.uom',string='Product UoM', store=True, select=True),
        'ship_reception_id': fields.many2one('sisb.ship.reception', 'Ship Reception', select=True, required=True),
        'date': fields.date('Date', help="Date", select=True, required=True),
        'note': fields.text('Notes', required=True),
        'valid_item':fields.integer('Valid Item'),
    }
    
    _defaults = {
        'partner_invoice_id': lambda self, cr, uid, context: context.get('partner_id', False) and self.pool.get('res.partner').address_get(cr, uid, [context['partner_id']], ['invoice'])['invoice'],
    }
    
    _sql_constraints = [
        ('uniq_ship_recept_detail_diff_id_valid_item_relation', 'unique (ship_reception_id, valid_item)', 'The ship reception detail differential and the valid item must be unique !')
    ]
    
    def copy(self, cr, uid, ids, defaults, context=None):
        raise osv.except_osv(_('Error !'), _('You can not duplicated'))
        
    def unlink(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Error !'), _('You can not deleted'))
    
sisb_ship_reception_detail_diferential()

