# -*- coding: utf-8 -*-


import time
from datetime import datetime, timedelta
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp


class sisb_ship_reception_notification_conf(osv.osv):
    _name = "sisb.ship.reception.notification.conf"
    _description = "Configuracion"
    _columns = {
        'name': fields.char('Nombre', size=256, required=True),
        'process_cron_id': fields.many2one('ir.cron', 'MTA Process', readonly=True),
        'interval_update': fields.integer('Numero de Intervalos'),
        'state_server': fields.selection([('start','Iniciado'), ('stop','Detenido')],'Estado',required=True,readonly=True),
        'time_interval': fields.selection( [('minutes', 'Minutos'), ('hours', 'Horas'), ('work_days','Dias laborales'), 
                                          ('days', 'Dias'),('weeks', 'Semanas'), ('months', 'Meses')], 'Unidad de intervalo', required=True),
        'users_ids':fields.many2many('res.users', 'users_notification_rel', 'user_id', 'notificacion_id', 'Usuarios', required=True),
        'activo': fields.boolean("Activo", readonly=True),
    }

    _defaults = {
    'state_server':'stop',
    'activo':False,
    }
    
    
    def funtion_cron_star(self, cr, uid, ids, context=None):
        if isinstance(ids, list):
            ids = ids[0]
        uid = 1
        ir_cron_obj = self.pool.get('ir.cron')
        brw = self.browse(cr, uid, ids)  
        res = {
        'name':brw.name,
        'model':'sisb.ship.reception.notification.cron',
        'args': '(%s,%s,%s)'%(ids, [users.user_email for users in brw.users_ids], context), 
        'function':'ship_reporting_status',
        'priority':5,
        'interval_number':brw.interval_update, 
        'interval_type':brw.time_interval,
        'user_id':uid,
        'numbercall': -1,
        'doall':False,
        'active':True
        }
        cron_id = ir_cron_obj.create(cr, uid, res)
        self.write(cr, uid, ids, {'process_cron_id':cron_id})
        self.write(cr, uid, ids, {'state_server':'start'})
        return True
    
    def funtion_cron_stop(self, cr, uid, ids, context=None):
            brw = self.browse(cr, uid, ids)
            for i in brw:
                self.pool.get('ir.cron').write(cr, uid, i.process_cron_id.id, {'active':False})   
                self.pool.get('ir.cron').unlink(cr, uid, i.process_cron_id.id)    
            self.write(cr, uid, ids, {'state_server':'stop'})
            return True


    def copy(self, cr, uid, ids, defaults, context=None):
        raise osv.except_osv(_('Error !'), _('No se puede duplicar'))
    
    def unlink(self, cr, uid, ids, context=None):
        if isinstance(ids, list):
            ids = ids[0]
        brw = self.browse(cr, uid, ids)
        if brw.state_server == 'start':
            raise osv.except_osv(_('ERROR!'), _('No se puede eliminar la configuracion si el cron esta activo'))
        return super(sisb_ship_reception_notification_conf, self).unlink(cr, uid, ids)
        
        
    def create(self, cr, uid, vals, context=None):
        ids = self.search(cr, uid, [('activo','=', True)]) 
        if ids:
            raise osv.except_osv(_('ERROR!'), _('No pueden haber dos configuraciones activas'))
        else:
            vals['activo']=True 
        return super(sisb_ship_reception_notification_conf, self).create(cr, uid, vals)
    
sisb_ship_reception_notification_conf()


class sisb_ship_reception_notification_cron(osv.osv):
    _name = "sisb.ship.reception.notification.cron"
    _description = "Funciones cron"
    
    def list_2_typle(self, list):
        list = [str(i) for i in list]    
        t = ()
        if len(list) == 1:
           t = "('%s')"%"".join(list)
        else:
            t = tuple(list)
        return str(t)
        
    def send_email_notification(self, cr, uid, ids, subject, message, email_to, context=None):
        smtp_client = self.pool.get('email.smtpclient')
        smtp_server = smtp_client.search(cr, uid, [('active','=',True),
                                        ('state','=','confirm')], context=False)
        if smtp_server:
            if email_to:
                smtp_server_id = smtp_server[0]
                server_name = smtp_client.browse(cr, uid, smtp_server_id).server
                email = {
                        'server_id': smtp_server_id,
                        'name': (subject),
                        'server': server_name,
                        'body': (message),
                        'to': email_to,
                        'from': uid,
                        }
                smtp_client.send_email(cr, uid, smtp_server_id, email['to'], email['name'], email['body'])

    
    def call_detail_move(self, cr, uid, object):
        list_details = []
        for line in object.ship_reception_lines:
            detail =  {'subject':'', 'message':''}
            if line.invoice_state == "2binvoiced":
               detail['subject'] = "Pendiente por facturar %s"%object.reference
               detail['message'] = """El Buque %s Referencia %s con %s %s 
                                      del producto %s se encuentra 
                                      en estado pendiente por facturar a la empresa
                                      %s
                                   """%(object.ship_id.name, object.reference,
                                        object.product_uom_qty,object.product_uom.name,
                                        object.product_id.name, line.partner_id.name)
               list_details.append(detail)
        return list_details

    def call_log(self, cr, uid, object):
        list_log = []
        lines_update = []
        for line in object.log_ship_reception_lines:
            log =  {'subject':'', 'message':''}
            if not line.is_notified:
               log['subject']="Cambio de estado %s"%line.name
               log['message']=line.description 
               lines_update.append(line.id)
               list_log.append(log)
        if lines_update:
            sql = "UPDATE sisb_log_ship_reception SET is_notified = True where id in %s"%self.list_2_typle(lines_update)
            cr.execute(sql)
            cr.commit()
        return list_log
		
    def alarm(self, cr, uid, state, object, context=None):
        mensajes = []
        if state in ['waiting','confirmed']:
            msg = getattr(self, 'alarm_'+state)(cr, uid, object, context=context)
            mensajes.append(msg)
        log_msg = self.call_log(cr, uid, object)
        movel_detail = self.call_detail_move(cr, uid, object)
        mensajes = mensajes + log_msg + movel_detail
        return mensajes
        
    def alarm_cancel(self, cr, uid, object, context=None):
        res =  {'subject':'', 'message':''}
        return res
        
    def alarm_done(self, cr, uid, object, context=None):
        res =  {'subject':'', 'message':''}
        return res
        
    def alarm_assigned(self, cr, uid, object, context=None):
        res =  {'subject':'', 'message':''}
        return res
        
    def alarm_draft(self, cr, uid, object, context=None):
        res =  {'subject':'', 'message':''}
        return res

    def alarm_waiting(self, cr, uid, object, context=None):
        res =  {'subject':'', 'message':''}
        format = "%Y-%m-%d"
        date_now = datetime.now().strftime(format)
        date_now = datetime.strptime(date_now, format)
        interval_date = 1
        arrival_date = datetime.strptime(object.arrival_date, format)
        diff_alarm = abs(arrival_date - date_now)
        if diff_alarm.days == interval_date:
           res['subject'] = "Recepcion %s"%object.reference
           res['message'] = """El Buque %s Referencia %s tiene una fecha 
                    planificada para el %s, falta %s dia(s) por llegar.
                    """%(object.ship_id.name, object.reference, arrival_date.strftime("%d/%m/%Y"), diff_alarm.days)
                    
        if date_now > arrival_date:
            res['subject'] = "Retardo %s"%object.reference
            res['message'] = """El Buque %s Referencia %s tiene una fecha planificada
                    para el %s, tiene %s dia(s) de retardo.
                    """%(object.ship_id.name, object.reference, 
                        arrival_date.strftime("%d/%m/%Y"), diff_alarm.days)
        return res
    
    def alarm_confirmed(self, cr, uid, object, context=None):
        res =  {'subject':'', 'message':''}
        if not object.is_cost:
            res['subject'] = "Pendiente por Asignacion de Costo %s"%object.reference
            res['message'] = "El Buque %s Referencia %s no se le ha asignado aun el costo"%(object.ship_id.name, object.reference)
        return res
            
    
    def search_states(self, cr, uid, context):
        res_states = {}
        ship_obj = self.pool.get('sisb.ship.reception')
        states_ids = ship_obj.search(cr, uid, [])
        states_brw = ship_obj.browse(cr, uid, states_ids)
        for sb in states_brw:
            res_states[sb.state] = []
        for sb in states_brw:
            res_states[sb.state].append(sb)
        return res_states 
    
    def ship_reporting_status(self, cr, uid, ids, emails, context=None):
        states_cron = self.search_states(cr, uid, context)
        list_msjs=[]
        for state in states_cron:
            for obj in states_cron[state]:
                msjs = self.alarm(cr, uid, state, obj, context=context)
                list_msjs = list_msjs + msjs
        
        for mail in list_msjs:
            if  mail['subject'] != "" and mail['message'] != "":
				self.send_email_notification(cr, uid, ids, mail['subject'], mail['message'], emails, context=context)
        return True 
        
sisb_ship_reception_notification_cron()
    
    
    


