# -*- coding: utf-8 -*-
"""
Módulo utilizado para incluir presupuesto en el proceso de recepcion de buque del SISB.
Al facturar el detalle de recepción de buque se registre la partida presupuestaria y la accion especifica.
"""

import time
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp


#----------------------------------------------------------
# Detalle de Recepcion
#----------------------------------------------------------

class sisb_ship_reception_detail(osv.osv):
    _name = 'sisb.ship.reception.detail'
    _inherit = 'sisb.ship.reception.detail'
   

    def action_create_invoice_from_detail(self, cr, uid, ids, journal_id=False, invoice_type='out_invoice', context=None):
        #Crear factura
        """ Crea la factura de los detalles de movimientos para facturar
        @param ids: ids de detalle de recepcion
        @param journal_id: diario de venta para la factura
        @param invoice_type: Tipo de factura que debe crearse
        @return: Ids de la factura creada para el detalle de movimiento
        """

        if context is None:
            context = {}
            
        if isinstance(ids, (int, long)):
            ids = [ids]
            
        if not ids:
            return {}

        res = super(sisb_ship_reception_detail, self).action_create_invoice_from_detail(cr, uid,  ids, journal_id, invoice_type, context)
        
        inv_obj = self.pool.get('account.invoice')
        inv_line_obj = self.pool.get('account.invoice.line')
        
        #Actualizar las lineas de las facturas creadas, colocando la accion especifica y la partida presupuestaria en sus lineas
        facturas_creadas = list(set(res.values()))
        
        
        
        #Consutar facturas creadas
        for fact in inv_obj.browse(cr, uid, facturas_creadas):
            #Consultar Accion Especifica
            accion_esp = fact.journal_id.sale_shop_id.accion_especifica_id.id or False
            for linea_fact in fact.invoice_line:
                #Consultar producto
                producto_partida = linea_fact.product_id.partida_ingreso and linea_fact.product_id.partida_ingreso.id or False
                #Actualizar
                values = {
                    'accion_especifica_id': accion_esp,
                    'partida_presupuesto': producto_partida,
                }
                line_update = inv_line_obj.write(cr, uid, linea_fact.id, values, context)

        return res

sisb_ship_reception_detail()
