#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import fields, osv
from lxml import etree
from tools.translate import _


class hr_variable_concepts(osv.osv):
    _inherit = 'hr.variable.concepts'

    def __init__(self, *args, **kargs):
        super(hr_variable_concepts, self).__init__(*args, **kargs)
        if 'tipo_de_proceso' in self._columns:
            if ('turnos', 'Calculo de turno') not in self._columns['tipo_de_proceso'].selection:
                self._columns['tipo_de_proceso'].selection.append(('turnos', 'Calculo de turno'))
        else:
            self._columns['tipo_de_proceso'] = fields.selection([('turnos', 'Calculo de turno')], "Tipo de proceso")

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        view = super(hr_variable_concepts, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar, submenu=submenu)
        if view_type == 'form':
            doc = etree.XML(view['arch'])
            # In case 'tipo_de_proceso' field is not in the view, add it to compare it
            node = doc.xpath("//field[@name='tipo_de_proceso']")
            if not node:
                tdp = etree.Element('field', name='tipo_de_proceso', invisible="1")
                if 'tipo_de_proceso' not in view['fields']:
                    view['fields'].update(self.pool.get('hr.variable.concepts').fields_get(cr, uid, fields=['tipo_de_proceso'], context=context))
                doc.append(tdp)
            # Now, we must iterate all fields to set them all readonly
            for n in doc.xpath("//field"):
                attrs = eval(n.get('attrs', '{}'))
                if attrs:
                    if 'readonly' in attrs:
                        attrs['readonly'].insert(0, '|')
                        for i, d in enumerate(attrs['readonly']):
                            f, o, v = d if type(d) == tuple else False, False, False
                            if f == 'tipo_de_proceso':
                                if o == 'in':
                                    v = v + ['turnos'] if type(v) == list else v + ('turnos',)
                                else:  # assuming '=' is the operator, 'not in' or another operator should not be used
                                    o, v = 'in', ['turnos', v]
                                attrs['readonly'][i] = (f, v, o)
                                break
                        else:
                            attrs['readonly'].append(('tipo_de_proceso', 'in', ('turnos',)))
                    else:
                        attrs.update({'readonly': [('tipo_de_proceso', 'in', ('turnos',))]})
                    n.set('attrs', str(attrs))
                else:
                    n.set('attrs', "{'readonly': [('tipo_de_proceso', 'in', ('turnos',))]}")
            view['arch'] = etree.tostring(doc)
        return view

    def unlink(self, cr, uid, ids, context=None):
        context = {} if context is None else context
        for vc in self.browse(cr, uid, ids, context=context):
            if vc.tipo_de_proceso == 'turnos':
                context.update({"eliminar_turno": True})
                self.pool.get('hr.variable.concept.line').unlink(cr, uid, map(int, vc.variable_concept_line_ids), context=context)
            else:
                context.pop('eliminar_turno', None)
        return super(hr_variable_concepts, self).unlink(cr, uid, ids, context=context)


hr_variable_concepts()


class hr_variable_concept_line(osv.osv):
    _inherit = 'hr.variable.concept.line'

    def unlink(self, cr, uid, ids, context=None):
        ids = ids if isinstance(ids, list) else [ids]
        for line in self.browse(cr, uid, ids, context=context):
            if line.variable_concept_id.tipo_de_proceso == 'turnos' and not context.get('eliminar_turno', False):
                raise osv.except_osv(_('ERROR'), _('Las líneas de conceptos variables de turnos no pueden ser borradas directamente'))
        return super(hr_variable_concept_line, self).unlink(cr, uid, ids, context=context)


hr_variable_concept_line()
