#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import fields, osv
from tools.translate import _


class sisb_turn_calcs(osv.osv):
    def _check_overlap(self, cr, uid, ids, context=None):
        return all(map(lambda r: self.search(cr, uid, [('day_start', '<=', r.day_end), ('day_end', '>=', r.day_start), ('concept_id', '=', r.concept_id.id), ('id', '!=', r.id), ('hr_hours_per_day_id', '=', r.hr_hours_per_day_id.id)], context=context, count=True) == 0, self.browse(cr, uid, ids, context=context)))

    def _check_qty(self, cr, uid, ids, context=None):
        return all(map(lambda x: x.qty > 0, self.browse(cr, uid, ids, context=context)))

    def _check_dates(self, cr, uid, ids, context=None):
        return all(map(lambda r: r.day_start <= r.day_end, self.browse(cr, uid, ids, context=context)))

    _name = "sisb.turn.calcs"
    _description = "Turn Calcs"
    _rec_name = "concept_id"
    _columns = {
        'day_start': fields.selection([(1, 'Monday'), (2, 'Tuesday'), (3, 'Wednesday'), (4, 'Thursday'), (5, 'Friday'), (6, 'Saturday'), (7, 'Sunday')], 'Start day', required=True),
        'day_end': fields.selection([(1, 'Monday'), (2, 'Tuesday'), (3, 'Wednesday'), (4, 'Thursday'), (5, 'Friday'), (6, 'Saturday'), (7, 'Sunday')], 'End day', required=True),
        'concept_id': fields.many2one('hr.concept', 'Assignation Concept', required=True, domain=[('type_calc', '=', 'allocation'), ('type', '=', 'variable')]),
        'deduct_concept_id': fields.many2one('hr.concept', 'Deduction Concept', required=True, domain=[('type_calc', '=', 'deduction'), ('type', '=', 'variable')]),
        'qty': fields.float('Quantity', required=True),
        'holidays': fields.boolean('Holidays', help='Check if this rule must be applied on holidays also'),
        'hr_hours_per_day_id': fields.many2one('hr.hours.per.day', 'Turn')
    }
    _defaults = {
        'day_start': 1,
        'day_end': 7
    }
    _constraints = [
        (_check_qty, 'Quantity must be positive', ['qty']),
        (_check_overlap, 'This rule already exists for this turn', ['day_start', 'day_end', 'concept_id']),
        (_check_dates, 'Start day must be greater than end day', ['day_start', 'day_end'])
    ]


sisb_turn_calcs()
