#!/usr/bin/env python
# -*- coding: utf-8 -*-
from osv import fields, osv
from tools.translate import _
from logging import getLogger
from datetime import datetime, timedelta
from re import search as regex


class turns_variable_concepts(osv.osv_memory):
    _name = 'turns.variable.concepts'
    _description = 'Wizard to create turns variable concepts sheet'
    _columns = {
        'hr_process_id': fields.many2one('hr.process', 'Process', domain=[('type', '=', 'nomina'), ('state', 'in', ['draft', 'load', 'review'])], required=True),
    }
    _logger = getLogger('turns.variable.concepts')

    def days_in_range(self, start, end):
        start, end = regex('\d{4}-\d{2}-\d{2}', start).group(0), regex('\d{4}-\d{2}-\d{2}', end).group(0)
        res = set()
        while start <= end:
            res.add(start)
            start = (datetime.strptime(start, '%Y-%m-%d') + timedelta(days=1)).strftime('%Y-%m-%d')
        return res

    def create_sheet(self, cr, uid, ids, context=None):
        _stc_obj = self.pool.get('sisb.turn.calcs')
        _hem_obj = self.pool.get('hr.employee')
        _hvc_obj = self.pool.get('hr.variable.concepts')
        _hvcl_obj = self.pool.get('hr.variable.concept.line')
        _hsw_obj = self.pool.get('hr.shift.week')
        _hpr_obj = self.pool.get('hr.process')
        _hpe_obj = self.pool.get('hr.period')
        _hho_obj = self.pool.get('hr.holidays')
        _hps_obj = self.pool.get('hr.payslip')
        _srmcvh_obj = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')
        days = {1: 'monday', 2: 'tuesday', 3: 'wednesday', 4: 'thursday', 5: 'friday', 6: 'saturday', 7: 'sunday'}
        # Getting all turns bonuses parameters
        bonuses_ids = _stc_obj.search(cr, uid, [], context=context)
        hsw_args = []  # args to query hr_shift_week table
        bonuses = {}  # Dic for bonuses in weekday:hour_per_day format
        for bonus in _stc_obj.browse(cr, uid, bonuses_ids, context=context):
            for day in range(bonus.day_start, bonus.day_end + 1):
                hsw_args.append((days[day] + '_hours_day_id', '=', bonus.hr_hours_per_day_id.id))
                if str(days[day]) + ':' + str(bonus.hr_hours_per_day_id.id) not in bonuses:
                    bonuses[str(days[day]) + ':' + str(bonus.hr_hours_per_day_id.id)] = []
                bonuses[str(days[day]) + ':' + str(bonus.hr_hours_per_day_id.id)].append(bonus)
        # Building args in polish notation
        hsw_args = ['|' for hsw in range(len(hsw_args) - 1)] + hsw_args
        for record in self.browse(cr, uid, ids, context=context):
            process_id = record.hr_process_id.id
            # Dic of employee:concepts for hr.variable.concept.line
            vc_lines = {}
            # Checking if there is already a sheet for turns created
            sheets = _hvc_obj.search(cr, uid, [('hr_process_id', '=', process_id), ('tipo_de_proceso', '=', 'turnos'), ('state', 'in', ['draft', 'load', 'review'])], context=context)
            if not sheets:
                sheet = _hvc_obj.create(cr, uid, {'hr_process_id': process_id, 'tipo_de_proceso': 'turnos', 'state': 'load'}, context=context)
                # Iterating per period type
                for period_brw in _hpr_obj.browse(cr, uid, process_id, context=context).hr_period_ids:
                    # Getting start and end date per process for last period, not the current one
                    # date_end of past period is date_start -1 from current period
                    past_date_end = (datetime.strptime(period_brw.date_start, '%Y-%m-%d') - timedelta(days=1)).strftime('%Y-%m-%d')
                    pde_id = _hpe_obj.search(cr, uid, [('date_end', '=', past_date_end), ('type', '=', period_brw.type)], context=context)
                    try:
                        period = _hpe_obj.browse(cr, uid, pde_id[0], context=context)
                        date_start, date_end = period.date_start, period.date_end
                    except:
                        raise osv.except_osv(_('Error'), _('Past period not found for process %s') % _hpr_obj.browse(cr, uid, process_id, context=context).name)
                    # Getting holidays from periods
                    holidays_ids = self.pool.get('hr.holidays.calendar').search(cr, uid, [('date', '>=', date_start), ('date', '<=', date_end)], context=context)
                    holidays = [x.date for x in self.pool.get('hr.holidays.calendar').browse(cr, uid, holidays_ids, context=context)]
                    # Getting employees cod_number from current process
                    employees_cods = {r.cod_number for r in _hpr_obj.browse(cr, uid, process_id, context=context).hr_payslip_ids}
                    employees_cods |= {r.cod_number for r in _hpr_obj.browse(cr, uid, process_id, context=context).hr_employee_ids}
                    # Getting employees absence days
                    absences = {i: set() for i in employees_cods}
                    absence_ids = _hho_obj.search(cr, uid, [('employee_id', 'in', list(employees_cods)), ('date_from', '<=', date_end), ('date_to', '>=', date_start), ('type', '=', 'remove'), ('state', '=', 'validate')], context=context)
                    for absence in _hho_obj.browse(cr, uid, absence_ids, context=context):
                        absences[absence.employee_id.cod_number] = absences[absence.employee_id.cod_number] | self.days_in_range(absence.date_from, absence.date_to)
                    # Getting days from period
                    days_from_period = self.days_in_range(date_start, date_end)
                    hsw_ids = _hsw_obj.search(cr, uid, ['&', '&', '&'] + hsw_args + [('period_id.date_end', '>=', date_start), ('period_id.date_start', '<=', date_end), ('employee_id.cod_number', 'in', list(employees_cods)), ('employee_id.hr_payroll_id.hr_payment_calendar_id.type', '=', period_brw.type)], context=context)
                    duped_hsw = {}
                    # Saving assignation concepts
                    for hsw_data in _hsw_obj.browse(cr, uid, hsw_ids, context=context):
                        if str(hsw_data.employee_id.cod_number) + ':' + str(hsw_data.period_id.id) not in duped_hsw:
                            duped_hsw[str(hsw_data.employee_id.cod_number) + ':' + str(hsw_data.period_id.id)] = True
                            check_date = hsw_data.period_id.date_start
                            for k, day in days.iteritems():
                                if day + ':' + str(hsw_data[day + '_hours_day_id'].id) in bonuses and check_date in days_from_period and check_date not in absences[hsw_data.employee_id.cod_number]:
                                    for bonus in bonuses[day + ':' + str(hsw_data[day + '_hours_day_id'].id)]:
                                        # Checking if the bonus must be applied on holidays
                                        if check_date not in holidays or bonus.holidays:
                                            if str(hsw_data.employee_id.cod_number) + ':' + str(bonus.concept_id.id) in vc_lines:
                                                vc_lines[str(hsw_data.employee_id.cod_number) + ':' + str(bonus.concept_id.id)]['value'] += bonus.qty
                                            else:
                                                vc_lines[str(hsw_data.employee_id.cod_number) + ':' + str(bonus.concept_id.id)] = {
                                                    'employee_id': hsw_data.employee_id.id,
                                                    'concept_id': bonus.concept_id.id,
                                                    'type': 'qty',
                                                    'value': bonus.qty,
                                                    'variable_concept_id': sheet
                                                }
                                check_date = (datetime.strptime(check_date, '%Y-%m-%d') + timedelta(days=1)).strftime('%Y-%m-%d')
                    # Next is to calculate bonuses that had not to be paid
                    reposo_medico_ids = _hvc_obj.search(cr, uid, [('hr_process_id', '=', process_id), ('para_reposos_medicos', '=', True)], context=context)
                    if reposo_medico_ids:
                        employees_ids, employees_cods = zip(*{(vc_line.employee_id.id, vc_line.employee_id.cod_number) for vc_line in sum([rmb.variable_concept_line_ids for rmb in _hvc_obj.browse(cr, uid, reposo_medico_ids, context=context)], [])})
                        srmcvh_ids = _srmcvh_obj.search(cr, uid, [('trabajador_id', 'in', list(employees_ids)), ('variable_concepts_id', 'in', reposo_medico_ids)], context=context)
                        delayed_holidays = {e: set() for e in employees_cods}
                        for srmcvh in _srmcvh_obj.browse(cr, uid, srmcvh_ids, context=context):
                            delayed_holidays[srmcvh.trabajador_id.cod_number] = delayed_holidays[srmcvh.trabajador_id.cod_number] | set(filter(lambda x: x < date_end, self.days_in_range((datetime.strptime(regex('\d{4}-\d{2}-\d{2}', srmcvh.reposo_medico_id.fecha_inicio).group(0), '%Y-%m-%d') + timedelta(days=srmcvh.dias_reposo_acumulados - srmcvh.dias_reposo_cargados)).strftime('%Y-%m-%d'), srmcvh.reposo_medico_id.fecha_fin))) - days_from_period
                        # Now we must get each period matching the employees absences
                        for emp_code in delayed_holidays:
                            min_date = min(delayed_holidays[emp_code]) if delayed_holidays[emp_code] else None
                            max_date = max(delayed_holidays[emp_code]) if delayed_holidays[emp_code] else None
                            # --------------- must find employee payroll ----------------
                            emp_payroll_type = _hem_obj.browse(cr, uid, _hem_obj.search(cr, uid, [('cod_number', '=', emp_code)], context=context)[0], context=context).hr_payroll_id.hr_payment_calendar_id.type
                            nom_period_ids = _hpe_obj.search(cr, uid, [('date_start', '<=', max_date), ('date_end', '>=', min_date), ('type', '=', emp_payroll_type)], context=context)
                            # Getting holidays from periods
                            holidays_ids = self.pool.get('hr.holidays.calendar').search(cr, uid, [('date', '>=', min_date), ('date', '<=', max_date)], context=context)
                            holidays = [x.date for x in self.pool.get('hr.holidays.calendar').browse(cr, uid, holidays_ids, context=context)]
                            # Looping for each nom_period
                            for p in _hpe_obj.browse(cr, uid, nom_period_ids, context=context):
                                # Paid bonuses for current employee
                                post_nom_period_ids = _hpe_obj.search(cr, uid, [('date_start', '=', (datetime.strptime(p.date_end, '%Y-%m-%d') + timedelta(days=1)).strftime('%Y-%m-%d')), ('type', '=', emp_payroll_type)])
                                hvcl_ids = _hvcl_obj.search(cr, uid, [('variable_concept_id.tipo_de_proceso', '=', 'turnos'), ('variable_concept_id.hr_process_id.hr_period_ids', 'in', post_nom_period_ids), ('employee_id.cod_number', '=', emp_code), ('variable_concept_id.state', '=', 'done')], context=context)
                                paid_bonuses = dict()
                                for vc in _hvcl_obj.browse(cr, uid, hvcl_ids, context=context):
                                    if vc.concept_id.id in paid_bonuses:
                                        paid_bonuses[vc.concept_id.id] += vc.value
                                    else:
                                        paid_bonuses[vc.concept_id.id] = vc.value
                                date_start, date_end = p.date_start, p.date_end
                                period_ids = _hpe_obj.search(cr, uid, [('date_start', '<=', date_end), ('date_end', '>=', date_start), ('type', '=', 'diario')], context=context)
                                emp_hsw_ids = _hsw_obj.search(cr, uid, [('period_id', 'in', period_ids), ('employee_id.cod_number', '=', emp_code)], context=context)
                                # Getting employees absence days
                                absences = []
                                absence_ids = _hho_obj.search(cr, uid, [('employee_id.cod_number', '=', emp_code), ('date_from', '>=', date_end), ('date_to', '>=', date_start), ('type', '=', 'remove'), ('state', '=', 'validate')], context=context)
                                for absence in _hho_obj.browse(cr, uid, absence_ids, context=context):
                                    absences += self.days_in_range(absence.date_from, absence.date_to)
                                # calculating real bonuses
                                days_from_period = self.days_in_range(date_start, date_end)
                                real_bonuses = {}
                                for bc in bonuses:
                                    for b in bonuses[bc]:
                                        real_bonuses[b.concept_id.id] = {
                                            'concept_id': b.deduct_concept_id.id,
                                            'value': 0
                                        }
                                duped_hsw = {}
                                for hsw_data in _hsw_obj.browse(cr, uid, emp_hsw_ids, context=context):
                                    if str(hsw_data.employee_id.cod_number) + ':' + str(hsw_data.period_id.id) not in duped_hsw:
                                        duped_hsw[str(hsw_data.employee_id.cod_number) + ':' + str(hsw_data.period_id.id)] = True
                                        check_date = hsw_data.period_id.date_start
                                        for k, day in days.iteritems():
                                            if day + ':' + str(hsw_data[day + '_hours_day_id'].id) in bonuses and check_date in days_from_period and check_date not in absences and check_date not in delayed_holidays[emp_code]:
                                                for bonus in bonuses[day + ':' + str(hsw_data[day + '_hours_day_id'].id)]:
                                                    if check_date not in holidays or bonus.holidays:
                                                        real_bonuses[bonus.concept_id.id]['value'] += bonus.qty
                                            check_date = (datetime.strptime(check_date, '%Y-%m-%d') + timedelta(days=1)).strftime('%Y-%m-%d')
                                # Getting diff between paid and real bonuses
                                for b in paid_bonuses:
                                    if real_bonuses[b]['value'] < paid_bonuses[b]:
                                        vc_lines[str(emp_code) + ':' + str(real_bonuses[b]['concept_id'])] = {
                                            'employee_id': hsw_data.employee_id.id,
                                            'concept_id': real_bonuses[b]['concept_id'],
                                            'type': 'qty',
                                            'value': paid_bonuses[b] - real_bonuses[b]['value'],
                                            'variable_concept_id': sheet
                                        }
            else:
                raise osv.except_osv(_('Error'), _('Turns variable concept sheets found for process %s, delete it to recalculate') % _hpr_obj.browse(cr, uid, process_id, context=context).name)
            if not vc_lines:
                raise osv.except_osv(_('Error'), _('No concepts were calculated for the process payrolls'))
            self._logger.info(_('On queue %d variable concept lines for creation') % len(vc_lines))
            for i in sorted(vc_lines):
                self.pool.get('hr.variable.concept.line').create(cr, uid, vc_lines[i], context=context)
            self.log(cr, uid, record.id, _('Turns concepts calculated for process %s') % record.hr_process_id.name)
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'hr.variable.concepts',
            'res_id': sheet,
            'target': 'self',
            'context': context,
        }


turns_variable_concepts()
