# -*- coding: utf-8 -*-
#!/usr/bin/env python
from osv import fields, osv
from tools.translate import _
from datetime import datetime

class sisb_turn_list(osv.osv_memory):
    def get_default_dept(self, cr, uid, context=None):
        th_group_model_data_id = self.pool.get('ir.model.data').search(cr, uid, [('name', '=', 'group_sisb_turn_list_th')])
        th_group = self.pool.get('ir.model.data').browse(cr, uid, th_group_model_data_id[0]).res_id
        groups = [group.id for group in self.pool.get('res.users').browse(cr, uid, uid).groups_id]
        if th_group in groups or uid == 1:
            return None
        else:
            emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)])
            if len(emp_id) < 1: raise osv.except_osv('ERROR', _('%s has no employee profile configured')%(self.pool.get('res.users').browse(cr, uid, uid).name))
            else: emp_id = emp_id[0]
            childs = [dept.id for dept in self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.child_ids]
            return self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.id if len(childs) == 0 else None

    def get_departments(self, cr, uid, context=None):
        th_group_model_data_id = self.pool.get('ir.model.data').search(cr, uid, [('name', '=', 'group_sisb_turn_list_th')])
        th_group = self.pool.get('ir.model.data').browse(cr, uid, th_group_model_data_id[0]).res_id
        groups = [group.id for group in self.pool.get('res.users').browse(cr, uid, uid).groups_id]
        if th_group in groups or uid == 1:
            return str(self.pool.get('hr.department').search(cr, uid, []))
        else:
            emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)])
            if len(emp_id) < 1: raise osv.except_osv('ERROR', _('%s has no employee profile configured')%(self.pool.get('res.users').browse(cr, uid, uid).name))
            else: emp_id = emp_id[0]
            childs = [dept.id for dept in self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.child_ids]
            pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
            while len(pending_childs) > 0:
                next_round = []
                for child in pending_childs:
                    childs += [c.id for c in child.child_ids]
                    next_round += [c.id for c in child.child_ids]
                pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
            childs.append(self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.id)
            return str(childs)

    _name = 'sisb.turn.list'
    _columns = {
        'period_id': fields.many2one('hr.period', 'Week', required=True),
        'department_id': fields.many2one('hr.department', 'Department', required=True),
        'depts_ids': fields.one2many('sisb.turn.list.depts', 'list_id', 'Departments'),
        'child_departments': fields.text('Allowed departments'),
        'allowed_weeks': fields.text('Allowed weeks')
    }
    _defaults = {
        'child_departments': lambda self, cr, uid, context=None: self.get_departments(cr, uid, context=context),
        'allowed_weeks': lambda self, cr, uid, context=None: str(self.pool.get('hr.period').search(cr, uid, [('date_start', '<=', datetime.now()), ('type', '=', 'diario'), ('date_start', '>', '2017-06-18')], context=context)),
        'period_id': lambda self, cr, uid, context=None: self.pool.get('hr.period').search(cr, uid, [('type', '=', 'diario'), ('date_start', '<=', datetime.now()), ('date_end', '>=', datetime.now())], context=context)[0],
        'department_id': lambda self, cr, uid, context=None: self.get_default_dept(cr, uid, context=context)
    }

    def print_results(self, cr, uid, ids, context=None):
        query = """
            SELECT hsw_id
            FROM (
                SELECT hsw.employee_id as emp_id, hsw.id as hsw_id,
                ROW_NUMBER() OVER(PARTITION BY hsw.employee_id) rn
                FROM
                    hr_shift_week as hsw
                    join hr_employee as he on hsw.employee_id = he.id
                    join hr_department as hd on he.department_id = hd.id
                WHERE
                    hd.id = %s and
                    he.state not in ('egresado','borrador') and
                    hsw.period_id = %s
            ) as sub
            WHERE rn = 1
        """
        for data in self.browse(cr, uid, ids, context=context):
            department, week = data.department_id.id, data.period_id.id
            childs = [department]
            pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
            while len(pending_childs) > 0:
                next_round = []
                for child in pending_childs:
                    childs += [c.id for c in child.child_ids]
                    next_round += [c.id for c in child.child_ids]
                pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
            for dept in childs:
                # We are not showing depts without employees
                if len(self.pool.get('hr.employee').search(cr, uid, [('department_id', '=', dept),('state', 'not in', ['borrador', 'egresado'])])) > 0:
                    dept_line_id = self.pool.get('sisb.turn.list.depts').create(cr, uid, {'list_id': data.id, 'department_id': dept})
                    cr.execute(query%(dept, week))
                    shift_week_ids = cr.fetchall()
                    shift_week_ids = [sw[0] for sw in shift_week_ids]
                    for hsw_brw in self.pool.get('hr.shift.week').browse(cr, uid, shift_week_ids, context=context):
                        self.pool.get('sisb.turn.list.depts').write(cr, uid, [dept_line_id],{
                        'lines_ids': [(0, 0, {
                            'dept_id': dept_line_id,
                            'employee_id': hsw_brw.employee_id.id,
                            'shift_week_id': hsw_brw.id
                        })]}, context=context)
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'sisb.turn.list'
        }

sisb_turn_list()

class sisb_turn_list_depts(osv.osv_memory):
    _name = 'sisb.turn.list.depts'
    _columns = {
        'list_id': fields.many2one('sisb.turn.list', 'Report ID'),
        'department_id': fields.many2one('hr.department', 'Department'),
        'lines_ids': fields.one2many('sisb.turn.list.lines', 'dept_id', 'Employees'),
    }

sisb_turn_list_depts()

class sisb_turn_list_lines(osv.osv_memory):
    _name = 'sisb.turn.list.lines'
    _columns = {
        'dept_id': fields.many2one('sisb.turn.list.depts', 'Department Line ID'),
        'employee_id': fields.many2one('hr.employee', 'Employee'),
        'shift_week_id': fields.many2one('hr.shift.week', 'Turn')
    }

sisb_turn_list_lines()

class hr_period(osv.osv):
    # Inherit over search from hr.period to allow ordering in the period view for sisb_turn_list_form
    _inherit = 'hr.period'
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context if context else {}
        return super(hr_period, self).search(cr, uid, args, offset=offset, limit=limit, order=context.get('order_period',order), context=context, count=count)

hr_period()