# -*- coding: utf-8 -*-
#!/usr/bin/env python
from osv import fields, osv
from tools.translate import _
from datetime import datetime, timedelta
import json

class sisb_turn_swap(osv.osv):
    def get_default_dept(self, cr, uid, context=None):
        th_group_model_data_id = self.pool.get('ir.model.data').search(cr, uid, [('name', '=', 'group_sisb_turn_swap_th')])
        th_group = self.pool.get('ir.model.data').browse(cr, uid, th_group_model_data_id[0]).res_id
        groups = [group.id for group in self.pool.get('res.users').browse(cr, uid, uid).groups_id]
        if th_group in groups or uid == 1:
            return None
        else:
            emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)])
            if len(emp_id) < 1: raise osv.except_osv('ERROR', _('%s has no employee profile configured')%(self.pool.get('res.users').browse(cr, uid, uid).name))
            else: emp_id = emp_id[0]
            childs = [dept.id for dept in self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.child_ids]
            return self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.id if len(childs) == 0 else None

    def get_departments(self, cr, uid, context=None):
        th_group_model_data_id = self.pool.get('ir.model.data').search(cr, uid, [('name', '=', 'group_sisb_turn_swap_th')])
        th_group = self.pool.get('ir.model.data').browse(cr, uid, th_group_model_data_id[0]).res_id
        groups = [group.id for group in self.pool.get('res.users').browse(cr, uid, uid).groups_id]
        if th_group in groups or uid == 1:
            return str(self.pool.get('hr.department').search(cr, uid, []))
        else:
            emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)])
            if len(emp_id) < 1: raise osv.except_osv('ERROR', _('%s has no employee profile configured')%(self.pool.get('res.users').browse(cr, uid, uid).name))
            else: emp_id = emp_id[0]
            childs = [dept.id for dept in self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.child_ids]
            pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
            while len(pending_childs) > 0:
                next_round = []
                for child in pending_childs:
                    childs += [c.id for c in child.child_ids]
                    next_round += [c.id for c in child.child_ids]
                pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
            childs.append(self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.id)
            return str(childs)

    def _check_duplicates_draft(self, cr, uid, ids, context=None):
        for record in self.browse(cr, uid, ids, context=context):
            if len(self.search(cr, uid, [('state', '=', 'draft'), ('ref', '=', 'CT-#'), ('department', '=', record.department.id), ('requester', '=', record.requester.id)], context=context)) > 1: return False
        return True

    def _get_departments(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        for i in ids:
            th_group_model_data_id = self.pool.get('ir.model.data').search(cr, uid, [('name', '=', 'group_sisb_turn_swap_th')])
            th_group = self.pool.get('ir.model.data').browse(cr, uid, th_group_model_data_id[0]).res_id
            groups = [group.id for group in self.pool.get('res.users').browse(cr, uid, uid).groups_id]
            if th_group in groups or uid == 1:
                res[i] = str(self.pool.get('hr.department').search(cr, uid, []))
            else:
                emp_id = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)])
                if len(emp_id) < 1: raise osv.except_osv('ERROR', _('%s has no employee profile configured')%(self.pool.get('res.users').browse(cr, uid, uid).name))
                else: emp_id = emp_id[0]
                childs = [dept.id for dept in self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.child_ids]
                pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
                while len(pending_childs) > 0:
                    next_round = []
                    for child in pending_childs:
                        childs += [c.id for c in child.child_ids]
                        next_round += [c.id for c in child.child_ids]
                    pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
                childs.append(self.pool.get('hr.employee').browse(cr, uid, emp_id).department_id.id)
                res[i] = str(childs)
        return res

    def _check_lines_set(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = True if len(record.swap_lines) > 0 else False
        return res

    _name = "sisb.turn.swap"
    _description = "Turn Swap"
    _rec_name = "ref"
    _order = 'id desc'
    _columns = {
        'ref': fields.char('Ref', size=32, readonly=True, help="Reference number"),
        'requester': fields.many2one('res.users', 'Requester', readonly=True, help="User who requested the swap"),
        'department': fields.many2one('hr.department', 'Department', required=True, help="Department where the employees belong"),
        'state': fields.selection([('draft', 'Draft'), ('confirmed', 'Confirmed'), ('rejected', 'Rejected'), ('approved', 'Approved')], 'State', readonly=True, help="State of this line"),
        'confirmed_by': fields.many2one('res.users', 'Confirmed by', readonly=True, help="User who confirmed this line"),
        'approved_by': fields.many2one('res.users', 'Approved by', readonly=True, help="User who approved this line"),
        'rejected_by': fields.many2one('res.users', 'Rejected by', readonly=True, help="User who rejected this line"),
        'swap_lines': fields.one2many('sisb.turn.swap.lines', 'swap_id', 'Swaps lines'),
        'child_departments': fields.function(_get_departments, string="Child Departments", type="text", object="hr.department", method=True, store=False),
        'lines_set': fields.function(_check_lines_set, string="Check if lines are set", type="boolean", method=True, store=False)
    }
    _defaults = {
        'ref': lambda *a: 'CT-#',
        'requester': lambda self, cr, uid, context: uid,
        'state': lambda *a: 'draft',
        'child_departments': lambda self, cr, uid, context=None: self.get_departments(cr, uid, context=context),
        'department': lambda self, cr, uid, context=None: self.get_default_dept(cr, uid, context=context),
        'lines_set': lambda *a: False
    }
    _constraints = [
        (_check_duplicates_draft, 'You already have a document in draft state for this department', ['department'])
    ]

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        context = context if context else {}
        if 'no_filter' in context: return super(sisb_turn_swap, self).search(cr, uid, args, context=context)
        th_group_model_data_id = self.pool.get('ir.model.data').search(cr, uid, [('name', '=', 'group_sisb_turn_swap_th')])
        th_group = self.pool.get('ir.model.data').browse(cr, uid, th_group_model_data_id[0]).res_id
        groups = [group.id for group in self.pool.get('res.users').browse(cr, uid, uid).groups_id]
        if th_group not in groups and uid != 1:
            manager_emp_ids = self.pool.get('hr.employee').search(cr, uid, [('user_id', '=', uid)], context=context)
            if len(manager_emp_ids) < 1: raise osv.except_osv('ERROR', _('%s has no employee profile configured')%(self.pool.get('res.users').browse(cr, uid, uid).name))
            childs = [dept.id for dept in self.pool.get('hr.employee').browse(cr, uid, manager_emp_ids[0]).department_id.child_ids]
            pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
            while len(pending_childs) > 0:
                next_round = []
                for child in pending_childs:
                    childs += [c.id for c in child.child_ids]
                    next_round += [c.id for c in child.child_ids]
                pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
            childs.append(self.pool.get('hr.employee').browse(cr, uid, manager_emp_ids[0]).department_id.id)
            arg = 'department', 'in', childs
            args.append(arg)
        return super(sisb_turn_swap, self).search(cr, uid, args, context=context)

    def validate_lines(self, cr, uid, swap_lines, context=None):
        for line_data in swap_lines:
            # Rejecting lines where swaping with another employee outside current week
            today_00_00 = datetime.strptime(datetime.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
            if line_data.swaped_with and datetime.strptime(line_data.since, '%Y-%m-%d') < today_00_00 - timedelta(days=today_00_00.weekday()):
                return _('You cant swap %s with %s. Swaps between employees outside current week are not allowed')%(line_data.employee.name, line_data.swaped_with.name)
            # Checking if employee has overtime lines for the days set
            if line_data.date_to:
                set_a = self.pool.get('horas.extras.planificacion.lineas').search(cr, uid, [('empleado_id', '=', line_data.employee.id), ('fecha_desde', '<=', line_data.date_to), ('fecha_hasta', '>=', line_data.date_to)])
                set_b = self.pool.get('horas.extras.planificacion.lineas').search(cr, uid, [('empleado_id', '=', line_data.employee.id), ('fecha_hasta', '>=', line_data.since), ('fecha_hasta', '<=', line_data.date_to)])
                set_c = self.pool.get('horas.extras.planificacion.lineas').search(cr, uid, [('empleado_id', '=', line_data.employee.id), ('fecha_desde', '>=', line_data.since), ('fecha_hasta', '<=', line_data.date_to)])
                if len(set_a) + len(set_b) + len(set_c) > 0:
                    return _('The employee %s has overtime lines for the period set')%(line_data.employee.name)
            else:
                if len(self.pool.get('horas.extras.planificacion.lineas').search(cr, uid, [('empleado_id', '=', line_data.employee.id), ('fecha_hasta', '>=', line_data.since)])) > 0:
                    return _('The employee %s has overtime lines for the period set')%(line_data.employee.name)
            # ------ End of validations with overtime ------------
            # Checking if employees still belong to the current department
            if line_data.employee.department_id.id != line_data.swap_id.department.id:
                return _('%s does not belong to the document department')%(line_data.employee.name)
            elif line_data.swaped_with:
                if line_data.swaped_with.department_id.id != line_data.swap_id.department.id:
                    return _('%s does not belong to the document department')%(line_data.swaped_with.name)
            # Now validating that is a good day for swaping (based on their payrolls)
            # Checking first if today is a good day to swap employees (based on their payrolls)
            # Employees payroll type
            emp_nom_type = line_data.employee.hr_payroll_id.hr_payment_calendar_id.type
            swp_nom_type = line_data.swaped_with.hr_payroll_id.hr_payment_calendar_id.type if line_data.swaped_with else False
            # Employees allowed days and hours to make swaps
            emp_allowed_days, emp_allowed_hours = line_data.employee.hr_payroll_id.swap_days, line_data.employee.hr_payroll_id.swap_hours
            swp_allowed_days = line_data.swaped_with.hr_payroll_id.swap_days if line_data.swaped_with else 0
            swp_allowed_hours  = line_data.swaped_with.hr_payroll_id.swap_hours if line_data.swaped_with else 0
            # Employees actual period
            emp_period_id = self.pool.get('hr.period').search(cr, uid, [('type', '=', emp_nom_type), ('state', '=', 'active')])
            swp_period_id = self.pool.get('hr.period').search(cr, uid, [('type', '=', swp_nom_type), ('state', '=', 'active')]) if swp_nom_type else False
            # Checking the previous period from 'employee' field
            emp_period = self.pool.get('hr.period').browse(cr, uid, emp_period_id[0])
            if emp_period.period_in_year > 1:
                emp_period = self.pool.get('hr.period').browse(cr, uid, emp_period.id - 1)
                if datetime.strptime(line_data.since, '%Y-%m-%d') <= datetime.strptime(emp_period.date_end, '%Y-%m-%d'):
                    if datetime.strptime(emp_period.date_end, '%Y-%m-%d') + timedelta(days=(emp_allowed_days+1), hours=emp_allowed_hours) < datetime.now():
                        return _('You cant swap %s because you exceded the maximum days allowed to make swaps through his/her payroll and for the period set')%(line_data.employee.name)
            # Checking the previous period from 'swaped_with' field
            swp_period = self.pool.get('hr.period').browse(cr, uid, swp_period_id[0]) if swp_period_id else False
            if swp_period:
                if swp_period.period_in_year > 1:
                    swp_period = self.pool.get('hr.period').browse(cr, uid, swp_period.id - 1)
                    if datetime.strptime(line_data.since, '%Y-%m-%d') <= datetime.strptime(swp_period.date_end, '%Y-%m-%d'):
                        if datetime.strptime(swp_period.date_end, '%Y-%m-%d') + timedelta(days=(swp_allowed_days+1), hours=swp_allowed_hours) < datetime.now():
                            return _('You cant swap %s because you exceded the maximum days allowed to make swaps through his/her payroll and for the period set')%(line_data.swaped_with.name)
            # ----------- End of payrolls allowed days to swap -----------
        return False

    def confirm(self, cr, uid, ids, context=None):
        if context is None: context = {}
        for data in self.browse(cr, uid, ids):
            if data.state == 'draft':
                if len(data.swap_lines) == 0:
                    raise osv.except_osv('ERROR', _('There are no lines loaded'))
                error = self.validate_lines(cr, uid, data.swap_lines, context=context)
                if error: raise osv.except_osv('ERROR', error)
                if data.ref == 'CT-#':
                    sequence = self.pool.get('ir.sequence').get(cr, uid, 'sisb.turn.swap')
                else:
                    sequence = data.ref
                self.write(cr, uid, data.id, {'state': 'confirmed', 'confirmed_by': uid, 'ref': sequence})
            else:
                return False
        return False

    def approve(self, cr, uid, ids, context=None):
        if context is None: context = {}
        weekday = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday']
        for data in self.browse(cr, uid, ids):
            if data.state == 'confirmed':
                error = self.validate_lines(cr, uid, data.swap_lines, context=context)
                if error: raise osv.except_osv('ERROR', error)
                for line_data in data.swap_lines:
                    date_start = datetime.strptime(line_data.since, '%Y-%m-%d')
                    date_end = datetime.strptime(line_data.date_to, '%Y-%m-%d') if line_data.date_to else datetime.now() + timedelta(days=(6 - datetime.now().weekday()))
                    date_actual = date_start
                    today = datetime.strptime(datetime.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
                    emp_nom_type = line_data.employee.hr_payroll_id.hr_payment_calendar_id.type
                    swp_nom_type = line_data.swaped_with.hr_payroll_id.hr_payment_calendar_id.type if line_data.swaped_with else False
                    start_date = datetime.now() + timedelta(days=365)
                    period_ids = self.pool.get('hr.period').search(cr, uid, [('state', '=', 'active')], context=context)
                    for period in self.pool.get('hr.period').browse(cr, uid, period_ids, context=context):
                        if period.period_in_year > 1:
                            period = self.pool.get('hr.period').browse(cr, uid, period.id - 1)
                        period_date = datetime.strptime(period.date_start, '%Y-%m-%d')
                        start_date = period_date if period_date < start_date else start_date
                    if date_start < start_date:
                        raise osv.except_osv('ERROR', _('This line is outdated and cannot be approved, it can only be rejected'))
                    # We gotta get the actual turn sequence
                    for rotation_line in line_data.new_rota.rotation_line_ids:
                        if line_data.new_turn.id == rotation_line.shift_id.id:
                            next_turn_sequence = rotation_line
                            break
                    # Also the past turn sequence
                    for rotation_line in line_data.past_rota.rotation_line_ids:
                        if line_data.past_turn.id == rotation_line.shift_id.id:
                            past_turn_sequence = rotation_line
                            break
                    first_monday = True
                    while date_actual <= date_end:
                        period_id = self.pool.get('hr.period').search(cr, uid, ['&', ('date_start', '<=', date_actual.strftime('%Y-%m-%d')), ('date_end', '>=', date_actual.strftime('%Y-%m-%d'))], context=context)
                        if len(period_id) < 1:
                            raise osv.except_osv('ERROR', _('There is no period created for the day: %s.')%(date_actual.strftime('%d-%m-%Y')))
                        # We get the shift week for the employee
                        shift_week_ids_emp = self.pool.get('hr.shift.week').search(cr, uid, ['&', ('employee_id', '=', line_data.employee.id), ('period_id', 'in', period_id)], context=context)
                        if len(shift_week_ids_emp) < 1:
                            raise osv.except_osv('ERROR', _('%s has no shift week lines for the day: %s.')%(line_data.employee.name, date_actual.strftime('%d-%m-%Y')))
                        # If is monday, but not the first monday, change to the new sequence
                        if date_actual.weekday() == 0 and not first_monday:
                            rotation_line_found = False
                            for rotation_line in line_data.new_rota.rotation_line_ids:
                                if rotation_line.sequence > next_turn_sequence.sequence:
                                    rotation_line_found = rotation_line
                                    break
                            next_turn_sequence = rotation_line_found if rotation_line_found else line_data.new_rota.rotation_line_ids[0]
                        new_turn_obj = self.pool.get('hr.shift').read(cr, uid, next_turn_sequence.shift_id.id, context=context)
                        # If swaped with another employee
                        if line_data.swaped_with:
                            # Also shift week for swaped with
                            past_turn_obj = self.pool.get('hr.shift.week').read(cr, uid, shift_week_ids_emp[0], context=context)
                            shift_week_ids_swp = self.pool.get('hr.shift.week').search(cr, uid, ['&', ('employee_id', '=', line_data.swaped_with.id), ('period_id', 'in', period_id)], context=context)
                            if len(shift_week_ids_swp) < 1:
                                raise osv.except_osv('ERROR', _('%s has no shift week lines for the day: %s.')%(line_data.swaped_with.name, date_actual.strftime('%d-%m-%Y')))
                            new_turn_obj = self.pool.get('hr.shift.week').read(cr, uid, shift_week_ids_swp[0], context=context)
                            swp_values = {
                                weekday[date_actual.weekday()]+'_hours_day_id': past_turn_obj[weekday[date_actual.weekday()]+'_hours_day_id'][0],
                                'swaps': [(4, line_data.swap_id.id)]
                            }
                            # Saving swaps for swaped_with employee
                            self.pool.get('hr.shift.week').write(cr, uid, shift_week_ids_swp, swp_values, context=context)
                        emp_values = {
                            weekday[date_actual.weekday()]+'_hours_day_id': new_turn_obj[weekday[date_actual.weekday()]+'_hours_day_id'][0],
                            'swaps': [(4, line_data.swap_id.id)]
                        }
                        # Saving swaps for employee
                        self.pool.get('hr.shift.week').write(cr, uid, shift_week_ids_emp, emp_values, context=context)
                        date_actual += timedelta(days=1)
                        first_monday = False
                    if not line_data.date_to:
                        context['is_a_swap'] = True
                        self.pool.get('hr.employee').write(cr, uid, line_data.employee.id, {
                            'shift_id': next_turn_sequence.shift_id.id,
                            'rotation_id': line_data.new_rota.id,
                            'sequence': next_turn_sequence.sequence
                            }, context=context)
                        if line_data.swaped_with:
                            self.pool.get('hr.employee').write(cr, uid, line_data.swaped_with.id, {
                                'shift_id': past_turn_sequence.shift_id.id,
                                'rotation_id': line_data.past_rota.id,
                                'sequence': past_turn_sequence.sequence
                                }, context=context)
                self.write(cr, uid, data.id, {'state': 'approved', 'approved_by': uid}, context=context)
            else:
                return False
        return False

    def back_draft(self, cr, uid, ids, context=None):
        for data in self.browse(cr, uid, ids):
            if data.state == 'confirmed':
                self.write(cr, uid, data.id, {'state': 'draft', 'confirmed_by': None}, context=context)
        return False

    def reject(self, cr, uid, ids, context=None):
        for data in self.browse(cr, uid, ids):
            if data.state == 'confirmed':
                self.write(cr, uid, data.id, {'state': 'rejected', 'rejected_by': uid}, context=context)
        return False

    def unlink(self, cr, uid, ids, context=None):
        for record in self.browse(cr, uid, ids):
            if record.ref == 'CT-#':
                if record.requester.id == uid:
                    super(sisb_turn_swap, self).unlink(cr, uid, ids, context=context)
                else:
                    raise osv.except_osv('ERROR', _('You cannot delete this document because it was created by %s (Only requester may delete their own documents)')%(record.requester.name))
            else:
                raise osv.except_osv('ERROR', _('Documents with ref number confirmed cannot be deleted'))
        return False

    def copy(self, cr, uid, id, defaults, context=None):
        raise osv.except_osv('ERROR', _('You cannot duplicate lines'))

    def onchange_lines(self, cr, uid, ids, swap_lines, context=None):
        if len(swap_lines) > 0:
            return {
                'value': {
                    'lines_set': True
                }
            }
        else:
            return {
                'value': {
                    'lines_set': False
                }
            }

sisb_turn_swap()

class sisb_turn_swap_lines(osv.osv):
    def get_departments(self, cr, uid, context=None):
        context = context if context else {}
        dept_id = context.get('dept', False)
        if not dept_id: return '[]'
        childs = [dept.id for dept in self.pool.get('hr.department').browse(cr, uid, dept_id).child_ids]
        pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
        while len(pending_childs) > 0:
            next_round = []
            for child in pending_childs:
                childs += [c.id for c in child.child_ids]
                next_round += [c.id for c in child.child_ids]
            pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
        childs.append(dept_id)
        return str(childs)

    def _get_departments(self, cr, uid, ids, field_name, args, context=None):
        context = context if context else {}
        dept_id = context.get('dept', False)
        res = {}
        if not dept_id:
            for i in ids:
                res[i] = '[]'
            return res
        for i in ids:
            childs = [dept.id for dept in self.pool.get('hr.department').browse(cr, uid, dept_id).child_ids]
            pending_childs = self.pool.get('hr.department').browse(cr, uid, childs, context=context)
            while len(pending_childs) > 0:
                next_round = []
                for child in pending_childs:
                    childs += [c.id for c in child.child_ids]
                    next_round += [c.id for c in child.child_ids]
                pending_childs = self.pool.get('hr.department').browse(cr, uid, next_round, context=context)
            childs.append(dept_id)
            res[i] = str(childs)
        return res

    def _get_turn_domain(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = str([rota.shift_id.id for rota in record.new_rota.rotation_line_ids])
        return res

    def get_past_rota(self, cr, uid, ids, field, args, context=None):
        res = {}
        for record in self.browse(cr, uid, ids, context):
            res[record.id] = record.employee.rotation_id.id
        return res

    def get_past_turn(self, cr, uid, ids, field, args, context=None):
        res = {}
        for record in self.browse(cr, uid, ids, context):
            res[record.id] = record.employee.shift_id.id
        return res

    def check_expired(self, cr, uid, ids, context=None):
        data = self.browse(cr, uid, ids[0], context)
        # Checking first if today is a good day to swap employees (based on their payrolls)
        # Employees payroll type
        emp_nom_type = data.employee.hr_payroll_id.hr_payment_calendar_id.type
        swp_nom_type = data.swaped_with.hr_payroll_id.hr_payment_calendar_id.type if data.swaped_with else False
        # Employees allowed days and hours to make swaps
        emp_allowed_days, emp_allowed_hours = data.employee.hr_payroll_id.swap_days, data.employee.hr_payroll_id.swap_hours
        swp_allowed_days = data.swaped_with.hr_payroll_id.swap_days if data.swaped_with else 0
        swp_allowed_hours  = data.swaped_with.hr_payroll_id.swap_hours if data.swaped_with else 0
        # Employees actual period
        emp_period_id = self.pool.get('hr.period').search(cr, uid, [('type', '=', emp_nom_type), ('state', '=', 'active')])
        swp_period_id = self.pool.get('hr.period').search(cr, uid, [('type', '=', swp_nom_type), ('state', '=', 'active')]) if swp_nom_type else False
        # Checking the previous period from 'employee' field
        emp_period = self.pool.get('hr.period').browse(cr, uid, emp_period_id[0])
        if emp_period.period_in_year > 1:
            emp_period = self.pool.get('hr.period').browse(cr, uid, emp_period.id - 1)
            if datetime.strptime(data.since, '%Y-%m-%d') <= datetime.strptime(emp_period.date_end, '%Y-%m-%d'):
                if datetime.strptime(emp_period.date_end, '%Y-%m-%d') + timedelta(days=(emp_allowed_days+1), hours=emp_allowed_hours) < datetime.now():
                    return False
        # Checking the previous period from 'swaped_with' field
        swp_period = self.pool.get('hr.period').browse(cr, uid, swp_period_id[0]) if swp_period_id else False
        if swp_period:
            if swp_period.period_in_year > 1:
                swp_period = self.pool.get('hr.period').browse(cr, uid, swp_period.id - 1)
                if datetime.strptime(data.since, '%Y-%m-%d') <= datetime.strptime(swp_period.date_end, '%Y-%m-%d'):
                    if datetime.strptime(swp_period.date_end, '%Y-%m-%d') + timedelta(days=(swp_allowed_days+1), hours=swp_allowed_hours) < datetime.now():
                        return False
        return True

    def check_dates(self, cr, uid, ids, context=None):
        # Checking if the days set to swap are within their previous periods
        data = self.browse(cr, uid, ids[0], context)
        emp_nom_type = data.employee.hr_payroll_id.hr_payment_calendar_id.type
        swp_nom_type = data.swaped_with.hr_payroll_id.hr_payment_calendar_id.type if data.swaped_with else False
        start_date = datetime.now() - timedelta(days=365)
        period_ids = self.pool.get('hr.period').search(cr, uid, [('type', 'in', [emp_nom_type, swp_nom_type]), ('state', '=', 'active')])
        for period in self.pool.get('hr.period').browse(cr, uid, period_ids):
            # We need the lastest period before the current active one
            if period.period_in_year > 1:
                period = self.pool.get('hr.period').browse(cr, uid, period.id - 1)
            period_date = datetime.strptime(period.date_start, '%Y-%m-%d')
            start_date = period_date if period_date > start_date else start_date
        today = datetime.strptime(datetime.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
        if datetime.strptime(data.since, '%Y-%m-%d') < start_date or datetime.strptime(data.since, '%Y-%m-%d') > today + timedelta(days = 6 - today.weekday()):
            return False
        if not data.date_to:
            return True
        elif datetime.strptime(data.date_to, '%Y-%m-%d') > today + timedelta(days = 6 - today.weekday()):
            return False
        else:
            return True

    def check_since_greater(self, cr, uid, ids, context=None):
        data = self.browse(cr, uid, ids[0], context)
        if not data.date_to: return True
        if datetime.strptime(data.since, '%Y-%m-%d') > datetime.strptime(data.date_to, '%Y-%m-%d'):
            return False
        else:
            return True

    def check_departments(self, cr, uid, ids, context=None):
        data = self.browse(cr, uid, ids[0], context)
        if not data.swaped_with: return True
        return True if data.employee.department_id == data.swaped_with.department_id else False

    def check_employees(self, cr, uid, ids, context=None):
        data = self.browse(cr, uid, ids[0], context)
        if data.employee == data.swaped_with: return False
        else: return True

    def is_swaped_set(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = True if record.swaped_with else False
        return res

    _name = "sisb.turn.swap.lines"
    _description = "Turn swap lines"
    _rec_name = "employee"
    _columns = {
        'swap_id': fields.many2one('sisb.turn.swap', 'Swap Document'),
        'type': fields.selection([('rota', 'Rotation'), ('turn', 'Turn'), ('emp', 'Employee')], 'Type', readonly=True, help="Describes the type of swap that is being applied"),
        'employee': fields.many2one('hr.employee', 'Employee', required=True),
        'swaped_with': fields.many2one('hr.employee', 'Swap with'),
        'since': fields.date('Since', required=True),
        'date_to': fields.date('To'),
        'reason': fields.char('Reason', size=128, help="Short explanation of the reason for this swap"),
        'past_rota': fields.many2one('hr.rotation', 'Actual Rotation', readonly=True, help="Rotation that the employee have or had before this line was approved"),
        'past_turn': fields.many2one('hr.shift', 'Actual Turn', readonly=True, help="Turn that the employee have or had before this line was approved"),
        'new_rota': fields.many2one('hr.rotation', 'New Rotation', required=True, help="Rotation that the employee will have for the duration set"),
        'new_turn': fields.many2one('hr.shift', 'New Turn', required=True, help="Turn that the employee will have for the duration set"),
        'child_departments': fields.function(_get_departments, string="Child Departments", type="text", object="hr.department", method=True, store=False),
        'turn_domain': fields.function(_get_turn_domain, string="Turn Domain", type="text", object="hr.shift", method=True, store=False),
        'swaped_set': fields.function(is_swaped_set, string="Check if swaped set", type="boolean", method=True, store=False)
    }
    _defaults = {
        'child_departments': lambda self, cr, uid, context=None: self.get_departments(cr, uid, context),
        'turn_domain': lambda *a: str([]),
        'swaped_set': lambda *a: False
    }
    _constraints = [
        (check_dates, _('Dates must belong to this week or the last period from the employees nom'), ['since','date_to']),
        (check_since_greater, _('"Since" date cannot be greater than "To" date'), ['since','date_to']),
        (check_employees, _('Both employees fields have the same employee'), ['new_turn']),
        (check_departments, _('Both employees must belong to the same department'), ['employee','swaped_with']),
        (check_expired, _('You cant swap one of the employees because you exceded the maximum days allowed to make swaps through his/her payroll and for the period set'), ['since','date_to'])
    ]

    def onchange_employee(self, cr, uid, ids, emp_id, context=None):
        if not emp_id:
            return {
                'value': {
                    'past_turn': None,
                    'past_rota': None
                }
            }
        employee = self.pool.get('hr.employee').browse(cr, uid, emp_id, context=context)
        if not employee.rotation_id or not employee.shift_id:
            return {
                'value': {
                    'employee': None,
                    'past_turn': None,
                    'past_rota': None
                },
                'warning': {
                    'title': 'ERROR',
                    'message': _('%s must have both rotation and turn set on his profile')%(employee.name)
                }
            }
        return {
            'value': {
                'past_turn': employee.shift_id.id,
                'past_rota': employee.rotation_id.id
            }
        }

    def onchange_swaped_with(self, cr, uid, ids, emp_id, context=None):
        if not emp_id:
            return {
                'value': {
                    'new_turn': None,
                    'new_rota': None,
                    'swaped_set': False
                },
                'domain': {
                    'new_turn': [('id', 'in', [])]
                }
            }
        employee = self.pool.get('hr.employee').browse(cr, uid, emp_id, context=context)
        if not employee.rotation_id or not employee.shift_id:
            return {
                'value': {
                    'swaped_with': None,
                    'new_turn': None,
                    'new_rota': None,
                    'swaped_set': False
                },
                'domain': {
                    'new_turn': [('id', 'in', [])]
                },
                'warning': {
                    'title': 'ERROR',
                    'message': _('%s must have both rotation and turn set on his profile')%(employee.name)
                }
            }
        shift_ids = [rota.shift_id.id for rota in employee.rotation_id.rotation_line_ids]
        return {
            'value': {
                'new_turn': employee.shift_id.id,
                'new_rota': employee.rotation_id.id,
                'swaped_set': True
            },
            'domain': {
                'new_turn': [('id', 'in', shift_ids)]
            }
        }

    def onchange_rota(self, cr, uid, ids, rota_id, emp_id, context=None):
        if emp_id:
            return {
                'warning': {
                    'title': 'ERROR',
                    'message': _('You should not change this field if you are swaping with another employee')
                },
                'value': {
                    'swaped_with': None,
                    'new_turn': None,
                    'new_rota': None
                },
                'domain': {
                    'new_turn': [('id', 'in', [])]
                }
            }
        if not rota_id:
            return {
                'value': {
                    'new_turn': None
                },
                'domain': {
                    'new_turn': [('id', 'in', [])]
                }
            }
        rota = self.pool.get('hr.rotation').browse(cr, uid, rota_id, context)
        shift_ids = [rota.shift_id.id for rota in rota.rotation_line_ids]
        new_turn = None if len(shift_ids) > 1 else None if len(shift_ids) == 0 else shift_ids[0]
        return {
            'value': {
                'new_turn': new_turn
            },
            'domain': {
                'new_turn': [('id', 'in', shift_ids)]
            }
        }

    def onchange_turn(self, cr, uid, ids, emp_id, context=None):
        if emp_id:
            return {
                'warning': {
                    'title': 'ERROR',
                    'message': _('You should not change this field if you are swaping with another employee')
                },
                'value': {
                    'swaped_with': None,
                    'new_turn': None,
                    'new_rota': None
                },
                'domain': {
                    'new_turn': [('id', 'in', [])]
                }
            }
        else:
            return True

    def create(self, cr, uid, args, context=None):
        today_00_00 = datetime.strptime(datetime.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
        # Checking if the employees exist in other doc
        employee = args['employee']
        swaped_with = args['swaped_with'] if args['swaped_with'] else None
        if args['swaped_with'] and datetime.strptime(args['since'], '%Y-%m-%d') < today_00_00 - timedelta(days=today_00_00.weekday()):
            emp_obj = self.pool.get('hr.employee')
            raise osv.except_osv('ERROR', _('You cant swap %s with %s. Swaps between employees outside current week are not allowed')%(emp_obj.browse(cr, uid, employee).name, emp_obj.browse(cr, uid, swaped_with).name))
        dupes_outside = []
        dupes_inside = []
        context['no_filter'] = 1 # We pass no filter to context so search wont filter by department
        for record in self.search(cr, uid, [('employee', '=', employee), ('swap_id.state', 'in', ['draft', 'confirmed'])], context=context):
            if self.browse(cr, uid, record).swap_id.id != args['swap_id']:
                dupes_outside.append(record)
            else:
                dupes_inside.append(record)
        for record in self.search(cr, uid, [('swaped_with', '=', employee), ('swap_id.state', 'in', ['draft', 'confirmed'])], context=context):
            if self.browse(cr, uid, record).swap_id.id != args['swap_id']:
                dupes_outside.append(record)
            else:
                dupes_inside.append(record)
        if swaped_with:
            for record in self.search(cr, uid, [('employee', '=', swaped_with), ('swap_id.state', 'in', ['draft', 'confirmed'])], context=context):
                if self.browse(cr, uid, record).swap_id.id != args['swap_id']:
                    dupes_outside.append(record)
                else:
                    dupes_inside.append(record)
            for record in self.search(cr, uid, [('swaped_with', '=', swaped_with), ('swap_id.state', 'in', ['draft', 'confirmed'])], context=context):
                if self.browse(cr, uid, record).swap_id.id != args['swap_id']:
                    dupes_outside.append(record)
                else:
                    dupes_inside.append(record)
        if len(dupes_outside) > 0:
            dupe = self.browse(cr, uid, dupes_outside[0], context).swap_id
            raise osv.except_osv('ERROR', _('One of the employees listed for swap has already a line in other document (Ref: %s, requester: %s) in draft or confirmed state')%(dupe.ref, dupe.requester.name))
        # Checking for duplicates inside the same document to not overlap between dates
        since = datetime.strptime(args['since'], '%Y-%m-%d')
        date_to = datetime.strptime(args['date_to'], '%Y-%m-%d') if args['date_to'] else datetime.now() + timedelta(days=6 - datetime.now().weekday())
        # We first iniciate a list with all days in the period since - date_to
        days_in_new_line = []
        actual_day = since
        while actual_day <= date_to:
            days_in_new_line.append(actual_day)
            actual_day += timedelta(days=1)
        for dupe in self.browse(cr, uid, dupes_inside, context=context):
            # Now, for each duplicate line, we get the days of that line period             
            days_in_current_line = []
            actual_day = datetime.strptime(dupe.since, '%Y-%m-%d')
            date_to = datetime.strptime(dupe.date_to, '%Y-%m-%d') if dupe.date_to else datetime.now() + timedelta(days=6 - datetime.now().weekday())
            while actual_day <= date_to:
                days_in_current_line.append(actual_day)
                actual_day += timedelta(days=1)
            # Next, we check if find matches in both lists
            if len(set(days_in_new_line) & set(days_in_current_line)) > 0:
                raise osv.except_osv('ERROR', _('A employees already has a line in this document which dates overlaps'))            
        # Reading data to save, also getting values from rotas and turns
        emp_obj = self.pool.get('hr.employee').browse(cr, uid, employee, context=context)
        if swaped_with:
            swp_obj = self.pool.get('hr.employee').browse(cr, uid, swaped_with, context=context)
            args['new_rota'] = swp_obj.rotation_id.id
            args['new_turn'] = swp_obj.shift_id.id
        args['past_rota'] = emp_obj.rotation_id.id
        args['past_turn'] = emp_obj.shift_id.id
        # Now saving the type of swap
        if swaped_with:
            args['type'] = 'emp'
        elif args['past_rota'] != args['new_rota']:
            args['type'] = 'rota'
        else:
            args['type'] = 'turn'
        return super(sisb_turn_swap_lines, self).create(cr, uid, args, context=context) 

    def write(self, cr, uid, ids, vals, context=None):
        ids = [ids] if type(ids) != list else ids
        # Checking if the employees exist in other doc
        for data in self.read(cr, uid, ids, context=context):
            data.update(vals)
            employee = data['employee'][0] if type(data['employee']) is tuple else data['employee']
            swaped_with = data['swaped_with'][0] if type(data['swaped_with']) is tuple else data['swaped_with']
            today_00_00 = datetime.strptime(datetime.now().strftime('%Y-%m-%d'), '%Y-%m-%d')
            if vals['swaped_with'] and datetime.strptime(vals['since'], '%Y-%m-%d') < today_00_00 - timedelta(days=today_00_00.weekday()):
                emp_obj = self.pool.get('hr.employee')
                raise osv.except_osv('ERROR', _('You cant swap %s with %s. Swaps between employees outside current week are not allowed')%(emp_obj.browse(cr, uid, employee).name, emp_obj.browse(cr, uid, swaped_with).name))
            dupes_outside = []
            dupes_inside = []
            context['no_filter'] = 1 # We pass no filter to context so search wont filter by department
            for record in self.search(cr, uid, [('employee', '=', employee), ('swap_id.state', 'in', ['draft', 'confirmed'])], context=context):
                if self.browse(cr, uid, record).swap_id.id != data['swap_id'][0]:
                    dupes_outside.append(record)
                elif self.browse(cr, uid, record).swap_id.id == data['swap_id'][0] and record != data['id']:
                    dupes_inside.append(record)
            for record in self.search(cr, uid, [('swaped_with', '=', employee), ('swap_id.state', 'in', ['draft', 'confirmed'])], context=context):
                if self.browse(cr, uid, record).swap_id.id != data['swap_id'][0]:
                    dupes_outside.append(record)
                elif self.browse(cr, uid, record).swap_id.id == data['swap_id'][0] and record != data['id']:
                    dupes_inside.append(record)
            if swaped_with:
                for record in self.search(cr, uid, [('employee', '=', swaped_with), ('swap_id.state', 'in', ['draft', 'confirmed'])], context=context):
                    if self.browse(cr, uid, record).swap_id.id != data['swap_id'][0]:
                        dupes_outside.append(record)
                    elif self.browse(cr, uid, record).swap_id.id == data['swap_id'][0] and record != data['id']:
                        dupes_inside.append(record)
                for record in self.search(cr, uid, [('swaped_with', '=', swaped_with), ('swap_id.state', 'in', ['draft', 'confirmed'])], context=context):
                    if self.browse(cr, uid, record).swap_id.id != data['swap_id'][0]:
                        dupes_outside.append(record)
                    elif self.browse(cr, uid, record).swap_id.id == data['swap_id'][0] and record != data['id']:
                        dupes_inside.append(record)
            if len(dupes_outside) > 0:
                dupe = self.browse(cr, uid, dupes_outside[0], context).swap_id
                raise osv.except_osv('ERROR', _('One of the employees listed for swap has already a line in other document (Ref: %s, requester: %s) in draft or confirmed state')%(dupe.ref, dupe.requester.name))
            # Checking for duplicates inside the same document to not overlap between dates
            since = datetime.strptime(data['since'], '%Y-%m-%d')
            # If there is no date_to, we set this week sunday as date_to
            date_to = datetime.strptime(data['date_to'], '%Y-%m-%d') if data['date_to'] else datetime.now() + timedelta(days=6 - datetime.now().weekday())
            # We first iniciate a list with all days in the period since - date_to
            days_in_new_line = []
            actual_day = since
            while actual_day <= date_to:
                days_in_new_line.append(actual_day)
                actual_day += timedelta(days=1)
            for dupe in self.browse(cr, uid, dupes_inside, context=context):
                # Now, for each duplicate line, we get the days of that line period                
                days_in_current_line = []
                actual_day = datetime.strptime(dupe.since, '%Y-%m-%d')
                date_to = datetime.strptime(dupe.date_to, '%Y-%m-%d') if dupe.date_to else datetime.now() + timedelta(days=6 - datetime.now().weekday())
                while actual_day <= date_to:
                    days_in_current_line.append(actual_day)
                    actual_day += timedelta(days=1)
                # Next, we check if find matches in both lists
                if len(set(days_in_new_line) & set(days_in_current_line)) > 0:
                    raise osv.except_osv('ERROR', _('A employees already has a line in this document which dates overlaps'))
        # Reading data to save, also getting values from rotas and turns
        emp_obj = self.pool.get('hr.employee').browse(cr, uid, employee, context=context)
        if swaped_with:
            swp_obj = self.pool.get('hr.employee').browse(cr, uid, swaped_with, context=context)
            vals['new_rota'] = swp_obj.rotation_id.id
            vals['new_turn'] = swp_obj.shift_id.id
        vals['past_rota'] = emp_obj.rotation_id.id
        vals['past_turn'] = emp_obj.shift_id.id
        if swaped_with:
            vals['type'] = 'emp'
        elif vals['past_rota'] != vals['new_rota']:
            vals['type'] = 'rota'
        else:
            vals['type'] = 'turn'
        return super(sisb_turn_swap_lines, self).write(cr, uid, ids, vals, context=context)
        
sisb_turn_swap_lines()